;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Pix (Blue), by Yoshicookiezeus
; Based on mikeyk's Spiky Ball sprite
;
; Description: Bounces around diagonally. Can only be hurt by blue Koopa shells.
;
; Uses first extra bit: NO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	RAM_SpriteDir   = $157C
	EXTRA_BITS = $7FAB10
	EXTRA_PROP_1 = $7FAB28
	NEW_SPRITE_NUM = $7FAB9E    ;0C bytes   custom sprite number           

	Hit_Counter = $1528			; The hit counter for any sprite that takes lots of hits to kill
	GetSpriteClippingA = $03B69F
	CheckForContact = $03B72B

	GetSpriteClippingB	= $03B6E5	
	ShowSprContactGfx = $01AB72

	Invuln_Timer = $1564		; The RAM address for Acro's stun duration
	Acro_Hits = $05			; This is how many hits it takes to kill Acro (default == 3)
			
	SPRITE_TO_GEN		dcb $57,$46,$4D 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "INIT"
			RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "MAIN"                                    
			PHB			; \
			PHK			;  | main sprite function, just calls local subroutine
			PLB			;  |
			JSR SPRITE_CODE_START	;  |
			PLB			;  |
			RTL			; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TIME_TO_SHOW = $22      ; time to display the sprite before it is thrown

TIME_TILL_THROW:	dcb $92,$92,$92,$F7     ; change it to dcb $F7,$62 so it throws two times

X_SPEED			dcb $10,$E0
Y_SPEED			dcb $10,$E0

RETURN			RTS
SPRITE_CODE_START	JSR SUB_GFX		; graphics routine
			LDA $14C8,x		; if sprite is dead, return
			CMP #$08
			BNE RETURN
			LDA $9D			;if sprites locked, return
			BNE RETURN
			JSL $01A7DC		;interact with Mario

LDA $1558,x             ; \ if time until throw < TIME_TO_SHOW
                    CMP #TIME_TO_SHOW       ;  |
                    BCS NO_THROW            ; / 
                    LDA $14                 ; \ 
                    ;LSR A                  ;  |
                    LSR A                   ;  |
                    CLC                     ;  |
                    ADC $15E9               ;  |
                    AND #$01                ;  |
                    STA $1602,x  	    ;  |
                    INC $1602,x             ;  | change image (hammer will be displayed)
                    INC $1602,x             ; /

                    LDA $1558,x             ; \ throw sprites if it's time
                    BNE NO_RESET3           ;  |
                    LDY $C2,x		    ;  |
                    LDA TIME_TILL_THROW,y   ;  | set the timer
                    STA $1558,x		    ; /
NO_RESET3           CMP #$01                ; \ call the sprite routine if the timer is 
                    BNE NO_THROW            ; / about to tun out
                    LDA $C2,x               ; \ handle multiple throws
                    EOR #$01                ;  |
                    STA $C2,x               ; /


NORMAL_SPRITE       JSR SUB_HAMMER_THROW    ; spawn normal sprite routine  
                    BRA NO_THROW

NO_THROW	    JSL $01803A

		    ;JSR SUB_HORZ_POS        ; \ always face Mario
		    ;TYA                     ;  |
		    ;STA $157C,x             ; /

		    LDA $14                 ; \ calculate frame
		    AND #$03              

	LDA $14                 ; \ make tilemap animated
	LSR A                   ;  |
	LSR A                   ;  |
	LSR A                   ;  |
	CLC                     ;  |
	ADC $15E9               ;  |
	AND #$01                ;  |
	STA $1602,x             ; /      


			LDA $1588,x		; \ if sprite is in contact with an object...
			AND #$03		;  |
			BEQ NO_CONTACT_x		;  |
			LDA $157C,x		;  | flip the direction status
			EOR #$01		;  |
			STA $157C,x		; /

NO_CONTACT_x		LDA $1588,x		; \ if sprite is in contact with an object...
			AND #$0C		;  |
			BEQ NO_CONTACT_Y		;  |
			LDA $151C,x		;  | flip the direction status
			EOR #$01		;  |
			STA $151C,x		; /

NO_CONTACT_Y		LDY $157C,x		; \ set x speed based on direction
			LDA X_SPEED,y		;  |
			STA $B6,x		; /
			LDY $151C,x		; \ set x speed based on direction
			LDA Y_SPEED,y		;  |
			STA $AA,x		; /                    
                                        
			JSL $01802A		; update position based on speed values

		JSR SPRITE_INTERACT     ; sprite interaction routine

NO_CONTACT 	    JSL $018032		    ; interact with sprites
		    JSL $01A7DC             ; check for Mario/sprite contact (carry set = contact)
                    BCC RETURN_24           ; return if no contact
                    JSR SUB_VERT_POS        ; 
                    LDA $0E                 ; \ if Mario isn't above sprite, and there's vertical contact...
                    CMP #$E6                ;  |     ... sprite wins
                    BPL SPRITE_WINS         ; /
                    LDA $7D                 ; \ if Mario speed is upward, return
                    BMI RETURN_24           ; /  
                  
		    LDA $140D               ; \ if Mario is spin jumping, goto SPIN_KILL		    
		    BNE SPIN_KILL           ; /

SPRITE_WINS         LDA $154C,x             ; \ if riding sprite...
                    ORA $15D0,x             ;  |   ...or sprite being eaten...
                    BNE RETURN_24           ; /   ...return
                    LDA $1490               ; \ if Mario star timer > 0, goto HAS_STAR 
                    BNE HAS_STAR            ; / NOTE: branch to RETURN_24 to disable star killing
		    JSL $00F5B7             ; hurt Mario                  
RETURN_24           RTS                     ; final return
         
SPIN_KILL           JSR SUB_STOMP_PTS       ; give Mario points
                    JSL $01AA33             ; set Mario speed, NOTE: remove call to not bounce off sprite
                    JSL $01AB99             ; display contact graphic
                    LDA #$04                ; \ status = 4 (being killed by spin jump)
                    STA $14C8,x             ; /   
                    LDA #$1F                ; \ set spin jump animation timer
                    STA $1540,x             ; /
                    JSL $07FC3B             ; show star animation
                    LDA #$08                ; \ play sound effect
                    STA $1DF9               ; /
                    RTS                     ; return

HAS_STAR            LDA #$02                ; \ status = 2 (being killed by star)
                    STA $14C8,x             ; /
                    LDA #$D0                ; \ set y speed
                    STA $AA,x               ; /
                    JSR SUB_HORZ_POS        ; get new direction
                    LDA KILLED_X_SPEED,y    ; \ set x speed based on direction
                    STA $B6,x               ; /
                    INC $18D2               ; increment number consecutive enemies killed
                    LDA $18D2               ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET2           ;  |
                    LDA #$08                ;  |
                    STA $18D2               ; /   
NO_RESET2           JSL $02ACE5             ; give Mario points
                    LDY $18D2               ; \ 
                    CPY #$08                ;  | if consecutive enemies stomped < 8 ...
                    BCS NO_SOUND2           ;  |
                    LDA STAR_SOUNDS,y       ;  |    ... play sound effect
                    STA $1DF9               ; /
NO_SOUND2           RTS                     ; final return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

X_OFFSET            dcb $06,$FA
X_OFFSET2           dcb $00,$FF

SUB_HAMMER_THROW
		JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
		BMI RETURN67            ; / after: Y has index of sprite being generated

		PHX
		TYX

		LDA #$20                ; \ sound effect
		STA $1DF9 

		LDA #$08
		STA $14C8,x

		PHX
		LDA #$02
		JSL RANDOM
		TAX
		LDA SPRITE_TO_GEN,x	; \ set sprite number for new sprite
		PLX

		;LDA #$3A
		STA $7FAB9E,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX

		PHY
                    LDA $157C,x
                    TAY
                    LDA $E4,x               ; \ set xlo position for new sprite
                    CLC
                    ADC X_OFFSET,y
                    PLY
                    STA $00E4,y

                    PHY
                    LDA $157C,x
                    TAY
                    LDA $14E0,x             ;  | set xhi position for new sprite
                    ADC X_OFFSET2,y
                    PLY
                    STA $14E0,y             ; /

                    LDA $D8,x               ; \ set y position for new sprite
                    SEC                     ;  | (y position of generator - 1)
                    SBC #$0E                ;  |
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    SBC #$00                ;  |
                    STA $14D4,y             ; /

                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    JSL $0187A7             ;  get table values for custom sprite
                    LDA #$88
                    STA EXTRA_BITS,x
                    PLX                       ; / 

                    LDA $157C,x
                    STA $157C,y
	
RETURN67	RTS		; return

RANDOM			PHX
			PHP
			SEP #$30
			PHA
			JSL $01ACF9		; Random number generation routine
			PLX
			CPX #$FF		;\ Handle glitch if max is FF
			BNE NORMALRT		;|
			LDA $148B		;|
			BRA ENDRANDOM		;/
NORMALRT		INX			; Amount in plus 1
			LDA $148B		;\
			STA $4202		;| Multiply with hardware regsisters
			STX $4203		;|
			NOP			;|
			NOP			;|
			NOP			;|
			NOP			;/
			LDA $4217
ENDRANDOM		PLP
			PLX
			RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Be killed by shells
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

KILLED_X_SPEED      dcb $F0,$10
	

SPRITE_INTERACT     LDY #$0B                ; sprite is being kicked
INTERACT_LOOP       LDA $14C8,y             ; \ if the sprite status is..
	            CMP #$09                ;  | ...shell-like
	            BCS PROCESS_SPRITE      ; /
NEXT_SPRITE         DEY
	            BPL INTERACT_LOOP
	            RTS

PROCESS_SPRITE      PHX                       
                    TYX                       
                    JSL $03B6E5             ; get sprite clipping B routine   
                    PLX                       
                    JSL $03B69F             ; get sprite clipping A routine  
                    JSL $03B72B             ; check for contact routine
	            BCC NEXT_SPRITE

	            PHX
	            TYX
	
	            JSL $01AB72             ; show sprite contact gfx routine

	            LDA #$02		    ; \ Kill thrown sprite
	            STA $14C8,x             ; /
	
	            LDA #$D0                ; \ Set killed Y speed
                    STA $AA,x               ; /
	
	            LDY #$00		    ; Set killed X speed
	            LDA $B6,x
	            BPL SET_SPEED
	            INY 

SET_SPEED	    LDA KILLED_X_SPEED,y
	            STA $B6,x

NO_KILL	            PLX

HANDLE_HIT	    LDA #$28                ; \ Play sound effect 
                    STA $1DFC               ; /

	            LDA #$01		    ; \ Set stunned state
	            STA $C2,x               ; /
	
	            LDA #$20		    ; \ Set stunned timer
	            STA $1564,x             ; /
		
	            INC $1528,x	            ; Increase hit counter and... 
	            LDA $1528,x	            ; \ Check if the sprite has been hit the max. times
	            CMP #Acro_Hits        ; /
	            BCC RETURN3

		LDA #$02            	; Kill Acro
		STA $14C8,x
		STZ $B6,x				; Set X speed
		LDA #$F0				; Set Y speed
		STA $AA,x

GOAL    	DEC $13C6               ; prevent Mario from walking at the level end
		LDA #$FF                ; \ set goal
		STA $1493               ; /
		LDA #$0B                ; \ set ending music
		STA $1DFB               ; /
        	RTS                     ; return

		LDA #$03
        	STA $1602,x             ; / write frame to show

RETURN3             RTS

PROCESS_CACHED_HIT  LDA $1528,x
	            BPL RETURN3
	            AND #$7F
	            STA $1528,x
	            BRA HANDLE_HIT


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite clipping routine
; ripped and modified by Roy, then modified more by Sonikku
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GetSpriteClipping:
	PHY
	PHX
	TXY
	LDA $00E4,y
	CLC
	ADC #$04		; Starting X pos of sprite clipping = sprite center position - $0C ($0C pixels to the left)
	STA $04
	LDA $14E0,y
	ADC #$00
	STA $0A
	LDA #$30                ; Width of sprite clipping
	STA $06
	LDA $00D8,y
	SEC
	SBC #$20                ; Starting Y pos of sprite clipping = sprite center position - $2C ($2C pixels above)
	STA $05
	LDA $14D4,y
	SBC #$00
	STA $0B
	LDA #$30                ; Height of sprite clipping
	STA $07	
	PLX
	PLY
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;WALKING1: dcb $00,$02,$04,$20,$22,$24,$40,$42,$44
;WALKING2: dcb $6A,$6C,$04,$20,$22,$24,$60,$62,$64

TILEMAP: db $04,$02,$00	; facing right
		db $24,$22,$20
		db $44,$42,$40

		db $00,$02,$04	; facing left
		db $20,$22,$24
		db $40,$42,$44
		
		db $0C,$0A,$08	; facing right
		db $2C,$2A,$28
		db $4C,$4A,$48

		db $08,$0A,$0C	; facing left
		db $28,$2A,$2C
		db $48,$4A,$4C

X_OFFSET: db $E8,$F8,$08
		db $E8,$F8,$08
		db $E8,$F8,$08

Y_OFFSET: db $E8,$E8,$E8
		db $F8,$F8,$F8
		db $08,$08,$08

PROPERTIES: db $4F,$4F,$4F
		db $4F,$4F,$4F
		db $4F,$4F,$4F

		db $0F,$0F,$0F
		db $0F,$0F,$0F
		db $0F,$0F,$0F

SUB_GFX:    
	JSR GET_DRAW_INFO       ; sets y = OAM offset
	LDA $157C,x
	STA $04
	ASL A
	ASL A
	ASL A
	CLC
	ADC $04
	STA $02

                    PHX
                    LDX #$08
LOOP_START:         PHX
                    LDA X_OFFSET,x
                    CLC
                    ADC $00                 ; \ tile x position = sprite y location ($01)
                    STA $0300,y             ; /

                    LDA Y_OFFSET,x
                    CLC
                    ADC $01                 ; \ tile y position = sprite x location ($00)
                    STA $0301,y             ; /

                    TXA
                    CLC
                    ADC $02
                    TAX

					PHX
					LDA $14
					LSR A
					LSR A
					LSR A
					AND #$01
					BNE SKIP
					TXA
					CLC
					ADC #$12
					TAX
SKIP:               LDA TILEMAP,x
                    STA $0302,y
                    PLX 

                    PHX
                    LDX $15E9
                    LDA $15F6,x             ; tile properties xyppccct, format
                    PLX
                    ORA PROPERTIES,x
                    ORA $64                 ; add in tile priority of level
                    STA $0303,y             ; store tile properties
                    PLX

                    INY                     ; \ increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ; /    ...so increment 4 times
                    DEX
                    BPL LOOP_START

                    PLX
                    LDY #$02                ; \ 460 = 2 (all 16x16 tiles)
                    LDA #$08               ;  | A = (number of tiles drawn - 1)
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS       PHY                     ; 
                    LDA $1697               ; \
                    CLC                     ;  | 
                    ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
                    INC $1697               ; increase consecutive enemies stomped
                    TAY                     ;
                    INY                     ;
                    CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
                    BCS NO_SOUND            ; /    ... don't play sound 
                    LDA STAR_SOUNDS,y       ; \ play sound effect
                    STA $1DF9               ; /   
NO_SOUND            TYA                     ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET            ;  |
                    LDA #$08                ; /
NO_RESET            JSL $02ACE5             ; give Mario points
                    PLY                     ;
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical Mario/sprite position check - shared
; Y = 1 if Mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal Mario/sprite check - shared
; Y = 1 if Mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
