;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Bowser Statue Disassembly, by imamelia
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dcb "INIT"
INC $157C,x	; set initial sprite direction to left
JSR SubSetXPos	; this actually jumps to the init routine of the exploding block
STY $C2,x	; the Y register determines which statue it will be
CPY #$02		; if it is sprite state 02...
BNE EndInit	;
LDA #$01		; then change its palette
STA $15F6,x
EndInit:
RTL

SubSetXPos:	; this determines which kind of statue it will be based on its X position
LDA $E4,x	; sprite X position (low byte)
LSR
LSR
LSR
LSR		; divide by 16 to get it in terms of tiles
AND #$03	; we want only 4 possible values
TAY		; transfer this into Y (to determine the sprite that the exploding block would spawn)
RTS		; but since this isn't an exploding block, we can just end right here

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BwsrStatueDispX:		dcb $08,$F8,$00,$00,$08,$00
BwsrStatueDispY:		dcb $10,$F8,$00
BwsrStatueTiles:		dcb $56,$30,$41,$56,$30,$35
BwsrStatueGfxProp:		dcb $00,$00,$00,$40,$40,$40
BwsrStatueTileSize:		dcb $00,$02,$02
BwsrFireDispXLo:		dcb $10,$F0
BwsrFireDispXHi:		dcb $00,$FF
BwsrStatueSpeed:		dcb $10,$F0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dcb "MAIN"
PHB
PHK
PLB
JSR BowserStatueMain
PLB
RTL

BowserStatueMain:

JSR SubStatueGFX		; draw the sprite

LDA $9D		; if sprites are locked...
BNE ReturnMain	; return
JSR SubOffscreen	; offscreen processing code
LDA $C2,x	; use the sprite state to index a pointer table;
JSL $0086DF	; this pointer table tells us which kind of Bowser statue this should be

dcw FrozenInPlace&$FFFF
dcw SpitFire&$FFFF
dcw JumpAround&$FFFF
dcw SpitFire&$FFFF

SpitFire:
JSR SubFireSpit	; prepare the sprite for spitting a fireball
FrozenInPlace:
JSR $01B44F	; invisible block routine (make sprite solid)
JSL $01802A	; update sprite position
LDA $1588,x	; check sprite blocked status
AND #$04	; if the sprite is not on the ground...
BEQ ReturnMain	; return
STZ $AA,x	; set sprite Y speed to 0
ReturnMain:
RTS

JumpAround:

ASL $167A,x	; change fourth Tweaker byte
LSR $167A,x	; um...I guess these two lines were just to clear bit 7?
JSL $01A7DC	; interact with the player (and with bit 7 cleared, that means default interaction)
STZ $1602,x	; set...graphics pointer table to 0?
LDA $AA,x	; if the sprite's Y speed
CMP #$10		; is greater than or equal to 10...
BPL NoInc1602	; don't increment $1602,x
INC $1602,x	; if not, don't not increment $1602,x
NoInc1602:	;
JSL $01802A	; don't not update sprite position
LDA $1588,x	; load sprite blocked status
AND #$03	; if it is not not touching a wall...
BEQ NoChange	; don't not not change its speed or direction
LDA $B6,x	; don't not not not load the sprite's X speed
EOR #$FF		; and don't not not not not flip it
INC A		; don't not not not not not increment it so that it will be properly inversed...
STA $B6,x		; don't *shot* Okay, I'll quit with the extra negatives.  Store new sprite speed.
LDA $157C,x	; load sprite direction
EOR #$01		; flip it
STA $157C,x	; store new sprite direction
NoChange:	
LDA $1588,x	; check sprite blocked status
AND #$04	; if not on the ground...
BEQ Return1	; return
LDA #$10		; set the sprite's Y speed
STA $AA,x	; to 10
STZ $B6,x		; and its X speed to 0
LDA $1540,x	; load that wretched sprite death frame counter, the one that I can never be sure about
BEQ RandomLabel	; if zero, skip this next part
DEC A		; if it isn't zero, than decrement it
BNE Return1	; if it still isn't zero, then skip this next part as well
LDA #$C0		; set sprite Y speed
STA $AA,x	;
JSR SubHorzPos	; get sprite direction
TYA		; transfer to the accumulator
STA $157C,x		; so the sprite always follows the player
LDA BwsrStatueSpeed,y	; set sprite X speed
STA $B6,x			; depending on direction
RTS		; return

RandomLabel:
LDA #$30
STA $1540,x
Return1:
RTS

SubFireSpit:	; I actually had to modify this routine a bit, since the original one didn't work
TXA		; for unknown reasons.  I replaced it with the fire-spawning code from my Giant Fire Bro.
ASL		; sprite index into A
ASL		; multiply by 4...
ADC $13		; add frame counter plus one...
AND #$7F	; clear out bit 7...
BNE Return1	; and return if the result is not 0.  Uh...what??
	    LDA $15A0,x             ; \ no egg if off screen
                    ORA $186C,x             ;  |
                    ORA $15D0,x
                    BNE Return1
                    
                    JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI Return1            ; / after: Y has index of sprite being generated

                    LDA #$08                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /

	LDA #$17
	STA $1DFC	; play fire sound effect

                    LDA #$B3
                    STA $9E,y

                    PHY                     ; set x position for new sprite
                    LDA $157C,x
                    TAY
                    LDA $E4,x               
                    CLC
                    ADC BwsrFireDispXLo,y
                    PLY
                    STA $00E4,y
    
                    PHY                     ; set x position for new sprite
                    LDA $157C,x
                    TAY
                    LDA $14E0,x             
                    ADC BwsrFireDispXHi,y
                    PLY
                    STA $14E0,y      

                    LDA $D8,x               ; \ set y position for new sprite
                    SEC                     ;  | (y position of generator - 1)
                    SBC #$02                ;  |
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    SBC #$00                ;  |
                    STA $14D4,y             ; /

                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    PLX                     ; /
	LDA $157C,x
	STA $157C,y

	RTS

SubStatueGFX:

JSR GET_DRAW_INFO

LDA $1602,x	; store wretched sprite graphics pointer table
STA $04		; to scratch RAM
EOR #$01		; flip bit 0...
DEC A		; decrement...
STA $03		; and store this to more scratch RAM (huh?)
LDA $15F6,x	; store sprite palette and GFX page info
STA $05		; to more scratch RAM
LDA $157C,x	; store sprite direction
STA $02		; to scratch RAM
PHX		; preserve sprite index
LDX #$02		; load X with #$02, since we're drawing three tiles

GFX_Loop:	;

PHX		; preserve tile index    
LDA $02		; if the sprite is facing left...
BNE NoInx	; don't increment the tile index
INX		; if the sprite is facing right,
INX		; increment the tile index
INX		; by 3
NoInx:
LDA $00		;
CLC			;
ADC BwsrStatueDispX,x	;
STA $0300,y		; set X position/displacement
LDA BwsrStatueGfxProp,x	; set horizontal flip of the tile...
ORA $05			; add in the palette and GFX page info
ORA $64		; add in the priority bits
STA $0303,y	; store tile properties
PLX		; get original index back
LDA $01		;
CLC			;
ADC BwsrStatueDispY,x	; set sprite Y position/displacement
STA $0301,y		;
PHX		; preserve tile index again
LDA $04		; load the value from the wretched sprite graphics pointer table
BEQ NoInx2	; if zero, don't change the tilemap index
INX		; if non-zero...
INX		; increment the tilemap index
INX		; by 3
NoInx2:			;
LDA BwsrStatueTiles,x	; load tile from tilemap
STA $0302,y		; store tile
PLX		; get original sprite index back
PHY		; preserve OAM index
TYA		; transfer Y to A...
LSR		;
LSR		; divide by 4,
TAY			; then transfer the new value back to Y
LDA BwsrStatueTileSize,x	; Ooh, tricky...we're setting the size of the tiles here.
STA $0460,y		; store to that mysterious fifth OAM slot, $0460,y 
PLY		; get OAM index back from the depths of chaos
INY
INY
INY
INY		; increment Y 4 times, setting it up for the next tile                     
DEX		; decrement X, which currently holds the tile index
CPX $03		; compare to twisted mutated wretched sprite graphics pointer value from before
BNE GFX_Loop	; if the values aren't equal, then we need to draw more tiles

PLX		; get sprite index back

LDY #$FF		; we already set the tile size, so load FF here
LDA #$02		; we drew 3 tiles
JSL $01B7B3	; finish off the OAM write
RTS		; end GFX routine

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

Table3:              dcb $40,$B0
Table6:              dcb $01,$FF 
Table4:              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
Table5:              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreen:   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA Store03            ;  | 
SubOffscreenX1:   LDA #$04                ;  |
                    BRA Store03            ;  |
SubOffscreenX2:   LDA #$02                ;  |
Store03:       
	    STA $03                 ;  |
                    BRA StartSub           ;  |
SubOffscreenX3:   STZ $03                 ; /

StartSub:           JSR SubIsOffscreen   ; \ if sprite is not off screen, return
                    BEQ Return2            ; /    
                    LDA $5B                 ; \  goto VerticalLevel if vertical level
                    AND #$01                ;  |
                    BNE VerticalLevel      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL EraseSprite        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE Return2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;
                    CLC                     ;
                    ADC Table4,y            ;
                    ROL $00                 ;
                    CMP $E4,x               ;x pos ;
                    PHP                     ;
                    LDA $1B                 ;x boundry hi ;
                    LSR $00                 ;
                    ADC Table5,y            ;
                    PLP                     ;
                    SBC $14E0,x             ;x pos high
                    STA $00                 ;
                    LSR $01                 ;
                    BCC Label20             ;
                    EOR #$80                ;
                    STA $00                 ;
Label20:             LDA $00                 ;
                    BPL Return2            ;
EraseSprite:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KillSprite         ; /
                    LDY $161A,x             ;
                    CPY #$FF                ;
                    BEQ KillSprite        ;
                    LDA #$00                ; \ 
                    STA $1938,y             ; /     
KillSprite:         STZ $14C8,x             ; erase sprite
Return2:            RTS                     ; return

VerticalLevel:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE Return2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS Return2            ; /
                    AND #$01                ;
                    STA $01                 ;
                    TAY                     ;
                    LDA $1C                 ;
                    CLC                     ;
                    ADC Table3,y            ;
                    ROL $00                 ;
                    CMP $D8,x               ;
                    PHP                     ;
                    LDA $001D             ;
                    LSR $00                 ;
                    ADC Table6,y            ;
                    PLP                     ;
                    SBC $14D4,x             ;
                    STA $00                 ;
                    LDY $01                 ;
                    BEQ Label22             ;
                    EOR #$80                ;
                    STA $00                 ;
Label22:             LDA $00                 ;
                    BPL Return2            ;
                    BMI EraseSprite        ;

SubIsOffScreen:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SubHorzPos:        LDY #$00                ;
                    LDA $94                 ;
                    SEC                     ;
                    SBC $E4,x               ;
                    STA $0F                 ;
                    LDA $95                 ;
                    SBC $14E0,x             ;
                    BPL Label16             ;
                    INY                     ;
Label16:             RTS                     ;


