;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Controllable Carrot Top Lift          ;;
;; By Roy			         ;;
;; Description: This is a carrot top     ;;
;; like in SMW, except that you can      ;;
;; control it with the controller        ;;
;; buttons. Speed can be adjusted in     ;;
;; the corresponding 'XSpeed' and        ;;
;; 'YSpeed' table. Depending on the X    ;;
;; position of where you place the sprite;;
;; in Lunar Magic, it faces either left  ;;
;; or right.                             ;;
;;                                       ;;
;; Uses extra bit: YES                   ;;
;; If extra bit is set, the carrot lift  ;;
;; will only last for a set amount of    ;;
;; frames.                               ;;
;;					 ;;
;; Based off SMW's carrot lift.          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Defines           ;
;-------------------;

		    FRAMES = $FF ; How many frames the Carrot Lift can be used. (Extra bit set.)
                    EXTRA_BITS = $7FAB10

;-------------------;
; Initial Routine   ;
;-------------------;

dcb "INIT"
LDA $E4,x
LSR
LSR
LSR
LSR
AND #$01
STA $1528,x
RTL

;-------------------;
; Main Routine      ;
;-------------------;

dcb "MAIN"
PHB			; \
PHK			;  |
PLB			;  | Wrapper.
JSR CarrotTopLift	;  |
PLB			; /
RTL			

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actual main routine                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Format of speed table:
; 1st value - no buttons
; 2nd value - right button
; 3rd value - left button
; 4th value - right and left buttons
; 5th value - down buttons
; 6th value - down and right buttons
; 7th value - down and left buttons
; 8th value - down, right and left buttons
; 9th value - up button
; 10th value - up and right buttons
; 11th value - up and left buttons
; 12th value - up, right and left buttons
; 13th value - up and down buttons
; 14th value - up, down and right buttons
; 15th value - up, down and left buttons
; 16th value - up, down, right and left buttons

XSpeed
db $00,$08,$F8,$00,$00,$08,$F8,$00,$00,$08,$F8,$00,$00,$08,$F8,$00

YSpeed
db $00,$00,$00,$00,$08,$08,$08,$08,$F8,$F8,$F8,$F8,$00,$00,$00,$00

MarioSpeedTable ; how fast Mario moves when on the platform
db $00,$02,$FE,$00 ; none, right, left, right & left.

InteractionTbl
db $54,$1C

Return
RTS

CarrotTopLift
JSR SUB_GFX
LDA $9D
BNE Return
JSR SUB_OFF_SCREEN_X0
STZ $AA,x
STZ $B6,x
JSR MarioSpriteCheck
JSL $03B69F
JSL $03B72B
BCC Return
LDA $1534,x
BNE AlreadyDecrementing
LDA #FRAMES
STA $1534,x

AlreadyDecrementing:
LDA $E4,x
STA $151C,x
LDA EXTRA_BITS,x
AND #$04
BEQ NoKill
LDA $1534,x
CMP #$01
BNE NoKill
LDA #$04
STA $14C8,x
LDA #$25 ; Blargg growl
STA $1DF9 ; Could be seen as an explosion of some sort.
LDA #$1F
STA $1540,x
JSL $07FC3B
RTS

NoKill:
;LDA $13
;AND #$01           ; Power of 2 minus one. The higher the value, the slower the timer decrements.
;BNE NoDecrementz   ; Get rid of the semicolons in front to make this work.
DEC $1534,x

NoDecrementz:
LDA $7D
BMI Return2
LDY $1528,x
LDA $94
SEC
SBC $151C,x
CLC
ADC InteractionTbl,y
TAY
LDA $187A
CMP #$01
LDA #$20
BCC NotOnYoshi2
LDA #$30

NotOnYoshi2
CLC
ADC $96
STA $00
LDA $D8,x
CLC
ADC DATA_038BAA,y
CMP $00
BPL Return2
LDA $187A
CMP #$01
LDA #$1D
BCC NotOnYoshi3
LDA #$2D

NotOnYoshi3
STA $00
LDA $D8,x
CLC
ADC DATA_038BAA,y
PHP
SEC
SBC $00
STA $96
LDA $14D4,x
SBC #$00
PLP
ADC #$00
STA $97
STZ $7D
LDA #$01
STA $1471
LDA $15
AND #$0F
TAY
LDA XSpeed,y
STA $B6,x
LDA YSpeed,y
STA $AA,x
JSL $01801A
JSL $018022
STZ $7B
LDY #$00
LDA $1491
BPL NoDecrement
DEY

NoDecrement
CLC
ADC $94
STA $94
TYA
ADC $95
STA $95

Return2
RTS

;------------------;
; Mario Sprite Int.;
;------------------;

MarioSpriteCheck
LDA $94
CLC
ADC #$04
STA $00
LDA $95
ADC #$00
STA $08
LDA #$08
STA $02
STA $03
LDA #$20
LDY $187A
BEQ NotOnYoshi
LDA #$30

NotOnYoshi
CLC
ADC $96
STA $01
LDA $97
ADC #$00
STA $09
RTS


DATA_038BAA                      dcb $20,$20,$20,$20,$20,$20,$20,$20
                                 dcb $20,$20,$20,$20,$20,$20,$20,$20
                                 dcb $20,$1F,$1E,$1D,$1C,$1B,$1A,$19
                                 dcb $18,$17,$16,$15,$14,$13,$12,$11
                                 dcb $10,$0F,$0E,$0D,$0C,$0B,$0A,$09
                                 dcb $08,$07,$06,$05,$04,$03,$02,$01
                                 dcb $00,$00,$00,$00,$00,$00,$00,$00
                                 dcb $00,$00,$00,$00,$00,$00,$00,$00
                                 dcb $00,$00,$00,$00,$00,$00,$00,$00
                                 dcb $00,$00,$00,$00,$00,$00,$00,$00
                                 dcb $01,$02,$03,$04,$05,$06,$07,$08
                                 dcb $09,$0A,$0B,$0C,$0D,$0E,$0F,$10
                                 dcb $11,$12,$13,$14,$15,$16,$17,$18
                                 dcb $19,$1A,$1B,$1C,$1D,$1E,$1F,$20
                                 dcb $20,$20,$20,$20,$20,$20,$20,$20
                                 dcb $20,$20,$20,$20,$20,$20,$20,$20
;------------------;
; Graphics Routine ;
;------------------;

HORIZ_DISP          dcb $10,$00,$00,$00,$10,$10
VERT_DISP           dcb $10,$10,$00,$10,$10,$00
TILEMAP             dcb $E0,$E2,$E4,$E0,$E2,$E4


SUB_GFX             JSR GET_DRAW_INFO       ;A:87BF X:0007 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1102 VC:066 00 FL:665
                    PHX                     ; /
		    LDA $1528,x
                    ASL A
                    CLC
                    ADC $1528,x
                    STA $03
                    LDX #$02                ; run loop 4 times, cuz 4 tiles per frame
LOOP_START_2        PHX                     ; push, current tile
                    TXA
                    CLC
                    ADC $03
                    TAX
                    PHY                     ; set tile to be 8x8 or 16x16
                    TYA                     ;
                    LSR A                   ;
                    LSR A                   ;
                    TAY                     ;
                    LDA #$02 
                    STA $0460,y             ;
                    PLY                     ;
                    
                    LDA $00                 ; \ tile x position = sprite x location ($00) + tile displacement
                    CLC                     ; |
                    ADC HORIZ_DISP,x        ; |
                    STA $0300,y             ; /
                    
                    LDA $01                 ; |
                    CLC                     ; | tile y position = sprite y location ($01) + tile displacement
                    ADC VERT_DISP,x         ; |
                    STA $0301,y             ; /

                    LDA TILEMAP,x           ; \ store tile
                    STA $0302,y             ; /

                    PHX                     ;
                    LDX $15E9               ;
                    LDA $1528,x             ;
                    BNE NoFlip              ;
                    LDA #$40                ;
                    BRA Branch

NoFlip              LDA #$00

Branch              ORA $15F6,x             ; get palette info
                    PLX                     ;
                    ORA $64                 ; add in tile priority of level
                    STA $0303,y             ; store tile properties

                    PLX                     ; \ pull, current tile
                    INY                     ; | increase index to sprite tile map ($300)...
                    INY                     ; |    ...we wrote 1 16x16 tile...
                    INY                     ; |    ...sprite OAM is 8x8...
                    INY                     ; |    ...so increment 4 times
                    DEX                     ; | go to next tile of frame and loop
                    BPL LOOP_START_2        ; /

                    PLX                     ; pull, X = sprite index
                    LDY #$FF                ; \ we've already set 460 so use FF
                    LDA #$02                ; | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; AC0D - off screen processing code - shared
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    ;org $01AC0D
                    
TABLE12             dcb $40,$B0
TABLE13             dcb $01,$FF
TABLE14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90
TABLE15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
                    STA $03                 ;  |
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2   LDA #$02                ;  |
STORE_03            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X3   STZ $03                 ; /

START_SUB           JSR SUB_80CB            ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0328 VC:176 00 FL:205
                    BEQ LABEL35             ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0480 VC:176 00 FL:205
                    LDA $5B                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0496 VC:176 00 FL:205
                    AND #$01                ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0520 VC:176 00 FL:205
                    BNE LABEL36             ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0536 VC:176 00 FL:205
                    LDA $D8,x               ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0552 VC:176 00 FL:205
                    CLC                     ;A:8A60 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0582 VC:176 00 FL:205
                    ADC #$50                ;A:8A60 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0596 VC:176 00 FL:205
                    LDA $14D4,x             ;A:8AB0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNVMXdizcHC:0612 VC:176 00 FL:205
                    ADC #$00                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:enVMXdizcHC:0644 VC:176 00 FL:205
                    CMP #$02                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0660 VC:176 00 FL:205
                    BPL LABEL32             ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0676 VC:176 00 FL:205
                    LDA $167A,x             ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0692 VC:176 00 FL:205
                    AND #$04                ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0724 VC:176 00 FL:205
                    BNE LABEL35             ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0740 VC:176 00 FL:205
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC TABLE14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC TABLE15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC LABEL31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
LABEL31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL LABEL35             ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
LABEL32             LDA $9E,x               
                    CMP #$1F                
                    BNE NOT_MAGIKOOPA       
                    STA $18C1                   
                    LDA #$FF                    
                    STA $18C0
NOT_MAGIKOOPA       LDA $14C8,x             ;A:FF1A X:0007 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1044 VC:059 00 FL:2878
                    CMP #$08                ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:1076 VC:059 00 FL:2878
                    BCC LABEL34             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1092 VC:059 00 FL:2878
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ LABEL34             ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
LABEL34             STZ $14C8,x             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1220 VC:059 00 FL:2878
LABEL35             RTS                     ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1336 VC:176 00 FL:205
LABEL36             LDA $167A,x             ;A:0101 X:0003 Y:006C D:0000 DB:01 S:01ED P:envMXdizCHC:0758 VC:136 00 FL:5307
                    AND #$04                ;A:0100 X:0003 Y:006C D:0000 DB:01 S:01ED P:envMXdiZCHC:0790 VC:136 00 FL:5307
                    BNE LABEL35             ;A:0100 X:0003 Y:006C D:0000 DB:01 S:01ED P:envMXdiZCHC:0806 VC:136 00 FL:5307
                    LDA $13                 ;A:0100 X:0003 Y:006C D:0000 DB:01 S:01ED P:envMXdiZCHC:0822 VC:136 00 FL:5307
                    LSR A                   ;A:0115 X:0003 Y:006C D:0000 DB:01 S:01ED P:envMXdizCHC:0846 VC:136 00 FL:5307
                    BCS LABEL35             ;A:010A X:0003 Y:006C D:0000 DB:01 S:01ED P:envMXdizCHC:0860 VC:136 00 FL:5307
                    LDA $E4,x               ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1092 VC:250 00 FL:5379
                    CMP #$00                ;A:0048 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1122 VC:250 00 FL:5379
                    LDA $14E0,x             ;A:0048 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizCHC:1138 VC:250 00 FL:5379
                    SBC #$00                ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1170 VC:250 00 FL:5379
                    CMP #$02                ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1186 VC:250 00 FL:5379
                    BCS LABEL32             ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1202 VC:250 00 FL:5379
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    BEQ LABEL37             ;A:0001 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:1310 VC:250 00 FL:5379
                    LDA $9E,x               ;A:0001 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:1326 VC:250 00 FL:5379
                    CMP #$22                ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:1356 VC:250 00 FL:5379
                    BEQ LABEL35             ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0004 VC:251 00 FL:5379
                    CMP #$24                ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0020 VC:251 00 FL:5379
                    BEQ LABEL35             ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0036 VC:251 00 FL:5379
LABEL37             LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC TABLE12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC TABLE13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ LABEL38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
LABEL38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL LABEL35             ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI LABEL32             ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $0180CB 
                    
SUB_80CB            LDA $15A0,x
                    ORA $186C,x
                    RTS