;==================================================
;   F L A S H Y     S P R I T E     D O O R
;   by Mathos
;
;   A door sprite that flash at regular intervals a "RE" or "GOAL" mention
;	Preferably for puzzle levels.
;	Also has gravity, so that you can build gimmicks around that.
;	Setting the extra bit makes it use a different sound and teleport to a specific level,
;	instead of using the current screen exit like any door.
;
;==================================================

!LVNumber = $0144	; level number to go to if extra bit is set
!ExtraBits = $7FAB10 ; Don't touch this.
!FreeRAM_A = $04	; Scratch RAM to use for custom facing routine
!GFXBeg = $04    ; \
!GFXEnd = $05    ; | Scratch RAM used temporarly to draw the sprite's graphics. Most "empty" adresses are sufficient.
!GFXNum = $06    ; / You'd better leave it as it is, unless you already use them for some reason.
!Timer = #$10      ; - How many frames between every graphical change. Any 2^n value supported.
					; Setting it to 0 will cause the sprite to flash as long as possible
!Use1Sound = #$0F    ; \ SFX to play when item is touched and disappears, with SFX bank
!Use1SBank = $1DFC  ; /
!Use2Sound = #$05    ; \ SFX to play when item is touched and disappears, with SFX bank
!Use2SBank = $1DF9  ; /
!FlashDistance = #$0040	; The max distance mario must be from the sprite for it to flash
						; Note that the check operates with a BOX, not a CIRCLE
!FlashTable = $1570	; Sprite table to use for flashing purposes
!IsAGoal = #$01	; If set to a non-zero value, the sprite flashes "GOAL" instead of "RE"

print "INIT ",pc
	STZ !FlashTable,x
	RTL

print "MAIN ",pc
	PHB				;\
	PHK				; | Change the data bank to the one our code is running from.
	PLB				; | This is a good practice.
	JSR SpriteCode	; | Jump to the sprite's function.
	PLB				;/ Restore old data bank.
	RTL		; And RETURN.

;===================================
;Sprite Function
;===================================
FlashAlways:
	SEP #$20
	LDA #$01
	STA !FlashTable,x
Return:
	JSR Graphics
	RTS

SpriteCode:
	LDA $14C8,x			;\
	CMP #$08			; | If sprite dead,
	BNE Return			;/ RETURN.
	LDA $9D				;\
	BNE Return			;/ If locked, RETURN.
	
	JSR SubOffScreen_X3	; handle offscreen
	
	JSL $01802A
	
	REP #$20
	LDA !FlashDistance
	BEQ FlashAlways
	SEP #$20
CheckX:
	JSR Facing
	REP #$20
	LDA !FreeRAM_A
	CMP !FlashDistance
	BCC XOkay
	SEP #$20
	BRA NoFlash
XOkay:
	SEP #$20
CheckY:
	JSR Yacing
	REP #$20
	LDA !FreeRAM_A
	CMP !FlashDistance
	BCC YOkay
	SEP #$20
	BRA NoFlash
YOkay:
	SEP #$20
	BRA DoFlash
NoFlash:
	STZ !FlashTable,x
	BRA Return
DoFlash:
	LDA #$01
	STA !FlashTable,x
	
CheckContact:	
	JSL $01A7DC
	BCC Return
CheckGround:
	LDA $13EF
	BEQ Return
CheckProximity:
	JSR Facing
	REP #$20
	LDA !FreeRAM_A
	CMP #$000A
	BCC CheckInput
	SEP #$20
	BRA Return
CheckInput:
	SEP #$20
	LDA $15
	ORA $16
	AND #$08
	BEQ Return
InteractionOK:
	LDA !ExtraBits,x
	AND #$04
	BNE ExtraBitSet
ExtraBitClear:
	LDA !Use1Sound
	STA !Use1SBank
	LDA #$06				; \
	STA $71					; | Teleport the player via screen exit.
	STZ $88					; |
	STZ $89					; /
	BRA EndInteraction
ExtraBitSet:
	LDA !Use2Sound
	STA !Use2SBank
	PHX					; \
	LDX $95					; |
	PEA !LVNumber				; |
	PLA					; |
	STA $19B8,x				; |
	PLA					; | Teleport the player to level XXXX.
	ORA #$04				; |
	STA $19D8,x				; |
	LDA #$06				; |
	STA $71					; |
	STZ $89					; |
	STZ $88					; |
	PLX					; /
EndInteraction:
	JMP Return


;===CUSTOM FACING ROUTINE===
; 2 bytes of free RAM return distance between sprite & Mario
; Results: A = #$00 if player is to the right, else to the left

Facing:
	LDA $00E4,x
	STA !FreeRAM_A
	LDA $14E0,x
	STA !FreeRAM_A+1
	REP #$20
	LDA !FreeRAM_A : CLC : ADC #$0008 : STA !FreeRAM_A
	LDA $94 : CLC : ADC #$0008
	SEC
	SBC !FreeRAM_A
	STA !FreeRAM_A
	SEP #$20
	LDA !FreeRAM_A+1
	BMI F_Left
F_Right:
	LDA #$00
	RTS
F_Left:
	REP #$20
	LDA #$0000
	SEC
	SBC !FreeRAM_A
	STA !FreeRAM_A
	SEP #$20
	LDA #$01
	RTS
	
Yacing:
	LDA $00D8,x
	STA !FreeRAM_A
	LDA $14D4,x
	STA !FreeRAM_A+1
	REP #$20
	LDA !FreeRAM_A : CLC : ADC #$0008 : STA !FreeRAM_A
	LDA $96 : CLC : ADC #$0008
	SEC
	SBC !FreeRAM_A
	STA !FreeRAM_A
	SEP #$20
	LDA !FreeRAM_A+1
	BMI Y_Left
Y_Right:
	LDA #$00
	RTS
Y_Left:
	REP #$20
	LDA #$0000
	SEC
	SBC !FreeRAM_A
	STA !FreeRAM_A
	SEP #$20
	LDA #$01
	RTS
	
;===================================
;Graphics Code (CUSTOM, NOT OPTIMIZED)
;===================================

; =FRAME TABLE=

PROPS:	db $0F,$0F,$0F,$4F,$4F
		db $0F,$0F,$0F,$0F,$4F,$4F
TILES:	db $84,$80,$82,$80,$82
		db $A0,$A2,$80,$82,$80,$82
XDISP:	db $00,$F8,$F8,$08,$08
		db $F8,$08,$F8,$F8,$08,$08
YDISP:	db $F0,$F0,$00,$F0,$00
		db $F0,$F0,$F0,$00,$F0,$00

Graphics:
	JSR GET_DRAW_INFO

	PHX		; Let's push X in order to not destroy everything.

	;===Actual frame drawing decision here===
		LDA !IsAGoal
		BEQ GFX_RE
	GFX_GOAL:
		LDA #$0A
		STA !GFXEnd
		LDA !FlashTable,x
		BEQ GFX_Frame4
		LDA $14
		AND !Timer
		BNE GFX_Frame4
	GFX_Frame3:
		LDA #$05
		STA !GFXBeg
		BRA GFX_FrameEnd
	GFX_Frame4:
		LDA #$07
		STA !GFXBeg
		BRA GFX_FrameEnd
	GFX_RE:
		LDA #$04
		STA !GFXEnd
		LDA !FlashTable,x
		BEQ GFX_Frame2
		LDA $14
		AND !Timer
		BNE GFX_Frame2
	GFX_Frame1:
		STZ !GFXBeg
		BRA GFX_FrameEnd
	GFX_Frame2:
		LDA #$01
		STA !GFXBeg
	GFX_FrameEnd:
		
	; ===End of decision with an interval===

	LDA !GFXEnd
	SEC
	SBC !GFXBeg
	STA !GFXNum
	INC !GFXEnd
	LDX !GFXBeg
LOOP:
	LDA $00			; Application of X displacement
	CLC
	ADC XDISP,x
	STA $0300,y

	LDA $01			; Application of Y displacement
	CLC
	ADC YDISP,x
	STA $0301,y

	LDA TILES,x		; Set tile to use
	STA $0302,y

	LDA PROPS,x		; Set PROPERTIES.
	STA $0303,y

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
	
	INX
	CPX !GFXEnd
	BCC LOOP
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02			; Y ends with the tile size .. 02 means it's 16x16
	LDA !GFXNum

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS				; Never forget this!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:
	STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ .ON_SCREEN_X        ;  |
	INC $15A0,x             ; /

.ON_SCREEN_X
	LDA $14E0,x             ; \
	XBA                     ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC                     ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC                     ;  |
	ADC.w #$0040            ;  |
	CMP.w #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE .INVALID           ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ .ON_SCREEN_LOOP     ;  | else, go through loop once
	INY                     ; / 
.ON_SCREEN_LOOP
	LDA $D8,x               ; \ 
	CLC                     ;  | set vertical offscreen if necessary
	ADC SPR_T1,y            ;  |
	PHP                     ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP                     ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ .ON_SCREEN_Y        ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA SPR_T2,y            ;  |
	STA $186C,x             ;  |
.ON_SCREEN_Y
	DEY                     ;  |
	BPL .ON_SCREEN_LOOP     ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC                     ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC                     ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

.INVALID
	PLA                     ; \ return from *main gfx routine* subroutine...
	PLA                     ;  |    ...(not just this subroutine)
	RTS                     ; /
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS RETURN_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / return