;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shy Guy, by mikeyk
;;
;; Description: A Shy Guy with many available configurations.
;;
;; Note: When rideable, clipping tables values should be: 03 0A FE 0E
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Uses first extra bit: NO
;;
;; Extra Property Byte 1
;;    bit 0 - enable spin killing
;;    bit 1 - stay on ledges
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	SPRITE_Y_SPEED  = $AA
	SPRITE_X_SPEED  = $B6
	SPRITE_Y_POS    = $D8
	ORIG_Y_POS      = $151C
	MAX_Y_SPEED     = $3E
	EXTRA_BITS	= $7FAB10
	EXTRA_PROP_1	= $7FAB28
	HIT_POINTS	= $05        	; being stomped by Mario
	H_OFFSCREEN	= $15A0
	V_OFFSCREEN	= $186C
	FREEZE_TIMER    = $1540
	SPR_OBJ_STATUS  = $1588
	SMASH_STATUS	= $1602

	IS_ON_GROUND    = $04
	SPRITE_GRAVITY 	= $20
	SPRITE_GRAVITY2 = $04
	RISE_SPEED      = $E0
	FALL_SPEED      = $10

	TIME_TO_SHAKE   = $18
	SOUND_EFFECT    = $09 
	TIME_ON_GROUND  = $40

	SPRITE_TO_GEN	= $16

	NEXT_STATE	dcb $01,$02
	NEXT_STATE2a	dcb $04,$05
	NEXT_STATE2b	dcb $04,$05,$07
	X_SPEED2	dcb $40,$C0
	X_SPEED3	dcb $18,$E8
	KILLED_X_SPEED	dcb $F0,$10
	SPRITE_STATE    = $C2

	XMAX		dcb $20,$D0
	XACCEL		dcb $02,$FE

X_SPEED		dcb $08,$F8,$08,$F8,$0C,$F4,$10,$F0,$14,$EC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
                    PHY
                    JSR SUB_HORZ_POS
                    TYA
                    STA $157C,x
                    PLY
                    LDA $1588,x             ; if on the ground, reset the turn counter
                    ORA #$04
                    STA $1588,x             ; if on the ground, reset the turn counter

                    RTL                 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"                                    
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR SPRITE_CODE_START   ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
RETURN              RTS
SPRITE_CODE_START   

		LDA SPRITE_STATE,x
		CMP #$02
		BNE ON_CEILING
		JSR UPSIDE_GFX
		BRA CONTINUE

ON_CEILING	LDA SPRITE_STATE,x	; Rebuilding the sprite
		CMP #$03
		BNE UPSIDE_WAIT
		JSR UPSIDE_GFX	; graphics routine when extra bit is set
		BRA CONTINUE

UPSIDE_WAIT	LDA SPRITE_STATE,x	; Rebuilding the sprite
		CMP #$04
		BNE WHEN_SPINNING
		JSR UPSIDE_GFX	; graphics routine when extra bit is set
		BRA CONTINUE

WHEN_SPINNING	LDA SPRITE_STATE,x
		CMP #$06
		BNE WHEN_KILLED
		JSR SHELL_GFX
		BRA CONTINUE

WHEN_KILLED	LDA SPRITE_STATE,x
		CMP #$09
		BNE TOSS_FIREBALLS
		JSR SHELL_GFX
		BRA CONTINUE

TOSS_FIREBALLS
		LDA SPRITE_STATE,x
		CMP #$07
		BNE CLEAR
		JSR SHELL_GFX
		BRA CONTINUE

CLEAR		JSR SUB_GFX	; graphics routine when extra bit is clear

CONTINUE
		LDA $14C8,x		; return if sprite status != 8

ALIVE		CMP #$08                ; \ if status != 8, return
		BNE RETURN              ; /          

		LDA $9D			; return if sprites locked
		BNE RETURN    

NO_JUMP		JSR SUB_OFF_SCREEN_X0	; only process sprite while on screen           

		JSL $018032             ; interact with sprites
               	JSL $01A7DC		; interact with mario 
		BCC NO_CONTACT          ; return if no contact

		LDA $154C,x             ; \ if sprite invincibility timer > 0 ...
		BNE NO_CONTACT          ; /    ... goto NO_CONTACT
		LDA #$08                ; \ sprite invincibility timer = $08
		STA $154C,x             ; /
		LDA $7D                 ; \  if Mario's y speed < 10 ...
		CMP #$10                ;  }   ... sprite will hurt Mario
		BMI SPRITE_WINS         ; /    

;		LDA $140D               ; \ if mario is spin jumping, goto SPIN_KILL
;		BNE SPIN_KILL           ; /

MARIO_WINS		
		JSL $01AA33             ; set Mario speed
		JSL $01AB99             ; display contact graphic

		LDA SPRITE_STATE,x	; return if sprite is invulnerable
		CMP #$00		
		BNE SPRITE_WINS

		JSR SUB_STOMP_PTS       ; give Mario points
		LDA #$28                ; \ sound effect
		STA $1DFC               ; /
		LDA #$A0		    ; \ Set THROW_FIRE timer
		STA $1564,x             ; /
		INC $1534,x             ; increment sprite hit counter
		LDA $1534,x             ; \ if sprite hit counter == 3
		CMP #HIT_POINTS         ;  |   

		BEQ SPRITE_DEAD		; /
			
		LDA #$06
		STA SPRITE_STATE,x
		BRA NEW_RETURN
SPRITE_DEAD	LDA #$09
		STA SPRITE_STATE,x

NO_COUNT
		LDA #$02                ; \ sound effect
		STA $1DF9               ; /
			
NEW_RETURN	RTS                     ; return

SPRITE_WINS	LDA $1497               ; \ if Mario is invincible...
		ORA $187A               ;  }  ... or Mario on yoshi...
		BNE NO_CONTACT          ; /   ... return
		JSR SUB_HORZ_POS        ; \  set new sprite direction
		TYA                     ;  }  
		STA $157C,x             ; /
		JSL $00F5B7             ; hurt Mario

NO_CONTACT	LDA SPRITE_STATE,x
		CMP #$00
		BEQ WALKING
		CMP #$01
		BEQ RETREATING0
		CMP #$02
		BEQ RISING0
		CMP #$03
		BEQ FLYING0
		CMP #$04
		BEQ WAITING0
		CMP #$05
		BEQ DROPPING0
		CMP #$06
		BEQ SPINNING0
		CMP #$07
		BEQ NOW_FIRE0
		CMP #$08
		BEQ DEAD0
		CMP #$09
		BEQ COUNTDOWN0

;-----------------------------------------------------------------------------------------
; state 0
;-----------------------------------------------------------------------------------------

RETREATING0	JMP RETREATING
RISING0		JMP RISING
FLYING0		JMP FLYING
WAITING0	JMP WAITING
DROPPING0	JMP DROPPING
SPINNING0	JMP SPINNING
NOW_FIRE0	JMP NOW_FIRE
DEAD0		JMP DEAD
COUNTDOWN0	JMP COUNTDOWN

WALKING

		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT      ;  |
		LDA $157C,x             ;  | flip the direction status
		EOR #$01                ;  |
		STA $157C,x             ; /

NO_OBJ_CONTACT  
		LDA EXTRA_BITS,x
		AND #$04
		BEQ FALLING             ;

		LDA $1588,x             ; run the subroutine if the sprite is in the air...
		ORA $151C,x             ; ...and not already turning
		BNE ON_GROUND           ;
		JSR SUB_CHANGE_DIR      ;
		LDA #$01                ; set that we're already turning
		STA $151C,x             ;

ON_GROUND	LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ IN_AIR
		STZ $151C,x
		STZ $AA,x
		BRA X_TIME

FALLING	LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ IN_AIR
		LDA #$10                ; \  y speed = 10
		STA $AA,x               ; /

;		STZ $AA,x

X_TIME		LDA $1534,x             ; \ set x speed based on total HP
		ASL
		CLC
		ADC $157C,x		; and direction
		TAY

		LDA X_SPEED,y           ;  |
		STA $B6,x               ; /            ; /

		JSR Hop		; Jump to custom code

IN_AIR		JSL $01802A             ; update position based on speed values

Hop:
		LDA #$D0		; If timer isn't D0..
		CMP $1504,x		; 
		BNE IncreaseHop	; Increase it.
		STZ $1504,x		; Reset timer.

		PHX
		LDA #$01
		JSL RANDOM
		TAX
		LDA NEXT_STATE,x	; \ set sprite number for new sprite
		PLX

		STA SPRITE_STATE,x

		RTS		; Return
IncreaseHop:
		INC $1504,x		; Increase timer.

RETURN2		RTS	


RANDOM			PHX
			PHP
			SEP #$30
			PHA
			JSL $01ACF9		; Random number generation routine
			PLX
			CPX #$FF		;\ Handle glitch if max is FF
			BNE NORMALRT		;|
			LDA $148B		;|
			BRA ENDRANDOM		;/
NORMALRT		INX			; Amount in plus 1
			LDA $148B		;\
			STA $4202		;| Multiply with hardware regsisters
			STX $4203		;|
			NOP			;|
			NOP			;|
			NOP			;|
			NOP			;/
			LDA $4217
ENDRANDOM		PLP
			PLX
			RTL
;-----------------------------------------------------------------------------------------
; state 1
;-----------------------------------------------------------------------------------------

RETREATING	LDA #$00     
		STA SPRITE_Y_SPEED,x	;set initial speed

		JSR Hop2		; Jump to custom code

		JSL $01801A		;apply speed

		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEED
		ADC #SPRITE_GRAVITY2
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEED
		JSL $019138		;interact with objects
		
		LDA $1564,x	; return if sprite is invulnerable
		CMP #$00
		BNE RETURN68

Hop2:
		LDA #$A0		; If timer isn't A0..
		CMP $1528,x		; 
		BNE IncreaseHop2	; Increase it.
		STZ $1528,x		; Reset timer.
		JSR SUB_HAMMER_THROW
		DEC SPRITE_STATE,x	; decrease sprite state to 01			
		RTS			; Return
IncreaseHop2:
		INC $1528,x		; Increase timer.

RETURN68	RTS     	 

;-----------------------------------------------------------------------------------------
; state 2
;-----------------------------------------------------------------------------------------

RISING		
		JSR SUB_HORZ_POS        ;determine if mario is close and act accordingly
		TYA        
		STA $157C,x

		LDA FREEZE_TIMER,x      ;if we're still waiting on the ground, return
		BNE RETURN21

		LDA SPRITE_Y_POS,x      ;check if the sprite is in original position
		CMP ORIG_Y_POS,x  
		BNE RISE

		INC SPRITE_STATE,x

		RTS                     

RISE		LDA #RISE_SPEED         ;set rising speed and apply it
		STA SPRITE_Y_SPEED,x     
		JSL $01801A             
RETURN21	RTS

;-----------------------------------------------------------------------------------------
; state 3
;-----------------------------------------------------------------------------------------

FLYING
			
	LDA H_OFFSCREEN,x	;return if offscreen horizontally
	BNE RETURN3
			
	STZ SPRITE_Y_SPEED,x

	LDA $1588,x		; \ if sprite is in contact with an object...
	AND #$03		;  |
	BEQ NO_OBJ_CONTACT3	;  |
	LDA $157C,x		;  | flip the direction status
	EOR #$01		;  |
	STA $157C,x		; /


	STZ $AA,x

NO_OBJ_CONTACT3
	LDY $157C,x             ; \ set x speed based on direction
	LDA X_SPEED3,y           ;  |
	STA $B6,x               ; /

	JSR Bolt
	JSL $01802A		; update position based on speed values

Bolt:
        LDA #$D0		; If timer isn't C0..
        CMP $1504,x		; 
        BNE IncreaseBolt		; Increase it.

		LDA $1534,x
		CMP #$02
		BCC HEALTHY
		
		PHX
		LDA #$02
		JSL RANDOM2
		TAX
		LDA NEXT_STATE2b,x	; \ set sprite number for new sprite
		PLX

		STA SPRITE_STATE,x
		BRA FINISH_STATE

HEALTHY
		PHX
		LDA #$01
		JSL RANDOM2
		TAX
		LDA NEXT_STATE2a,x	; \ set sprite number for new sprite
		PLX

		STA SPRITE_STATE,x

FINISH_STATE	STZ $1504,x		; Reset timer.

        RTS		; Return
IncreaseBolt:
        INC $1504,x		; Increase timer.

RETURN3		RTS

RANDOM2			PHX
			PHP
			SEP #$30
			PHA
			JSL $01ACF9		; Random number generation routine
			PLX
			CPX #$FF		;\ Handle glitch if max is FF
			BNE NORMALRT2		;|
			LDA $148B		;|
			BRA ENDRANDOM2		;/
NORMALRT2		INX			; Amount in plus 1
			LDA $148B		;\
			STA $4202		;| Multiply with hardware regsisters
			STX $4203		;|
			NOP			;|
			NOP			;|
			NOP			;|
			NOP			;/
			LDA $4217
ENDRANDOM2		PLP
			PLX
			RTL

;-----------------------------------------------------------------------------------------
; state 4
;-----------------------------------------------------------------------------------------

WAITING		LDA #$00     
		STA SPRITE_Y_SPEED,x	;set initial speed

		JSR Wait		; Jump to custom code

		JSL $01801A		;apply speed

		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEED2
		ADC #SPRITE_GRAVITY2
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEED2
		JSL $019138		;interact with objects
		
		LDA $1564,x	; return if sprite is invulnerable
		CMP #$00
		BNE RETURN69

Wait:
		LDA #$A0		; If timer isn't A0..
		CMP $1528,x		; 
		BNE IncreaseWait	; Increase it.
		STZ $1528,x		; Reset timer.
		JSR SUB_HAMMER_THROW
		DEC SPRITE_STATE,x	; decrease sprite state to 01			
		RTS			; Return
IncreaseWait:
		INC $1528,x		; Increase timer.

RETURN69	RTS

;-----------------------------------------------------------------------------------------
; state 5
;-----------------------------------------------------------------------------------------

DROPPING	JSL $01801A		;apply speed
			
		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEEDD
		ADC #SPRITE_GRAVITY2
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEEDD
		JSL $019138		;interact with objects
			
		LDA $1588,x	;return if not on the ground
		AND #IS_ON_GROUND
		BEQ RETURN9
		    
		JSR SUB_HORZ_POS        ; \ always face Mario
		TYA                     ;  |
		STA $157C,x             ; /

		LDA #TIME_ON_GROUND	;set time to stay on ground
		STA FREEZE_TIMER,x  
									
		STZ SPRITE_STATE,x	

RETURN9		RTS

;-----------------------------------------------------------------------------------------
; state 6
;-----------------------------------------------------------------------------------------

SPINNING

NOSPRC	JSR SUB_HORZ_POS
	TYA
	LDA XMAX,y
	CMP $B6,x
	BEQ SETSPD
	LDA $B6,x
	CLC
	ADC XACCEL,y
	STA $B6,x
SETSPD	LDA $1588,x
	AND #$04
	BEQ NOWALLS
	STZ $AA,x
NOWALLS	LDA $1588,x
	AND #$03
	BEQ INAIR

	LDA $B6,x
	EOR #$FF
	INC A
	STA $B6,x

NOSMSH	LDA $77
	AND #$04
	BEQ INITIALIZE
	LDA #$20
	STA $18BD

INITIALIZE
	STZ $1504,x		; Reset timer.
	LDA #$02
	STA SPRITE_STATE,x

INAIR	
	JSL $01802A

RETURN24
	RTS

;-----------------------------------------------------------------------------------------
; state 7
;-----------------------------------------------------------------------------------------

NOW_FIRE	LDA #$00     
		STA SPRITE_Y_SPEED,x	;set initial speed

		JSR Fire		; Jump to custom code

		JSL $01801A		;apply speed

		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEEDF
		ADC #SPRITE_GRAVITY2
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEEDF
		JSL $019138		;interact with objects
		
		LDA $1564,x	; return if sprite is invulnerable
		CMP #$00
		BNE RETURN61

Fire:
		LDA #$A0		; If timer isn't A0..
		CMP $1504,x		; 
		BNE IncreaseFire	; Increase it.
		STZ $1504,x		; Reset timer.
		LDA #$03
		STA SPRITE_STATE,x		
		RTS			; Return
IncreaseFire:
		LDA $1504,x			; Check if the timer is greater than $60
		CMP #$20
		BNE ONE_SHOT
		JSR SUB_FIRE_THROW
		BRA INC_TIME2
ONE_SHOT	CMP #$40
		BNE TWO_SHOTS
		JSR SUB_FIRE_THROW
		BRA INC_TIME2
TWO_SHOTS	CMP #$60
		BNE THREE_SHOTS
		JSR SUB_FIRE_THROW
		BRA INC_TIME2
THREE_SHOTS	CMP #$80
		BNE FOUR_SHOTS
		JSR SUB_FIRE_THROW
		BRA INC_TIME2
FOUR_SHOTS	CMP #$A0
		BNE NO_FIRE
		JSR SUB_FIRE_THROW
NO_FIRE
INC_TIME2	INC $1504,x		; Increase timer.

RETURN61	RTS

;-----------------------------------------------------------------------------------------
; state 8
;-----------------------------------------------------------------------------------------

DEAD		LDA FREEZE_TIMER,x
		BNE RETURN10

		STZ SPRITE_X_SPEED,x

		LDA $0F30
		BNE RETURN10

		LDA $0F33
		AND $01
		BNE RETURN10

		INC SMASH_STATUS,X
		LDA SMASH_STATUS,x
		CMP #$02
		BNE RETURN10

		INC SPRITE_STATE,x	;chage state to falling

RETURN10	RTS	

;-----------------------------------------------------------------------------------------
; state 9
;-----------------------------------------------------------------------------------------

COUNTDOWN	
		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT1      ;  |
		LDA #$04            	; Kill Lololo or Lalala
		STA $14C8,x
		JSL $07FC3B             ; show star animation
		LDA #$08                ; \ play sound effect
		STA $1DF9               ; /

		LDA EXTRA_BITS,x
		AND #$04
		BEQ CLEAR_END

GOAL    	DEC $13C6               ; prevent Mario from walking at the level end
		LDA #$FF                ; \ set goal
		STA $1493               ; /
		LDA #$0B                ; \ set ending music
		STA $1DFB               ; /
        	RTS                     ; return

		BRA CONTINUE_END
CLEAR_END
CONTINUE_END

NO_OBJ_CONTACT1

		LDY $157C,x             ; \ set x speed based on direction
		LDA X_SPEED2,y           ;  |
		STA $B6,x               ; /

RETURN77	JSL $01802A		;apply speed
		RTS			; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

X_OFFSET            dcb $06,$FA
X_OFFSET2           dcb $00,$FF

SUB_HAMMER_THROW
		JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
		BMI RETURN62            ; / after: Y has index of sprite being generated

		PHX
		TYX

		LDA #$20                ; \ sound effect
		STA $1DF9 

		LDA #$08
		STA $14C8,x

		LDA #SPRITE_TO_GEN
		STA $7FAB9E,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX

		PHY
                    LDA $157C,x
                    TAY
                    LDA $E4,x               ; \ set xlo position for new sprite
                    CLC
                    ADC X_OFFSET,y
                    PLY
                    STA $00E4,y

                    PHY
                    LDA $157C,x
                    TAY
                    LDA $14E0,x             ;  | set xhi position for new sprite
                    ADC X_OFFSET2,y
                    PLY
                    STA $14E0,y             ; /

                    LDA $D8,x               ; \ set y position for new sprite
                    SEC                     ;  | (y position of generator - 1)
                    SBC #$0E                ;  |
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    SBC #$00                ;  |
                    STA $14D4,y             ; /

                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    JSL $0187A7             ;  get table values for custom sprite
                    LDA #$88
                    STA EXTRA_BITS,x
                    PLX                       ; / 

                    LDA $157C,x
                    STA $157C,y
	
RETURN62	RTS		; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

X_SPEED_HAMMERa      dcb $10,$F0
Y_SPEED_HAMMERa      dcb $07,$07

SUB_FIRE_THROW
LDY #$E8                ;A:0218 X:0009 Y:0009 D:0000 DB:03 S:01E8 P:envMXdizcHC:0522 VC:104 00 FL:19452
                    LDA $157C,x             ;A:0218 X:0009 Y:00E8 D:0000 DB:03 S:01E8 P:eNvMXdizcHC:0538 VC:104 00 FL:19452
                    BNE LABEL9a              ;A:0201 X:0009 Y:00E8 D:0000 DB:03 S:01E8 P:envMXdizcHC:0570 VC:104 00 FL:19452
                    LDY #$18                ;A:0200 X:0009 Y:00E8 D:0000 DB:03 S:01E8 P:envMXdiZcHC:0210 VC:098 00 FL:21239
LABEL9a             STY $00                 ;A:0201 X:0009 Y:00E8 D:0000 DB:03 S:01E8 P:envMXdizcHC:0592 VC:104 00 FL:19452
                    LDY #$07                ;A:0201 X:0009 Y:00E8 D:0000 DB:03 S:01E8 P:envMXdizcHC:0616 VC:104 00 FL:19452
LABEL8a              LDA $170B,y             ;A:0201 X:0009 Y:0007 D:0000 DB:03 S:01E8 P:envMXdizcHC:0632 VC:104 00 FL:19452
                    BEQ LABEL7a              ;A:0200 X:0009 Y:0007 D:0000 DB:03 S:01E8 P:envMXdiZcHC:0664 VC:104 00 FL:19452
                    DEY                     ;A:0204 X:0009 Y:0007 D:0000 DB:03 S:01E8 P:envMXdizcHC:0088 VC:103 00 FL:19638
                    BPL LABEL8a              ;A:0204 X:0009 Y:0006 D:0000 DB:03 S:01E8 P:envMXdizcHC:0102 VC:103 00 FL:19638
                    RTS                     ;
LABEL7a              LDA #$02                ; \ projectile is a fireball
                    STA $170B,y             ; /

                    LDA $E4,x               ; \ set x position
                    CLC                     ;  |
                    ADC #$05                ;  |
                    STA $171F,y             ;  |
                    LDA $14E0,x             ;  |
                    ADC #$00                ;  |
                    STA $1733,y             ; /
                    
                    LDA $D8,x               ; \ set y position
                    CLC                     ;  |
                    ADC #$00                ;  |
                    STA $1715,y             ;  |
                    LDA $14D4,x             ;  |
                    ADC #$00                ;  |
                    STA $1729,y             ; /

		LDA #$50
		JSR CODE_01BF6A
		LDX $15E9
		LDA $00
		STA $173D,y
		LDA $01
		STA $1747,y

LABEL6a              RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; aiming routine
; input: accumulator should be set to total speed (x+y)
; output: $00 = y speed, $01 = x speed
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CODE_01BF6A:		STA $01
			PHX					;\ preserve sprite indexes of Magikoopa and magic
			PHY					;/
			JSR SUB_VERT_POS				; $0E = vertical distance to Mario
			STY $02					; $02 = vertical direction to Mario
			LDA $0E					;\ $0C = vertical distance to Mario, positive
			BPL CODE_01BF7C				; |
			EOR #$FF				; |
			CLC					; |
			ADC #$01				; |
CODE_01BF7C:		STA $0C					;/
			JSR SUB_HORZ_POS			; $0F = horizontal distance to Mario
			STY $03					; $03 = horizontal direction to Mario
			LDA $0F					;\ $0D = horizontal distance to Mario, positive
			BPL CODE_01BF8C				; |
			EOR #$FF				; |
			CLC					; |
			ADC #$01				; |
CODE_01BF8C:		STA $0D					;/
			LDY #$00
			LDA $0D					;\ if vertical distance less than horizontal distance,
			CMP $0C					; |
			BCS CODE_01BF9F				;/ branch
			INY					; set y register
			PHA					;\ switch $0C and $0D
			LDA $0C					; |
			STA $0D					; |
			PLA					; |
			STA $0C					;/
CODE_01BF9F:		LDA #$00				;\ zero out $00 and $0B
			STA $0B					; | ...what's wrong with STZ?
			STA $00					;/
			LDX $01					;\ divide $0C by $0D?
CODE_01BFA7:		LDA $0B					; |\ if $0C + loop counter is less than $0D,
			CLC					; | |
			ADC $0C					; | |
			CMP $0D					; | |
			BCC CODE_01BFB4				; |/ branch
			SBC $0D					; | else, subtract $0D
			INC $00					; | and increase $00
CODE_01BFB4:		STA $0B					; |
			DEX					; |\ if still cycles left to run,
			BNE CODE_01BFA7				;/ / go to start of loop
			TYA					;\ if $0C and $0D was not switched,
			BEQ CODE_01BFC6				;/ branch
			LDA $00					;\ else, switch $00 and $01
			PHA					; |
			LDA $01					; |
			STA $00					; |
			PLA					; |
			STA $01					;/
CODE_01BFC6:		LDA $00					;\ if horizontal distance was inverted,
			LDY $02					; | invert $00
			BEQ CODE_01BFD3				; |
			EOR #$FF				; |
			CLC					; |
			ADC #$01				; |
			STA $00					;/
CODE_01BFD3:		LDA $01					;\ if vertical distance was inverted,
			LDY $03					; | invert $01
			BEQ CODE_01BFE0				; |
			EOR #$FF				; |
			CLC					; |
			ADC #$01				; |
			STA $01					;/
CODE_01BFE0:		PLY					;\ retrieve Magikoopa and magic sprite indexes
			PLX					;/
			RTS					; return

;===================================
;Graphics Code
;===================================

PROPERTIES: db $7F,$3F

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAP:
	db $00,$02,$20,$22,$40,$42,$60,$62 ; WALKING 1 ;\ RIGHT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 2 ;\ RIGHT
	db $00,$02,$28,$2A,$48,$4A,$68,$6A ; WALKING 3 ;\ RIGHT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 4 ;\ RIGHT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 1 ;\ RIGHT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 2 ;\ RIGHT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 3 ;\ RIGHT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 4 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 1 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 2 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 3 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 4 ;\ RIGHT
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E

	db $00,$02,$20,$22,$40,$42,$60,$62 ; WALKING 1 ;\ LEFT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 2 ;\ LEFT
	db $00,$02,$28,$2A,$48,$4A,$68,$6A ; WALKING 3 ;\ LEFT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 4 ;\ LEFT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 1 ;\ LEFT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 2 ;\ LEFT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 3 ;\ LEFT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 4 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 1 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 2 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 3 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 4 ;\ LEFT
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E

YDISP:
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ RIGHT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ RIGHT

	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 1 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 2 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 3 ;\ LEFT
	db $D0,$D0,$E0,$E0,$F0,$F0,$00,$00 ; WALKING 4 ;\ LEFT
 
XDISP:
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT

	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT

SUB_GFX:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR A			; |
	LSR A			; | Add in frame animation rate; More LSRs for slower animation.
	LSR A
	AND #$03		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte
	ASL A
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

SPIKES_UP:
	LDA $1528,x			; Check if the timer is greater than $60
	CMP #$80
	BCC PREPARE_FIRE
	LDA $03
	CLC
	ADC #$40				; | ...use stun frames.
	STA $03					; /
	BRA DONE_WALKING

PREPARE_FIRE:
	LDA SPRITE_STATE,x		; if yelling at Mario
	CMP #$01
	BNE WHEN_DROPPING				; |
	LDA $03					; |
	CLC						; |
	ADC #$20				; | ...use stun frames.
	STA $03					; /
	BRA DONE_WALKING

WHEN_DROPPING
	LDA SPRITE_STATE,x
	CMP #$05
	BNE DONE_WALKING
	LDA $03
	CLC
	ADC #$60
	STA $03

DONE_WALKING
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE NoAdd
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$80		; | Adding 8 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAdd:
	PHX			;\ Push sprite index ..
	LDX #$07		;/ And load X with number of tiles to loop through.
Loop:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			;\ Get it back into X for an index.

	LDA $00			;\
	CLC			; | Apply X displacement of the sprite.
	ADC XDISP,x		; |
	STA $0300,y		;/ 

	LDA $01			;\
	CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
	ADC YDISP,x		; | Otherwise, both tiles would have been drawn to the same position!
	STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
				;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

	LDA TILEMAP,x
	STA $0302,y

	PHX			; Push number of times to go through loop + "left" displacement if necessary.
	LDX $02			;\
	LDA PROPERTIES,x	; | Set properties based on direction.
	STA $0303,y		;/
	PLX			; Pull number of times to go through loop.

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
		
	PLX			; Pull current tile back.
	DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
				; is drawn, then loop again to draw the first tile.

	BPL Loop		; Loop until X becomes negative (FF).
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
	LDA #$07		; A -> number of tiles drawn - 1.
				; I drew 2 tiles, so 2-1 = 1. A = 01.

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			; Never forget this!

;===================================
;Graphics Code
;===================================

PROPERTIESU: db $FF,$BF

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAPU:
	db $00,$02,$20,$22,$40,$42,$60,$62 ; WALKING 1 ;\ RIGHT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 2 ;\ RIGHT
	db $00,$02,$28,$2A,$48,$4A,$68,$6A ; WALKING 3 ;\ RIGHT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 4 ;\ RIGHT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 1 ;\ RIGHT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 2 ;\ RIGHT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 3 ;\ RIGHT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 4 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 1 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 2 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 3 ;\ RIGHT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 4 ;\ RIGHT
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E

	db $00,$02,$20,$22,$40,$42,$60,$62 ; WALKING 1 ;\ LEFT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 2 ;\ LEFT
	db $00,$02,$28,$2A,$48,$4A,$68,$6A ; WALKING 3 ;\ LEFT
	db $00,$02,$24,$26,$44,$46,$64,$66 ; WALKING 4 ;\ LEFT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 1 ;\ LEFT
	db $00,$02,$A0,$A2,$C0,$C2,$E0,$E2 ; WALKING 2 ;\ LEFT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 3 ;\ LEFT
	db $00,$02,$A8,$AA,$C8,$CA,$E8,$EA ; WALKING 4 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 1 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 2 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 3 ;\ LEFT
	db $00,$02,$A4,$A6,$C4,$C6,$E4,$E6 ; WALKING 4 ;\ LEFT
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E
	db $00,$02,$2C,$2E,$4C,$4E,$6C,$6E

YDISPU:
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ RIGHT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ RIGHT

	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 1 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 2 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 3 ;\ LEFT
	db $0D,$0D,$FD,$FD,$ED,$ED,$DD,$DD ; WALKING 4 ;\ LEFT
 
XDISPU:
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 1 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 2 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 3 ;\ RIGHT
	db $00,$10,$00,$10,$00,$10,$00,$10 ; WALKING 4 ;\ RIGHT

	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 1 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 2 ;/ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 3 ;\ LEFT
	db $10,$00,$10,$00,$10,$00,$10,$00 ; WALKING 4 ;/ LEFT

UPSIDE_GFX:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR A			; |
	LSR A			; | Add in frame animation rate; More LSRs for slower animation.
	LSR A
	AND #$03		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte
	ASL A
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

SPIKES_UPU:
	LDA $1528,x			; Check if the timer is greater than $60
	CMP #$80
	BCC PREPARE_FIREU
	LDA $03
	CLC
	ADC #$40				; | ...use stun frames.
	STA $03					; /
	BRA DONE_WALKINGU

PREPARE_FIREU:
	LDA SPRITE_STATE,x		; if yelling at Mario
	CMP #$04
	BNE WHEN_RISING			; |
	LDA $03					; |
	CLC						; |
	ADC #$20				; | ...use stun frames.
	STA $03					; /
	BRA DONE_WALKINGU

WHEN_RISING
	LDA SPRITE_STATE,x
	CMP #$02
	BNE DONE_WALKINGU
	LDA $03
	CLC
	ADC #$60
	STA $03
	
DONE_WALKINGU
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE NoAddU
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$80		; | Adding 8 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAddU:
	PHX			;\ Push sprite index ..
	LDX #$07		;/ And load X with number of tiles to loop through.
LoopU:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			;\ Get it back into X for an index.

	LDA $00			;\
	CLC			; | Apply X displacement of the sprite.
	ADC XDISPU,x		; |
	STA $0300,y		;/ 

	LDA $01			;\
	CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
	ADC YDISPU,x		; | Otherwise, both tiles would have been drawn to the same position!
	STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
				;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

	LDA TILEMAPU,x
	STA $0302,y

	PHX			; Push number of times to go through loop + "left" displacement if necessary.
	LDX $02			;\
	LDA PROPERTIESU,x	; | Set properties based on direction.
	STA $0303,y		;/
	PLX			; Pull number of times to go through loop.

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
		
	PLX			; Pull current tile back.
	DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
				; is drawn, then loop again to draw the first tile.

	BPL LoopU		; Loop until X becomes negative (FF).
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
	LDA #$07		; A -> number of tiles drawn - 1.
				; I drew 2 tiles, so 2-1 = 1. A = 01.

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			; Never forget this!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

SHELL_TILEMAP:
	dcb $84,$86,$88,$8A ; SPINNING 1 ;\ RIGHT
	dcb $CC,$CE,$EC,$EE ; SPINNING 2 ; | RIGHT
	dcb $84,$86,$88,$8A ; SPINNING 3 ; | RIGHT
	dcb $CC,$CE,$EC,$EE ; SPINNING 4 ;/ RIGHT
	dcb $84,$86,$88,$8A ; FIRING 1 ;\ RIGHT
	dcb $CC,$CE,$EC,$EE ; FIRING 2 ; | RIGHT
	dcb $84,$86,$88,$8A ; FIRING 3 ; | RIGHT
	dcb $CC,$CE,$EC,$EE ; FIRING 4 ;/ RIGHT

	dcb $84,$86,$88,$8A ; SPINNING 1 ;\ RIGHT
	dcb $CC,$CE,$EC,$EE ; SPINNING 2 ; | RIGHT
	dcb $84,$86,$88,$8A ; SPINNING 3 ; | RIGHT
	dcb $CC,$CE,$EC,$EE ; SPINNING 4 ;/ RIGHT
	dcb $84,$86,$88,$8A ; FIRING 1 ;\ RIGHT
	dcb $CC,$CE,$EC,$EE ; FIRING 2 ; | RIGHT
	dcb $84,$86,$88,$8A ; FIRING 3 ; | RIGHT
	dcb $CC,$CE,$EC,$EE ; FIRING 4 ;/ RIGHT

VERT_DISP: 	
	dcb $F0,$F0,$00,$00 ; SPINNING 1 ;\ RIGHT
	dcb $F0,$F0,$00,$00 ; SPINNING 2 ; | RIGHT
	dcb $F0,$F0,$00,$00 ; SPINNING 3 ; | RIGHT
	dcb $F0,$F0,$00,$00 ; SPINNING 4 ;/ RIGHT
	dcb $ED,$ED,$DD,$DD ; FIRING 1 ;\ RIGHT
	dcb $ED,$ED,$DD,$DD ; FIRING 2 ; | RIGHT
	dcb $ED,$ED,$DD,$DD ; FIRING 3 ; | RIGHT
	dcb $ED,$ED,$DD,$DD ; FIRING 4 ;/ RIGHT

	dcb $F0,$F0,$00,$00 ; SPINNING 1 ;\ LEFT
	dcb $F0,$F0,$00,$00 ; SPINNING 2 ; | LEFT
	dcb $F0,$F0,$00,$00 ; SPINNING 3 ; | LEFT
	dcb $F0,$F0,$00,$00 ; SPINNING 4 ;/ LEFT
	dcb $ED,$ED,$DD,$DD ; FIRING 1 ;\ LEFT
	dcb $ED,$ED,$DD,$DD ; FIRING 2 ; | LEFT
	dcb $ED,$ED,$DD,$DD ; FIRING 3 ; | LEFT
	dcb $ED,$ED,$DD,$DD ; FIRING 4 ;/ LEFT

HORZ_DISP: 	
	dcb $00,$10,$00,$10 ; SPINNING 1 ;\ RIGHT
	dcb $00,$10,$00,$10 ; SPINNING 2 ; | RIGHT
	dcb $10,$00,$10,$00 ; SPINNING 3 ; | RIGHT
	dcb $10,$00,$10,$00 ; SPINNING 4 ;/ RIGHT
	dcb $00,$10,$00,$10 ; FIRING 1 ;\ RIGHT
	dcb $00,$10,$00,$10 ; FIRING 2 ; | RIGHT
	dcb $10,$00,$10,$00 ; FIRING 3 ; | RIGHT
	dcb $10,$00,$10,$00 ; FIRING 4 ;/ RIGHT

	dcb $10,$00,$10,$00 ; SPINNING 1 ;\ LEFT
	dcb $10,$00,$10,$00 ; SPINNING 2 ; | LEFT
	dcb $00,$10,$00,$10 ; SPINNING 3 ; | LEFT
	dcb $00,$10,$00,$10 ; SPINNING 4 ;/ LEFT
	dcb $10,$00,$10,$00 ; FIRING 1 ;\ LEFT
	dcb $10,$00,$10,$00 ; FIRING 2 ; | LEFT
	dcb $00,$10,$00,$10 ; FIRING 3 ; | LEFT
	dcb $00,$10,$00,$10 ; FIRING 4 ;/ LEFT

SHELL_PROP
	dcb $3F,$3F,$3F,$3F ; SPINNING 1 ;\ RIGHT
	dcb $3F,$3F,$3F,$3F ; SPINNING 2 ; | RIGHT
	dcb $7F,$7F,$7F,$7F ; SPINNING 3 ; | RIGHT
	dcb $7F,$7F,$7F,$7F ; SPINNING 4 ;/ RIGHT
	dcb $BF,$BF,$BF,$BF ; FIRING 1 ;\ RIGHT
	dcb $BF,$BF,$BF,$BF ; FIRING 2 ; | RIGHT
	dcb $FF,$FF,$FF,$FF ; FIRING 3 ; | RIGHT
	dcb $FF,$FF,$FF,$FF ; FIRING 4 ;/ RIGHT

	dcb $7F,$7F,$7F,$7F ; SPINNING 1 ;\ LEFT
	dcb $7F,$7F,$7F,$7F ; SPINNING 2 ; | LEFT
	dcb $3F,$3F,$3F,$3F ; SPINNING 3 ; | LEFT 
	dcb $3F,$3F,$3F,$3F ; SPINNING 4 ;/ LEFT
	dcb $FF,$FF,$FF,$FF ; FIRING 1 ;\ LEFT
	dcb $FF,$FF,$FF,$FF ; FIRING 2 ; | LEFT
	dcb $BF,$BF,$BF,$BF ; FIRING 3 ; | LEFT 
	dcb $BF,$BF,$BF,$BF ; FIRING 4 ;/ LEFT
	
SHELL_GFX:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	LSR			; |
	AND #$03		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

IN_SHELL:
	LDA SPRITE_STATE,x		; if hit
	CMP #$07
	BNE DONE_WALKING3				; |
	LDA $03					; |
	CLC						; |
	ADC #$10				; | ...use stun frames.
	STA $03					; /
;	BRA DONE_WALKING3

DONE_WALKING3:
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE LOOP_START
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$20		; | Adding 16 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.

LOOP_START:
	PHX                     ; push sprite index
        LDX #$03                ; loop counter = (number of tiles per frame) - 1

LOOP_START_2:
        PHX						; push current tile number
        
        TXA                     ; \ X = index of frame start + current tile
        CLC                     ;  |
        ADC $03                 ;  |
        TAX                     ; /
        
        PHY                     ; \
        TYA                     ; |
        LSR A                   ; |
        LSR A                   ; |
        TAY                     ; | Set tile to be 16x16 ($02)
        LDA #$02				; |
        STA $0460,y             ; |
        PLY                     ; /
        
        LDA $00                 ; \ tile x position = sprite x location ($00)
        CLC                     ;  |
        ADC HORZ_DISP,x			;  |
        STA $0300,y             ; /
        
        LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
        CLC                     ;  |
        ADC VERT_DISP,x			;  |
        STA $0301,y             ; /
        
        LDA SHELL_TILEMAP,x           ; \ store tile
        STA $0302,y             ; / 
        
        PHX
        LDX $15E9
        LDA $15F6,x             ; get palette info
        PLX
        ORA SHELL_PROP,x	; flip tile if necessary
        ORA $64                 ; add in tile priority of level
        STA $0303,y             ; store tile properties
        
        PLX                     ; \ pull, X = current tile of the frame we're drawing
        INY                     ;  | increase index to sprite tile map ($300)...
        INY                     ;  |    ...we wrote 1 16x16 tile...
        INY                     ;  |    ...sprite OAM is 8x8...
        INY                     ;  |    ...so increment 4 times
        DEX                     ;  | go to next tile of frame and loop
        BPL LOOP_START_2        ; / 

        PLX                     ; pull, X = sprite index
        LDY #$FF                ; \ We've already set 460, so use FF!
        LDA #$03                ; | A = number of tiles drawn - 1
        JSL $01B7B3             ; / don't draw if offscreen
        RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; speed related
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SUB_9A04            LDA SPR_OBJ_STATUS,x
                    BMI THWOMP_1
                    LDA #$00                
                    LDY $15B8,x
                    BEQ THWOMP_2
THWOMP_1            LDA #$18
THWOMP_2            STA SPRITE_Y_SPEED,x  
                    RTS           

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_CHANGE_DIR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $019098

SUB_CHANGE_DIR      ;LDA $15AC,x
                    ;BNE LABEL41
                    ;LDA #$08   
                    ;STA $15AC,x
                    LDA $B6,x  
                    EOR #$FF   
                    INC A      
                    STA $B6,x  
                    LDA $157C,x
                    EOR #$01   
                    STA $157C,x
LABEL41             RTS        



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS       PHY                     ; 
                    LDA $1697               ; \
                    CLC                     ;  | 
                    ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
                    INC $1697               ; increase consecutive enemies stomped
                    TAY                     ;
                    INY                     ;
                    CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
                    BCS NO_SOUND            ; /    ... don't play sound 
                    LDA STAR_SOUNDS,y       ; \ play sound effect
                    STA $1DF9               ; /   
NO_SOUND            TYA                     ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET            ;  |
                    LDA #$08                ; /
NO_RESET            JSL $02ACE5             ; give mario points
                    PLY                     ;
                    RTS                     ; return
                
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical mario/sprite position check - shared
; Y = 1 if mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2   LDA #$02                ;  |
STORE_03            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X3   STZ $03                 ; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
                    CLC                     ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
                    ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
                    ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
                    CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
                    PHP                     ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
                    LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
                    LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
                    ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
                    PLP                     ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
                    SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
                    STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
                    LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
                    BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
                    EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
                    STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20             LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
                    BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
                    CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
                    BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
                    LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
                    STA $1938,y             ; /                             A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
                    STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
                    TAY                     ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
                    LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
                    CLC                     ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
                    ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
                    ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
                    CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
                    PHP                     ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
                    LDA.W $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
                    LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
                    ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
                    PLP                     ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
                    SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
                    STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
                    LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
                    BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
                    EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
                    STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22             LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
                    BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
                    BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return
