;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Flying Kamek sprite, from Yoshi's Island.
;;  Flies left in the BG, at the height positioned at in LM.
;;  Then flies right in the FG, at Mario's height.
;;
;; Coded by kaizoman666. No credit necessary.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!waitTime		=	$80
	; How many frames to wait before going from BG to FG, and vice versa.
	
!bgSpeed		=	$E0
	; X speed while in the background.

!fgSpeed		=	$30
	; X speed while in the foreground.


!warningSound	=	$28
!warningPort	=	$1DFC
	; Sound effect to play when about to appear (32 frames prior).

!spawnSound		=	$26
!spawnPort		=	$1DF9
	; Sound effect to play when actually appearing.


SmallTilemap:
	db $00,$10,$01,$11,$02		; Frame 1
	db $00,$10,$01,$11,$12		; Frame 2
SmallProperties:
	db $0F,$0F,$0F,$0F,$0F		; Frame 1
	db $0F,$0F,$0F,$0F,$0F		; Frame 2
	; Tilemap for the small sprite used in the BG.
	;  [A][C]	 Each set of five bytes assembles as shown,
	;  [B][D][E]  where each block is 8x8.
	; First table is tile numbers. Second table is YXPPCCCT.

BigTilemap:
	db $24,$44,$03,$22,$20		; Frame 1
	db $24,$44,$05,$42,$40		; Frame 2
BigProperties:
	db $3F,$3F,$3F,$3F,$3F		; Frame 1
	db $3F,$3F,$3F,$3F,$3F		; Frame 2
	; Tilemap for the big sprite used in the FG.
	;     [C][A]  Each set of five bytes assembles as shown,
	;  [E][D][B]  where each block is 16x16.
	; First table is tile numbers. Second table is YXPPCCCT.





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; actual code begins
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
	LDA #!bgSpeed		;\ Set initial X speed.
	STA $B6,x			;/
	LDA $D8,x			;\ 
	STA $151C,x			;| Preserve BG Y position.
	LDA $14D4,x			;|
	STA $1528,x			;/
	RTL

print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ScrOffs:				; Offsets for positioning in the screen.
	dw $FFE0,$0120

Main:
	LDA $1540,x			;\ Return if waiting offscreen.
	BEQ NotSpawning		;/
	
	CMP #$20			;\ 
	BNE .noWarning		;|
	LDA $C2,x			;|
	BEQ Return			;| Play a warning sound if almost time to appear.
	LDA #!warningSound	;|
	STA !warningPort	;|
	BRA Return			;/

  .noWarning
	CMP #$01			;\ 
	BNE Return			;|
						;|
	LDA $C2,x			;|
	EOR #$01			;|
	ASL					;|
	TAY					;|
	REP #$20			;|
	LDA ScrOffs,y		;|
	CLC					;|
	ADC $1A				;|
	SEP #$20			;|
	STA $E4,x			;|
	XBA					;|
	STA $14E0,x			;|
						;| If coming back onscreen, set an appropriate position.
	CPY #$02			;|  In the BG, it's to the right, at the spawn Y position.
	BNE FindMario		;|  In the FG, it's to the left, at Mario's Y position.
	LDY $151C,x			;|
	LDA $1528,x			;|
	BRA StoreYPos		;|
						;|
  FindMario:			;|
	LDA #!spawnSound	;|
	STA !spawnPort		;|
	LDA $96				;|
	CLC					;|
	ADC #$10			;|
	TAY					;|
	LDA $97				;|
	ADC #$00			;|
						;|
  StoreYPos:			;|
	STY $D8,x			;|
	STA $14D4,x			;/
	

  NotSpawning:
	JSR Graphics
	LDA $14C8,x			;\ 
	CMP #$08			;|
	BNE Return			;| Terminate if killed or game frozen.
	LDA $9D				;|
	BEQ NotDead			;/
  Return:
	RTS



  NotDead:
	JSL $018022			; Update X position.
	
	LDA $C2,x			;\ Branch if in FG mode.
	BNE Foreground		;/
	
  Background:
	JSR CheckOffscreen	;\ 
	BPL Return			;| If off the left side, switch to FG.
	LDA #!fgSpeed		;|
	BRA LoopAround		;/

  Foreground:
	JSL $01A7DC			; Interact with Mario.
	JSR CheckOffscreen	;\ 
	BMI Return			;| If off the right side, switch to BG.
	LDA #!bgSpeed		;/
	
  LoopAround:
	STA $B6,x			;\ 
	LDA $C2,x			;|
	EOR #$01			;| Set new X speed, switch FG/BG, and wait to spawn there.
	STA $C2,x			;|
	LDA #!waitTime		;|
	STA $1540,x			;/
	RTS





CheckOffscreen:
	LDA $C2,x			;\ 
	ASL					;|
	TAY					;|
	LDA $14E0,x			;|
	XBA					;|
	LDA $E4,x			;| Check whether offscreen on the desired side.
	REP #$20			;|
	SEC					;|
	SBC $1A				;|
	CMP ScrOffs,y		;|
	SEP #$20			;/
	RTS






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gfx routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XDispBig:
	db $10,$10,$00,$00,$F0
YDispBig:
	db $F8,$08,$F8,$08,$08

XDispSmall:
	db $FC,$FC,$04,$04,$0C
YDispSmall:
	db $FC,$04,$FC,$04,$04


Graphics:
	JSR GetDrawInfo
	PHX
	LDA $C2,x
	STA $0E
	LDX #$04

  TileLoop:
	LDA XDispBig,x				;\ 
	PHY							;|
	LDY $0E						;|
	BNE .notSmallX				;|
	LDA XDispSmall,x			;| Set X position.
  .notSmallX					;|
	PLY							;|
	CLC							;|
	ADC $00						;|
	STA $0300,y 				;/
	LDA YDispBig,x				;\ 
	PHY							;|
	LDY $0E						;|
	BNE .notSmallY				;|
	LDA YDispSmall,x			;| Set Y position.
  .notSmallY					;|
	PLY							;|
	CLC							;|
	ADC $01						;|
	STA.w $0301,y         		;/
	
	PHX
	LDA $14						;\ 
	AND #$04					;|
	BEQ +						;|
	TXA							;| Handle animation.
	CLC							;|
	ADC #$05					;|
	TAX							;/
  +
	LDA $0E
	BEQ SmallGFX
	
  BigGFX:
	LDA BigTilemap,x			;\ 
	STA $0302,y					;| Handle the tilemap for Kamek in the foreground.
	LDA BigProperties,x			;|
	STA $0303,y					;/
	BRA NextLoop
	
  SmallGFX:
	LDA SmallTilemap,x			;\ 
	STA $0302,y					;| Handle the tilemap for Kamek in the background.
	LDA SmallProperties,x		;|
	STA $0303,y					;/
	
  NextLoop:
	PLX							;\ 
	INY                       	;|
	INY                       	;|
	INY                       	;| Loop for all tiles.
	INY                       	;|
	DEX                       	;|
	BPL TileLoop	 			;/
	
	PLX
	LDA $0E						;\ 
	ASL							;|
	TAY							;| Draw 5 tiles; 8x8s if small, 16x16s if big.
	LDA #$04					;|
	JSL $01B7B3					;/
	RTS





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; other subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GetDrawInfo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Table1:              db $0C,$1C
Table2:              db $01,$02

GetDrawInfo:
	STZ $186C,x
	STZ $15A0,x
	LDA $E4,x
	CMP $1A
	LDA $14E0,x
	SBC $1B
	BEQ OnscreenX
	INC $15A0,x
OnscreenX:
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	SEC
	SBC $1A
	CLC
	ADC.w #$0040
	CMP #$0180
	SEP #$20
	ROL A
	AND #$01
	STA $15C4,x
	BNE Invalid

	LDY #$00
	LDA $1662,x
	AND #$20
	BEQ OnscreenLoop
	INY
OnscreenLoop:
	LDA $D8,x
	CLC
	ADC Table1,y
	PHP
	CMP $1C
	ROL $00
	PLP
	LDA $14D4,x
	ADC #$00
	LSR $00
	SBC $1D
	BEQ OnscreenY
	LDA $186C,x
	ORA Table2,y
	STA $186C,x
OnscreenY:
	DEY
	BPL OnscreenLoop
	LDY $15EA,x
	LDA $E4,x
	SEC
	SBC $1A
	STA $00
	LDA $D8,x
	SEC
	SBC $1C
	STA $01
	RTS
	
Invalid:
	PLA
	PLA
	RTS