;==========
;KooPhD
;==========

!Vial = $04			; Sprite number of the vial/potion.
!Pause = $15				; How long the Koopa stops before he throws the potion.
!Walk = 120				; How long the Koopa walks before he throws the potion (frames). 180 = 3s, 120 = 2s, 150 = 2.5s, etc.
WalkSp: db $08,$F8		; Walking speeds of the Koopa.

!ThrowSFX = $10			; The sound KooPhD makes when throwing the vial.
!ThrowPort = $1DF9

Tilemap:
	db $20,$42		; Walking 1 (upper, lower)
	db $20,$40		; Walking 2 (upper, lower)

TilemapThrow:
	db $24,$44		; Throwing 1 (upper, lower)
	db $24,$44		; These should be the same as above.

TilemapKick:
	db $24,$42
	db $24,$42		; These should be the same as above.
	
Properties:
	db $69,$29		; YXPPCCCT format.	

!Proximity = $0002		; Don't touch that.	
	
print "INIT ", pc
	JSR SUB_HORZ_POS
	TYA
	STA $157C,x
	LDA #$01			; This is to prevent the sprite from initially turning when we see it, alternatively could add EOR #$01.
	STA $151C,x
	RTL

Return:
	RTS

print "MAIN ", pc
	PHB
	PHK
	PLB
	JSR Run
	LDA $1504,x
	BEQ +
	DEC $1504,x
+
	PLB
	RTL

Run:
	JSR GFX
	JSR SUB_OFF_SCREEN_X0
	LDA $14C8,x		;If the sprite is dead..
	CMP #$08
	BNE Return		;..return
	LDA $9D
	BNE Return		;Also return if sprites are locked.
	
	JSL $018032
	JSL $01A7DC
	BCC +
	JSR Interaction
+
	LDA $1588,x
	AND #$03
	BEQ +
	LDA $157C,x
	EOR #$01
	STA $157C,x
+
	LDA $C2,x
	ASL A
	PHX
	TAX
	JMP (Ptr,x)
Ptr:
	dw Walking
	dw Pause
	dw Throw
Throw:
	PLX
	LDA $1534,x
	BNE EndSpawn
	JSR SUB_HORZ_POS
	TYA
	STA $157C,x
	LDA #!ThrowSFX
	STA !ThrowPort
	LDA #$01
	STA $1534,x
	LDA $186C,x
	BNE EndSpawn
	JSL $02A9DE
	BMI EndSpawn
	LDA #$01		; Sprite state ($14C8,x).
	STA $14C8,y
	PHX
	TYX
	LDA #!Vial		; This the sprite number to spawn.
	STA $7FAB9E,x
	PLX
	LDA $E4,x
	STA $00E4,y
	LDA $14E0,x
	STA $14E0,y
	LDA $D8,x
	SEC
	SBC #$10
	STA $00D8,y
	LDA $14D4,x
	SBC #$00
	STA $14D4,y
	PHX
	TYX
	JSL $07F7D2
	JSL $0187A7
	LDA #$08
	STA $7FAB10,x
	PLX	

EndSpawn:
	LDA $1510,x
	INC $1510,x
	CMP.b #!Pause/3
	BCC +
	STZ $1510,x
	STZ $C2,x
+	RTS
	
Pause:
	PLX
	LDA $1510,x
	INC $1510,x
	CMP #!Pause
	BCC +
	STZ $1510,x
	INC $C2,x
+	
	RTS
	
Walking:
	PLX
	STZ $1534,x
	INC $1510,x
	LDA $1510,x
	CMP #!Walk
	BCC +	
	INC $C2,x
	STZ $1510,x
+

;KooPhD's are smart, so they stay on ledges.

	JSR MaybeJumpShells
	LDA $1588,x
	ORA $151C,x
	BNE OnGround
	LDA $157C,x
	EOR #$01
	STA $157C,x
	LDA #$01
	STA $151C,x
OnGround:
	LDA $1588,x
	AND #$04
	BEQ InAir
	STZ $151C,x
InAir:
	LDA $1594,x
	BNE Skippy
	LDA $1540,x
	BEQ +
	STZ $B6,x
	BRA SpdUpdate
+
	LDY $157C,x
	LDA WalkSp,y
	STA $B6,x
	LDA #$25
	STA $AA,x
SpdUpdate:
	JSL $01802A			; UPDATE SPRITE SPEED	
	
	LDA $1504,x
	BNE +
	JSR SUB_HORZ_POS
	TYA
	CMP $157C,x
	BEQ +
	LDA #$20
	STA $1504,x
+
	LDA $1504,x
	CMP #$01
	BNE Skippy
	LDA $157C,x
	EOR #$01
	STA $157C,x
Skippy:
	RTS	
	
Interaction:
	LDA $1490
	BEQ +
	JMP StarP
+
	LDA $7D
	CMP #$10
	BMI KoopaWin
	
	JSR SUB_STOMP_PTS
	JSL $01AA33
	JSL $01AB99
	LDA $140D
	ORA $187A
	BEQ +
	JMP SPIN_KILL
+	
Main:
	LDA $186C,x
	BNE Ender
	JSL $02A9DE
	BMI Ender
	LDA #$01
	STA $14C8,y
	PHX
	TYX
	LDA #!Vial
	STA $7FAB9E,x
	PLX
	LDA $E4,x
	STA $00E4,y
	LDA $14E0,x
	STA $14E0,y
	LDA $D8,x
	STA $00D8,y
	LDA $14D4,x
	STA $14D4,y
	PHX
	TYX
	JSL $07F7D2
	JSL $0187A7
	LDA #$0C
	STA $7FAB10,x
	PLX
Ender:
	LDA #$05
	STA $9E,x
	LDA #$09
	STA $14C8,x
	JSL $07F7D2
	LDA #$09
	STA $1DFC
	RTS
	
KoopaWin:
	LDA $1497
	ORA $187A
	BNE NoContact
	JSR SUB_HORZ_POS
	TYA
	STA $157C,x
	JSL $00F5B7
NoContact:
	RTS
	
SUB_STOMP_PTS:       
	PHY                     
	LDA $1697               
	CLC                     
	ADC $1626,x             
	INC $1697               
	TAY                     
	INY                     
	CPY #$08                
	BCS NO_SOUND            
	LDA STAR_SOUNDS,y             
	STA $1DF9               
NO_SOUND:            
	TYA                     
	CMP #$08                
	BCC NO_RESET            
	LDA #$08                
NO_RESET:            
	JSL $02ACE5             
	PLY                     
	RTS                     
	
STAR_SOUNDS:
	db $00,$13,$14,$15,$16,$17,$18,$19
	
SPIN_KILL:          
	LDA #$04                
	STA $14C8,x             
	LDA #$1F                
	STA $1540,x             
	JSL $07FC3B             
	LDA #$08                
	STA $1DF9  
	JMP Main
	RTS                     
	
StarP:
	INC $1594,x
	LDA #$02                
	STA $14C8,x    
	LDA #$D0                
	STA $AA,x               
	JSR SUB_HORZ_POS         
	LDA KILLED_X_SPEED,y    
	STA $B6,x               
	JSL $01802A
	INC $18D2               
	LDA $18D2               
	CMP #$08                
	BCC NO_RESET2           
	LDA #$08                
	STA $18D2               
NO_RESET2:          
	JSL $02ACE5             
	LDY $18D2               
	CPY #$08                
	BCS NO_SOUND2           
	LDA STAR_SOUNDS,y             
	STA $1DF9               
NO_SOUND2:           
	RTS                    ; final return

;Sprite Routines

MaybeJumpShells:
	JSR Proximity
	JSR YProximity
	PHY
	LDY #$00
.Loop
	LDA !Proximity,y
	BEQ .NotSlash
	LDA $14C8,y
	CMP #$09
	BEQ .Slash
	CMP #$0A
	BNE .NotSlash
.Slash	
	LDA $1540,x
	BNE .NotSetSlash
	LDA #$0A
	STA $1540,x
.NotSetSlash
	LDA #$13
	STA $1DF9
	LDA $157C,x
	BEQ .Right
	LDA #$C0
	STA $00B6,y
	LDA #$D0
	STA $00AA,y
	BRA .NotSlash
.Right
	LDA #$30
	STA $00B6,y
	LDA #$D0
	STA $00AA,y
.NotSlash
	INY
	CPY #$0C
	BCC .Loop
	PLY
	RTS
	
Proximity:
	PHY
	LDY #$00
ProxLoop:
	STY $0F
	CPX $0F
	BEQ RangeOut
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	PHA
	SEP #$20
	LDA $14E0,y
	XBA
	LDA $00E4,y
	REP #$20
	STA $60
	PLA
	SEC
	SBC $60
	SEP #$20
	PHY
	PHA
	JSR SUB_HORZ_POS
	PLA
	EOR InvertAbility,y 
	CMP #$12
	BCS OutofRange
	PLY
	LDA #$01
	STA !Proximity,y
	INY
	CPY #$0C
	BCC ProxLoop
	PLY
	RTS	
OutofRange:					
	PLY
	LDA $00E4,y
	CMP $E4,x
	BNE RangeOut
	LDA #$01
	STA !Proximity,y
	INY
	CPY #$0C
	BCC ProxLoop
	PLY
	RTS
RangeOut:
	LDA #$00
	STA !Proximity,y
	INY
	CPY #$0C
	BCC ProxLoop
	PLY
	RTS

InvertAbility:
	db $FF,$00	
	
YProximity:						;Zero out the sprites if they are too far above/below the penguin
	PHY
	LDY #$00
YProxLoop:
	STY $0F
	CPX $0F
	BEQ YRangeOut
	LDA $14D4,x
	XBA
	LDA $D8,x
	REP #$20
	PHA
	SEP #$20
	LDA $14D4,y
	XBA
	LDA $00D8,y
	REP #$20
	STA $60
	PLA
	SEC
	SBC $60
	SEP #$20
	PHY
	PHA
	JSR SUB_VERT_POS
	PLA
	EOR InvertAbility,y
	CMP #$10
	BCS YOutofRange
	PLY
	INY
	CPY #$0C
	BCC YProxLoop
	PLY
	RTS
YOutofRange:
	PLY
	LDA $00D8,y
	CMP $D8,x
	BNE YRangeOut
	INY
	CPY #$0C
	BCC YProxLoop
	PLY
	RTS
YRangeOut:
	LDA #$00
	STA !Proximity,y
	INY
	CPY #$0C
	BCC YProxLoop
	PLY
	RTS	
	
KILLED_X_SPEED:
	db $F0,$10
	
YDisp:
	db $F0,$00
	db $F1,$01

GFX:
	JSR GET_DRAW_INFO
	LDA $1602,x
	ASL A
	STA $03
	
	LDA $1540,x
	STA $05
	LDA $C2,x
	STA $04
	BNE +
	LDA $05
	BNE +
	LDA $14
	LSR #3
	AND #$01
	ASL A
	STA $03
+	
	LDA $157C,x
	STA $02
	PHX
	LDX #$01				; # of loops to go through..
Looper:
	PHX
	TXA
	ORA $03
	TAX
	
	LDA $00
	STA $0300,y			; X position.
	LDA $01
	CLC
	ADC YDisp,x			; Add Y displacement.
	STA $0301,y
	
	LDA $05
	BEQ +
	LDA TilemapKick,x
	BRA SetTmp
+
	LDA $04
	BEQ Standard
	LDA TilemapThrow,x
	BRA SetTmp
Standard:
	LDA Tilemap,x			; ---> Takes first 4 tiles.
SetTmp:
	STA $0302,y
	
	LDX $02
	LDA Properties,x
	LDX $15E9
	ORA $15F6,x
	ORA $64
	STA $0303,y

	TYA
	LSR #2
	TAX
	LDA #$02
	STA $0460,x

	PLX
	INY #4
	DEX
	BPL Looper

	PLX
	LDY #$FF
	LDA #$02
	JSL $01B7B3
	RTS

;=================;
;SUB_HORZ_POS
;=================;

SUB_HORZ_POS:
	LDY #$00
	LDA $D1
	SEC
	SBC $E4,x
	STA $0F
	LDA $D2
	SBC $14E0,x
	BPL SPR_L16
	INY
SPR_L16:
	RTS

;=================;
;SUB_VERT_POS
;=================;

SUB_VERT_POS:
	LDY #$00
	LDA $D3
	SEC
	SBC $D8,x
	STA $0F
	LDA $D4
	SBC $14D4,x
	BPL SPR_L11
	INY
SPR_L11:
	RTS

;=================;
;SUB_OFF_SCREEN
;=================;

SPR_T12:
	db $40,$B0
SPR_T13:
	db $01,$FF
SPR_T14:
	db $30,$C0,$A0,$C0,$A0,$F0,$60,$90
	db $30,$C0,$A0,$80,$A0,$40,$60,$B0
SPR_T15:
	db $01,$FF,$01,$FF,$01,$FF,$01,$FF
	db $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X1:
	LDA #$02
	BRA STORE_03
SUB_OFF_SCREEN_X2:
	LDA #$04
	BRA STORE_03
SUB_OFF_SCREEN_X3:
	LDA #$06
	BRA STORE_03
SUB_OFF_SCREEN_X4:
	LDA #$08
	BRA STORE_03
SUB_OFF_SCREEN_X5:
	LDA #$0A
	BRA STORE_03
SUB_OFF_SCREEN_X6:
	LDA #$0C
	BRA STORE_03
SUB_OFF_SCREEN_X7:
	LDA #$0E
STORE_03:
	STA $03
	BRA START_SUB
SUB_OFF_SCREEN_X0:
	STZ $03

START_SUB:
	JSR SUB_IS_OFF_SCREEN
	BEQ RETURN_35
	LDA $5B
	AND #$01
	BNE VERTICAL_LEVEL
	LDA $D8,x
	CLC
	ADC #$50
	LDA $14D4,x
	ADC #$00
	CMP #$02
	BPL ERASE_SPRITE
	LDA $167A,x
	AND #$04
	BNE RETURN_35
	LDA $13
	AND #$01
	ORA $03
	STA $01
	TAY
	LDA $1A
	CLC
	ADC SPR_T14,y
	ROL $00
	CMP $E4,x
	PHP
	LDA $1B
	LSR $00
	ADC SPR_T15,y
	PLP
	SBC $14E0,x
	STA $00
	LSR $01
	BCC SPR_L31
	EOR #$80
	STA $00
SPR_L31:
	LDA $00
	BPL RETURN_35
	ERASE_SPRITE:
	LDA $14C8,x
	CMP #$08
	BCC KILL_SPRITE
	LDY $161A,x
	CPY #$FF
	BEQ KILL_SPRITE
	LDA #$00
	STA $1938,y
KILL_SPRITE:
	STZ $14C8,x
RETURN_35:
	RTS

VERTICAL_LEVEL:
	LDA $167A,x
	AND #$04
	BNE RETURN_35
	LDA $13
	LSR A
	BCS RETURN_35
	LDA $E4,x
	CMP #$00
	LDA $14E0,x
	SBC #$00
	CMP #$02
	BCS ERASE_SPRITE
	LDA $13
	LSR A
	AND #$01
	STA $01
	TAY
	LDA $1C
	CLC
	ADC SPR_T12,y
	ROL $00
	CMP $D8,x
	PHP
	LDA $001D
	LSR $00
	ADC SPR_T13,y
	PLP
	SBC $14D4,x
	STA $00
	LDY $01
	BEQ SPR_L38
	EOR #$80
	STA $00
SPR_L38:
	LDA $00
	BPL RETURN_35
	BMI ERASE_SPRITE

SUB_IS_OFF_SCREEN:
	LDA $15A0,x
	ORA $186C,x
	RTS

;================;
;GET_DRAW_INFO
;================;

SPR_T1:
	db $0C,$1C
SPR_T2:
	db $01,$02

GET_DRAW_INFO:
	STZ $186C,x
	STZ $15A0,x
	LDA $E4,x
	CMP $1A
	LDA $14E0,x
	SBC $1B
	BEQ ON_SCREEN_X
	INC $15A0,x

ON_SCREEN_X:
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	SEC
	SBC $1A
	CLC
	ADC.w #$0040
	CMP.w #$0180
	SEP #$20
	ROL A
	AND #$01
	STA $15C4,x
	BNE INVALID

	LDY #$00
	LDA $1662,x
	AND #$20
	BEQ ON_SCREEN_LOOP
	INY
ON_SCREEN_LOOP:
	LDA $D8,x
	CLC
	ADC SPR_T1,y
	PHP
	CMP $1C
	ROL $00
	PLP
	LDA $14D4,x
	ADC #$00
	LSR $00
	SBC $1D
	BEQ ON_SCREEN_Y
	LDA $186C,x
	ORA SPR_T2,y
	STA $186C,x
ON_SCREEN_Y:
	DEY
	BPL ON_SCREEN_LOOP
	LDY $15EA,x
	LDA $E4,x
	SEC
	SBC $1A
	STA $00
	LDA $D8,x
	SEC
	SBC $1C
	STA $01
	RTS

INVALID:
	PLA
	PLA
	RTS