;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Custom Thwomp, by Mirumo
;;
;; Description:Just like a normal thwomp but it has a smoke
;; animation when touching the ground 
;;   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        
                    ; symbolic names for RAM addresses (don't change these)
		SPRITE_Y_SPEED  = $AA
		SPRITE_X_SPEED  = $B6
		SPRITE_STATE    = $C2
		SPRITE_Y_POS    = $D8
		ORIG_Y_POS      = $151C
		EXPRESSION      = $1528
		FREEZE_TIMER    = $1540
		SPR_OBJ_STATUS  = $1588
		H_OFFSCREEN     = $15A0
		V_OFFSCREEN     = $186C            
                    
		; sprite data
		MAX_Y_SPEED     = $3E 
		TIME_TO_SHAKE   = $18
		SOUND_EFFECT    = $09   
		InvTimer = $7F9989 ; Used for many sprite misc. stuff. Make it atleast 12 bytes.

		X_SPEED             dcb $00,$F0,$00,$10     ;rest at bottom, forward speed, rest at top, backwards speed
		TIME_IN_POS         dcb $37,$18,$37,$18     ;moving forward, rest, moving backwards, rest ($48,$10,$48,$10)                                                  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
                    LDA SPRITE_Y_POS,x  
                    STA ORIG_Y_POS,x
                    LDA $E4,x  
                    CLC        
                    ADC #$08   
                    STA $E4,x  
                    RTL
      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"                                    
                    PHB                     
                    PHK                     
                    PLB                     
                    JSR SPRITE_CODE_START   
                    PLB                     
                    RTL                     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS  

SPRITE_CODE_START   JSR SPRITE_GRAPHICS

                    LDA $14C8,x             ; return if sprite status != 8
                    CMP #$08            	 
                    BNE RETURN           
                    
                    LDA $9D			        ; return if sprites locked
                    BNE RETURN    
                    
                    JSR SUB_OFF_SCREEN_X0   ; only process sprite while on screen           
                    
                    JSL $01A7DC             ; interact with mario 
                    
                    LDA SPRITE_STATE,x     
                    CMP #$01
                    BEQ CHASE

;-----------------------------------------------------------------------------------------
; state 0
;-----------------------------------------------------------------------------------------

SWIMMING            LDA V_OFFSCREEN,x       ;fall if offscreen vertically
                    BNE SET_FALLING
                    
                    LDA H_OFFSCREEN,x       ;return if offscreen horizontally
                    BNE RETURN0

Walk_Normal:
		LDA $151C,x             ;
		TAY                     ;
		LDA $1540,x             ;
		BEQ CHANGE_SPEED        ;
		LDA X_SPEED,y           ; | set y speed
		STA $B6,x               ; /
		STZ $AA,x
		BRA APPLY_SPEED
                    
CHANGE_SPEED        LDA TIME_IN_POS,y       ;A:0001 X:0007 Y:0000 D:0000 DB:01 S:01F5 P:envMXdiZCHC:0654 VC:057 00 FL:24235
                    STA $1540,x             ;A:0020 X:0007 Y:0000 D:0000 DB:01 S:01F5 P:envMXdizCHC:0686 VC:057 00 FL:24235
                    LDA $151C,x
                    INC A
                    AND #$03
                    STA $151C,x
                    
APPLY_SPEED         JSL $01802A             ; update position based on speed values
                    
                    JSR SUB_HORZ_POS        ;determine if mario is close and act accordingly
                    TYA        
                    STA $157C,x
                    STZ EXPRESSION,x
                    LDA $0F    
                    CLC        
                    ADC #$40   
                    CMP #$80                
                    BCS THWOMP_4
                    LDA #$01                
                    STA EXPRESSION,x 
THWOMP_4            LDA $0F     
                    CLC         
                    ADC #$24    
                    CMP #$50    
                    BCS RETURN0

SET_FALLING         LDA #$02                    ;set expression    
                    STA EXPRESSION,x  

                    INC SPRITE_STATE,x          ;chage state to falling
 
                    LDA #$00     
                    STA SPRITE_Y_SPEED,x        ;set initial speed

RETURN0
		RTS                     

;-----------------------------------------------------------------------------------------
; state 1
;-----------------------------------------------------------------------------------------
SPEED				dcb $10,$F0

CHASE:
				LDA #$19                ; \ sound effect
				STA $1DFC 

				LDA #$04                ; \ sound effect
				STA $1DFA 

				JSL $01A7DC             ; check for mario/sprite contact
					LDA $AA,x
					JSL $01802A             ; update position based on speed values
					STA $AA,x
					LDA $1528,x
					BNE NOT_MODE0
					JSR MODE0
					RTS
NOT_MODE0			CMP #$01
					BNE NOT_MODE1
					JSR MODE1
					RTS
NOT_MODE1			JSR MODE2
					RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ETC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MODE0				LDA $94
					STA $1570,x
					LDA $95
					STA $1504,x
					LDA $96
					CLC
					ADC #$10
					STA $1594,x
					LDA $97
					ADC #$00
					STA $1602,x
					LDA #$01
					STA $1528,x
					JSR SUB_HORZ_POS
                    TYA
                    STA $157C,x
					RTS

MODE1				JSR SPEED_DEC
					LDA $157C,x
					BNE MODE1_2
					LDA #$30
					STA $B6,x
					BRA MODE1_3
MODE1_2				LDA #$D0
					STA $B6,x
MODE1_3				LDA $1570,x
					AND #$F0
					STA $0F
					LDA $E4,x
					AND #$F0
					CMP $0F
					BNE MODE1_E
					LDA #$02
					STA $1528,x
MODE1_E				RTS

MODE2				;LDA $13
					;AND #$01
					;BNE MODE2_E
					LDA $B6,x
					BEQ MODE2_2
					CMP #$80
					BCC MODE2_1_2
					INC A
					STA $B6,x
					BRA MODE2_2
MODE2_1_2			DEC A
					STA $B6,x
MODE2_2				LDA $AA,x
					BEQ MODE2_3
					CMP #$80
					BCC MODE2_2_2
					INC A
					STA $AA,x
					BRA MODE2_3
MODE2_2_2			DEC A
					STA $AA,x
MODE2_3				LDA $B6,x
					BNE MODE2_E
					LDA $AA,x
					BNE MODE2_E
					STZ $1528,x
MODE2_E				RTS





SPEED_DEC           LDA $E4,x
                    SEC
                    SBC $1570,x
                    STA $0F
                    BNE LBL11
                    LDA #$01
                    STA $0F
LBL11               LDA $14E0,x
                    SBC $1504,x
                    BPL FACE_LEFT
                    JSR RIGHT_SPEED
                    BRA VERC
FACE_LEFT           CMP #$01
                    BEQ LEFT2
                    JSR LEFT_SPEED1
                    BRA VERC
LEFT2               JSR LEFT_SPEED2
VERC                LDA $D8,x
                    SEC
                    SBC $1594,x
                    STA $0F
                    BNE LBL12
                    LDA #$01
                    STA $0F
LBL12               LDA $14D4,x
                    SBC $1602,x
                    BPL FACE_UP
                    JSR DOWN_SPEED
                    BRA VER_E
FACE_UP             JSR UP_SPEED
VER_E               RTS

UP_SPEED            LDA $0F38
                    CMP #$00
                    BNE UP3
                    LDA $0F
                    CMP #$80
                    BCC UP2
                    LDA #$7F
UP2                 STA $0F
                    BRA UP5
UP3                 CMP #$02
                    BNE UP4
                    JSR UP1_1
                    BRA UP5
UP4                 JSR UP1_2
UP5                 LDA #$00
                    SEC
                    SBC $0F
                    STA $AA,x
                    RTS

DOWN_SPEED          LDA #$00
                    SEC
                    SBC $0F
                    STA $0F
                    LDA $0F38
                    CMP #$00
                    BNE DOWN3
                    LDA $0F
                    CMP #$80
                    BCC DOWN2
                    LDA #$7F
DOWN2               STA $AA,x
                    RTS
DOWN3               CMP #$02
                    BNE DOWN4
                    JSR UP1_1
                    BRA DOWN5
DOWN4               JSR UP1_2
DOWN5               LDA $0F
                    STA $AA,x
                    RTS






UP1_1
LOOPUP1             LDA $03
                    BEQ LOOPUP1E
                    LDA $0F
                    LSR A
                    STA $0F
                    DEC $03
                    LDA $03
                    BNE LOOPUP1
LOOPUP1E            LDA $0F
                    CMP #$80
                    BCC LOOPUP1EE
                    LDA #$7F
                    STA $0F
LOOPUP1EE           RTS

UP1_2               LDA $0F
                    CMP #$80
                    BCC LOOPUP3
                    LDA #$7F
                    STA $0F
                    STZ $03
                    RTS
LOOPUP3             LDA $03
                    BEQ LOOPUP2E
                    LDA $0F
                    ASL A
                    STA $0F
                    DEC $03
                    LDA $03
                    BNE UP1_2
LOOPUP2E            LDA $0F
                    CMP #$80
                    BCC LOOPUP2EE
                    LDA #$7F
                    STA $0F
LOOPUP2EE           RTS







LEFT_SPEED1         STZ $02
                    LDA $0F
                    CMP #$40
                    BCC LEFT1_2
                    LDA #$02
                    STA $02
                    STZ $03
                    JSR LEFT1_40
                    BRA LEFT1_3
LEFT1_2             CMP #$20
                    BCS LEFT1_3
                    LDA #$01
                    STA $02
                    STZ $03
                    JSR LEFT1_20
                    BRA LEFT1_3
LEFT1_3             LDA #$00
                    SEC
                    SBC $0F
                    STA $B6,x
                    RTS

LEFT1_40            
LOOP140             LDA $0F
                    CMP #$40
                    BCC LOOP140E
                    LSR A
                    STA $0F
                    INC $03
                    BRA LOOP140
LOOP140E            RTS
LEFT1_20            
LOOP120             LDA $0F
                    CMP #$20
                    BCS LOOP120E
                    ASL A
                    STA $0F
                    INC $03
                    BRA LOOP140
LOOP120E            RTS

LEFT_SPEED2         STZ $03
                    INC $03
                    LDA $0F
                    LSR A
                    CLC
                    ADC #$80
                    STA $0F
                    LDA #$02
                    STA $02
                    JSR LEFT1_40
                    LDA #$00
                    SEC
                    SBC $0F
                    STA $B6,x
                    RTS

RIGHT_SPEED         STZ $02
                    LDA #$00
                    SEC
                    SBC $0F
                    STA $0F
                    CMP #$40
                    BCC RIGHT_2
                    LDA #$02
                    STA $02
                    STZ $03
                    JSR LEFT1_40
                    BRA RIGHT_3
RIGHT_2             CMP #$20
                    BCS RIGHT_3
                    LDA #$01
                    STA $02
                    STZ $03
                    JSR LEFT1_20
                    BRA RIGHT_3
RIGHT_3             LDA $0F
                    STA $B6,x

                   STZ SPRITE_STATE,x      ;reset state to swimming
RETURN2             RTS 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;PROPERTIES: dcb $7F,$3F

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAP:
	dcb $00,$02,$20,$22 ; SWIM 1 	;\ RIGHT
	dcb $04,$06,$24,$26 ; SWIM 2 	;/ RIGHT
	dcb $0C,$0E,$2C,$2E ; CHASE 1 	;\ RIGHT
	dcb $08,$0A,$28,$2A ; CHASE 2 	;/ RIGHT

	dcb $00,$02,$20,$22 ; SWIM 1 	;\ LEFT
	dcb $04,$06,$24,$26 ; SWIM 2 	;/ LEFT
	dcb $0C,$0E,$2C,$2E ; CHASE 1 	;\ LEFT
	dcb $08,$0A,$28,$2A ; CHASE 2 	;/ LEFT

YDISP: 	
	dcb $F0,$F0,$00,$00 ; SWIM 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; SWIM 2 	;/ RIGHT
	dcb $F0,$F0,$00,$00 ; CHASE 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; CHASE 2 	;/ RIGHT

	dcb $F0,$F0,$00,$00 ; SWIM 1 	;\ LEFT
	dcb $F0,$F0,$00,$00 ; SWIM 2 	;/ LEFT
	dcb $F0,$F0,$00,$00 ; CHASE 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; CHASE 2 	;/ RIGHT

XDISP: 	
	dcb $F8,$08,$F8,$08 ; SWIM 1 	;\ RIGHT
	dcb $F8,$08,$F8,$08 ; SWIM 2 	;/ RIGHT
	dcb $F8,$08,$F8,$08 ; CHASE 1 	;\ RIGHT
	dcb $F8,$08,$F8,$08 ; CHASE 2 	;/ RIGHT

	dcb $08,$F8,$08,$F8 ; SWIM 1 	;\ LEFT
	dcb $08,$F8,$08,$F8 ; SWIM 2 	;/ LEFT
	dcb $08,$F8,$08,$F8 ; CHASE 1 	;\ LEFT
	dcb $08,$F8,$08,$F8 ; CHASE 2 	;/ LEFT

PROPERTIES
	dcb $3F,$3F,$3F,$3F ;
	dcb $3F,$3F,$3F,$3F
	dcb $3F,$3F,$3F,$3F
	dcb $3F,$3F,$3F,$3F

	dcb $7F,$7F,$7F,$7F
	dcb $7F,$7F,$7F,$7F
	dcb $7F,$7F,$7F,$7F
	dcb $7F,$7F,$7F,$7F

SPRITE_GRAPHICS:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	AND #$01		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

ACTIVATED:
		LDA SPRITE_STATE,x			; \ If sprite state...
		CMP #$01				; | ...is 1...
		BCC DONE_WALKING		; | 
		LDA $03					; |
		CLC						; | ...invert.
		ADC #$08				; |
		STA $03					; /
		;BRA DONE_WALKING

DONE_WALKING:
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE NoAdd
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$10		; | Adding 16 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAdd:
	PHX			;\ Push sprite index ..
	LDX #$03		;/ And load X with number of tiles to loop through.
Loop:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			;\ Get it back into X for an index.

	LDA $00			;\
	CLC			; | Apply X displacement of the sprite.
	ADC XDISP,x		; |
	STA $0300,y		;/ 

	LDA $01			;\
	CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
	ADC YDISP,x		; | Otherwise, both tiles would have been drawn to the same position!
	STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
				;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

	LDA TILEMAP,x
	STA $0302,y

	PHX
        LDX $15E9
        LDA $15F6,x             ; get palette info
        PLX
        ORA PROPERTIES,x		; flip tile if necessary
        ORA $64                 ; add in tile priority of level
        STA $0303,y             ; store tile properties

	;PHX			; Push number of times to go through loop + "left" displacement if necessary.
	;LDX $02			;\
	;LDA PROPERTIES,x	; | Set properties based on direction.
	;STA $0303,y		;/
	;PLX			; Pull number of times to go through loop.

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
		
	PLX			; Pull current tile back.
	DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
				; is drawn, then loop again to draw the first tile.

	BPL Loop		; Loop until X becomes negative (FF).
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
	LDA #$03		; A -> number of tiles drawn - 1.
				; I drew 2 tiles, so 2-1 = 1. A = 01.

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			; Never forget this!     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; speed related
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SUB_9A04            LDA SPR_OBJ_STATUS,x
                    BMI THWOMP_1
                    LDA #$00                
                    LDY $15B8,x
                    BEQ THWOMP_2
THWOMP_1            LDA #$18
THWOMP_2            STA SPRITE_Y_SPEED,x  
                    RTS                                   


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; display smoke effect for bullet bill shooter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_SMOKE           LDY #$03                ; \ find a free slot to display effect
FINDFREE            LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open

X_OFFSET            dcb $F4,$0C

FOUNDONE            LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    CLC
                    ADC #$12
                    STA $17C4,y             ; /
                    LDA #$1B                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
                    CLC
                    ADC #$F0
                    STA $17C8,y             ; /
                    LDX $15E9               
                    RTS            
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; display smoke effect for bullet bill shooter 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_SMOKE2          LDY #$03                ; \ find a free slot to display effect
FINDFREE2           LDA $17C0,y             ;  |
                    BEQ FOUNDONE2           ;  |
                    DEY                     ;  |
                    BPL FINDFREE2           ;  |
                    RTS                     ; / return if no slots open

X_OFFSET2           dcb $F4,$0C

FOUNDONE2           LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    CLC
                    ADC #$14
                    STA $17C4,y             ; /
                    LDA #$1B                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
                    CLC
                    ADC #$0F
                    STA $17C8,y             ; /
                    LDX $15E9               
                    RTS            
                    
;###############################################################################################
;###############################################################################################
;###############################################################################################
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;###############################################################################################
;###############################################################################################
;###############################################################################################


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical mario/sprite position check - shared
; Y = 1 if mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA SPRITE_Y_POS,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP SPRITE_Y_POS,x      ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA H_OFFSCREEN,x       ; \ if sprite is on screen, accumulator = 0 
                    ORA V_OFFSCREEN,x       ; |  
                    RTS                     ; / return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ V_OFFSCREEN,x       ; reset sprite offscreen flag, vertical
                    STZ H_OFFSCREEN,x       ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC H_OFFSCREEN,x       ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA SPRITE_Y_POS,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA V_OFFSCREEN,x       ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA V_OFFSCREEN,x       ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA SPRITE_Y_POS,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
                    