;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sidestepper (Mario-Bros)
;; By Sonikku
;; Description: A sprite that walks; if stomped, hit by a cape, bounce block, etc.,
;; it will move faster. Doing any of these actions again will stun it for a short 
;; period, leaving it open for defeat. Be warned; by default, after this period it'll 
;; hop back up in a new color and move faster. It goes from red -> green -> blue.
;; A shell will defeat it instantly; same with a stomp if "Can be jumped on" and 
;; "Dies when jumped on" are both enabled in the CFG file.
;; 
;; This sprite uses custom interaction with most forms of damage, barring sprites.
;; I have documented which CFG editor features are enabled through additional code.
;; 
;; Supported Features:
;;;; Extra Bit:
;;;;;; - If set, the sprite will be either a green or blue Sidestepper based on the X position.
;;;; 
;;;; 1665:
;;;;;; - Can be jumped on
;;;;;; - Dies when jumped on
;;;;;; 
;;;; Extra Property Byte 1: xBCFxxSL ********************************************************* Bit value
;;;;;; - L: Stays on ledges --------------------------------------------------------------------- $01
;;;;;; - S: Won't shift color ------------------------------------------------------------------- $02
;;;;;; - x: unused ------------------------------------------------------------------------------ $04
;;;;;; - x: unused ------------------------------------------------------------------------------ $08
;;;;;; - F: Dies from fire (unless "Disable fireball killing" is checked) ----------------------- $10
;;;;;; - C: Dies from cape (unless "Disable cape killing" is checked) --------------------------- $20
;;;;;; - B: Dies from blocks -------------------------------------------------------------------- $40
;;;;;; - E: Can exit through horizontal pipes --------------------------------------------------- $80
;;;;;; 
;;;; Other Features:
;;;;;; - If "Spawns a new sprite" is checked and it can die to fire, it will spawn a coin if killed by a fireball.
;;;;;; - If "takes 5 fireballs to kill" is checked and it can die to fire, it'll have 5 fireball HP.
;;;;;; - Spawning the sprite with $163E,x set will cause it to walk (without gravity) and appear behind objects.
;;;;;; - The highest bit of $C2,x (1xxxxxxx/#$80) determines that when $163E,x is up, the sprite gets deleted (used for an exit animation).
;;;; Default CFG Files:
;;;; - SideStepper.cfg:				No special qualities whatsoever.
;;;; - SideStepper_Ledge.cfg:			Attempts to stay on ledges.
;;;; - SideStepper_Unstompable.cfg:		Cannot be stomped; you must rely on any other method to defeat it.
;;;; - SideStepper_Unstompable_Ledge.cfg:	Same as above two; cannot be stomped and attempts to stay on ledges.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	JSR SubHorzPos	; \
	TYA		;  | set direction based on where mario is
	STA $157C,x	; /

	LDA $7FAB10,x	; \
	AND #$04	;  | branch if extra bit not set
	BEQ +		; /
	LDA $E4,x	; \
	LSR : LSR	;  | 
	LSR : LSR	;  | load color based on current x position x16
	AND #$01	;  | 
	INC		;  | 
	STA $1594,x	; /
+	RTL

	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR SideStepperMain
	PLB
	RTL
SideStepperMain:
	JSR SUB_OFF_SCREEN_X0
	LDA $64		; \
	PHA		;  | 
	LDA $C2,x	;  | \
	AND #$80	;  |  | go behind objects if state set
	BNE .behindobj	;  | /
	LDA $163E,x	;  | 
	BEQ +		;  | force sprite behind foreground if needed
.behindobj
	LDA #$10	;  | 
	STA $64		;  | 
+	JSR SubGfx	;  |  load graphics
	PLA		;  | 
	STA $64		; /
	LDA $9D		; \ return if sprites locked
	BNE .return	; /
	LDA $14C8,x	; \
	CMP #$08	;  | return if sprite isn't "normal"
	BNE .return	; /
	LDA $163E,x	; \ branch if spawn timer isn't set
	BEQ .normal	; /
	JSL $018022	; \ set x/y speed without gravity
	JSL $01801A	; /
	INC $1570,x	; increment frame counter
	LDA $1570,x	; \
	LSR : LSR	;  | set frame based on frame counter
	AND #$03	;  | 
	TAY		;  | 
	LDA $C2,x	;  | \
	AND #$01	;  |  | 
	BEQ +		;  |  | increment animation by 4 if it should display angry frames
	INY : INY	;  |  | 
	INY : INY	;  | /
+	TYA		;  | 
	STA $1602,x	; /
	LDY $157C,x	; \
	LDA .xspd_pipe,y ; | set x speed
	STA $B6,x	; /
.return	RTS
.xspd_pipe
	db $08,$F8
.normal	LDA $C2,x	; \
	AND #$80	;  | kill sprite if state is set
	BEQ .stayalive	;  | 
	LDY $161A,x	;  | \
	CPY #$FF	;  |  | 
	BEQ +		;  |  | kill sprite (but allow it to respawn if it can)
	LDA #$00	;  |  | 
	STA $1938,y	;  | /
+	STZ $14C8,x	; /
	RTS		; 
.stayalive
	JSR BounceBlockContact
	JSR CapeContact
	JSR FireballContact
	JSR MarioContact
	JSL $018032

	LDA $1570,x	; \
	LSR : LSR : LSR	;  | 
	AND #$03	;  | 
	LDY $C2,x	;  | 
	BEQ .setframe	;  | 
	CPY #$02	; /
	BNE .animate_normal
	LDA $14		; \
	AND #$01	;  | increment $1504,x every other frame
	BNE +		;  | 
	INC $1504,x	; /
+	LDA $1570,x	; \
	LSR : LSR : LSR : LSR
	AND #$01	;  | 
	CLC		;  | load frame based on state and frame count
	ADC #$08	;  | 
	BRA .setframe	;  | 
.animate_normal
	CLC		;  | 
	ADC #$04	;  | 
.setframe
	STA $1602,x	; /

	JSL $01802A	; move sprite based on x/y speeds with gravity

	LDA $1FD6,x	; \
	AND #$80	;  | damage sprite if flag is set
	BEQ +		;  | 
	STZ $1FD6,x	; /
	JSR DamageSprite
+	LDA $1588,x	; \ turn around if it hits a wall
	AND #$03	; /
	BEQ +
	JSR .handlewalls
+	LDA $1588,x	; \
	AND #$04	;  | detect if sprite is on the ground
	BEQ .in_air	; /
	LDY #$00	; \
	LDA $15B8,x	;  | 
	BEQ +		;  | manage sprite y speed; force it downwards if it's on a slope (to properly walk down them if it stays on ledges)
	LDY #$18	;  | 
+	STY $AA,x	; /
	JSR .sprite_speed
	LDA #$01	; \ sprite is on the ground (will turn around when it starts to fall)
	STA $1510,x	; /
	LDA $C2,x	; \ 
	CMP #$02	;  | load flipped animation if state matches 
	BCS .flipped_anim ;| 
	AND #$01	;  | \ set animation speed based on value in $C2,x +1
	INC		;  | /
	LDY $1594,x	;  | \
	CPY #$02	;  |  | factor in "fast" color to apply animation speed to
	BCC .setrate	;  |  | 
	INC		;  | /
	BRA .setrate	; /
.flipped_anim
	LDY #$01	; load animation rate
	LDA $1504,x	; \
	CMP #$F0	;  | branch if below #$F0
	BCC +		; /
	STZ $C2,x	; clear state
	STZ $1504,x	; clear timer
	LDA #$E0	; \ set y speed
	STA $AA,x	; /
	LDA #$10	; \ disable contact with sprites
	STA $1564,x	; /
	RTS		; 
+	CMP #$A0	; \ branch if sprite should change clor
	BEQ .increase_speed	; / 
	BCC +		; 
	LDY #$04	; load animation rate
+	TYA		; \
.setrate
	CLC		;  | 
	ADC $1570,x	;  | 
	STA $1570,x	;  | 
	RTS		; /
.in_air	LDA $1588,x	; \
	AND #$08	;  | nullify y speed if hitting a ceiling
	BEQ +		;  | 
	STZ $AA,x	; /
+	LDA $1510,x	; \ branch if "was on ground" state is not set
	BEQ +		; /
	LDA $7FAB28,x	; \
	AND #$01	;  | branch if sprite shouldn't turn around on ledges
	BEQ +		; /
	JSR .turn_around
+	STZ $1510,x	; clear "was on ground" state
	RTS		; 
.increase_speed
	LDA $7FAB28,x	; \
	AND #$02	;  | don't shift color if it isn't able to
	BNE +		; /
	LDA $1558,x	; \ don't run if timer is set
	BNE +		; /
	LDA #$80	; \ set timer
	STA $1558,x	; /
	LDA $1594,x	; \
	CMP #$02	;  | increase color value
	BCS +		;  | 
	INC $1594,x	; /
+	RTS		; 
.sprite_speed
	LDA $1594,x	; \
	ASL : ASL	;  | 
	ADC $1594,x	;  | create index for color
	ADC $1594,x	;  | 
	STA $00		; /
	LDA $C2,x	; \
	ASL		;  | 
	CLC		;  | 
	ADC $00		;  | index sprite speed by current state, color, and direction
	TAY		;  | 
	LDA $157C,x	;  | 
	BEQ +		;  | 
	INY		; /
+	LDA .xspd,y	; \ set sprite speed
	STA $B6,x	; /
	RTS

.xspd	db $0C,$F4	; slow (red)
	db $14,$EC	; fast (red)
	db $00,$00	; stunned (red) (note: leave this alone)

	db $10,$F0	; slow (green)
	db $18,$E8	; fast (green)
	db $00,$00	; stunned (green) (note: leave this alone)

	db $14,$EC	; slow (blue)
	db $20,$E0	; fast (blue)
	db $00,$00	; stunned (blue) (note: leave this alone)

.handlewalls
	LDA $7FAB28,x	; \
	AND #$80	;  | branch if sprite should enter pipes
	BEQ .turn_around ; /
	STZ $00		; clear flag
	LDY #$03	; load times to loop
-	LDA $18A7	; \ get tile number
	CMP .tile,y	; /
	BNE +		; branch if not matching
	TYA		; \
	INC		;  | set flag
	STA $00		;  | 
	BRA ++		; /
+	DEY		; decrement y
	BPL -		; loop
++	LDA $00		; \ flip direction if it never found a match
	BEQ .turn_around	; /
	LDA $C2,x	; \ 
	ORA #$80	;  | set "leaving via pipe" flag
	STA $C2,x	; /
	LDA #$20	; \ set timer
	STA $163E,x	; /
	STZ $AA,x	; no y speed
	STZ $B6,x	; no x speed

	LDA $00		; \
	DEC		;  | index by $00
	TAY		; /
	LDA $D8,x	; \ 
	DEC		;  | load y position into scratch ram
	STA $00		; /
	LDA #$0F	; \ set all bits up to #$0F i guess
	TSB $00		; /
	LDA $00		; \
	CLC		;  | offset sprite position by $00
	ADC .offset,y	;  | (this just aligns it to the tile)
	STA $D8,x	; /
	RTS		; 
.tile	db $3C,$56,$3F,$58
.offset	db $FE,$00,$FE,$00
.turn_around
	LDA $B6,x	; \
	EOR #$FF	;  | flip sprite speed
	INC		;  | 
	STA $B6,x	; /
	LDA $157C,x	; \
	EOR #$01	;  | flip sprite direction
	STA $157C,x	; /
	RTS		; 

BounceBlockContact:
	LDA $1564,x	; \ return if sprite shouldn't interact with other sprites
	BNE .no_contact	; /
	LDY #$03	; get # of slots to check for
-	LDA $1699,y	; \ don't check clipping with nonexistent slots
	BEQ .loop	; /
	LDA $18F8,y	; \
	CMP #$03	;  | don't check clipping with bounce blocks that currently shouldn't interact with sprites
	BCS .loop	; /
	LDA $16CD,y	; \ load index for bounce block sprite clipping/position (?)
	BEQ .loop	; / don't interact if it's zero
	PHX		; \ 
	TAX		;  | 
	LDA $16D1,y	;  | 
	CLC		;  | 
	ADC $029656,x	;  | set bounce block clipping x position
	STA $00		;  | 
	LDA $16D5,y	;  | 
	ADC $029658,x	;  | 
	STA $08		; /
	LDA $02965A,x	; \ set bounce block clipping width
	STA $02		; /
	LDA $16D9,y	; \
	CLC		;  | 
	ADC $02965C,x	;  | 
	STA $01		;  | set bounce block clipping y position
	LDA $16DD,y	;  | 
	ADC $02965E,x	;  | 
	STA $09		; /
	LDA $029660,x	; \ set bounce block clipping height
	STA $03		; /
	PLX		; 
	JSL $03B69F	; get sprite clipping
	JSL $03B72B	; check for contact
	BCS +		; 
.loop	DEY		; \ loop if needed
	BPL -		; /
.no_contact
	RTS		; 
+	PHY		; 
	STZ $00		; 
	LDA $16D1,y	; \
	SEC		;  | 
	SBC $E4,x	;  | get bounce block x offset low from sprite
	STA $0C		;  | 
	LDA $16D5,y	; /
	SBC $14E0,x	; \
	BPL +		;  | set flag for which side sprite is on
	INC $00		; /
+	LDA $00		; \ set sprite direction
	STA $157C,x	; /
	LDA $0C		; \
	EOR #$FF	;  | 
	INC		;  | 
	PLY		;  | 
	BPL +		;  | 
	LSR		;  | 
	CMP #$EC	;  | set sprite x speed based on it's distance from the bounce block
	BPL .setspd	;  | 
	LDA #$EC	;  | 
	BRA .setspd	;  | 
+	ASL		;  | 
	CMP #$14	;  | 
	BMI .setspd	;  | 
	LDA #$14	;  | 
.setspd	STA $B6,x	; /
DamageSprite:
	LDA $7FAB28,x	; \
	AND #$40	;  | damage sprite if it shouldn't actually die
	BEQ +		; /
	LDA #$C0	; \ set y speed
	STA $AA,x	; /
	LDA #$03	; \ kill sprite
	STA $1DF9	; /
	JMP KillSprite2
+	LDA #$D8	; \ set sprite y speed
	STA $AA,x	; /
	LDA #$18	; \ set timer to disable contact
	STA $1564,x	; /
	LDA #$03	; \ play sound
	STA $1DF9	; /
	LDA #$00	; \ give score
	JSL $02ACE5	; /
DamageSprite2:
	LDY $C2,x	; \
	LDA $C2,x	;  | 
	CMP #$02	;  | 
	BCC +		;  | set sprite state based on which one it already is
	LDY #$FF	;  | 
+	INY		;  | 
	TYA		;  | 
	STY $C2,x	; /
	STZ $1504,x	; end stun timer
	STZ $1510,x	; don't mark sprite as "in the air"
	LDA $7FAB28,x	; \
	AND #$02	;  | don't shift color back if it shouldn't shift colors
	BNE +		; /
	LDA $1558,x	; \
	BEQ +		;  | 
	LDA $1594,x	;  | allow color change reversal if the timer is set
	BEQ +		;  | 
	DEC $1594,x	;  | 
+	STZ $1558,x	; /
	RTS		; 

CapeContact:
	LDA $166E,x
	AND #$20
	ORA $1FE2,x
	BNE .no_contact
	LDA $1632,x	; \
	PHY		;  | 
	LDY $74		;  | 
	BEQ +		;  | don't interact with cape if mario and the sprite
	EOR #$01	;  | are not both above/behind the foreground together
+	PLY		;  | 
	EOR $13F9	;  | 
	BNE .no_contact	; /
	LDA $13E8	; \ don't interact with cape if cape contact is disabled
	BEQ .no_contact	; /
	LDA $15D0,x	; \
	ORA $1FE2,x	;  | don't interact with cape if sprite is being eaten or it just shouldn't
	BNE .no_contact ; /
	LDA $13E9	; \
	SEC		;  | 
	SBC #$02	;  | 
	STA $00		;  | set cape clipping x position
	LDA $13EA	;  | 
	SBC #$00	;  | 
	STA $08		; /
	LDA #$14	; \ set cape clipping width
	STA $02		; /
	LDA $13EB	; \
	STA $01		;  | set cape clipping y position
	LDA $13EC	;  | 
	STA $09		; / 
	LDA #$10	; \ set cape clipping height
	STA $03		; /
	JSL $03B69F	; get sprite clipping
	JSL $03B72B	; check for contact
	BCC .no_contact	; 
	JSR SubHorzPos	; get mario/sprite distance information
	TYA		; \ set sprite direction
	STA $157C,x	; /
	LDA Killed_X_Speed,y	; \ set sprite speed
	STA $B6,x	; /
	LDA #$C0	; \ set sprite vertical speed
	STA $AA,x	; /
	LDA $7FAB28,x	; \
	AND #$20	;  | damage sprite if it shouldn't actually die
	BEQ +		; /
	LDA #$03	; \ play sound
	STA $1DF9	; /
	JMP KillSprite2	; kill sprite
+	JSR DamageSprite	; damage the sprite
	LDA #$18	; \ set timer to disable contact
	STA $1FE2,x	; /
.no_contact
	RTS

FireballContact:
	LDA $166E,x
	AND #$10
	BNE .no_contact
	LDY #$09	; times to loop
-	LDA $170B,y	; \
	CMP #$05	;  | 
	BEQ +		;  | only detect yoshi and mario fireballs
	CMP #$11	;  |
	BNE .loop	; /
+	LDA $171F,y	; \
	SEC		;  | 
	SBC #$02	;  | 
	STA $00		;  | fireball clipping x position
	LDA $1733,y	;  | 
	SBC #$00	;  | 
	STA $08		; /
	LDA #$0C	; \ fireball clipping width
	STA $02		; /
	LDA $1715,y	; \
	SEC		;  | 
	SBC #$04	;  | 
	STA $01		;  | fireball clipping y position
	LDA $1729,y	;  | 
	SBC #$00	;  | 
	STA $09		; /
	LDA #$0C	; \ fireball clipping height
	STA $03		; /
	JSL $03B69F	; get sprite clipping
	JSL $03B72B	; check for contact
	BCS +		; 
.loop	DEY		; 
	BPL -		; 
	RTS		; 
+	LDA #$01	; \
	STA $170B,y	;  | turn fireball to smoke
	LDA #$10	;  | 
	STA $176F,y	; /
	LDA $7FAB28,x	; \
	AND #$10	;  | branch if it should die to fire
	BNE .kill_fire	; /
	PHX		; 
	LDX #$10	; \
	LDA $1747,y	;  | 
	BPL +		;  | 
	LDX #$F0	;  | set sprite x speed based on fireball x direction
+	TXA		;  | 
	PLX		;  | 
	STA $B6,x	; /
	LDA #$E8	; \ set sprite y speed
	STA $AA,x	; /
	JMP DamageSprite	; damage sprite
.kill_fire
	LDA $190F,x	; \
	AND #$08	;  | check if sprite takes 5 fireballs to kill
	BEQ .kill	; /
	INC $1528,x	; increase "hits" counter
	LDA $1528,x	; \
	CMP #$05	;  | kill sprite if hits reaches max
	BCS .kill	; /
+	LDA #$01	; \ play sound
	STA $1DF9	; /
	RTS		; 
.no_contact
	RTS		; 
.kill	LDA #$03	; \ play sound
	STA $1DF9	; /
	LDY #$00	; \
	LDA $1747,y	;  | 
	BPL +		;  | 
	INY		;  | set x speed based on fireball direction
+	LDA Killed_X_Speed,y	;  | 
	STA $B6,x	;  | 
	TYA		;  | 
	STA $157C,x	; /
	LDA $1686,x	; \
	AND #$40	;  | turn to coin if it spawns a new sprite
	BNE .turn_to_coin	; /
	LDA #$E0
	STA $AA,x
	JMP KillSprite2
.turn_to_coin
	LDA #$21	; \ new sprite number
	STA $9E,x	; /
	LDA #$08	; \ new sprite status
	STA $14C8,x	; /
	JSL $07F7D2	; initialize sprite
	LDA #$D0	; \ set y speed
	STA $AA,x	; /
	RTS		; 

MarioContact:
	LDA $154C,x	; \ return if mario can't interact with sprite
	BNE .no_contact	; /
	JSL $01A7DC	; \ return if no contact
	BCC .no_contact	; /
	LDA $C2,x	; \
	CMP #$02	;  | branch if not stunned
	BCC +		; /
	LDA #$03	; \ play sound
	STA $1DF9	; /
	LDA #$10	; \ set mario kick frame timer
	STA $149A	; /
	JSR SubHorzPos	; \
	LDA Killed_X_Speed,y	;  |  set x speed
	STA $B6,x	; /
	LDA #$E0	; \ set y speed
	STA $AA,x	; /
	JMP KillSprite2	; kill sprite
+	LDA $1490	; \ branch if not using a star
	BEQ .nostar	; /
	JSR GiveScore_Star
	LDA #$02	; \ kill sprite
	STA $14C8,x	; /
	LDA #$E0	; \ set y speed
	STA $AA,x	; /
	LDY $157C,x	; \
	LDA Killed_X_Speed,y	; | set x speed
	STA $B6,x	; /
.no_contact
	RTS
.nostar	JSR SubVertPos	; get mario vertical position from sprite's
	REP #$20	; \
	LDA $0E		;  | 
	LDY $187A	;  | \
	BEQ +		;  |  | account for yoshi
	SEC		;  |  | 
	SBC #$0010	;  | /
+	CMP #$0014	;  | hurt mario if below interaction point
	SEP #$20	;  | 
	BCC .spritewins	; /
	LDA $7D		; \ no contact if moving upwards
	BMI .no_contact	; /
	LDA #$10	; \ disable interaction for a bit
	STA $154C,x	; /
	LDA $1656,x	; \
	AND #$10	;  | branch if "can be stomped" is checked
	BNE .stompable	; /
	LDA $140D	; \ hurt mario when not spinjumping
	ORA $187A	;  | or on yoshi
	BEQ .spritewins	; /
	LDA #$02	; \ play sound
	STA $1DF9	; /
	BRA .endcontact	; finish contact
.stompable
	JSL $01AB99	; display contact graphic
	JSL $01AA33	; set mario speed
	LDY #$00	; \
	LDA $187A	;  | set y to 1 if riding yoshi
	BEQ +		;  | 
	INY		; /
+	LDA $140D	; \ branch if not spinjumping
	ORA $187A	;  | or not riding yoshi
	BEQ .normalstomp ;/
.killspinjump
	TYA
	BNE +
	LDA #$F0	; \ set mario speed
	STA $7D		; /
+	JSR GiveScore	; give score
	LDA #$04	; \ set sprite state (turn to smoke)
	STA $14C8,x	; /
	LDA #$1F	; \ set sprite timer (smoke timer)
	STA $1540,x	; /
	JSL $07FC3B	; killed sprite stars
	LDA #$08	; \ play sound
	STA $1DF9	; /
	RTS		; 
.normalstomp
	LDA $1656,x	; \
	AND #$20	;  | kill sprite if "dies when stomped" is set
	BNE .killsprite	; / 
	JSR GiveScore	; give score
	JSR DamageSprite2	; damage the sprite
.endcontact
	JSL $01AA33	; set mario speed
	JSL $01AB99	; display contact graphic
	RTS
.spritewins
	LDA #$10	; \ time to disable contact with sprite
	STA $154C,x	; /
	LDA $187A	; \ hurt mario if not riding yoshi
	BEQ .hurtmario	; /
	PHX		; 
	LDX $18DF	; load index of yoshi
	DEX		; 
	PHK		; 
	PEA.w .loseyoshi-1	; \
	PEA.w $0180CA-1		;  | JSL2RTS; lose yoshi routine
	JML $01F70E		; /
.loseyoshi
	PHX		; \
	LDX $76		;  | set x speed based on
	LDA $01EBBE,x	;  | flipped mario direction
	PLX		;  | 
	STA $B6,x	; /
	PLX		; 
	RTS		; 
.hurtmario
	JSL $00F5B7	; hurt mario
	RTS		; 
.killsprite
	LDA #$03	; \ play sound
	STA $1DF9	; /
	LDA #$02	; \ kill sprite
	STA $14C8,x	; /
	STZ $B6,x	; no x speed
	STZ $AA,x	; no y speed
	LDA #$02	; \ give score
	JSR GiveScore	; /
	BRA .endcontact	; 
KillSprite:
	LDA #$C0	; \ set y speed
	STA $AA,x	; /
	LDY $157C,x	; \
	LDA Killed_X_Speed,y	; | set x speed
	STA $B6,x	; /
	LDA #$03
	STA $1DF9
KillSprite2:
	LDA #$02	; \ kill sprite
	STA $14C8,x	; /
	LDA #$00	; \ give score
	JSL $02ACE5	; /
	RTS
Killed_X_Speed:
	db $F0,$10
GiveScore_Star:
	PHY		; \
	LDA $18D2	;  | load star kill counter
	INC $18D2	;  | increment star kill counter
	BRA GiveScoreParent
GiveScore:
	PHY		; 
	LDA $1697	; 
	CLC		; 
	ADC $1626,x	; 
	INC $1697	; 
GiveScoreParent:
	TAY		; 
	INY		; 
	CPY #$08	; \ don't play sound if above #$08
	BCS +		; /
	TYA		; \
	CLC		;  | play sound based on current value
	ADC #$12	;  | 
	STA $1DF9	; /
+	TYA		; 
	CMP #$08	; \ 
	BCC +		;  | don't continue incrementing score if above #$08
	LDA #$08	; /
+	JSL $02ACE5	; give score
	PLY		; 
	RTS		; 

SubGfx:
	JSR GET_DRAW_INFO

	PHY		; 
	LDY $1602,x	; \
	LDA $14C8,x	;  | 
	CMP #$08	;  | set frame index
	BEQ +		;  | override frame index if killed
	LDY #$08	;  | 
+	STY $02		; /
	PLY		; 

	LDA $00
	STA $0300,y

	LDA $01
	STA $0301,y

	PHX
	LDX $02		; index by current frame
	LDA .tilemap,x	; load tilemap
	PLX
	STA $0302,y

	LDA $1594,x	; load sprite color
	PHX
	TAX
	LDA .palette,x	; set to palette
	PHA
	LDX $02		; index by current frame
	PLA
	ORA .flip,x	; set x flip
	ORA #$01	; set second gfx page (change to #$00 to set to first)
	PLX		; 
	ORA $64		; factor in level settings
	STA $0303,y	; 

	LDY #$02
	LDA #$00
	JSL $01B7B3
	RTS
.tilemap
	db $80,$82,$80,$82
	db $A0,$A2,$A0,$A2
	db $C0,$C2

.flip	db $00,$00,$40,$40
	db $00,$00,$40,$40
	db $00,$00

.palette
	db $08,$0A,$06,$06

;; NOTE: I tried to keep the tilemap as customizable as possible. If you wish
;;;;;;;; to save graphic file space, replace the "$C0,$C2" in .tilemap with
;;;;;;;; "$80,$82", and also change the very last "$00,$00" in .flip to "$80,$80".
;;;;;;;; This will make the stunned/death frames use the regular walking frames,
;;;;;;;; but flipped vertically.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:            db $40,$B0
SPR_T13:            db $01,$FF
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:  STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0:  STZ $03					; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$30                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

SubHorzPos:

	LDY #$00	; 
	LDA $14E0,x	; \
	XBA		;  | 
	LDA $E4,x	;  | 
	REP #$20	;  | get subtract sprite's x position by mario's
	SEC		;  | 
	SBC $94		;  | 
	BMI +		;  | 
	INY		;    increment y if positive 
+	STA $0C		;  | 
	SEP #$20	; /
	RTS		; 


SubVertPos:
	LDY #$00	; 
	LDA $14D4,x	; 
	XBA		;  | 
	LDA $D8,x	;  | 
	REP #$20	;  | get subtract sprite's y position by mario's
	SEC		;  | 
	SBC $96		;  | 
	BMI +		;  | 
	INY		;    increment y if positive
+	STA $0E		;  | 
	SEP #$20	; /
	RTS		; 