;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SMB2 Carryable Item 2, by imamelia, mikeyk, and ICB
;
; This is a carryable item from SMB2 that interacts with objects.
;
; Extra bytes: 0
;
; Sprite states:
; - 00 = carryable bomb
; - 01 = Pow
; - 02 = mushroom block
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

incsrc subroutinedefs.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; 00 - carryable bomb
; 01 - Pow
; 02 - mushroom block
; 03-FF - unused
Tilemap:
db $CE,$C8,$C6

FuseSparkTiles:
db $AC,$AD,$BD,$AD

Palette:
db $0D,$09,$03

BombFlashPalettes:
db $0D,$07,$0D,$09,$0D,$05,$0D,$03

XSpeed:
db $20,$E0

KilledXSpeed:
db $F0,$10

!MushBlkMap16Num = $0502

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:
	LDA $14C8,x
	CMP #$09
	BCS HandleCarried
	JSR CarriableItem2Main
Init:
	RTL

HandleCarried:
	LDY $1510,x
	PHY
	JSL $81812B
	PLA
	STA $1510,x
	JSR CarriableItem2Main
	LDY $15EA,x
	LDA #$F0
	STA $0309,y
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ReturnMain:
RTS

CarriableItem2Main:

JSR CarriableItem2GFX

LDA $1510,x			;
JSL $8086DF			;

dw S00_CarryBomb		; 00 - carriable bomb
dw S01_Pow			; 01 - Pow
dw S02_MushBlock		; 02 - mushroom block

S00_CarryBomb:
	JSR CarriableItem2GFX
	LDA $9D
	BNE .Return1
	LDA $1534,x
	BNE .Explode
	LDA $1504,x
	BNE .NoExplode
	LDA #$09
	STA $1DFC
	LDA #$00
	STA $7FAB10,x
	LDA #$0D
	STA $9E,x
	LDA #$08
	STA $14C8,x
	JSL $87F7D2
	LDA #$40
	STA $1540,x
	INC $1534,x
	RTS
.NoExplode
	STZ $1540,x
	DEC $1504,x
.SkipDec
	JSL $83B664
	JSL $83B69F
	JSL $83B72B
	BCS .TryCarry
.Return1
	RTS
.TryCarry
	BIT $15
        BVC .Return1
        LDA $1470
        ORA $187A
        BNE .Return1
        LDA #$0B
        STA $14C8,x
	RTS
.Explode
	PHB
	LDA #$82
	PHA
	PLB
	JSL $828086
	PLB
	RTS
	
S01_Pow:
	STZ $1540,x
	JSR CarriableItem2GFX
	LDA $1588,x
	AND #$07
	BNE .ActivatePow
	JSL $83B664
	JSL $83B69F
	JSL $83B72B
	BCC .Return
.TryCarry
	BIT $15
        BVC .Return
        LDA $1470
        ORA $187A
        BNE .Return
        LDA #$0B
        STA $14C8,x
	STZ $1540,x
	RTS
.ActivatePow
	LDA #$28
	STA $1887
	LDA #$16
	STA $1DFC
	LDA #$02
	STA $14C8,x
	STZ $B6,x
	STZ $AA,x
	JSL $80FA80
.Return
	RTS
	
S02_MushBlock:
	STZ $1540,x
	JSR CarriableItem2GFX
	LDA $1686,x
	AND #$F7
	STA $1686,x
	LDA $9D
	BNE .Return1
	JSL $83B664
	JSL $83B69F
	JSL $83B72B
	BCS .TryCarry
	LDA $B6,x
	BNE .Return1
	LDA $1588,x
	AND #$04
	BEQ .SetTimer
	LDA $1540,x
	BNE .Return1
	STZ $14C8,x
	LDA $E4,x
	CLC
	ADC #$08
	AND #$F0
	STA $9A
	LDA $14E0,x
	ADC #$00
	STA $9B
	LDA $D8,x
	CLC
	ADC #$08
	AND #$F0
	STA $98
	LDA $14D4,x
	ADC #$00
	STA $99
	PHP
	REP #$30
	LDA #!MushBlkMap16Num
	STA $03
	JSL !SubSetMap16
	PLP
.Return1
	RTS
.SetTimer
	LDA #$05
	STA $1540,x
	RTS
.TryCarry
	LDA $1588,x
	AND #$04
	BNE .Return1
	BIT $15
        BVC .Return1
        LDA $1470
        ORA $187A
        BNE .Return1
	STZ $1540,x
        LDA #$0B
        STA $14C8,x
	LDA $1686,x
	ORA #$08
	STA $1686,x
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CarriableItem2GFX:

JSL !GetDrawInfo			;

PHY						;
LDY $1510,x				;
LDA Tilemap,y				;
STA $02					;
LDA Palette,y				;
ORA $64					;
STA $03					;
PLY						;

LDA $14C8,x				;
CMP #$02				;
BNE .NoVFlip				;
LDA #$80				;
TSB $03					;
.NoVFlip					;

LDA $00					;
STA $0300,y				;

LDA $01					;
STA $0301,y				;

LDA $02					;
STA $0302,y				;

LDA $03					;
STA $0303,y				;

LDA $1510,x				;
BEQ .BombGFX				;

LDY #$02					;
LDA #$00				;
JSL $81B7B3				;
RTS						;

.BombGFX

LDA $1504,x				;
CMP #$20				;
BCS .NoFlash				;
LSR						;
AND #$07				;
TAX						;
LDA.w BombFlashPalettes,x	;
ORA $64					;
STA $0303,y				;
.NoFlash					;

LDA $00					;
CLC						;
ADC #$FD				;
STA $0304,y				;

LDA $01					;
CLC						;
ADC #$FB				;
STA $0305,y				;

LDA $1504,x				;
LSR #2					;
AND #$03				;
TAX						;
LDA.w FuseSparkTiles,x		;
STA $0306,y				;

LDA #$01				;
ORA $64					;
STA $0307,y				;

TYA						;
LSR #2					;
TAX						;
LDA #$02				;
STA $0460,x				;
STZ $0461,x				;

LDX $15E9				;
LDY #$FF					;
LDA #$01				;
JSL $81B7B3				;
RTS						;

;------------------------------------------------
; init and main pointers
;------------------------------------------------

dl Init,Main



	