;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;SMW/SMB1 Mushroom Scale Platforms
;by Ice Man
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;No extra bit = short range platforms
;Extra bit set = long range platforms
;#$00 to $1E00 = SMB styled scales (tiles above platform)
;#$01 to $1E00 = SMW styled scales (tiles below platform)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Init & Main Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	dcb "INIT"
	LDA #$01					;#$00 = SMB scales
	STA $1E00					;#$01 = SMW scales
	
	LDA $D8,x					;Sprite Ypos Low Byte (table)
	STA $1534,x
	LDA $14D4,x					;Sprite Ypos High Byte (table)
	STA $151C,x					;Vertical direction

	LDA $7FAB10,x				;Long/short scale
	AND #$04					;platforms now depend
	BNE Init_X					;on the extra bit
	
	LDA $E4,x					;Sprite Xpos Low Byte (table)
	CLC							;Clear carry flag
	ADC #$60					;Add in #$60 (long range)
	STA $C2,x					;Sprite State
	BRA More_Init				;Continue Init

Init_X:
	LDA $E4,x					;Sprite Xpos Low Byte (table)
	CLC							;Clear carry flag
	ADC #$40					;Add in #$40 (short range)
	STA $C2,x					;Sprite State

More_Init:	
	LDA $14E0,x					;Sprite Xpos High Byte (table)
	ADC #$00					;Add in nothing
	STA $1602,x
	RTL							;Return

	dcb "MAIN"
	LDA $15EA,x					;Sprite's index to OAM
	PHA
	PHB
	PHK							;Wrapper
	PLB
	JSR Code_Start				;Execute sprite
	PLB
	PLA
	STA $15EA,x					;Sprite's index to OAM
	RTL							;Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
;Sprite code start
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Code_Start:
	JSR SUB_OFF_SCREEN_X2	 	;Sub off screen X2
	STZ $185E					;Keep track of a tile to generate			
	LDA $E4,x					;Sprite Xpos Low Byte (table)
	PHA							;Push A
	LDA $14E0,x					;Sprite Xpos High Byte (table)
	PHA							;Push A
	LDA $D8,x					;Sprite Ypos Low Byte (table)
	PHA							;Push A
	LDA $14D4,x					;Sprite Ypos High Byte (table)
	PHA							;Push A
	LDA $151C,x					;Vertical direction
	STA $14D4,x					;Sprite Ypos High Byte (table)
	LDA $1534,x
	STA $D8,x					;Sprite Ypos Low Byte (table)
	LDA $C2,x					;Sprite State
	STA $E4,x					;Sprite Xpos Low Byte (table)
	LDA $1602,x
	STA $14E0,x					;Sprite Xpos High Byte (table)
	LDY #$02					;#$02 into Y
	JSR Block_Init				;Generate tiles
	PLA							;Pull A
	STA $14D4,x					;Sprite Ypos High Byte (table)
	PLA							;Pull A
	STA $D8,x					;Sprite Ypos Low Byte (table)
	PLA							;Pull A
	STA $14E0,x					;Sprite Xpos High Byte (table)
	PLA							;Pull A
	STA $E4,x					;Sprite Xpos Low Byte (table)
	BCC Scale_Down
	INC $185E					;Keep track of a tile to generate
	LDA #$F8					;Up-speed (right platform)
	JSR Set_Position			;Set sprite movement in current frame

Scale_Down:
	LDA $15EA,x					;Sprite's index to OAM
	CLC							;Clear carry flag
	ADC #$08					;Add in #$08
	STA $15EA,x					;Sprite's index to OAM
	LDY #$00					;#$00 to Y
	JSR Block_Init				;Generate block init
	BCC Scale_Up
	INC $185E					;Keep track of a tile to generate
	LDA #$08					;Down-speed (left platform)
	JSR Set_Position			;Set sprite movement in current frame

Scale_Up:
	LDA $185E					;Keep track of a tile to generate
	BNE Return
	LDY #$02					;#$02 to Y
	LDA $D8,x					;Sprite Ypos Low Byte (table)
	CMP $1534,x
	BEQ Return
	LDA $14D4,x					;Sprite Ypos High Byte (table)
	SBC $151C,x					;Vertical direction
	BMI Do_Position

	LDY #$FE					;Restore speed
Do_Position:
	TYA							;Y -> A
	JSR Set_Position			;Set sprite movement in current frame

Return:
	RTS							;Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Generate tiles
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Scale_SMW:
	dcb $02,$07,$07,$02			;empty, scale, scale, empty

Scale_SMB:
	dcb $07,$02,$02,$07			;scale, empty, empty, scale	
	
Block_Init:
	LDA $D8,x					;Sprite Ypos Low Byte (table)
	AND #$0F					;Generate tile every 16 pixel
	BNE Invisible_Block
	LDA $AA,x					;Sprite Y Speed Table
	BEQ Invisible_Block
	LDA $AA,x					;Sprite Y Speed Table
	BPL Generate_Block
	INY							;Increase Y

Generate_Block:
	LDA $1E00					;Check free RAM
	BEQ SMB_Scales				;#$00 = SMB scales

	LDA Scale_SMW,y				;Set scale tiles
	STA $9C						;Generate Map16 tile
	BRA Continue_Block			;Continue block code

SMB_Scales:
	LDA Scale_SMB,y				;Set scale tiles
	STA $9C						;Generate Map16 tile

Continue_Block:
	LDA $E4,x					;Sprite Xpos Low Byte (table)
	STA $9A						;Block creation: X position (low)
	LDA $14E0,x					;Sprite Xpos High Byte (table)
	STA $9B						;Block creation: X position (high)
	LDA $D8,x					;Sprite Ypos Low Byte (table)
	STA $98						;Block creation: Y position (low)
	LDA $14D4,x					;Sprite Ypos High Byte (table)
	STA $99						;Block creation: Y position (high)
	JSL $00BEB0					;Generate tiles

Invisible_Block:
	JSR MushroomScaleGfx		;Draw scale graphics
	STZ $1528,x
	JSL $01B44F					;Invisible Block Main
	RTS							;Return

Set_Position:
	LDY $9D						;Lock sprites timer
	BNE Return2
	PHA							;Push A
	JSL $01801A					;ypos no gravity
	PLA							;Pull A
	STA $AA,x					;Sprite Y Speed Table
	LDY #$00					;#$00 to Y
	LDA $1491					;Set sprite movement in current frame
	EOR #$FF					;XOR #$FF
	INC A						;Increase A
	BPL More_Position
	DEY							;Decrease Y

More_Position:
	CLC							;Clear carry flag
	ADC $1534,x
	STA $1534,x
	TYA							;Y -> A
	ADC $151C,x					;Vertical direction
	STA $151C,x					;Vertical direction

Return2:
	RTS							;Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Draw GFX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	

MushroomScaleGfx:
	JSR GET_DRAW_INFO			;Get draw info

	LDA $00						;
	SEC							;Set X-position
	SBC #$08					;of mushroom scale
	STA $0300,y					;left and right.
	CLC							;
	ADC #$10					;
	STA $0304,y					;

	LDA $01						;Set Y-position of
	DEC A						;mushroom scale
	STA $0301,y					;left and right.
	STA $0305,y					;

	LDA #$80					;Tile to use for 
	STA $0302,y					;mushroom scale
	STA $0306,y					;left and right.

	LDA $15F6,x					;
	ORA $64						;Properties of
	STA $0303,y					;mushroom scale
	ORA #$40					;left and right.
	STA $0307,y					;

	LDA #$01                	;Tiles drawn -1
	LDY #$02                	;460 = 2 (all 16x16 tiles)
	JSL $01B7B3					;Don't draw off screen
	RTS							;Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;GET_DRAW_INFO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \ 
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \ 
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 

                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Sub Off Screen X0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return