;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Pix (Green), by Yoshicookiezeus
; Based on mikeyk's Spiky Ball sprite
;
; Description: Bounces around diagonally. Can only be hurt by green Koopa shells.
;
; Uses first extra bit: NO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "INIT"
			RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "MAIN"                                    
			PHB			; \
			PHK			;  | main sprite function, just calls local subroutine
			PLB			;  |
			JSR SPRITE_CODE_START	;  |
			PLB			;  |
			RTL			; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

X_SPEED			dcb $18,$E8
Y_SPEED			dcb $18,$E8

RETURN			RTS
SPRITE_CODE_START	JSR SUB_GFX		; graphics routine
			LDA $14C8,x		; if sprite is dead, return
			CMP #$08
			BNE RETURN
			LDA $9D			;if sprites locked, return
			BNE RETURN
			JSL $01A7DC		;interact with Mario

			LDA $1588,x		; \ if sprite is in contact with an object...
			AND #$03		;  |
			BEQ NO_CONTACT_x		;  |
			LDA $157C,x		;  | flip the direction status
			EOR #$01		;  |
			STA $157C,x		; /

NO_CONTACT_x		LDA $1588,x		; \ if sprite is in contact with an object...
			AND #$0C		;  |
			BEQ NO_CONTACT_Y		;  |
			LDA $151C,x		;  | flip the direction status
			EOR #$01		;  |
			STA $151C,x		; /

NO_CONTACT_Y		LDY $157C,x		; \ set x speed based on direction
			LDA X_SPEED,y		;  |
			STA $B6,x		; /
			LDY $151C,x		; \ set x speed based on direction
			LDA Y_SPEED,y		;  |
			STA $AA,x		; /                    
                                        
			JSL $01802A		; update position based on speed values

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hitpoints routine 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


			LDY #$0B
LABEL4			LDA $9E,y
			CMP #$04 
			BNE LABEL5
			LDA $14C8,y
			CMP #$09
			BEQ LABEL6
			CMP #$0A
			BEQ LABEL6
LABEL5			DEY
			BPL LABEL4
			RTS

LABEL6			PHX
			TYX
			JSL $03B6E5
			PLX
			JSL $03B69F
			JSL $03B72B
			BCC LABEL5

			PHX
			TYX
			STZ $14C8,x
			LDA $E4,x
			STA $9A
			LDA $14E0,x
			STA $9B
			LDA $D8,x
			STA $98
			LDA $14D4,x
			STA $99
			PHB
			LDA #$02
			PHA
			PLB
			LDA #$FF
			JSL $028663
			PLB
			PLX
			INC $1528,x
			LDA #$28
			STA $1DFC

CHECK_IF_DIE		LDA $1528,x		;if hits taken =/= 2, branch
			CMP #$02
			BNE LABEL8
			LDA #$04		;kill sprite
			STA $14C8,x
			LDA #$1F
			STA $1540,x
			JSL $07FC3B
			LDA #$23		;play sound effect
			STA $1DF9

LABEL8			RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP			dcb $80,$82
X_OFFSET		dcb $00,$00
Y_OFFSET		dcb $00,$00

SUB_GFX			JSR GET_DRAW_INFO	; sets y = OAM offset
			LDA $157C,x		; \ $02 = direction
			STA $02			; / 
			LDA $14			; \ 
			LSR A			;  |
			LSR A			;  |
			CLC			;  |
			ADC $15E9		;  |
			AND #$01		;  |
			STA $03			;  | $03 = index to frame start (0 or 1)
			PHX			; /
                    
			LDA $14C8,x
			CMP #$02
			BNE LOOP_START_2
			STZ $03
			LDA $15F6,x
			ORA #$80
			STA $15F6,x

LOOP_START_2		LDA $00			; \ tile x position = sprite x location ($00)
			STA $0300,y		; /

			LDA $01			; \ tile y position = sprite y location ($01)
			STA $0301,y		; /

			LDA $15F6,x		; tile properties xyppccct, format
			ORA $64			; add in tile priority of level
			STA $0303,y		; store tile properties

			LDX $03			; \ store tile
			LDA TILEMAP,x		;  |
			STA $0302,y		; /

			INY			; \ increase index to sprite tile map ($300)...
			INY			;  |    ...we wrote 1 16x16 tile...
			INY 			;  |    ...sprite OAM is 8x8...
			INY			; /    ...so increment 4 times

			PLX			; pull, X = sprite index
			LDY #$02		; \ 460 = 2 (all 16x16 tiles)
			LDA #$00		;  | A = (number of tiles drawn - 1)
			JSL $01B7B3		; / don't draw if offscreen
			RTS			; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
