;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Prize Koopas
;; By Erik557
;; For ASM Contest Round 0
;; Assembles with SpriteTool v1.41 (Asar)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; These koopas hold a prize depending on the extra property byte.
;; The prize is customizable, as is the amount of rounds.
;; Scroll below to see what's configurable and what is not.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Uses first extra property byte 1 - YES
;;   Value
;;     00  -  No prize
;;     01  -  Prize 1 (coins, customizable)
;;     02  -  Prize 2 (flower)
;;     03  -  Prize 3 (feather)
;;     04  -  Prize 4 (1-up)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!CoinAmount = 10                   ; coin amount in decimal
!MaxSpeed = $35                    ; max speed the shell can go (don't go crazy or above 7F)
!MinSpeed = $11                    ; minimun speed the shell needs (don't set this too high!)
!SpinningTime = $A0                ; should be more than 81.

tilemap:
       db $8A,$8C,$8A,$8E          ; tilemap of the shell
Prize:
       db $00,$01,$75,$77,$78      ; sprite number of the prize, except for coins (coins SHOULD be 1)
PrizeTiles:
       db $00,$E8,$26,$0E,$24      ; tile the prize uses (16x16)
PrizeProps:
       db $00,$04,$0A,$44,$0A      ; properties of the prize tile


print "INIT ",pc
       PHB
       PHK
       PLB
       LDA $7FAB28,x          ;\
       TAY                    ; | use extra property bit to get which prize to use
       LDA Prize,y            ; | and store it
       STA $1594,x            ;/
       LDA PrizeTiles,y       ;\ tiles the prizes use
       STA $1602,x            ;/
       LDA PrizeProps,y       ;\ tile properties of the prize
       STA $1FD6,x            ;/ 
       LDA #$7A               ; timer before the prize appears at the start
       STA $1540,x            ;
       LDA #$18               ; another timer wtf
       STA $1528,x            ;
       LDA #!SpinningTime-$80 ; another timer!
       JSR RANDOM             ;
       CLC                    ;
       ADC #$80               ;
       STA $151C,x            ;
       PLB
RTL

print "MAIN ",pc
       PHB
       PHK
       PLB
       JSR PrizeKoopa
       PLB
RTL

MainReturn:
       RTS

PrizeKoopa:
       JSR graphics         ; it's cancer
       LDA $14C8,x
       CMP #$08
       BNE MainReturn
       JSL $01802A          ; always update speed
       JSL $018032
       
       LDA $C2,x
       ASL
       TAX
       JMP.w (.youbet,x)
.youbet
       dw SetUp             ; set up and start the game
       dw Speed             ; speed up
       dw GivePrize         ; ready to give prize
       dw End               ; give prize, and end bonus game

SetUp:
       LDX $15E9            ; restore sprite index
       LDA $1540,x          ;\ wait until timer hits 0
       BEQ .SkipGFX         ;/
       CMP #$4D
       BCS +
       CMP #$4B
       BCC .Lower
       LDA #$10             ;\ play sound
       STA $1DF9            ;/
       JSR DrawSmoke        ; spawn smoke
       BRA +
.Lower
       LDA $14              ;\
       AND #$01             ; | decrease prize ypos every other frame
       BNE +                ; |
       DEC $1528,x          ;/
       BRA +
.SkipGFX
       INC $C2,x
+      RTS
Speed:
       LDX $15E9     ; restore sprite index
       LDA $151C,x   ;\ stop all movement
       BEQ .Stop     ;/
       LDA $14       ;\
       AND #$01      ; | decrease timer every certain frames
       BNE .Ignore   ;/
       DEC $151C,x
.Ignore
       LDA $14
       AND #$1F
       BNE .Return
       LDA #!MaxSpeed
       JSR RANDOM
       CMP #!MinSpeed
       BCC .Return
       STA $B6,x
       BRA .Return
.Stop
       LDA $E4,x
       CMP #$10
       BCC .Return
       CMP #$DE
       BCS .Return
       LDA $14
       AND #$01
       BNE .Return
       
       JSR CheckClipping
       
       LDA $B6,x     ;\ if speed is negative increase instead
       BEQ .Done     ; | if 0 we're done
       BMI .Increase ;/
       DEC $B6,x     ; decrease speed
       BRA .Return
.Increase
       INC $B6,x     ; increase speed
       BRA .Return
.Done
       INC $C2,x
.Return
       RTS

GivePrize:
       LDX $15E9            ; restore sprite index
       JSL $01A7DC          ;\ return if no contact
       BCC .Return          ;/
       LDA $15              ;\
       AND #$08             ; | return if not pressing up
       BEQ .Return          ;/
       JSR KillSprites      ; kill other shells
       LDA #$10             ;\ play sfx
       STA $1DF9            ;/
       LDA #$80             ;\ fadeout music
       STA $1DFB            ;/
       INC $C2,x
.Return
       RTS
End:
       LDX $15E9
       LDA $1528,x          ;\
       CMP #$1D             ; | advance if tile high enough
       BEQ +                ;/
       LDA $14              ;\
       AND #$01             ; | increase prize ypos every other frame
       BNE .Return          ; |
       INC $1528,x          ;/
       LDA #$10             ;\ set a timer
       STA $1540,x          ;/
       BRA .Return
+
       LDY $15EA,x          ; load sprite index in OAM
       LDA #$F0             ;\ kill prize tile
       STA $0305,y          ;/
       
       LDA $1540,x          ;\ return if not zero
       BNE .Return          ;/
       
       LDA $1594,x          ;\
       BEQ .nothing         ; | check prize Mario got
       CMP #$01             ; |
       BEQ .Coin            ; |
       JSR Spawn            ; |
       BRA .End             ;/
.nothing
       LDA #$2A             ;\ play sound
       STA $1DFC            ;/
       BRA .End
.Coin
       LDA #!CoinAmount     ;\ store coins
       STA $13CC            ;/
       LDA #$01
       STA $1DFC
.End
       LDA #$58
       STA $14AB
       
       INC $188F
       JSR DrawSmoke
       LDA $17C4,y
       ADC #$12
       STA $17C4,y
       STZ $14C8,x
       STZ $71
.Return
       RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; graphics shit goes here
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

props:
       db $00,$00,$40,$00

graphics:
       JSR GetDrawInfo      ; GET_DRAW_INFO
       
       LDA $00              ;\ xpos of the tile
       STA $0300,y          ;/
       LDA $01              ;\ ypos of the tile
       STA $0301,y          ;/
       LDA $14              ;\
       LSR #2               ; |
       AND #$03             ; | tile of the tile (!)
       TAX                  ; |
       LDA tilemap,x        ; |
       STA $0302,y          ;/
       LDA props,x          ;\
       LDX $15E9            ; > restore sprite index
       ORA $64              ; |
       ORA $15F6,x          ; | properties of the tile
       STA $0303,y          ;/
       
       PEI ($00)            ; preserve x and y pos relative to screen border
       LDY #$02             ; 16x16 tiles
       TDC                  ; 1 tile drawn (TDC = LDA #$00, 1 byte saved)
       JSL $01B7B3          ; finish OAM write
       
       LDY $15EA,x
       INY #4               ; get new set of OAM slots
       PLA                  ;\ restore xpos
       STA $00              ;/
       PLA                  ;\ restore ypos
       STA $01              ;/
       LDA $C2,x            ; if not time to draw prize, return
       BEQ +
       CMP #$03
       BNE .Return
+      LDA $1602,x          ;\ if no prize, no point on running this
       BEQ .Return          ;/

       LDA $00              ;\ xpos of the tile, should be the same
       STA $0300,y          ;/
       LDA $01              ;\
       SEC                  ; | ypos of the tile, go down every 2 frames
       SBC $1528,x          ; |
       STA $0301,y          ;/
       LDA $1602,x          ;\ we stored the tiles earlier
       STA $0302,y          ;/
       LDA $1FD6,x          ;\ so did we with the properties
       STA $0303,y          ;/
       
       LDY #$02             ; Locke: Again!
       LDA #$03             ; yes, but for some reason tiles get always drawn as 8x8...
       JSL $01B7B3          ;
.Return
       RTS

;============;
;RNG by 1024
;============;

RANDOM:
	PHX
	PHP
	SEP #$30
	PHA
	JSL $01ACF9
	PLX
	CPX #$FF
	BNE NORMALRT
	LDA $148B
	BRA ENDRANDOM
NORMALRT:
	INX
	LDA $148D
	STA $4202
	STX $4203
	NOP #4
	LDA $4217	
ENDRANDOM:
	PLP
	PLX
	RTS

;============================
KillSprites:
;============================

	LDY #$09             ; setup loop
-	LDA $14C8,y          ;\ if sprite non-existent,
	BEQ ++               ;/ branch

	LDA $7FAB9E,x        ;\ if custom sprite is same number as this one (i.e. one of the dummies),
	PHX                  ; |
	TYX                  ; |
	CMP $7FAB9E,x        ; |
	BEQ +                ;/ branch
.Kill
	LDA #$04             ;\ set sprite status (killed by spinjump)
	STA $14C8,y          ;/
	LDA #$1F             ;\ set time to show smoke cloud
	STA $1540,y          ;/
+	PLX
++	DEY							; decrease sprite index
	BPL -						; if still sprites left to check, branch
.Return
	RTS							; return

;======================
; Custom clipping check
;======================

CheckClipping:
       LDY #$0B
-      LDA $7FAB9E,x
       TYX
       CMP $7FAB9E,x
       BEQ +
       TXY
       LDX $15E9
       LDA.w $14C8,y
       BNE .FreeLoop
       BRA .CheckNext
+      TXY
       LDX $15E9
.CheckNext
       DEY
       BPL -
       RTS

.FreeLoop
       LDA.w $E4,y
       STA $00
       LDA $14E0,y
       STA $08
       LDA #$10
       STA $02
       LDA.w $D8,y
       STA $01
       LDA $14D4,y
       STA $09
       LDA #$10
       STA $03
       JSL $03B69F
       JSL $03B72B
       BCC .CheckNext
       
       LDA #$25
       JSR RANDOM
       INC
       XBA
       LDA $B6,x
       BMI .Neg
       CMP #$06
       BCS +
       XBA
       BRA .setX
.Neg   CMP #$FA
       BCC +
       XBA
       EOR #$FF
       INC A
.setX  STA $B6,x
+      BRA .CheckNext

;;;;;;;;;;;;;;;;;;;;;;;
;; Smoke Routine
;;;;;;;;;;;;;;;;;;;;;;;

DrawSmoke:
	LDY #$03
-
	LDA $17C0,y
	BEQ +
	DEY
	BPL -
	RTS
+
	LDA #$01
	STA $17C0,y
	LDA #$1C     
	STA $17CC,y
	LDA $D8,x
       SEC
       SBC #$13
	STA $17C4,y
	LDA $E4,x
	STA $17C8,y
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Spawn Sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Spawn:
	XBA
       LDA $1337
	JSL $02A9E4
	BMI .EndSpawn
	LDA #$08
	STA $14C8,y
	XBA
	STA.w $9E,y
	PHX
	TYX
	JSL $07F7D2
	PLX
	LDA $94
	STA.w $E4,y
	LDA $95
	STA $14E0,y
	LDA $96
       CLC
       ADC #$08
	STA.w $D8,y
	LDA $97
       ADC #$00
	STA $14D4,y
.EndSpawn
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GetDrawInfo - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GetDrawInfo:
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	SEC
	SBC $1A
	STA $00
	CLC
	ADC.w #$0040
	CMP.w #$0180
	SEP #$20
	LDA $01
	BEQ +
	LDA #$01
+	STA $15A0,x
	TDC
	ROL A
	STA $15C4,x
	BNE .Invalid
	LDA $14D4,x
	XBA
	LDA $1662,x
	AND #$20
	BEQ .CheckOnce
.CheckTwice
	LDA $D8,x
	REP #$21
	ADC.w #$001C
	SEC
	SBC $1C
	SEP #$20
	LDA $14D4,x
	XBA
	BEQ .CheckOnce
	LDA #$02
.CheckOnce
	STA $186C,x
	LDA $D8,x
	REP #$21
	ADC.w #$000C
	SEC
	SBC $1C
	SEP #$21
	SBC #$0C
	STA $01
	XBA
	BEQ .OnScreenY
	INC $186C,x
.OnScreenY
	LDY $15EA,x
	RTS
		
.Invalid
	PLA
	PLA
	RTS
