;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Rocketbarrel, by yoshicookiezeus
; 
; Description: A barrel that once triggered moves constantly to the right and
; is affected by gravity, exploding if it touches anything. Once Mario has touched 
; it he is stuck inside until the screen defined below, and after pressing the jump
; button thrice to activate it can be accelerated upwards by holding the jump button.
;
; NOTE: Make sure Mario can't enter the barrel while on Yoshi!
;
; Uses the extra bit: Yes
; Depending on the extra bit, either the first or the second value in the
; ScreenToStop table is used to determine on which screen Mario is released
; from the barrel.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite constants - feel free to change these
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!SpeedX		= $20
	!MaxSpeedY	= $40
	!MinSpeedY	= $C0

ScreenToStop:
	db $08,$10	; which screen to stop at - first value if extra bit is clear, second if it is set

Tilemap:
	db $80,$82,$A0,$A2
	db $84,$86,$A4,$A6
	db $88,$8A,$A8,$AA
OffsetX:
	db $08,$F8,$08,$F8
OffsetY:
	db $08,$08,$F8,$F8

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROM and RAM defines - don't mess with these
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!RAM_ControllerA	= $15
	!RAM_ControllerAFrame	= $16
	!RAM_ScreenBndryYLo	= $1C
	!RAM_ScreenBndryYHi	= $1D
	!RAM_MarioSpeedX	= $7B
	!RAM_MarioSpeedY	= $7D
	!RAM_MarioXPos		= $94
	!RAM_MarioXPosHi	= $95
	!RAM_MarioYPos		= $96
	!RAM_MarioYPosHi	= $97
	!RAM_SpritesLocked	= $9D
	!RAM_SpriteNum		= $9E
	!RAM_SpriteSpeedY	= $AA
	!RAM_SpriteSpeedX	= $B6
	!RAM_SpriteState	= $C2
	!RAM_SpriteYLo		= $D8
	!RAM_SpriteXLo		= $E4
	!OAM_DispX		= $0300
	!OAM_DispY		= $0301
	!OAM_Tile		= $0302
	!OAM_Prop		= $0303
	!RAM_SpriteStatus	= $14C8
	!RAM_SpriteYHi		= $14D4
	!RAM_SpriteXHi		= $14E0
	!RAM_SprObjStatus	= $1588
	!RAM_CurrentSprIndex	= $15E9
	!RAM_SpritePal		= $15F6
	!RAM_SmokeNum		= $17C0
	!RAM_SmokeYLo		= $17C4
	!RAM_SmokeXLo		= $17C8
	!RAM_SmokeTimer		= $17CC
	!RAM_ExtraBits		= $7FAB10
	!RAM_CustSpriteNum	= $7FAB9E

	!ExecutePtr		= $0086DF
	!KillMario		= $00F606
	!UpdateYPosNoGrvty	= $01801A
	!UpdateXPosNoGrvty	= $018022
	!SprObjInteract		= $019138
	!MarioSprInteract	= $01A7DC
	!FinishOAMWrite		= $01B7B3
	!GetSpriteClippingA	= $03B69F
	!GetSpriteClippingB	= $03B6E5
	!CheckForContact	= $03B72B
	!InitSpriteTables	= $07F7D2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
Init:
	PHB
	PHK
	PLB

	LDY #$09				; setup loop
.LoopStart
	CPY !RAM_CurrentSprIndex		;\ if sprite being checked is this one,
	BEQ .Continue				;/ branch
	LDA !RAM_SpriteStatus,y			;\ if sprite being checked is non-existant,
	BEQ .Continue				;/ branch
	PHX
	TYX
	LDA !RAM_CustSpriteNum,x		;\ if sprite being checked isn't the same as this one,
	PLX					; |	
	CMP !RAM_CustSpriteNum,x		; |
	BNE .Continue				;/ branch
	STZ !RAM_SpriteStatus,x			; if code gets here, there is another rocketbarrel active, so this one is destroyed
	PLB
	RTL

.Continue
	DEY					; decrease loop counter
	BPL .LoopStart				; if sprites left to check, branch
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:
	JSR SpriteGraphics
	LDA !RAM_SpritesLocked
	BNE .Return
	LDA !RAM_SpriteStatus,x
	CMP #$08
	BNE .Return

	LDA !RAM_SpriteState,x
	JSL !ExecutePtr

.StatePointers
	dw Inactive
	dw Waiting
	dw Active
	dw Finished

.Return
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Inactive:
	JSL !MarioSprInteract		;\ if Mario not touching sprite,
	BCC .Return			;/ branch
	INC !RAM_SpriteState,x		; go to next state
.Return
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Waiting:
	JSR MarioInBarrel
	LDA !RAM_ControllerAFrame	;\ if A or B isn't pressed,
	AND #$80			; |
	BEQ .Return			;/ branch
	JSR SpawnSmoke
	LDA #$09			;\ play sound effect
	STA $1DFC			;/
	LDA $1504,x			;\ increase ignition count
	INC A				; |
	STA $1504,x			;/
	CMP #$03			;\ if ignition count not at 3,
	BNE .Return			;/ branch
	LDA #$E0
	STA !RAM_SpriteSpeedY,x
	INC !RAM_SpriteState,x		; go to next state
.Return
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Active:
	JSR MarioInBarrel

	LDA #$08			;\ set camera position (right)
	STA $1400			;/

	LDA !RAM_SpriteYHi,x
	XBA
	LDA !RAM_SpriteYLo,x
	REP #$20
	CMP !RAM_ScreenBndryYLo
	SEP #$20
	BCC .AboveScreen
	REP #$20
	CMP #$0000
	SEP #$20
	BPL .NotAboveScreen

.AboveScreen
	LDA !RAM_ScreenBndryYHi
	STA !RAM_SpriteYHi,x
	LDA !RAM_ScreenBndryYLo
	STA !RAM_SpriteYLo,x
	STZ !RAM_SpriteSpeedY,x
	JMP .NotBelowScreen

.NotAboveScreen
	REP #$20
	SEC
	SBC !RAM_ScreenBndryYLo
	CMP #$00F0
	SEP #$20
	BCC .NotBelowScreen

	JSL $00F60A
	RTS

.NotBelowScreen

	LDA !RAM_SpriteSpeedX,x		;\ if max X speed has been reached,
	CMP #!SpeedX			; |
	BEQ .NoAccX			;/ branch
	CLC				;\ increase X speed
	ADC #$02			; |
	STA !RAM_SpriteSpeedX,x		;/
.NoAccX

	LDA !RAM_ControllerA		;\ if A or B button is not pressed,
	AND #$80			; |
	BEQ .Continue			;/ branch
	LDA #$04			;\ set acceleration timer
	STA $154C,x			;/
.Continue
	LDA $154C,x			;\ if acceleration timer nonzero,
	BNE .Up				;/ branch
	LDA !RAM_SpriteSpeedY,x
	CMP #!MaxSpeedY
	BEQ .SpeedDone
	INC !RAM_SpriteSpeedY,x
	INC !RAM_SpriteSpeedY,x
	BRA .SpeedDone
.Up
	LDA $1540,x
	BNE .NoSmoke
	LDA #$08
	STA $1540,x
	JSR SpawnSmoke
	LDA #$27			;\ play sound effect
	STA $1DFC			;/
.NoSmoke
	LDA !RAM_SpriteSpeedY,x
	CMP #!MinSpeedY
	BEQ .SpeedDone
	DEC !RAM_SpriteSpeedY,x
	DEC !RAM_SpriteSpeedY,x
.SpeedDone

	LDA !RAM_SpriteSpeedY,x
	BMI .Negative
	CMP #$20
	BCC .Neutral
	LDA #$02
	BRA .StoreFrame
.Negative
	CMP #$E0
	BCS .Neutral
	LDA #$00
	BRA .StoreFrame
.Neutral
	LDA #$01
.StoreFrame
	STA $1602,x

	JSL !SprObjInteract		; interact with objects
	LDA !RAM_SprObjStatus,x		;\ if sprite isn't touching any objects,
	AND #$EF			; |
	BEQ .NoObjectContact		;/ branch
	JSR Explode
	RTS

.NoObjectContact

	LDY #$09
.LoopStart
	CPY !RAM_CurrentSprIndex		;\ if sprite being checked is this one,
	BEQ .NoSpriteContact			;/ branch
	LDA !RAM_SpriteStatus,y			;\ if sprite being checked is non-existant,
	BEQ .NoSpriteContact			;/ branch

	JSL !GetSpriteClippingA			;\ if sprite being checked isn't touching this one,
	PHX					; |
	TYX					; |
	JSL !GetSpriteClippingB			; |
	PLX					; |
	JSL !CheckForContact			; |
	BCC .NoSpriteContact			;/ branch
	JSR Explode
	RTS
.NoSpriteContact
	DEY					; decrease loop counter
	BPL .LoopStart				; if sprites left to check, branch

	LDA !RAM_ExtraBits,x
	AND #$04
	LSR
	LSR
	TAY
	LDA !RAM_SpriteXHi,x
	CMP ScreenToStop,y
	BCC .DontStop
	
	INC !RAM_SpriteState,x
.DontStop
	JSL !UpdateYPosNoGrvty
	JSL !UpdateXPosNoGrvty
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Finished:
	JSR SubOffScreenX0
	LDA !RAM_SpriteSpeedY,x
	CMP #!MaxSpeedY
	BEQ .NoAcc
	INC !RAM_SpriteSpeedY,x
	INC !RAM_SpriteSpeedY,x
.NoAcc
	JSL !UpdateYPosNoGrvty
	JSL !UpdateXPosNoGrvty
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MarioInBarrel:
	LDA !RAM_SpriteYLo,x		;\ set Mario's position and speed to that of the barrel
	SEC				; |
	SBC #$10			; |
	STA !RAM_MarioYPos		; |
	LDA !RAM_SpriteYHi,x		; |
	SBC #$00			; |
	STA !RAM_MarioYPosHi		; |
	LDA !RAM_SpriteXLo,x		; |
	STA !RAM_MarioXPos		; |
	LDA !RAM_SpriteXHi,x		; |
	STA !RAM_MarioXPosHi		; |
	LDA !RAM_SpriteSpeedY,x		; |
	STA !RAM_MarioSpeedY		; |
	LDA !RAM_SpriteSpeedX,x		; |
	STA !RAM_MarioSpeedX		;/
	LDA #$FF			;\ hide Mario
	STA $78				;/
	LDA #$05			;\ make Mario invulnerable
	STA $1497			;/
	LDA #$01			;\ make camera follow Mario
	STA $1404			;/
	RTS

Explode:
	LDA #$00			;\ display Mario
	STA $78				;/
	JSL !KillMario

	LDA #$0D			;\ turn sprite into bob-omb explosion
	STA !RAM_SpriteNum,x		; |
	JSL !InitSpriteTables		; |
	LDA #$08			; |
	STA !RAM_SpriteStatus,x		; |
	LDA #$01			; |
	STA $1534,x			; |
	LDA #$30			; |
	STA $1540,x			; |
	LDA #$09			; |
	STA $1DFC			;/

	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteGraphics:
	JSR GetDrawInfo
	LDA $1602,x
	ASL
	ASL
	STA $03

	PHX				; preserve sprite index
	LDX #$03			; setup loop
.LoopStart
	LDA $00				;\ set x position of tile
	CLC				; |
	ADC OffsetX,x			; |
	STA !OAM_DispX,y		;/
	LDA $01				;\ set y position of tile
	CLC				; |
	ADC OffsetY,x			; |
	STA !OAM_DispY,y		;/
	PHX				; preserve loop counter
	LDX $03				;\ set tile number
	LDA Tilemap,x			; |
	STA !OAM_Tile,y			;/
	LDX !RAM_CurrentSprIndex	;\ set tile properties
	LDA !RAM_SpritePal,x		; |
	ORA $64				; |
	STA !OAM_Prop,Y			;/
	PLX				; retrieve loop counter
	INY				;\ increase OAM index so that the next tile can be written
	INY				; |
	INY				; |
	INY				;/
	INC $03
	DEX				; decrease loop counter
	BPL .LoopStart			; if still tiles left to draw, go to start of loop
	PLX				; retrieve sprite index
	LDY #$02			; the tiles written were 16x16
	LDA #$03			; we wrote four tiles
	JSL !FinishOAMWrite
	RTS				; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; display smoke effect
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SmokeX:
	db $00,$F8,$F0
SmokeY:
	db $10,$08,$00

SpawnSmoke:
	LDY $1602,x
	LDA SmokeX,y
	STA $00
	LDA SmokeY,y
	STA $01

	LDY #$03			; find a free slot to display effect
.LoopStart
	LDA !RAM_SmokeNum,y
	BEQ .FoundSlot
	DEY
	BPL .LoopStart
.Return
	RTS				; return if no slots open

.FoundSlot
	LDA #$01			;\ set effect graphic to smoke graphic
	STA !RAM_SmokeNum,y		;/
	LDA !RAM_SpriteXLo,x		;\ set smoke x position
	CLC				; |
	ADC $00				; |
	STA !RAM_SmokeXLo,y		;/
	LDA !RAM_SpriteYLo,x		;\ set smoke y position
	CLC				; |
	ADC $01				; |
	STA !RAM_SmokeYLo,y		;/
	LDA #$1B			;\ set time to show smoke
	STA !RAM_SmokeTimer,y		;/
	RTS	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GetDrawInfo
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;org $03B75C

Table1:	db $0C,$1C
Table2:	db $01,$02

GetDrawInfo:
	STZ $186C,x		; reset sprite offscreen flag, vertical
	STZ $15A0,x		; reset sprite offscreen flag, horizontal
	LDA $E4,x		;\
	CMP $1A			; | set horizontal offscreen if necessary
	LDA $14E0,x		; |
	SBC $1B			; |
	BEQ .OnScreenX		; |
	INC $15A0,x		;/

.OnScreenX
	LDA $14E0,x		;\
	XBA			; |
	LDA $E4,x		; |
	REP #$20		; |
	SEC			; |
	SBC $1A			; | mark sprite invalid if far enough off screen
	CLC			; |
	ADC #$0040		; |
	CMP #$0180		; |
	SEP #$20		; |
	ROL A			; |
	AND #$01		; |
	STA $15C4,x		;/ 
	BNE .Invalid		; 

	LDY #$00		;\ set up loop:
	LDA $1662,x		; | 
	AND #$20		; | if not smushed (1662 & 0x20), go through loop twice
	BEQ .OnScreenLoop	; | else, go through loop once
	INY			;/ 
.OnScreenLoop
	LDA $D8,x		;\ 
	CLC			; | set vertical offscreen if necessary
	ADC Table1,y		; |
	PHP			; |
	CMP $1C			; | (vert screen boundary)
	ROL $00			; |
	PLP			; |
	LDA $14D4,x		; | 
	ADC #$00		; |
	LSR $00			; |
	SBC $1D			; |
	BEQ .OnScreenY		; |
	LDA $186C,x		; | (vert offscreen)
	ORA Table2,y		; |
	STA $186C,x		; |
.OnScreenY
	DEY			; |
	BPL .OnScreenLoop	;/

	LDY $15EA,x		; get offset to sprite OAM
	LDA $E4,x		;\ 
	SEC			; | 
	SBC $1A			; | $00 = sprite x position relative to screen boarder
	STA $00			;/ 
	LDA $D8,x		;\ 
	SEC			; | 
	SBC $1C			; | $01 = sprite y position relative to screen boarder
	STA $01			;/ 
	RTS			; return

.Invalid
	PLA			;\ return from *main gfx routine* subroutine...
	PLA			; |    ...(not just this subroutine)
	RTS			;/

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubOffScreen
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
Spr_T12:
	db $40,$B0
Spr_T13:
	db $01,$FF
Spr_T14:
	db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
	db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
Spr_T15:
	db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
	db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SubOffScreenX1:
	LDA #$02		;\ entry point of routine determines value of $03
	BRA Store03		; | (table entry to use on horizontal levels)
SubOffScreenX2:
	LDA #$04		; | 
	BRA Store03		; |
SubOffScreenX3:
	LDA #$06		; |
	BRA Store03		; |
SubOffScreenX4:
	LDA #$08		; |
	BRA Store03		; |
SubOffScreenX5:
	LDA #$0A		; |
	BRA Store03		; |
SubOffScreenX6:
	LDA #$0C		; |
	BRA Store03		; |
SubOffScreenX7:
	LDA #$0E		; |
Store03:
	STA $03			; |            
	BRA StartSub		; |
SubOffScreenX0:
	STZ $03			;/

StartSub:
	JSR SubIsOffScreen	;\ if sprite is not off screen, return
	BEQ .Return		;/
	LDA $5B			;\  goto VERTICAL_LEVEL if vertical level
	AND #$01		; |
	BNE .VerticalLevel	;/     
	LDA $D8,x		;\
	CLC			; | 
	ADC #$50		; | if the sprite has gone off the bottom of the level...
	LDA $14D4,x		; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
	ADC #$00		; | 
	CMP #$02		; | 
	BPL .EraseSprite	;/    ...erase the sprite
	LDA $167A,x		;\ if "process offscreen" flag is set, return
	AND #$04		; |
	BNE .Return		;/
	LDA $13
	AND #$01
	ORA $03
	STA $01
	TAY
	LDA $1A
	CLC
	ADC Spr_T14,y
	ROL $00
	CMP $E4,x
	PHP
	LDA $1B
	LSR $00
	ADC Spr_T15,y
	PLP
	SBC $14E0,x
	STA $00
	LSR $01
	BCC .Label31
	EOR #$80
	STA $00
.Label31
	LDA $00
	BPL .Return
.EraseSprite
	LDA $14C8,x		;\ if sprite status < 8, permanently erase sprite
	CMP #$08		; |
	BCC .KillSprite		;/    
	LDY $161A,x
	CPY #$FF
	BEQ .KillSprite
	LDA #$00
	STA $1938,y
.KillSprite
	STZ $14C8,x		; erase sprite
.Return
	RTS			; return

.VerticalLevel
	LDA $167A,x		;\ if "process offscreen" flag is set, return
	AND #$04		; |
	BNE .Return		;/
	LDA $13			;\
	LSR A			; | 
	BCS .Return		;/
	LDA $E4,x		;\ 
	CMP #$00		; | if the sprite has gone off the side of the level...
	LDA $14E0,x		; |
	SBC #$00		; |
	CMP #$02		; |
	BCS .EraseSprite	;/  ...erase the sprite
	LDA $13
	LSR A
	AND #$01
	STA $01
	TAY
	LDA $1C
	CLC
	ADC Spr_T12,y
	ROL $00
	CMP $D8,x
	PHP
	LDA $001D
	LSR $00
	ADC Spr_T13,y
	PLP
	SBC $14D4,x
	STA $00
	LDY $01
	BEQ .Label38
	EOR #$80
	STA $00
.Label38
	LDA $00
	BPL .Return
	BMI .EraseSprite

SubIsOffScreen:
	LDA $15A0,x		;\ if sprite is on screen, accumulator = 0 
	ORA $186C,x		; |  
	RTS			;/ return