;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SMB1 Bowser, by mikeyk, modified by imamelia
;
; This is a Bowser sprite that behaves like the one in SMB1.
;
; Extra bytes: 1
;
; Extra byte 1:
; - Bit 0: Throw hammers.  0 = no, 1 = yes.
; - Bit 1: Stun the player when it hits the ground.  0 = no, 1 = yes.
; - Bits 2-7: Unused.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

incsrc subroutinedefs.asm

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; defines and tables
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!JumpTimer = $163E
!HammerTimer = $15AC
!FireTimer = $1528
!DirTimer = $1540
!SpriteImg = $1602

!TimeToStun = $28
!TimeToJump = $88
!TimeToShow = $12
!JumpHeight = $D0

!HammerTile = $6D

XSpeed:
	db $00,$FA,$00,$06
TimeInPos:
	db $78,$14,$74,$14
TimeTillFire:
	db $78,$A8
TimeTillHammer:
	db $0C,$0C,$0C,$0C,$0C,$43,$0C,$0C
	db $0C,$0C,$0C,$0C,$73,$0C,$0C,$0C
	db $0C,$0C,$63,$0C,$0C,$0C,$0C,$0C
	db $0C,$83,$0C,$0C,$0C,$0C,$0C,$53

Tilemap:
	db $00,$02,$27,$29,$00,$06,$4C,$4E,$04,$02,$27,$29,$04,$06,$4C,$4E
	db $00,$02,$27,$29,$00,$06,$4C,$4E,$04,$02,$27,$29,$04,$06,$4C,$4E
HorizDisp:
	db $00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10
	db $10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00
VertDisp:
	db $F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00
	db $F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00

FireXOffsetLo:
	db $13,$FC
FireXOffsetHi:
	db $00,$FF
HammerXOffset:
	db $02,$0C
HammerXSpeed:
	db $14,$EC

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; init routine wrapper
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Init:
	JSR SMB1BowserInitRt
	RTL

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; init routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SMB1BowserInitRt:

	JSL !SubHorizPos		;
	TYA					;
	STA $157C,x			; make the sprite initially face the player
	LDA $7FAB40,x		; first extra byte to misc. table
	STA $1510,x			;
	LSR					; if bit 0 is set...
	BCC .NoHammerInit		;
	LDA #$48			; then set the hammer-throwing timer
	STA !HammerTimer,x	;
.NoHammerInit			;
	TXA					;
	AND #$03			;
	ASL #4				;
	PHA					;
	ASL					;
	ADC #$04			;
	STA !FireTimer,x		; initialize the fire-spitting timer
	PLA					;
	ADC #$00			;
	STA !JumpTimer,x		; initialize the jump timer
	RTS					;

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; main routine wrapper
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Main:
	JSR SMB1BowserMainRt
	RTL

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; main routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

KilledByStar:				;
	INC !HammerTimer,x	;
ReturnMain:				;
	RTS					;

SMB1BowserMainRt:

	JSR SMB1BowserGFX		; draw the sprite

	LDA $14C8,x			;
	CMP #$02			; if the sprite status is 02...
	BEQ KilledByStar		;
	CMP #$08			; return if the sprite status is not normal
	BNE ReturnMain		;
	LDA $9D				; or sprites are locked
	BNE ReturnMain		;
	
	JSL !SubHorizPos		;
	TYA					;
	STA $157C,x			; make the sprite always face the player
	JSL !SubOffscreenX3	; offscreen processing
	INC $1570,x			; increment the frame counter
	LDA !FireTimer,x		; decrement the fire timer
	BEQ $03				; unless it is already at 0
	DEC !FireTimer,x		;

	LDA $1570,x			;
	LSR #4				;
	AND #$01			; determine which frame to show (change every 16 frames)
	STA !SpriteImg,x		;
	
	LDA !FireTimer,x		;
	CMP #!TimeToShow	; if it's almost time to spit fire...
	BCS .TryHammer		;
	INC !SpriteImg,x		; then change the image
	INC !SpriteImg,x		; to the fire-spitting one
	LDA !FireTimer,x		;
	BNE .NoReset			; if the fire-spitting timer has run out...
	LDY $C2,x			;
	LDA TimeTillFire,y		; then reset it
	STA !FireTimer,x		;
.NoReset					;
	CMP #$01			;
	BNE .TryHammer		;
	LDA $C2,x			;
	EOR #$01				;
	STA $C2,x			;
	
	JSR SubFireSpit			;

.TryHammer				;
	LDA $1510,x			;
	LSR					; throw hammers only if bit 0 is set
	BCC .JumpSprite		;
	LDA !HammerTimer,x	; if the hammer-throwing timer has run out...
	BNE .NoReset2			;
	LDY $1504,x			;
	LDA TimeTillHammer,y	; reset it
	STA !HammerTimer,x	;
.NoReset2				;
	CMP #$01			; if it is time to throw a hammer...
	BNE .JumpSprite		;
	LDA $1504,x			;
	INC					;
	AND #$1F			; increment the hammer timer pattern counter
	STA $1504,x			;
	
	JSR SubHammerThrow	;

.JumpSprite				;
	LDA !JumpTimer,x		;
	CMP #$40			; if the jump timer is greater than 40...
	BCS .WalkSprite		; just go to normal walking code
	LDA !SpriteImg,x		; if the sprite is about to jump...
	ORA #$01			; force-set bit 0 of the image index
	STA !SpriteImg,x		;
	INC !DirTimer,x		; cancel out the auto-decrement of the direction timer
	;STZ $B6,x			; zero out the sprite X speed
	LDA !JumpTimer,x		;
	CMP #$38			; if the jump timer has reached 38...
	BNE .NoJump2			;
	LDA #!JumpHeight		; then set the sprite's Y speed
	STA $AA,x			;
	BRA .ApplySpeed		;
.NoJump2				;
	CMP #$00			; if the jump timer has reached 00...
	BNE .ApplySpeed		; then reset it
	LDA #!TimeToJump		;
	STA !JumpTimer,x		;
	BRA .ApplySpeed		;
.WalkSprite				;
	LDY $151C,x			;
	LDA !DirTimer,x		; if it is not yet time to change the speed state...
	BEQ .ChangeSpeed		;
	LDA XSpeed,y			;
	STA $B6,x			; then just set the X speed
	BRA .ApplySpeed		;
.ChangeSpeed				;
	LDA TimeInPos,y		; if it is time to change the speed state...
	STA !DirTimer,x		; then reset the direction timer
	LDA $151C,x			;
	INC					; and increment the speed state
	AND #$03			;
	STA $151C,x			;
.ApplySpeed				;
	LDA $1588,x			;
	AND #$04			; save the blocked status in relation to the ground
	STA $08				; so we can check it later
	
	JSL $81802A			;
	
	LDA $1510,x			;
	AND #$02			; if bit 1 of the sprite behavior table is not set...
	BEQ .NoShake			; then don't shake the ground
	LDA $1588,x			;
	AND #$04			; if the sprite is on the ground now...
	BEQ .NoShake			;
	EOR $08				; but it was not on the ground before updating its position...
	BEQ .NoShake			;
	LDA #$09			;
	STA $1DFC			; play a sound effect
	LDA $77				;
	AND #$04			; if the player is not on the ground...
	BEQ .NoShake			; then don't stun him/her
	LDA #!TimeToStun		; shake the ground and stun the player
	STA $18BD			;
.NoShake					;
	LDA $1588,x			;
	AND #$03			; if the sprite is touching a wall...
	BEQ .NoChangeDir		;
	LDA $151C,x			;
	INC					; then increment the speed state
	AND #$03			;
	STA $151C,x			;
.NoChangeDir				;
	JSL $818032			; interact with other sprites
	JSL $81A7DC			; interact with the player
	RTS					;

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; graphics routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SMB1BowserGFX:

	JSL !GetDrawInfo		; prepare variables
	
	LDA !SpriteImg,x		;
	ASL #2				;
	STA $03				; index to tilemap tables into $03
	LDA $157C,x			;
	STA $02				; sprite direction into $02
	BNE .NoAdd			;
	LDA $03				;
	CLC					;
	ADC #$10			;
	STA $03				;
.NoAdd					;
	LDA $15F6,x			;
	ORA $64				;
	STA $04				;
	LDA $02				;
	ROR #3				;
	AND #$40			;
	EOR #$40				;
	TSB $04				;
	LDA #$03			;
	STA $05				;
	ORA $03				;
	TAX					;
.Loop					;
	LDA $00				;
	CLC					;
	ADC.w HorizDisp,x		;
	STA $0300,y			;
	LDA $01				;
	CLC					;
	ADC.w VertDisp,x		;
	STA $0301,y			;
	LDA.w Tilemap,x		;
	STA $0302,y			;
	LDA $04				;
	STA $0303,y			;
	
	INY #4				;
	DEX					;
	DEC $05				;
	BPL .Loop				;
	
	LDX $15E9			;
	LDA !HammerTimer,x	;
	CMP #$02			; if the hammer timer is between 02 and 1E...
	BCC .NoShowHammer	;
	CMP #$1E			;
	BCC .ShowHammerToo	; then draw the hammer tile also
.NoShowHammer			;
	LDA #$03			;
	BRA .FinishGFX		;

.ShowHammerToo			;
	LDA $00				;
	LDX $02				;
	CLC					;
	ADC.w HammerXOffset,x;
	STA $0300,y			;
	LDA $01				;
	CLC					;
	ADC #$E8			;
	STA $0301,y			;
	LDA #!HammerTile		;
	STA $0302,y			;
	LDA #$07			;
	CPX #$00				;
	BNE $02				;
	ORA #$40			;
	ORA $64				;
	STA $0303,y			;
	
	LDX $15E9			;
	LDA #$04			;
.FinishGFX				;
	LDY #$02				;
	JSL $81B7B3			;
	RTS					;

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; fire-spitting routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Return68:				;
	RTS					;
	
SubFireSpit:

	LDA $186C,x			;
	ORA $15D0,x			;
	BNE Return68			;
	
	JSL $82A9E4			;
	BMI Return68			;
	
	LDA #$01			;
	STA $14C8,y			;
	LDA #$B3				;
	STA $009E,y			;
	
	PHY					;
	LDY $157C,x			;
	LDA FireXOffsetLo,y	;
	STA $00				;
	LDA FireXOffsetHi,y	;
	STA $01				;
	PLY					;
	LDA $E4,x			;
	CLC					;
	ADC $00				;
	STA $00E4,y			;
	LDA $14E0,x			;
	ADC $01				;
	STA $14E0,y			;
	LDA $D8,x			;
	SEC					;
	SBC #$05				;
	STA $00D8,y			;
	LDA $14D4,x			;
	SBC #$00				;
	STA $14D4,y			;
	
	PHX					;
	TYX					;
	JSL $87F7D2			;
	PLX					;

	LDA #$17			;
	STA $1DFC			;
	RTS					;


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; hammer-throwing routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SubHammerThrow:

	LDY #$07				;
.FindFree					;
	LDA $170B,y			;
	BEQ .FoundSlot		;
	DEY					;
	BPL .FindFree			;
	RTS					;
.FoundSlot				;
	LDA #$04			;
	STA $170B,y			;

	PHY					;
	LDY $157C,x			;
	LDA HammerXSpeed,y	;
	STA $00				;
	LDA $E4,x			;
	CLC					;
	ADC HammerXOffset,y	;
	PLY					;
	STA $171F,y			;
	LDA $14E0,x			;
	ADC #$00			;
	STA $1733,y			;
	LDA $D8,x			;
	CLC					;
	ADC #$E7			;
	STA $1715,y			;
	LDA $14D4,x			;
	ADC #$FF			;
	STA $1729,y			;
	
	LDA #$D0			;
	STA $173D,y			;
	LDA $00				;
	STA $1747,y			;
	RTS					;

dl Init,Main





