;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Boss Fireball (sprite 34), by imamelia
;;
;; This is a disassembly of sprite 34 in SMW, the fireball that Ludwig spits.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:
db $0C,$1C
Table2:
db $01,$02

;OAMIndexes:
;db $A8,$B0,$B8,$C0,$C8
BossFireballTilemap:
db $4A,$4C,$6A,$6C
XDisp:
db $00,$F0,$00,$10
Properties:
db $45,$45,$05,$05

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR BossFireballMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BossFireballMain:

LDA $9D			; if sprites are locked...
ORA $13FB		; or the player is frozen...
BNE BossFireballGFX		; skip the movement code
LDA $1540,x		; check a timer of some sort (must be set during spawning; it is never changed in the sprite code)
CMP #$10			; if it hasn't dropped lower than 10...
BCS BossFireballGFX		; skip the movement code
TAY			; get the timer into Y
BNE SkipFrameSet		; if it hasn't reached 0, freeze the sprite's animation

JSR SetAnimationFrame	; change the sprite's animation frame every 8 frames

JSL $01A7DC		; interact with the player

SkipFrameSet:		;

JSL $018022		; update sprite X position without gravity

LDA $E4,x		; sprite X position
CLC			;
ADC #$20		; +20
XBA			;
LDA $14E0,x		;
ADC #$00		; set the high byte
XBA			; A now contains the complete X position of the sprite +20
REP #$20			; I rewrote the routine slightly, though...learn to XBA, Nintendo
CMP #$0230		; if the sprite has reached a certain point offscreen to the right...
SEP #$20			;
BCC BossFireballGFX	;
STZ $14C8,x		; erase it

BossFireballGFX:		;

JSR GetDrawInfo		; Support your local shared subroutine today!

LDA $1602,x		;
ASL			; current frame x2
STA $03			;
LDA $157C,x		;
ASL			; direction x2
STA $02			;
;LDA OAMIndexes,x		; Okay, why are we setting the OAM index manually?
;STA $15EA,x		; Forget that.
PHX			; preserve the sprite index
LDA $1540,x		; load the frame
LDX #$00			; start X at 00 (was 01, but I'm going the other way)
CMP #$08			; if the frame timer is greater than 08...
BCS GFXLoop		; keep X at 00 so that only one tile will be drawn
INX			;
GFXLoop:			;
PHX			; preserve X
PHX			; twice
TXA			; transfer X to A...
CLC			;
ADC $02			; add the value from the sprite direction
TAX			; and transfer the result back to X

LDA $00			;
CLC			;
ADC XDisp,x		; set the tile X displacement
STA $0300,y		;

LDA $14			; sprite frame counter
ROR #3			;
AND #$80		; make the sprite X/Y-flip to animate
ORA Properties,x		;
STA $0303,y		;

LDA $01			;
INC			;
INC			; set the Y displacement
STA $0301,y		; 2 pixels down

PLA			; pull back the previous tilemap index
CLC			;
ADC $03			; add in the current frame
TAX			; to get the index for the tilemap
LDA BossFireballTilemap,x	;
STA $0302,y		; set the tile number

PLX			;
INY #4			;
DEX			; decrement the index
BPL GFXLoop		; loop if we're drawing 2 tiles
PLX			; pull back the sprite index

LDY #$02			; the tiles are 16x16
LDA #$01			; and we drew 2 tiles (shouldn't this be a problem for when X was set to 00?)
JSL $81B7B3		; finish the write to OAM
RTS

SetAnimationFrame:		;

INC $1570,x		; increment the number of frames the sprite has been on the screen
LDA $1570,x		;
LSR			; divide this by 4
LSR			; (was 8, but incremented twice)
AND #$01		;
STA $1602,x		; set the sprite's current animation frame
RTS

GetDrawInfo:		; Support your local shared subroutine TODAY!

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:

PLA
PLA
RTS

