;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Invisible Solid Block (sprite 6D), by imamelia
;;
;; This is a disassembly of sprite 6D in SMW, the invisible solid block.  This sprite's
;; main routine in its entirety is also used to make sprites such as platforms solid.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XOffsetLo:
db $0E,$F1,$10,$E0,$1F,$F1

XOffsetHi:
db $00,$FF,$00,$FF,$00,$FF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR InvisibleSolidBlockMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

InvisibleSolidBlockMain:

PHK		;
PER $0006	;
PEA $8020	;
JML $01A7F7	; process interaction between the player and the sprite
BCC NoContact	; return if there was no contact

LDA $D8,x	; sprite Y position low byte
SEC		;
SBC $1C		; minus screen boundary Y position low byte
STA $00		; save the result

LDA $80		; player Y position relative to the screen
CLC		;
ADC #$18	; + 18
CMP $00		; if the player is within a certain range of the sprite...
BPL SecondCheck	; skip to the second part of the code

LDA $7D		; if the player is moving upward...
BMI NoContact	;
LDA $77		; or if the player is touching the ceiling...
AND #$08	;
BNE NoContact	; there was no contact

LDA #$10		;
STA $7D		; set the player Y speed to 10
LDA #$01		;
STA $1471	; and set the "on a sprite" flag

LDA #$1F		; base Y offset: 1F
LDY $187A	; if the player is on Yoshi...
BEQ StoreYOffset	;
LDA #$2F		; then the Y offset is 2F
StoreYOffset:	;
STA $01		;
LDA $D8,x	; sprite Y position low byte
SEC		;
SBC $01		; minus the offset
STA $96		; into the player Y position low byte
LDA $14D4,x	; sprite Y position high byte
SBC #$00		; handle overflow
STA $97		; into the player Y position high byte

LDA $77		;
AND #$03	; if the player is touching a wall...
BNE IsContact	; there is still contact, but don't move the player with the sprite
LDY #$00		; start Y at 00
LDA $1528,x	; $1528,x holds the number of pixels to move the player when riding a sprite
BPL ShiftRight	; if it is positive, the sprite and player are moving to the right
DEY		;
ShiftRight:	;
CLC		;
ADC $94		; add this value to the player's X position
STA $94		;
TYA		; and handle the high byte
ADC $95		;
STA $95		;

IsContact:

SEC		; set the carry flag, indicating that there was contact between the player and the sprite
RTS		;

NoContact:
CLC		; clear the carry flag, indicating that there was no contact
RTS		;

SecondCheck:	;

;LDA $190F,x	; if bit 0 of $190F,x (Tweaker table 6) is set,
;LSR		; indicating that the sprite is passable from the sides and bottom...
;BCS NoContact	; then the sprite acts as if there were no contact

LDA #$00		; A = 00
LDY $73		; if the player is ducking...
BNE Offset08	;
LDY $19		; or small...
BNE SetOffset	;
Offset08:		; then the offset initial value is 08
LDA #$08		;
SetOffset:		;
LDY $187A	; if the player is on Yoshi...
BEQ NoAdd8	;
ADC #$08	; then add 8 more to the offset value
NoAdd8:		;
CLC		;
ADC $80		; add the player's Y position relative to the screen
CMP $00		; if the result is less than the value from the sprite Y position/screen boundary...
BCC Offset3	; jump to the third part of code

LDA $7D		; if the player is moving down...
BPL NoContact2	; there will be no contact
LDA #$10		;
STA $7D		; if not, set the player's Y speed to 10

;LDA $9E,x	; check the sprite number
;CMP #$83	; if the sprite is a flying question-mark block...
;BCC NotQBlock	;
;LDA #$0F	; then set some timer
;STA $1564,x	;
;LDA $C2,x	; if the sprite state is zero...
;BNE NotQBlock	;
;INC $C2,x	; increment it
;LDA #$10	;
;STA $1558,x	; set another timer

NotQBlock:	;

LDA #$01		;
STA $1DF9	; play the "hit head on solid surface" sound effect

NoContact2:

CLC		;
RTS		;

Offset3:

JSR SubHorzPos	;

;LDA $9E,x	; check the sprite number
;CMP #$A9	; if it is a Reznor...
;BEQ Index1	; use the second pair of bytes in the offset table
;CMP #$9C	; if it is flying blocks...
;BEQ Index2	; use the third pair
;CMP #$BB	; if it is a moving castle block...
;BEQ Index2	; use the third pair
;CMP #$60	; if it is a flat switch...
;BEQ Index2	; use the third pair
;CMP #$49	; if it is anything else except a growing/shrinking pipe...
;BNE Index0	; use the first pair

;Index2:		;
;INY #2		;

;Index1:		;
;INY #2		;

Index0:		;

LDA XOffsetLo,y	;
CLC		;
ADC $E4,x	; set up the low byte of the X offset
STA $94		;
LDA XOffsetHi,y	; set up the high byte
ADC $14E0,x	;
STA $95		; shift the player horizontally
STZ $7B		; and set his/her X speed to zero
CLC		;
RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS