;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Mechakoopa (sprite A2), by imamelia
;;
;; This is a disassembly of sprite A2 in SMW, the Mechakoopa.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Speed:
db $08,$F8

XDisp:
db $F8,$08,$F8,$00,$08,$00,$10,$00

YDisp:
db $F8,$F8,$08,$00,$F9,$F9,$09,$00
db $F8,$F8,$08,$00,$F9,$F9,$09,$00
db $FD,$00,$05,$00,$00,$00,$08,$00

Tilemap:
db $40,$42,$60,$51,$40,$42,$60,$0A
db $40,$42,$60,$0C,$40,$42,$60,$0E
db $00,$02,$10,$01,$00,$02,$10,$01

TileProp:
db $00,$00,$00,$00,$40,$40,$40,$40

TileSize:
db $02,$00,$00,$02

Palette:
db $0B,$05

KeyXDisp:
db $F9,$0F

KeyTileProp:
db $4D,$0D

KeyTilemap:
db $70,$71,$72,$71

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR MechakoopaMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MechakoopaMain:

LDA $14C8,x	; check the sprite state
CMP #$09		; if it is stunned...
BCS Stunned	; run a different routine

JSR MechakoopaGFX	;

LDA $14C8,x	;
CMP #$08		; if the sprite status is not normal...
BNE Return00	;
LDA $9D		; or sprites are locked...
BNE Return00	; return

JSR SubOffscreenX0	;
JSL $01803A	; interact with the player and other sprites
JSL $01802A	; update sprite position

LDA $1588,x	;
AND #$04	; if the sprite is not on ground...
BEQ NoSpeed	; don't set any speed values

STZ $AA,x	; Y speed = 0
LDY $157C,x	;
LDA Speed,y	; set the X speed depending on direction
STA $B6,x		;
LDA $C2,x	;
INC $C2,x	;
AND #$3F	; every 40 frames...
BNE NoSpeed	;
JSR SubHorzPos	;
TYA		; turn to face the player
STA $157C,x	;

NoSpeed:		;

LDA $1588,x	;
AND #$03	; if the sprite is touching a wall...
BEQ NoFlipDir	;
LDA $B6,x	;
EOR #$FF		; flip its X speed
INC		;
STA $B6,x		;
LDA $157C,x	;
EOR #$01		; and direction
STA $157C,x	;

NoFlipDir:	;

INC $1570,x	;
LDA $1570,x	;
AND #$0C	; set the sprite's frame
LSR #2		;
STA $1602,x	;

Return00:		;
RTS

Stunned:

LDA $1A		;
PHA		;
LDA $1540,x	;
CMP #$30		; if the stun timer is below 30...
BCS NoShake	;
AND #$01	;
EOR $1A		; make the Mechakoopa shake back and forth 1 pixel
STA $1A		;
NoShake:		;

JSR MechakoopaGFX	;

LDY $15EA,x	;
LDA #$F0		;
STA $0309,y	; prevent a glitched tile from showing up

PLA		;
STA $1A		;

LDA $14C8,x	;
CMP #$0B		; if the sprite is being carried...
BNE Return01	;
LDA $76		;
EOR #$01		;
STA $157C,x	; set the sprite direction

Return01:		;
RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MechakoopaGFX:

LDA #$0B		;
STA $15F6,x	;

LDA $1540,x	; if the stun timer is set...
BEQ NotStunnedGFX	;

LDY #$05		;
CMP #$05		;
BCC StunFrame4	;
CMP #$FA		;
BCC StoreStunFrame	;
LDY #$04		;
StunFrame4:	;
StoreStunFrame:	;
TYA		;
STA $1602,x	;
LDA $1540,x	;
CMP #$30		; if the stun timer is below 30...
BCS NotStunnedGFX	;
AND #$01	;
TAY		;
LDA Palette,y	; make the Mechakoopa flash palettes
STA $15F6,x	;

NotStunnedGFX:	;

JSR GetDrawInfo	;

LDA $15F6,x	;
STA $04		;

TYA		;
CLC		;
ADC #$0C	;
TAY		;

LDA $1602,x	;
ASL #2		;
STA $03		;
LDA $157C,x	;
ASL #2		;
EOR #$04		;
STA $02		;

PHX		;
LDX #$03		; 4 tiles to draw

GFXLoop:		;

PHX		;
PHY		;
TYA		;
LSR #2		;
TAY		;
LDA TileSize,x	;
STA $0460,y	;     

PLY		;
PLA		;
PHA		;
CLC		;
ADC $02		;
TAX		;

LDA $00		;
CLC		;
ADC XDisp,x	;
STA $0300,y	;

LDA TileProp,x	;
ORA $04		;
ORA $64		;
STA $0303,y	;

PLA		;
PHA		;
CLC		;
ADC $03		;
TAX		;

LDA Tilemap,x	;
STA $0302,y	;

LDA $01		;
CLC		;
ADC YDisp,x	;
STA $0301,y	;

PLX		;
DEY #4		;
DEX		;
BPL GFXLoop	;

PLX
LDY #$FF		;
LDA #$03		;
JSL $81B7B3	;

JSR KeyGFX	;

RTS		;

KeyGFX:		;

LDA $15EA,x	;
CLC		;
ADC #$10	;
STA $15EA,x	;

JSR GetDrawInfo	;

PHX		;
LDA $1570,x	;
LSR #2		;
AND #$03	;
STA $02		;

LDA $157C,x	;
TAX		;
LDA $00		;
CLC		;
ADC KeyXDisp,x	;
STA $0300,y

LDA $01		;
STA $0301,y	;

LDA KeyTileProp,x	;
ORA $64		;
STA $0303,y	;

LDX $02		;
LDA KeyTilemap,x	;
STA $0302,y	;

PLX		;
LDY #$00		;
LDA #$00		;
JSL $81B7B3	;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS











