;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Checkerboard Platforms (sprites 55 & 57), by imamelia
;;
;; This is a disassembly of sprites 55 and 57 in SMW, the checkerboard platforms.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, the sprite will move horizontally, acting like 55.
;; If the first extra bit is clear, the sprite will move vertically, acting like 57.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!ExtraBit = $04

Acceleration:
db $FF,$01

MaxSpeed:
db $F0,$10

!CheckerboardTile1 = $EA	; the left tile of the checkerboard platform
!CheckerboardTile2 = $EB	; the middle tile of the checkerboard platform
!CheckerboardTile3 = $EC	; the right tile of the checkerboard platform

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
INC $1602,x
LDA $7FAB10,x
AND #!ExtraBit
STA $1504,x
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR CheckerboardPlatformsMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CheckerboardPlatformsMain:

JSR CheckerboardPlatformGFX

LDA $9D			; if sprites are locked...
BNE Return0		; return

LDA $1540,x		; if the timer is set...
BNE ContinueMain		; skip to the position-updating and player interaction routines

INC $C2,x		; increment the sprite state
LDA $C2,x		;
AND #$03		; if the sprite state isn't 0...
BNE ContinueMain		; skip to the position-updating and player interaction routines

LDA $151C,x		;
AND #$01		;
TAY			;
LDA $AA,x		;
CLC			;
ADC Acceleration,y		; give the sprite some acceleration
STA $AA,x		; set its Y speed
STA $B6,x			; set its X speed
CMP MaxSpeed,y		; if the sprite has reached maximum speed...
BNE ContinueMain

INC $151C,x		;
LDA #$18			; timer = 18 if the platform moves vertically
LDY $1504,x		;
BNE StoreTimer		;
LDA #$08			; timer = 08 if the platform moves horizontally
StoreTimer:		;
STA $1540,x		;

ContinueMain:

LDY $1504,x		; if the extra bit is set...
BNE UpdateY		; make the sprite move vertically

JSL $018022		; else, make the sprite move horizontally
BRA EndOfMain		;

UpdateY:			;

JSL $01801A		;
STZ $1491		;

EndOfMain:		;

LDA $1491		;
STA $1528,x		; prevent the player from sliding

JSL $01B44F		; make it solid

JSR SubOffscreenX1		;

Return0:			;
RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CheckerboardPlatformGFX:

JSR GetDrawInfo		;

LDA $1602,x		;
STA $01

LDA $D8,x		;
SEC			;
SBC $1C			;
STA $0301,y		;
STA $0305,y		;
STA $0309,y		;
STA $030D,y		;
STA $0311,y		;
LDX $15E9		;
LDA $E4,x		;
SEC			;
SBC $1A			;
STA $0300,y		; set the tile X displacement
CLC			;
ADC #$10		;
STA $0304,y		; second tile 16 pixels to the right of the first
CLC			;
ADC #$10		;
STA $0308,y		; third tile 16 pixels to the right of the second
CLC			;
ADC #$10		;
STA $030C,y		; fourth tile 16 pixels to the right of the third
CLC			;
ADC #$10		;
STA $0310,y		; fifth tile 16 pixels to the right of the fourth

LDA #!CheckerboardTile1	;
STA $0302,y		; first tile
LDA #!CheckerboardTile2	;
STA $0306,y		; second,
STA $030A,y		; third,
STA $030E,y		; and fourth tiles
LDA #!CheckerboardTile3		;
STA $0312,y		; fifth tile

LDA $64			;
ORA $15F6,x		; no hardcoded palette this time, and the two platforms use the same one
STA $0303,y		; properties for the first...
STA $0307,y		; second...
STA $030B,y		; third...
STA $030F,y		; fourth...
STA $0313,y		; and fifth tile

LDA #$04			; 5 tiles to draw
LDY #$02			; all tiles are 16x16
JSL $81B7B3		;
RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

;SubOffscreenX0:
;LDA #$00
;BRA SubOffscreenMain
SubOffscreenX1:
LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS











