;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Normal/Custom Sprite Generator, based on mikeyk's code, further changed by Davros.
;;
;; Description: This sprite will generate normal or custom sprites when all the enemies on 
;; screen are killed. The x position determines the sprite it will spawn.
;;
;; Uses first extra bit: YES
;; When the first extra bit is clear, the sprite will turn into a normal sprite. When the 
;; first extra bit is set, it will turn into a custom sprite.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                            
		    NORMAL_SPRITE1 = $15    ; horizontal fish
		    NORMAL_SPRITE2 = $0F    ; goomba
		    NORMAL_SPRITE3 = $00    ; shelless green koopa
		    NORMAL_SPRITE4 = $04    ; green koopa

		    CUSTOM_SPRITE1 = $11    ; birdo
		    CUSTOM_SPRITE2 = $1B    ; boomerang bro
		    CUSTOM_SPRITE3 = $1C    ; boomerang
		    CUSTOM_SPRITE4 = $20    ; para-beetle

                    EXTRA_BITS = $7FAB10
                    NEW_SPRITE_NUM = $7FAB9E            
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
                    RTL                 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"  
                    LDA $1493               ; \ if the goal is set...
                    BNE RETURN              ; /
  
                    LDY #$0B		    ; \ load number of times to go through loop
LOOP_START          CPY $00                 ;  |
                    BEQ DONT_CHECK          ; /
                    LDA $190F,y             ; \ if the tweaker bits are set...
                    AND #$40                ;  |
                    BNE DONT_CHECK          ; /     
                    LDA $14C8,y             ; \ if no sprite is left alive...
                    BNE RETURN              ; /
DONT_CHECK          DEY                     ;
                    BPL LOOP_START          ;       

                    LDA EXTRA_BITS,x        ; \ generate custom sprite if the extra bit is set...
                    AND #$04                ;  |
                    BNE SET_CUST_GEN        ; /

                    JSR NORMAL_GEN          ; generate normal sprite routine
                    BRA RETURN

SET_CUST_GEN        JSR CUSTOM_GEN          ; generate custom sprite routine

RETURN              RTL                     ; return  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; generate custom sprite routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


NORMAL_GEN          LDA $E4,x               ; \ if the x position is set...
                    AND #$30                ;  |
                    BEQ NORM_SPRITE2        ;  |
                    CMP #$20                ;  |
                    BEQ NORM_SPRITE3        ;  |
                    CMP #$10                ;  |
                    BEQ NORM_SPRITE4        ; /

NORM_SPRITE1        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #NORMAL_SPRITE1     ; set normal sprite number
                    BRA STORE_SPRITE

NORM_SPRITE2        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #NORMAL_SPRITE2     ; set normal sprite number
                    BRA STORE_SPRITE

NORM_SPRITE3        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #NORMAL_SPRITE3     ; set normal sprite number
                    BRA STORE_SPRITE

NORM_SPRITE4        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #NORMAL_SPRITE4     ; set normal sprite number
                    BRA STORE_SPRITE

STORE_SPRITE        STA $9E,x               ; store normal sprite
                    JSL $07F7D2             ; reset sprite tables

                    JSR SUB_SMOKE           ; smoke routine
                    
                    JSR SUB_HORZ_POS        ; \ always face Mario
                    TYA                     ;  |
                    STA $157C,x             ; /

                    LDA #$10                ; \ sound effect
                    STA $1DF9               ; /
                    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; generate custom sprite routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


CUSTOM_GEN          LDA $E4,x               ; \ if the x position is set...
                    AND #$30                ;  | 
                    BEQ CUST_SPRITE2        ;  |
                    CMP #$20                ;  |
                    BEQ CUST_SPRITE3        ;  |
                    CMP #$10                ;  |
                    BEQ CUST_SPRITE4        ; /

CUST_SPRITE1        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #CUSTOM_SPRITE1     ; set custom sprite number
                    BRA STORE_SPRITE2

CUST_SPRITE2        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #CUSTOM_SPRITE2     ; set custom sprite number
                    BRA STORE_SPRITE2

CUST_SPRITE3        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #CUSTOM_SPRITE3     ; set custom sprite number
                    BRA STORE_SPRITE2

CUST_SPRITE4        LDA #$08                ; \ set sprite status
                    STA $14C8,x             ; /

                    LDA #CUSTOM_SPRITE4     ; set custom sprite number
                    BRA STORE_SPRITE2

STORE_SPRITE2       STA NEW_SPRITE_NUM,x    ; store custom sprite
                    JSL $07F7D2             ; reset sprite tables
                    JSL $0187A7             ; Get table values for custom sprite
                    LDA #$88                ; \ Mark as initialized
                    STA EXTRA_BITS,x        ; /

                    JSR SUB_SMOKE           ; smoke routine
                    
                    JSR SUB_HORZ_POS        ; \ always face Mario
                    TYA                     ;  |
                    STA $157C,x             ; /

                    LDA #$10                ; \ sound effect
                    STA $1DF9               ; /
                    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; display smoke effect
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SUB_SMOKE           LDY #$03                ; \ find a free slot to display effect
FINDFREE            LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE            LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA #$1B                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    STA $17C4,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
                    STA $17C8,y             ; /
                    RTS                     ; return
                    

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

