;;;;;;;;;;;;;;;;;;
; Treasure Chest ;
; By: Schwa      ;
;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; When touched, the chest opens, and out pop 12 coins, one at a time. ;
; Careful when using this with Fire Flowers.                          ;
;                                                                     ;
; Insert "treasure_coin.cfg" to be the sprite RIGHT AFTER this one.   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

EXTRA_BITS = $7FAB10
NEW_SPRITE_NUM = $7FAB9E

TILEMAP
	dcb $20,$40,$40		; Closed, Generating coins, Open

;;;;;;;;;;;;;;;;
; Init Routine ;
;;;;;;;;;;;;;;;;

	dcb "INIT"
	RTL

;;;;;;;;;;;;;;;;
; Main Routine ;
;;;;;;;;;;;;;;;;

	dcb "MAIN"
	PHB
	PHK
	PLB
	JSR SPRITE_CODE
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;
; Main Sprite Code ;
;;;;;;;;;;;;;;;;;;;;

SPRITE_CODE
	JSR SUB_GFX		; Call graphics routine

	LDA $C2,x		; \  Check to see if chest is "making coins"
	CMP #$01		;  | If it isn't...
	BNE DONT_MAKE_COIN	;  | ...don't make the coin.
	LDA $13			;  | Otherwise, check the Frame Number...
	AND #$1F		;  | 
	CMP #$01		;  | Every 4 frames...
	BEQ MAKE_COIN		; /  ...make the coin.

DONT_MAKE_COIN
	LDA $C2,x		; \  Check to see if the chest is closed
	CMP #$00		;  | If it's not closed...
	BNE RETURN_1		;  | ...do nothing
	JSL $01A7DC		;  | Otherwise interact with Mario
	BCC RETURN_1		;  | Do nothing if no contact
	LDA #$01		;  | If contact...
	STA $C2,x		;  | Set status to "Generating coins"
	LDA #$4A		;  | Play sound effect
	STA $1DFC		; /  
RETURN_1
	RTS

;;;;;;;;;;;;;;;;;;;;;;
; Open Chest Routine ;
;;;;;;;;;;;;;;;;;;;;;;

MAKE_COIN
	JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
	BMI RETURN_2            ; / after: Y has index of sprite being generated

	LDA #$01		; \ set sprite status for new sprite
	STA $14C8,y		; /

	;LDA #$21		; \ set sprite number for new sprite
	;STA $009E,y		; /

	PHX			; \ generate the next custom sprite on the list
	LDA NEW_SPRITE_NUM,x	;  |
	INC A			;  |
	TYX			;  |
	STA NEW_SPRITE_NUM,x	;  |
	PLX			; /

	LDA $00E4,x		; \ set x position for new sprite (same as the chest)
	STA $00E4,y		;  |
	LDA $14E0,x		;  |
	STA $14E0,y		; /

	LDA $00D8,x		; \ set y position for new sprite (same as chest, minus 3 blocks)
	SEC			;  |
	SBC #$20		;  |
	STA $00D8,y		;  |
	LDA $14D4,x		;  |
	SBC #$00		;  |
	STA $14D4,y		; /

	PHX			; \  before: X must have index of sprite being generated
	TYX			;  | routine clears *all* old sprite values...
	JSL $07F7D2		; /  ...and loads in new values for the 6 main sprite tables

	JSL $0187A7		; \  get table values for custom sprite
	LDA #$08		;  | $08 = 00001000 = no extra bit, yes custom sprite, not initialized
	STA EXTRA_BITS,x	; / 

	PLX			; restore the X Register again

	INC $1534,x		; increase number of coins generated

	LDA $1534,x		; load how many coins have been generated
	CMP #$0C		; if 12 coins have been generated...
	BEQ COIN_FINISH		; ...finish generating coins.
	RTS

COIN_FINISH
	LDA #$02		; \ Set the chest to "open" instead of closed
	STA $C2,x		; /
RETURN_2
	RTS

;;;;;;;;;;;;;;;;;;;;
; Graphics Routine ;
;;;;;;;;;;;;;;;;;;;;

SUB_GFX
	JSR GET_DRAW_INFO	; set sprite index to Y Register

	LDA $00			; \ Draw X position
	STA $0300,y		; /

	LDA $01			; \ Draw Y position
	STA $0301,y		; /

	PHY			; \  Push Y Register...
	LDY $C2,x		;  | Load Open/Closed status of chest (00 for closed, 01 for open)
	LDA TILEMAP,y		;  | 
	PLY			;  | ...Restore Y Register
	STA $0302,y		; /  Draw the chest whether it's open or closed

	LDA $15F6,x		; \  Set sprite properties to normal
	ORA $64			;  |
	STA $0303,y		; /

	INY
	INY
	INY
	INY

	LDY #$02		; \  Do some bookkeeping
	LDA #$00		;  |
	JSL $01B7B3		;  |
	RTS			; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
