;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Turn Block Bridge Horizontal and Vertical Disassembly
;
;If you don't want the tile on the right be X flipped, find
;CODE_01B79C:                      ORA #$60
;and comment it out
;
;USES FIRST EXTRA BIT: YES
;If set, it will start expanding vertically
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Defines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BlkBridgeLength:                  db $20,$00	;Length - I don't recommend changing this. At least not to a bigger value than 20
TurnBlkBridgeSpeed:               db $01,$FF	;Folding speed - This can be changed, but be careful with big values
BlkBridgeTiming:                  db $40,$40	;Timer - This can be changed to any value
			!TurnblockGFX		= $40	;Tile to use (40 = Turnblock)

			!RAM_MarioYPosHi	= $97
			!RAM_MarioYPos		= $96
			!RAM_MarioSpeedY	= $7D
			!RAM_MarioAnimation	= $71
			!RAM_MarioXPos		= $94
			!RAM_MarioXPosHi	= $95
			!RAM_SpritesLocked	= $9D
			!RAM_SpriteState	= $C2
			!RAM_SpriteYLo		= $D8
			!RAM_SpriteXLo		= $E4
			!OAM_DispX		= $0300
			!OAM_DispY		= $0301
			!OAM_Tile		= $0302
			!OAM_Prop		= $0303
			!OAM_TileSize		= $0460
			!RAM_SpriteYHi		= $14D4
			!RAM_SpriteXHi		= $14E0
			!RAM_SpriteDir		= $157C
			!RAM_SprOAMIndex	= $15EA
			!RAM_ScreenBndryYLo	= $1C
			!OAM_Tile3DispY		= $0309
			!OAM_Tile4DispY		= $030D
			!OAM_Tile2DispY		= $0305
			!RAM_ScreenBndryXLo	= $1A
			!OAM_Tile3DispX		= $0308
			!OAM_Tile4DispX		= $030C
			!OAM_Tile2DispX		= $0304
			!OAM_Tile2		= $0306
			!OAM_Tile3		= $030A
			!OAM_Tile4		= $030E
			!OAM_Tile4Prop		= $030F
			!OAM_Tile2Prop		= $0307
			!OAM_Tile3Prop		= $030B
			!RAM_OnYoshi		= $187A
			!EXTRA_BITS 		= $7FAB10 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
		    LDA !EXTRA_BITS,x		;Extra bit
		    AND #$04			;
		    BEQ BitClear		;
		    LDA #$02			;Start expanding vertically by setting sprite state to
		    STA !RAM_SpriteState,x	;02
BitClear:
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR TurnBlockBridge     ;  |
                    PLB                     ;  |
                    RTL                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Sprite Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TurnBlockBridge:                  JSR SUB_OFF_SCREEN_X0     ;Off screen processing
CODE_01B6A8:                      JSR CODE_01B710           ;GFX Routine
CODE_01B6AB:                      JSR CODE_01B852           ;Mario interaction stuff
CODE_01B6AE:                      JSR CODE_01B6B2           ;Timer
Return01B6B1:                     RTS                       ; Return 

CODE_01B6B2:                      LDA !RAM_SpriteState,X    ;Go out/in state
CODE_01B6B4:                      AND #$01                  ;00->00 01->01 02->00 03->01 usw.
CODE_01B6B6:                      TAY                       ;into Y
CODE_01B6B7:                      LDA $151C,X               ;\Length reached?
CODE_01B6BA:                      CMP BlkBridgeLength,Y     ; |go set time
CODE_01B6BD:                      BEQ CODE_01B6D1           ;/
CODE_01B6BF:                      LDA $1540,X               ;Timer
CODE_01B6C2:                      ORA !RAM_SpritesLocked    ;Sprites Locked
CODE_01B6C4:                      BNE Return01B6D0          ;not zero? Return
CODE_01B6C6:                      LDA $151C,X               ;\?
CODE_01B6C9:                      CLC                       ; |Add
CODE_01B6CA:                      ADC TurnBlkBridgeSpeed,Y  ; |speed to
CODE_01B6CD:                      STA $151C,X               ;/length
Return01B6D0:                     RTS                       ; Return 

CODE_01B6D1:                      LDA BlkBridgeTiming,Y     ;\Set timer
CODE_01B6D4:                      STA $1540,X               ;/
CODE_01B6D7:                      INC !RAM_SpriteState,X    ;Increase State
Return01B6D9:                     RTS                       ; Return 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Mario interaction code below
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CODE_01B852:                      LDA $15C4,X             	;Sprite off screen
CODE_01B855:                      BNE Return01B8B1          	;Return
CODE_01B857:                      LDA !RAM_MarioAnimation    	;Mario doing a special action?
CODE_01B859:                      CMP #$01                	;
CODE_01B85B:                      BCS Return01B8B1          	;Return
CODE_01B85D:                      JSR CODE_01B8FF         	;
CODE_01B860:                      BCC Return01B8B1          	;No contact, return
CODE_01B862:                      LDA !RAM_SpriteYLo,X          ;
CODE_01B864:                      SEC                       	;
CODE_01B865:                      SBC !RAM_ScreenBndryYLo    	;
CODE_01B867:                      STA $02                       ;
CODE_01B869:                      SEC                           ;
CODE_01B86A:                      SBC $0D                       ;
CODE_01B86C:                      STA $09                       ;
CODE_01B86E:                      LDA $80                       ;
CODE_01B870:                      CLC                       	;
CODE_01B871:                      ADC #$18                	;
CODE_01B873:                      CMP $09                   	;
CODE_01B875:                      BCS ADDR_01B8B2		;
CODE_01B877:                      LDA !RAM_MarioSpeedY          ;
CODE_01B879:                      BMI Return01B8B1              ;
CODE_01B87B:                      STZ !RAM_MarioSpeedY          ;Mario is on top of the sprite interaction(?)
CODE_01B87D:                      LDA #$01                	;platform type
CODE_01B87F:                      STA $1471               	;
CODE_01B882:                      LDA $0D                       ;
CODE_01B884:                      CLC                       	;
CODE_01B885:                      ADC #$1F                	;
CODE_01B887:                      LDY !RAM_OnYoshi         	;
CODE_01B88A:                      BEQ CODE_01B88F           	;
CODE_01B88C:                      CLC                       	;
CODE_01B88D:                      ADC #$10                	;
CODE_01B88F:                      STA $00                   	;
CODE_01B891:                      LDA !RAM_SpriteYLo,X       	;
CODE_01B893:                      SEC                       	;
CODE_01B894:                      SBC $00                   	;
CODE_01B896:                      STA !RAM_MarioYPos         	;
CODE_01B898:                      LDA !RAM_SpriteYHi,X     	;
CODE_01B89B:                      SBC #$00                	;
CODE_01B89D:                      STA !RAM_MarioYPosHi       	;
CODE_01B89F:                      LDY #$00                	;
CODE_01B8A1:                      LDA $1491               	;
CODE_01B8A4:                      BPL CODE_01B8A7           	;
ADDR_01B8A6:                      DEY                       	;
CODE_01B8A7:                      CLC                       	;
CODE_01B8A8:                      ADC !RAM_MarioXPos         	;
CODE_01B8AA:                      STA !RAM_MarioXPos         	;
CODE_01B8AC:                      TYA                       	;
CODE_01B8AD:                      ADC !RAM_MarioXPosHi       	;
CODE_01B8AF:                      STA !RAM_MarioXPosHi       	;
Return01B8B1:                     RTS                       ; Return 

ADDR_01B8B2:                      LDA $02                       ;
ADDR_01B8B4:                      CLC                       	;
ADDR_01B8B5:                      ADC $0D                   	;
ADDR_01B8B7:                      STA $02                   	;
ADDR_01B8B9:                      LDA #$FF                	;
ADDR_01B8BB:                      LDY $73         		;
ADDR_01B8BD:                      BNE ADDR_01B8C3           	;
ADDR_01B8BF:                      LDY $19      			;
ADDR_01B8C1:                      BNE ADDR_01B8C5           	;
ADDR_01B8C3:                      LDA #$08                	;
ADDR_01B8C5:                      CLC                       	;
ADDR_01B8C6:                      ADC $80                   	;
ADDR_01B8C8:                      CMP $02                   	;
ADDR_01B8CA:                      BCC ADDR_01B8D5           	;
ADDR_01B8CC:                      LDA !RAM_MarioSpeedY       	;
ADDR_01B8CE:                      BPL Return01B8D4          	;
ADDR_01B8D0:                      LDA #$10                	;
ADDR_01B8D2:                      STA !RAM_MarioSpeedY       	;
Return01B8D4:                     RTS                       ; Return 

ADDR_01B8D5:                      LDA $0E                   	;
ADDR_01B8D7:                      CLC                       	;
ADDR_01B8D8:                      ADC #$10                	;
ADDR_01B8DA:                      STA $00                   	;
ADDR_01B8DC:                      LDY #$00                	;
ADDR_01B8DE:                      LDA !RAM_SpriteXLo,X       	;
ADDR_01B8E0:                      SEC                       	;
ADDR_01B8E1:                      SBC !RAM_ScreenBndryXLo    	;
ADDR_01B8E3:                      CMP $7E                   	;
ADDR_01B8E5:                      BCC ADDR_01B8EF           	;
ADDR_01B8E7:                      LDA $00                   	;
ADDR_01B8E9:                      EOR #$FF                	;
ADDR_01B8EB:                      INC A                     	;
ADDR_01B8EC:                      STA $00                   	;
ADDR_01B8EE:                      DEY                       	;
ADDR_01B8EF:                      LDA !RAM_SpriteXLo,X       	;
ADDR_01B8F1:                      CLC                       	;
ADDR_01B8F2:                      ADC $00                   	;
ADDR_01B8F4:                      STA !RAM_MarioXPos         	;
ADDR_01B8F6:                      TYA                       	;
ADDR_01B8F7:                      ADC !RAM_SpriteXHi,X     	;
ADDR_01B8FA:                      STA !RAM_MarioXPosHi       	;
ADDR_01B8FC:                      STZ $7B       		;
Return01B8FE:                     RTS                       ; Return 

CODE_01B8FF:                      LDA $00                  ;
CODE_01B901:                      STA $0E                  ;
CODE_01B903:                      LDA $02                  ;
CODE_01B905:                      STA $0D                  ;
CODE_01B907:                      LDA !RAM_SpriteXLo,X     ;
CODE_01B909:                      SEC                      ;
CODE_01B90A:                      SBC $00                  ;
CODE_01B90C:                      STA $04                  ;
CODE_01B90E:                      LDA !RAM_SpriteXHi,X     ;
CODE_01B911:                      SBC #$00                 ;
CODE_01B913:                      STA $0A                  ;
CODE_01B915:                      LDA $00                  ;
CODE_01B917:                      ASL                      ;
CODE_01B918:                      CLC                      ;
CODE_01B919:                      ADC #$10                 ;
CODE_01B91B:                      STA $06                  ;
CODE_01B91D:                      LDA !RAM_SpriteYLo,X     ;
CODE_01B91F:                      SEC                      ;
CODE_01B920:                      SBC $02                  ;
CODE_01B922:                      STA $05                  ;
CODE_01B924:                      LDA !RAM_SpriteYHi,X     ;
CODE_01B927:                      SBC #$00                 ;
CODE_01B929:                      STA $0B                  ;
CODE_01B92B:                      LDA $02                  ;
CODE_01B92D:                      ASL                      ;
CODE_01B92E:                      CLC                      ;
CODE_01B92F:                      ADC #$10                 ;
CODE_01B931:                      STA $07                  ;
CODE_01B933:                      JSL $03B664    	   ;Mario clipping
CODE_01B937:                      JSL $03B72B    	   ;check contact
Return01B93B:                     RTS                       ; Return 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;GFX Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CODE_01B710:                      JSR GET_DRAW_INFO    
CODE_01B713:                      STZ $00                   
CODE_01B715:                      STZ $01                   
CODE_01B717:                      STZ $02                   
CODE_01B719:                      STZ $03                   
CODE_01B71B:                      LDA !RAM_SpriteState,X   ;Sprite State
CODE_01B71D:                      AND #$02		   ;00->00 01->00 02->02 03->02 04->00 05->00 06->02 etc.
CODE_01B71F:                      TAY                      ;into Y
CODE_01B720:                      LDA $151C,X              ;
CODE_01B723:                      STA $0000,Y              ;$0000 holds current length? Indexed by Y. If Y=02, $0002 is used, which is the length in vertical direction
CODE_01B726:                      LSR                      ;divide by 2
CODE_01B727:                      STA $0001,Y              ;Store into $0001, also indexed by Y. Same as above
CODE_01B72A:                      LDY !RAM_SprOAMIndex,X   ; Y = Index into sprite OAM 
CODE_01B72D:                      LDA !RAM_SpriteYLo,X       
CODE_01B72F:                      SEC                       
CODE_01B730:                      SBC !RAM_ScreenBndryYLo    
CODE_01B732:                      STA $0311,Y             
CODE_01B735:                      PHA                       
CODE_01B736:                      PHA                       
CODE_01B737:                      PHA                       
CODE_01B738:                      SEC                       
CODE_01B739:                      SBC $02                   
CODE_01B73B:                      STA !OAM_Tile3DispY,Y    
CODE_01B73E:                      PLA                       
CODE_01B73F:                      SEC                       
CODE_01B740:                      SBC $03                   
CODE_01B742:                      STA !OAM_Tile4DispY,Y    
CODE_01B745:                      PLA                       
CODE_01B746:                      CLC                       
CODE_01B747:                      ADC $02                   
CODE_01B749:                      STA !OAM_DispY,Y         
CODE_01B74C:                      PLA                       
CODE_01B74D:                      CLC                       
CODE_01B74E:                      ADC $03                   
CODE_01B750:                      STA !OAM_Tile2DispY,Y    
CODE_01B753:                      LDA !RAM_SpriteXLo,X       
CODE_01B755:                      SEC                       
CODE_01B756:                      SBC !RAM_ScreenBndryXLo    
CODE_01B758:                      STA $0310,Y             
CODE_01B75B:                      PHA                       
CODE_01B75C:                      PHA                       
CODE_01B75D:                      PHA                       
CODE_01B75E:                      SEC                       
CODE_01B75F:                      SBC $00                   
CODE_01B761:                      STA !OAM_Tile3DispX,Y     ;leftmost tile
CODE_01B764:                      PLA                       
CODE_01B765:                      SEC                       
CODE_01B766:                      SBC $01                   
CODE_01B768:                      STA !OAM_Tile4DispX,Y     ;second tile from the left
CODE_01B76B:                      PLA                       
CODE_01B76C:                      CLC                       
CODE_01B76D:                      ADC $00                   
CODE_01B76F:                      STA !OAM_DispX,Y          ;rightmost tile
CODE_01B772:                      PLA                       
CODE_01B773:                      CLC                       
CODE_01B774:                      ADC $01                   
CODE_01B776:                      STA !OAM_Tile2DispX,Y     ;second tile from the right
CODE_01B779:                      LDA !RAM_SpriteState,X    ;
CODE_01B77B:                      LSR                       ;
CODE_01B77C:                      LSR                       ;
CODE_01B77D:                      LDA #!TurnblockGFX                
CODE_01B77F:                      STA !OAM_Tile2,Y          ;second tile from the right
CODE_01B782:                      STA !OAM_Tile4,Y          ;second tile from the left
CODE_01B785:                      STA $0312,Y               ;Middle tile
CODE_01B788:                      STA !OAM_Tile3,Y          ;leftmost tile
CODE_01B78B:                      STA !OAM_Tile,Y           ;rightmost tile
CODE_01B78E:                      LDA $64                   
CODE_01B790:                      STA !OAM_Tile4Prop,Y      ;OXOOO
CODE_01B793:                      STA !OAM_Tile2Prop,Y      ;OOOXO
CODE_01B796:                      STA !OAM_Tile3Prop,Y      ;XOOOO
CODE_01B799:                      STA $0313,Y               ;OOXOO
CODE_01B79C:                      ORA #$60                  ;COMMENT THIS OUT AND THE TILE ON THE RIGHT WON'T BE X FLIPPED
CODE_01B79E:                      STA !OAM_Prop,Y           ;OOOOX
CODE_01B7A1:                      LDA $00                   
CODE_01B7A3:                      PHA                       
CODE_01B7A4:                      LDA $02                   
CODE_01B7A6:                      PHA                       
CODE_01B7A7:                      LDA #$04                
CODE_01B7A9:                      JSR CODE_01B37E         
CODE_01B7AC:                      PLA                       
CODE_01B7AD:                      STA $02                   
CODE_01B7AF:                      PLA                       
CODE_01B7B0:                      STA $00                   
Return01B7B2:                     RTS                       ; Return 

CODE_01B37E:                      LDY #$02                
CODE_01B380:                      JMP FinishOAMWriteRt 

FinishOAMWriteRt:                 STY $0B                   
CODE_01B7BD:                      STA $08                   
CODE_01B7BF:                      LDY !RAM_SprOAMIndex,X   ; Y = Index into sprite OAM 
CODE_01B7C2:                      LDA !RAM_SpriteYLo,X       
CODE_01B7C4:                      STA $00                   
CODE_01B7C6:                      SEC                       
CODE_01B7C7:                      SBC !RAM_ScreenBndryYLo    
CODE_01B7C9:                      STA $06                   
CODE_01B7CB:                      LDA !RAM_SpriteYHi,X     
CODE_01B7CE:                      STA $01                   
CODE_01B7D0:                      LDA !RAM_SpriteXLo,X       
CODE_01B7D2:                      STA $02                   
CODE_01B7D4:                      SEC                       
CODE_01B7D5:                      SBC !RAM_ScreenBndryXLo    
CODE_01B7D7:                      STA $07                   
CODE_01B7D9:                      LDA !RAM_SpriteXHi,X     
CODE_01B7DC:                      STA $03                   
CODE_01B7DE:                      TYA                       
CODE_01B7DF:                      LSR                       
CODE_01B7E0:                      LSR                       
CODE_01B7E1:                      TAX                       
CODE_01B7E2:                      LDA $0B                   
CODE_01B7E4:                      BPL CODE_01B7F0           
CODE_01B7E6:                      LDA !OAM_TileSize,X      
CODE_01B7E9:                      AND #$02                
CODE_01B7EB:                      STA !OAM_TileSize,X      
CODE_01B7EE:                      BRA CODE_01B7F3           

CODE_01B7F0:                      STA !OAM_TileSize,X      
CODE_01B7F3:                      LDX #$00                
CODE_01B7F5:                      LDA !OAM_DispX,Y         
CODE_01B7F8:                      SEC                       
CODE_01B7F9:                      SBC $07                   
CODE_01B7FB:                      BPL CODE_01B7FE           
CODE_01B7FD:                      DEX                       
CODE_01B7FE:                      CLC                       
CODE_01B7FF:                      ADC $02                   
CODE_01B801:                      STA $04                   
CODE_01B803:                      TXA                       
CODE_01B804:                      ADC $03                   
CODE_01B806:                      STA $05                   
CODE_01B808:                      JSR CODE_01B844         
CODE_01B80B:                      BCC CODE_01B819           
CODE_01B80D:                      TYA                       
CODE_01B80E:                      LSR                       
CODE_01B80F:                      LSR                       
CODE_01B810:                      TAX                       
CODE_01B811:                      LDA !OAM_TileSize,X      
CODE_01B814:                      ORA #$01                
CODE_01B816:                      STA !OAM_TileSize,X      
CODE_01B819:                      LDX #$00                
CODE_01B81B:                      LDA !OAM_DispY,Y         
CODE_01B81E:                      SEC                       
CODE_01B81F:                      SBC $06                   
CODE_01B821:                      BPL CODE_01B824           
CODE_01B823:                      DEX                       
CODE_01B824:                      CLC                       
CODE_01B825:                      ADC $00                   
CODE_01B827:                      STA $09                   
CODE_01B829:                      TXA                       
CODE_01B82A:                      ADC $01                   
CODE_01B82C:                      STA $0A                   
CODE_01B82E:                      JSR CODE_01C9BF         
CODE_01B831:                      BCC CODE_01B838           
CODE_01B833:                      LDA #$F0                
CODE_01B835:                      STA !OAM_DispY,Y         
CODE_01B838:                      INY                       
CODE_01B839:                      INY                       
CODE_01B83A:                      INY                       
CODE_01B83B:                      INY                       
CODE_01B83C:                      DEC $08                   
CODE_01B83E:                      BPL CODE_01B7DE           
CODE_01B840:                      LDX $15E9               ; X = Sprite index 
Return01B843:                     RTS                       ; Return 

CODE_01B844:                      REP #$20                  ; Accum (16 bit) 
CODE_01B846:                      LDA $04                   
CODE_01B848:                      SEC                       
CODE_01B849:                      SBC !RAM_ScreenBndryXLo    
CODE_01B84B:                      CMP #$0100              
CODE_01B84E:                      SEP #$20                  ; Accum (8 bit) 
Return01B850:                     RTS                       ; Return 

Return01B851:                     RTS                      


CODE_01C9BF:                      REP #$20                  ; Accum (16 bit) 
CODE_01C9C1:                      LDA $09                   
CODE_01C9C3:                      PHA                       
CODE_01C9C4:                      CLC                       
CODE_01C9C5:                      ADC #$0010              
CODE_01C9C8:                      STA $09                   
CODE_01C9CA:                      SEC                       
CODE_01C9CB:                      SBC !RAM_ScreenBndryYLo    
CODE_01C9CD:                      CMP #$0100              
CODE_01C9D0:                      PLA                       
CODE_01C9D1:                      STA $09                   
CODE_01C9D3:                      SEP #$20                  ; Accum (8 bit) 
Return01C9D5:                     RTS                       ; Return 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;org $03B75C

TABLE1:			db $0C,$1C
TABLE2:			db $01,$02

GET_DRAW_INFO:		STZ $186C,x             ; reset sprite offscreen flag, vertical
			STZ $15A0,x             ; reset sprite offscreen flag, horizontal
			LDA $E4,x               ; \
			CMP $1A                 ;  | set horizontal offscreen if necessary
			LDA $14E0,x             ;  |
			SBC $1B                 ;  |
			BEQ ON_SCREEN_X         ;  |
			INC $15A0,x             ; /

ON_SCREEN_X:		LDA $14E0,x             ; \
			XBA			;  |
			LDA $E4,x               ;  |
			REP #$20                ;  |
			SEC			;  |
			SBC $1A                 ;  | mark sprite invalid if far enough off screen
			CLC			;  |
			ADC #$0040            ;  |
			CMP #$0180            ;  |
			SEP #$20                ;  |
			ROL A                   ;  |
			AND #$01                ;  |
			STA $15C4,x             ; / 
			BNE INVALID             ; 
			
			LDY #$00                ; \ set up loop:
			LDA $1662,x             ;  | 
			AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
			BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
			INY			; / 
ON_SCREEN_LOOP:		LDA $D8,x               ; \ 
			CLC			;  | set vertical offscreen if necessary
			ADC TABLE1,y            ;  |
			PHP			;  |
			CMP $1C                 ;  | (vert screen boundry)
			ROL $00                 ;  |
			PLP			;  |
			LDA $14D4,x             ;  | 
			ADC #$00                ;  |
			LSR $00                 ;  |
			SBC $1D                 ;  |
			BEQ ON_SCREEN_Y         ;  |
			LDA $186C,x             ;  | (vert offscreen)
			ORA TABLE2,y            ;  |
			STA $186C,x             ;  |
ON_SCREEN_Y:		DEY			;  |
			BPL ON_SCREEN_LOOP      ; /

			LDY $15EA,x             ; get offset to sprite OAM
			LDA $E4,x               ; \ 
			SEC			;  | 
			SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
			STA $00                 ; / 
			LDA $D8,x               ; \ 
			SEC			;  | 
			SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
			STA $01                 ; / 
			RTS			; return

INVALID:		PLA			; \ return from *main gfx routine* subroutine...
			PLA			;  |    ...(not just this subroutine)
			RTS			; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			
SPR_T12:		db $40,$B0
SPR_T13:		db $01,$FF
SPR_T14:		db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
			db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:		db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
			db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:	LDA #$02                ; \ entry point of routine determines value of $03
			BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:	LDA #$04                ;  | 
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:	LDA #$06                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:	LDA #$08                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:	LDA #$0A                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:	LDA #$0C                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:	LDA #$0E                ;  |
STORE_03:		STA $03			;  |            
			BRA START_SUB		;  |
SUB_OFF_SCREEN_X0:	STZ $03			; /

START_SUB:		JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
			BEQ RETURN_35           ; /
			LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
			AND #$01                ; |
			BNE VERTICAL_LEVEL      ; /     
			LDA $D8,x               ; \
			CLC			; | 
			ADC #$50                ; | if the sprite has gone off the bottom of the level...
			LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
			ADC #$00                ; | 
			CMP #$02                ; | 
			BPL ERASE_SPRITE        ; /    ...erase the sprite
			LDA $167A,x             ; \ if "process offscreen" flag is set, return
			AND #$04                ; |
			BNE RETURN_35           ; /
			LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
			AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
			ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
			STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
			TAY			;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
			LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
			CLC			;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
			ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
			ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
			CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
			PHP			;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
			LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
			LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
			ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
			PLP			;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
			SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
			STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
			LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
			BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
			EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
			STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:		LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
			BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:		LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
			CMP #$08                ; |
			BCC KILL_SPRITE         ; /    
			LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
			CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
			BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
			LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
			STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:		STZ $14C8,x             ; erase sprite
RETURN_35:		RTS			 ; return

VERTICAL_LEVEL:		LDA $167A,x             ; \ if "process offscreen" flag is set, return
			AND #$04                ; |
			BNE RETURN_35           ; /
			LDA $13                 ; \
			LSR A                   ; | 
			BCS RETURN_35           ; /
			LDA $E4,x               ; \ 
			CMP #$00                ;  | if the sprite has gone off the side of the level...
			LDA $14E0,x             ;  |
			SBC #$00                ;  |
			CMP #$02                ;  |
			BCS ERASE_SPRITE        ; /  ...erase the sprite
			LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
			LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
			AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
			STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
			TAY			;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
			LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
			CLC			;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
			ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
			ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
			CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
			PHP			;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
			LDA $001D               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
			LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
			ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
			PLP			;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
			SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
			STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
			LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
			BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
			EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
			STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:		LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
			BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
			BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:	LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
			ORA $186C,x             ; |  
			RTS			; / return