;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Grinder that follows walls like a hothead.
;; If in an even X position, it'll move left first. If odd, it'll move right first.
;; If the extra bit is set, it'll always follow Mario.
;;  It's not very smart, though, and doesn't account for blocks being in the way.
;;
;; Coded by kaizoman666. No credit necessary.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!FollowTime	=	$07
	; How often to reorient the grinder toward Mario if the extra bit is set.
	; Valid values are $00, $01, $03, $07, $0F, $1F, $3F, or $7F.

!Speed		=	$10
	; For math to work out right, this number should divide into both xD0 and x80.
	; It can work fine if it doesn't, but there may be some small oddities in movement.


!TileA		=	$6C 
!TileB		=	$6E
	; 16x16 tiles to use for the grinder's animation.

Properties:
	db $03,$43,$83,$C3
	; YXPPCCCT properties for each of the sprite's four tiles. Change palette here.
	
!AniSpeed	=	$02
	; How fast the sprite animates.
	; Valid values are $01, $02, $04, $08, $10, $20, $40, or $80
	; Alternatively use #$00 and it won't animate, using only tile A.


!Sound		=	$04
!Port		=	$1DFA
	; Sound effect to play for the grinder.





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; actual code begins
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
	PHB
	PHK
	PLB
	JSR Init
	PLB
	RTL

print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Init:
	LDA $E4,x				;\ 
	LDY #$00				;|
	AND #$10				;|
	EOR #$10				;|
	LSR						;|
	LSR						;|
	STA $C2,x				;| Move left if on an even X coordinate, right if on an odd.
	BNE StartOutLeft		;|
	INY						;|
StartOutLeft:				;|
	LDA XSpeed,y			;|
	STA $B6,x				;|
	LDA YSpeed,y			;|
	STA $AA,x				;/
	
	LDA $7FAB10,x			;\ 
	AND #$04				;| Save the extra bit.
	STA $1510,x				;/
	RTS




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XSpeed:
	db !Speed,$00,256-!Speed,$00,256-!Speed,$00,!Speed,$00
YSpeed:
	db $00,!Speed,$00,256-!Speed,$00,!Speed,$00,256-!Speed

FakeX:
	db $01,$FF,$FF,$01,$FF,$01,$01,$FF
FakeY:
	db $01,$01,$FF,$FF,$01,$01,$FF,$FF

BlockedStatus:
	db $01,$04,$02,$08,$02,$04,$01,$08

!DelayA		=	$D0/!Speed
!DelayB		=	$80/!Speed



Main:
	JSL $018032			; Interact with sprites.
	JSR Graphics		; Draw graphics.

	LDA $14C8,x			;\ 
	CMP #$08			;|
	BNE Return			;| Terminate if dying or game frozen.
	LDA $9D				;|
	BEQ NotDead			;/
  Return:
	RTS
	
NotDead:
	LDA $13      		;\ 
	AND #$03        	;|
	BNE .noSound 		;| Play sound every 4 frames.
	LDA #!Sound			;|
	STA !Port			;/
	
  .noSound
	JSR SubOffscreenX0	; Process offscreen.
	JSL $01A7DC			; Interact with Mario.
	
	LDA $1540,x			;\ Skip down if in the process of turning an outer corner.
	BNE NoGap			;/
	
	LDA $1510,x			;\ 
	BEQ NoFollow		;|
	LDA $14				;| Attempt to follow Mario if the extra bit is set.
	AND #!FollowTime	;|
	BNE NoFollow		;|
						;|
	LDA $B6,x			;| 
	BEQ _vertical		;|
	JSR SubHorzPos		;|\ 
	LDA XSpeed,y		;||
	CMP $B6,x			;||
	BEQ NoFollow		;|| Change horizontal direction.
	LDA $C2,x			;||
	EOR #$04			;||
	STA $C2,x			;|/
	BRA NoFollow		;|
						;|
  _vertical:			;|
	JSR SubVertPos		;|\ 
	LDA YSpeed,y		;||
	CMP $AA,x			;|| Change vertical direction.
	BEQ NoFollow		;||
	LDA $C2,x			;||
	EOR #$06			;||
	STA $C2,x			;//
	
NoFollow:
	LDY $C2,x			;\ 
	LDA FakeY,y			;|
	STA $AA,x			;| Interact with objects.
	LDA FakeX,y			;|
	STA $B6,x			;|
	JSL $019138			;/
	
	LDA $1588,x			;\ 
	AND #$0F			;|
	BNE NoGap			;|
	LDA #$08			;| If no longer touching any blocks, set it to start turning a corner.
	STA $1564,x			;|
	LDA.b #!DelayA		;|
	STA $1540,x			;/

NoGap:
	LDA.b #!DelayB		;\ 
	CMP $1540,X			;|
	BNE NoTurn			;|
	INC $C2,X			;|
	LDA $C2,X			;|
	CMP.b #$04			;|
	BNE .counter		;| Change movement direction if time to turn.
	STZ $C2,X			;|
  .counter				;|
	CMP.b #$08			;|
	BNE NoTurn			;|
	LDA.b #$04			;|
	STA $C2,X			;/

NoTurn:
	LDY $C2,x			;\ 
	LDA $1588,x			;|
	AND BlockedStatus,y	;|
	BEQ NoWall			;|
	LDA #$08			;|
	STA $1564,x			;|
	DEC $C2,x			;|
	LDA $C2,x			;| If it's hit a wall, turn the corner.
	BPL .counter		;|
	LDA #$03			;|
	BRA _storeDir		;|
  .counter				;|
	CMP #$03			;|
	BNE NoGap			;|
	LDA #$07			;|
  _storeDir:			;|
	STA $C2,x			;/

NoWall:
	LDA YSpeed,y		;\ 
	STA $AA,x			;| Set actual X/Y speed.
	LDA XSpeed,y		;|
	STA $B6,x			;/
	
	JSL $018022			; Update X position.
	JSL $01801A			; Update Y position.
	
	RTS





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gfx routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XDisp:			; X displacements.
	db $F8,$08,$F8,$08
YDisp:			; Y displacements.
	db $F8,$F8,$08,$08

Graphics:
	JSR GetDrawInfo
	PHX
	LDX #$03					; Draw 4 tiles.
CODE_01DBA8:
	LDA $00                   	;\ 
	CLC                       	;| Set tile X position.
	ADC XDisp,x     		  	;|
	STA $0300,y 				;/        
	LDA $01                   	;\ 
	CLC                       	;| Set tile Y position.
	ADC YDisp,x    			   	;|
	STA.w $0301,y         		;/
	LDA $14						;\ 
	AND #!AniSpeed				;|
	BEQ +						;|
	LDA #!TileA					;|
	BRA ++						;| Set tile number.
  +								;|
	LDA #!TileB					;|
  ++							;|
	STA $0302,y					;/
	LDA Properties,x 			;\ Set YXPPCCCT.
	STA $0303,Y          		;/
	INY                       	;\ 
	INY                       	;|
	INY                       	;| Loop for all tiles.
	INY                       	;|
	DEX                       	;| 
	BPL CODE_01DBA8  			;/
	PLX
	LDY #$02      				;\ 
	LDA #$03                	;| Upload 4 16x16 tiles.
	JSL $01B7B3					;|
	RTS           				;/





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; other subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SubHorzPos (modified to return xspeed index)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,X
	LDA $95
	SBC $14E0,X
	BPL ReturnHorz
	INY #2
ReturnHorz:
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SubVertPos (modified to return yspeed index)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SubVertPos:
	LDY #$01
	LDA $D3
	SEC
	SBC $D8,X
	LDA $D4
	SBC.w $14D4,X
	BPL ReturnVert
	INY #2
ReturnVert:
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SubOffScreen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
	STZ $03
	JSR SubIsOffscreen
	BEQ ReturnSOS
	LDA $5B
	LSR
	BCS VerticalLevel
	LDA $D8,x
	CLC
	ADC #$50
	LDA $14D4,x
	ADC #$00
	CMP #$02
	BPL EraseSprite
	LDA $167A,x
	AND #$04
	BNE ReturnSOS
	LDA $13
	AND #$01
	ORA $03
	STA $01
	TAY
	LDA $1A
	CLC
	ADC Table4,y
	ROL $00
	CMP $E4,x
	PHP
	LDA $1B
	LSR $00
	ADC Table5,y
	PLP
	SBC $14E0,x
	STA $00
	LSR $01
	BCC Label20
	EOR #$80
	STA $00
Label20:
	LDA $00
	BPL ReturnSOS

EraseSprite:
	LDA $14C8,x
	CMP #$08
	BCC KillSprite
	LDY $161A,x
	CPY #$FF
	BEQ KillSprite
	LDA #$00
	STA $1938,y
	KillSprite:
	STZ $14C8,x
ReturnSOS:
	RTS

VerticalLevel:
	LDA $167A,x
	AND #$04
	BNE ReturnSOS
	LDA $13
	LSR
	BCS ReturnSOS
	AND #$01
	STA $01
	TAY
	LDA $1C
	CLC
	ADC Table3,y
	ROL $00
	CMP $D8,x
	PHP
	LDA $1D
	LSR $00
	ADC Table6,y
	PLP
	SBC $14D4,x
	STA $00
	LDY $02
	BEQ Label22
	EOR #$80
	STA $00
Label22:
	LDA $00
	BPL ReturnSOS
	BMI EraseSprite

SubIsOffscreen:
	LDA $15A0,x
	ORA $186C,x
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GetDrawInfo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GetDrawInfo:
	STZ $186C,x
	STZ $15A0,x
	LDA $E4,x
	CMP $1A
	LDA $14E0,x
	SBC $1B
	BEQ OnscreenX
	INC $15A0,x
OnscreenX:
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	SEC
	SBC $1A
	CLC
	ADC.w #$0040
	CMP #$0180
	SEP #$20
	ROL A
	AND #$01
	STA $15C4,x
	BNE Invalid

	LDY #$00
	LDA $1662,x
	AND #$20
	BEQ OnscreenLoop
	INY
OnscreenLoop:
	LDA $D8,x
	CLC
	ADC Table1,y
	PHP
	CMP $1C
	ROL $00
	PLP
	LDA $14D4,x
	ADC #$00
	LSR $00
	SBC $1D
	BEQ OnscreenY
	LDA $186C,x
	ORA Table2,y
	STA $186C,x
OnscreenY:
	DEY
	BPL OnscreenLoop
	LDY $15EA,x
	LDA $E4,x
	SEC
	SBC $1A
	STA $00
	LDA $D8,x
	SEC
	SBC $1C
	STA $01
	RTS
	
Invalid:
	PLA
	PLA
	RTS