;Egg Plant
;by smkdan

;IGNORES PALETTE IN CFG EDITOR

;USES EXTRA BIT
;If set, a custom sprite is generated.  Else, a standard sprite is generated.

;EXTRA PROP 1:
;Sprite number to generate

;EXTRA PROP 2:
;How many of chosen sprite (above) can be on screen before it stops generating.  A sprite is counted even if it wasn't generated.

;A plant from SMW2 that shoots out an item or enemy of your choice

;Tilemaps for SP3

DIR = $03
PROPRAM = $04
PROPINDEX = $05
XDISPINDEX = $06
YDISPINDEX = $07
FRAMEINDEX = $08
TEMP = $09

;1570: Frame # for GFX
;C2: action for shooting
;1602: counter for action

;===

;Data to use for the sprite to be generated

;=====

dcb "INIT"
	RTL

dcb "MAIN"
	PHB
	PHK
	PLB
	JSR Run
	PLB
	RTL	;also nothing


Run
	JSR SUB_OFF_SCREEN_X0
	JSR GET_DRAW_INFO
	JSR GFX			;draw sprite

	LDA $14C8,x
	CMP #$08          	 
	BNE Return           
	LDA $9D			;locked sprites?
	BNE Return

	LDA $C2,x		;load decision bit
	BEQ Idle		;doing nothing, keep counting
	CMP #$01
	BEQ Shooting

Idle
	LDA $1602,x
	INC A
	STA $1602,x
	CMP #$30
	BNE Return_S		;return if not yet 30h

	INC $C2,x		;+1 for behaviour
	STZ $1602,x		;reset counter, to be used in GFX also
	STZ $1570,x		;reset frame counter for inflating GFX

Shooting
	LDA $1602,x
	INC A
	STA $1602,x
	CMP #$0C		;return if counter not yet 6 x2
	BNE Return_S

	JSR GenerateX		;attempt to generate sprite

	STZ $C2,x		;back to idling
	STZ $1570,x		;reset frame counter for idling
	STZ $1602,x		;reset counter

Return_S
    	LDA $1588,x
        AND #$04
        BEQ InAir
        LDA #$10                ;falling yspd, if not already falling
        STA $AA,x
     	
InAir 
	JSL $01802A		;speed update
	
Return
	RTS     

GenerateX
	LDA $7FAB10,x
	AND #$04
	BNE GenCust		;custom sprite if extra bit is set

	JSR CountSpritesStd	;standard count sprite routine
	BEQ ReturnG		;Z = 1 if it's to return

	JSL $02A9E4		;grab free sprite slot
	BMI ReturnG		;return if none left before anything else...

	JSR SetupStd

	RTS			;return std

GenCust
	JSR CountSpritesCust	;standard count sprite routine
	BEQ ReturnG		;Z = 1 if it's to return

	JSL $02A9E4		;grab free sprite slot
	BMI ReturnG		;return if none left before anything else...

	JSR SetupCust

	RTS			;return custom
		
ReturnG
	JSR SUB_SMOKE		;smoke if not generating
	RTS     

;===

SetupStd		
	PHX			;preserve sprite index

	LDA $7FAB28,x		;compare to see if it's a throw block
	CMP #$53
	BEQ ThrowBlock

	LDA #$01		;init routine as normal
	STA $14C8,y
	LDA $7FAB28,x
	STA $9E,y		;sprite type
	BRA SkipStd

ThrowBlock
	LDA #$09		;status = stationary
	STA $14C8,y		;normal status, run init routine?

	LDA #$53		;load throwblock to generate
	STA $9E,y		;into sprite type table

SkipStd	
	LDA $E4,x		;same Xpos
	STA $E4,y
	LDA $14E0,x		;share same screen
	STA $14E0,y

	LDA $D8,x
	SEC
	SBC #$14
	STA $D8,y		;Ypos
	LDA $14D4,x
	SBC #$00
	STA $14D4,y		;same screen

	TYX			;new sprite slot into X
	JSL $07F7D2		;create sprite

	LDA $9E,y		;if throw block
	CMP #$53
	BNE NotThrowBlock

	LDA #$FF		;set stun timer
	STA $1540,y

NotThrowBlock
	JSL $01ACF9		;'random number'
	AND #$8F		;preserve direction bit and low 4 bits
	BPL PositiveRS
	ORA #$70		;set higher bits if negative

PositiveRS
	STA $B6,x		;set Xspd

	LDA #$B0		;set Yspd fixed
	STA $AA,x

	LDA #$3A		;gulping sound
	STA $1DFC

	PLX			;restore sprite index

	RTS			;return

;===

SetupCust
	PHX			;preserve sprite index

	LDA #$01
	STA $14C8,y		;normal status, run init routine?
	LDA $7FAB28,x		;load sprite to generate...from extra prop 1

	PHX
	TYX
	STA $7FAB9E,x		;into sprite type table for custom sprites
	PLX
	
	LDA $E4,x		;same Xpos
	STA $E4,y
	LDA $14E0,x		;share same screen
	STA $14E0,y

	LDA $D8,x
	SEC
	SBC #$14
	STA $D8,y		;Ypos
	LDA $14D4,x
	SBC #$00
	STA $14D4,y		;same screen

	TYX			;new sprite slot into X
	JSL $07F7D2		;create sprite
	JSL $0187A7
        LDA #$88		;set as custom sprite
        STA $7FAB10,X

	JSL $01ACF9		;'random number'
	AND #$8F		;preserve direction bit and low 4 bits
	BPL PositiveRC
	ORA #$70		;set higher bits if negative

PositiveRC
	STA $B6,x		;set Xspd

	LDA #$B0		;set Yspd fixed
	STA $AA,x

	LDA #$3A		;gulping sound
	STA $1DFC

	PLX			;restore sprite index

	RTS			;return

;===

CountSpritesStd
	STZ TEMP		;TEMP will be the sprite counter
	LDY #$00

StdLoop
	LDA $9E,y		;look into sprite type table
	CMP $7FAB28,x		;compare with generated sprite type
	BNE NoIncS		;if not equal, skip this part

	LDA $14C8,y		;load status
	BEQ NoIncS		;if sprite is dead, don't increment

	INC TEMP		;else increment
	LDA TEMP		;compare TEMP to count
	CMP $7FAB34,x		;if count is reached....
	BCS ReturnStdN		;just return

NoIncS
	INY			;advance index
	CPY #$0C		;if some left to check...
	BNE StdLoop		;loop again

	LDA #$01		;Z = 0
	RTS

ReturnStdN
	LDA #$00		;Z = 1
	RTS	
		
;=====

CountSpritesCust
	STZ TEMP		;TEMP will be the sprite counter
	LDY #$00

CustLoop
	PHX
	TYX			;Y into X
	LDA $7FAB9E,x		;look into sprite type table for custom sprites
	PLX

	CMP $7FAB28,x		;compare with generated sprite type
	BNE NoIncC		;if not equal, skip this part

	TYX			;Y->X, for long indexed
	LDA $7FAB10,x		;see if it's also a custom sprite...
	AND #$08
	BEQ NoIncC		;return if custom sprite flag is clear

	LDA $14C8,y		;load status
	BEQ NoIncC		;if sprite is dead, don't increment

	INC TEMP		;else increment
	LDA TEMP		;compare TEMP to count

	LDX $15E9		;original sprite index
	CMP $7FAB34,x		;if count is reached....
	BCS ReturnCustN		;just return

NoIncC
	LDX $15E9		;restore sprite index

	INY			;advance index
	CPY #$0C		;if some left to check...
	BNE CustLoop		;loop again

	LDA #$01		;Z = 0
	RTS

ReturnCustN
	LDA #$00		;Z = 1
	RTS	

;===

TILEMAP	dcb $4E,$3A,$2A		;body, stem, stem
	dcb $4E,$3A,$2A
	dcb $60,$3A,$2A

;shooting frames

	dcb $4A,$3A,$2A
	dcb $48,$3A,$2A
	dcb $4A,$3A,$2A
	dcb $4A,$3A,$2A
	dcb $4C,$3A,$2A		;generating frame
	dcb $4C,$3A,$2A

XDISP	dcb $01,$00,$08
	dcb $01,$00,$08
	dcb $00,$00,$08

	dcb $00,$00,$08
	dcb $00,$00,$08
	dcb $00,$00,$08
	dcb $00,$00,$08
	dcb $01,$00,$08
	dcb $01,$00,$08

YDISP	dcb $F8,$07,$07		;It's 8px down from it's origin
	dcb $F9,$07,$07
	dcb $F9,$07,$07

	dcb $F8,$07,$07
	dcb $F8,$07,$07
	dcb $F8,$07,$07
	dcb $F8,$07,$07
	dcb $F7,$07,$07
	dcb $F7,$07,$07

PROP	dcb $04,$06,$06		;pink,green,green

SIZE	dcb $02,$00,$00

GFX
	LDA $15F6,x	;load properties
	STA PROPRAM	;into TEMP

	LDA $157C,x	;direction...
	STA DIR

	STZ FRAMEINDEX	;zero, since it will be added

	LDA $C2,x	;action
	BEQ IdleG	;zero = idling

;shooting
	LDA $1602,x	;load counter
	LSR A		;every 2nd advance

	CLC
	ADC #$03	;skip past idle frames
	STA $1570,x
	BRA Skip	
	
IdleG
	LDA $14
	AND #$07	;every 8th frame
	BNE Skip
	LDA $9D		;due to odd way it animates, skip animating if sprites are locked
	BNE Skip
	LDA $1570,x	;grab frame no.
	INC A
	STA $1570,x
	CMP #$03	;3 = overflow
	BNE Skip
	STZ $1570,x
	
Skip
	LDA $1570,x	;frame #
	ASL A		;x2
	CLC
	ADC $1570,x	;x3
	CLC
	ADC FRAMEINDEX
	STA FRAMEINDEX

	LDX #$00	;loop index

OAM_Loop
	TYA
	LSR A
	LSR A
	PHY
	TAY
	LDA SIZE,x
	STA $0460,y		;size table
	PLY			;restore

	TXA	
	CLC
	ADC FRAMEINDEX
	PHX
	TAX

	LDA $00
	CLC
	ADC XDISP,x
	STA $0300,y
	PLX

	TXA	
	CLC
	ADC FRAMEINDEX
	PHX
	TAX

	LDA $01
	CLC
	ADC YDISP,x
	STA $0301,y

	PLX

	TXA	
	CLC
	ADC FRAMEINDEX
	PHX
	TAX
	LDA TILEMAP,x
	STA $0302,y
	PLX

	LDA PROPRAM
	AND #$01		;page bit only, ignore palette setting
	ORA $64
	ORA PROP,x		;palette, just using loop index
	STA $0303,y

	INY
	INY
	INY
	INY
	INX
	CPX #$03	;3 tiles for flower body and stem
	BNE OAM_Loop

	LDX $15E9

	LDY #$FF	;$460 written
	LDA #$02	;3 tiles
	JSL $01B7B3

	RTS	

;=================
;BORROWED ROUTINES
;=================

SUB_SMOKE           LDY #$03                ; \ find a free slot to display effect
FINDFREE            LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE            LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    CLC
                    ADC #$EC
                    STA $17C4,y             ; /
                    LDA #$14                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
		    CLC
		    ADC #$00
                    STA $17C8,y             ; /
                    LDX $15E9
             
                    RTS
;GET_DRAW_INFO
;==============

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642