;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Growing Vine Disassembly, by imamelia
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dcb "INIT"
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dcb "MAIN"
PHB
PHK
PLB
JSR GrowingVineMain
PLB
RTL

GrowingVineMain:

JSR SubOffscreen

LDA $64		; level priority bits
PHA		; save for later
LDA $1540,x	;
CMP #$20			; if this counter is less than 20...
BCC DontChangePriority	; don't change the sprite's priority
LDA #$10			;
STA $64		; set new priority bits
DontChangePriority:
JSR SubVineGFX	; the original graphics routine was so screwy that I had to rewrite part of it
PLA		;
STA $64		; restore original priority bits from before
LDA $9D		; if sprites are locked...
BNE ReturnMain	; return
LDA #$F0		; set the sprite's
STA $AA,x	; Y speed
JSL $01801A	; update sprite Y position without gravity      
LDA $1540,x	; load the counter
CMP #$20		; this time, branch if it is greater than or equal to #$20
BCS SkipStuff	;
JSR $019138	; interact with objects
LDA $1588,x	; load sprite blocked status
BNE SkipStuff	; branch if the sprite is touching anything  
LDA $14D4,x	; sprite Y position (high byte)
BPL DontEraseSprite	; don't erase the sprite if it hasn't gone offscreen

SkipStuff:
LDA $14C8,x		; if the sprite isn't in its normal status...
CMP #$08			; (more like, if the status is *less* than 08)
BCC SpriteDisappears	; then make it disappear
LDY $161A,x		; load sprite index to level table
CPY #$FF			; if FF...
BEQ SpriteDisappears	; then make the sprite disappear
LDA #$00			; mark the sprite to be loaded
STA $1938,y	; based on its index stored in $161A,x
SpriteDisappears:	;
STZ $14C8,x	;
RTS

DontEraseSprite:	;

LDA $D8,x	; sprite Y position (low byte)
AND #$0F	; we only want the lower 4 bits
;CMP #$00		; pointless line, but I'll leave it in anyway
BNE ReturnMain	; if this is 00, then return
LDA $E4,x	; sprite X position (low byte)
STA $9A		; this will be the low byte of the vine tile's X position
LDA $14E0,x	; sprite X position (high byte)
STA $9B		; this will be the high byte of the vine tile's X position
LDA $D8,x	; sprite Y position (low byte)
STA $98		; block Y position (low byte)
LDA $14D4,x	; sprite Y position (high byte)
STA $99		; block Y position (high byte)
LDA #$03		; block to generate:
STA $9C		; vine tile
JSL $00BEB0	; generate common Map16 tile routine
ReturnMain:	;
RTS

SubVineGFX:
JSR GET_DRAW_INFO	; set up variables for OAM
LDA $00		; sprite X displacement
STA $0300,y	; first OAM slot
LDA $01		; sprite Y displacement
STA $0301,y	; second OAM slot
LDA $14		; take the sprite frame counter
LSR
LSR
LSR
LSR		; and shift right 4 times
LDA #$AC	; tile used if bit 4 of the frame counter was clear
BCC StoreTile	;
LDA #$AE		; tile used if bit 4 of the frame counter was set
StoreTile:		;
STA $0302,y	; store the tile number      
LDA $157C,x	;
LSR		;
LDA $15F6,x	; load sprite palette and GFX page info
BCS NoFlip	; if the sprite's direction was right ($157C,x and $02=01),
ORA #$40		; flip the tile
NoFlip:
ORA $64		; add in priority bits
STA $0303,y	; store tile properties
LDY #$02		; the tiles are 16x16
LDA #$00		; and we drew one tile...
JSL $01B7B3	; finish OAM write
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

Table3:              dcb $40,$B0
Table6:              dcb $01,$FF 
Table4:              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
Table5:              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreen:   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA Store03            ;  | 
SubOffscreenX1:   LDA #$04                ;  |
                    BRA Store03            ;  |
SubOffscreenX2:   LDA #$02                ;  |
Store03:       
	    STA $03                 ;  |
                    BRA StartSub           ;  |
SubOffscreenX3:   STZ $03                 ; /

StartSub:           JSR SubIsOffscreen   ; \ if sprite is not off screen, return
                    BEQ Return2            ; /    
                    LDA $5B                 ; \  goto VerticalLevel if vertical level
                    AND #$01                ;  |
                    BNE VerticalLevel      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL EraseSprite        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE Return2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;
                    CLC                     ;
                    ADC Table4,y            ;
                    ROL $00                 ;
                    CMP $E4,x               ;x pos ;
                    PHP                     ;
                    LDA $1B                 ;x boundry hi ;
                    LSR $00                 ;
                    ADC Table5,y            ;
                    PLP                     ;
                    SBC $14E0,x             ;x pos high
                    STA $00                 ;
                    LSR $01                 ;
                    BCC Label20             ;
                    EOR #$80                ;
                    STA $00                 ;
Label20:             LDA $00                 ;
                    BPL Return2            ;
EraseSprite:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KillSprite         ; /
                    LDY $161A,x             ;
                    CPY #$FF                ;
                    BEQ KillSprite        ;
                    LDA #$00                ; \ 
                    STA $1938,y             ; /     
KillSprite:         STZ $14C8,x             ; erase sprite
Return2:            RTS                     ; return

VerticalLevel:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE Return2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS Return2            ; /
                    AND #$01                ;
                    STA $01                 ;
                    TAY                     ;
                    LDA $1C                 ;
                    CLC                     ;
                    ADC Table3,y            ;
                    ROL $00                 ;
                    CMP $D8,x               ;
                    PHP                     ;
                    LDA $001D             ;
                    LSR $00                 ;
                    ADC Table6,y            ;
                    PLP                     ;
                    SBC $14D4,x             ;
                    STA $00                 ;
                    LDY $01                 ;
                    BEQ Label22             ;
                    EOR #$80                ;
                    STA $00                 ;
Label22:             LDA $00                 ;
                    BPL Return2            ;
                    BMI EraseSprite        ;

SubIsOffScreen:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return
