;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Hammer Brother, by mikeyk
;; adapted by yoshicookiezeus
;;
;; Description: This guy walks back and forth, throwing hammers at Mario.
;;
;; Number of extra bytes used: 2
;; Extra byte 1: xxxxxxsj
;;		x - unused
;;		s - if set, the sprite will play a sound effect when it throws a hammer
;;		j - if set, the sprite won't jump down through platforms
;; Extra byte 2: amount of time between consecutive hammers (original hammer bro value: $28)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

incsrc ramdefines.asm
incsrc subroutinedefs.asm

			!RAM_ThrowTimer		= $1504
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			
			!HammerTile		= $6D
			!TimeToShow		= $18

			!SFX			= $14
			!SFXPort		= $1DF9
			
Tilemap:		db $46,$4A,$7F,$00
			db $46,$5A,$2F,$00
			db $48,$4A,$7F,$00
			db $48,$5A,$2F,$00

OffsetX:		db $00,$00,$08
			db $00,$08,$00
OffsetY:		db $F8,$08,$08
TileSize:		db $02,$00,$00
HammerOffset:		db $F6,$0A

Properties:		db $40,$00

SpeedX:			db $08,$F8

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Init:			PHY
			JSL !SubHorizPos		;\ make sprite face Mario
			TYA				; |
			STA !RAM_SpriteDir,x		;/
			PLY
			
			TXA				;\ set throw timer
			AND #$01			; |
			ASL A				; |
			ASL A				; |
			ASL A				; |
			ASL A				; |
			CLC				; |
			ADC #$20			; |
			STA !RAM_ThrowTimer,x		;/
			RTL
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main sprite JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:			PHB
			PHK
			PLB
			JSR DecrementTimers
			JSR HammerBro
			PLB
			RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DecrementTimers:	LDA $14C8,x			;\ if sprite status not normal,
			CMP #$08			; |
			BNE .Return			;/ branch
			LDA !RAM_SpritesLocked		;\ if sprites locked,
			BNE .Return			;/ branch
			LDA !RAM_ThrowTimer,x		;\ if throw timer is zero,
			BEQ .Return			;/ branch
			DEC !RAM_ThrowTimer,x		; decrement throw timer
.Return			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main sprite routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

IncrementTimer:		INC !RAM_ThrowTimer,x
Return:			RTS			

HammerBro:		JSR SpriteGraphics
			LDA $14C8,x			;\ if sprite status not normal,
			CMP #$08			; |
			BNE IncrementTimer		;/ branch
			LDA !RAM_SpritesLocked		;\ if sprites locked,
			BNE Return			;/ branch

			JSL !SubOffscreenX0
			INC $1570,x			; increase jumping timer
			LDA $1570,x			;\ determine graphics frame to use
			LSR A				; |
			LSR A				; |
			LSR A				; |
			AND #$01			; |
			STA $1602,x			;/

			LDA $1540,x			;\ if not time to start interacting with objects after jumping,
			BEQ .DisableNotSet		; |
			CMP #$01			; |
			BNE .DisableNotSet		;/ branch

			LDA !RAM_ExtraByte1,x		;\ if first bit of extra byte 1 set,
			AND #$01			; |
			BNE .DisableNotSet		;/ branch

			LDA !RAM_Tweaker1686,x		;\ clear "don't interact with objects" flag
			AND #$7F			; |
			STA !RAM_Tweaker1686,x		;/

.DisableNotSet		LDA !RAM_ThrowTimer,x		;\ if sufficient time left until throw,
			CMP #!TimeToShow		; |
			BCS .NoThrow			;/ branch 
			INC $1602,x			;\ set graphics frame to display hammer
			INC $1602,x			;/

			LDA !RAM_ThrowTimer,x		;\ if throw timer isn't zero,
			BNE .NoTimeSet			;/ branch
			LDA !RAM_ExtraByte2,x 		;\ set timer
			STA !RAM_ThrowTimer,x		;/
.NoTimeSet		CMP #$01			;\ if not time to throw hammer,
			BNE .NoThrow			;/ branch
			JSR SubHammerThrow

.NoThrow		LDA !RAM_SprObjStatus,x		;\ if sprite isn't on ground,
			AND #$04			; |
			BEQ .NoJump			;/ branch
			LDA #$10			;\ set sprite y speed
			STA !RAM_SpriteSpeedY,x		;/

			JSL !SubHorizPos		;\ make sprite face Mario
			TYA				; |
			STA !RAM_SpriteDir,x		;/

			LDA $1570,x			;\ if not time to jump,
			CLC				; |
			ADC #$77			; |
			AND #$E7			; |
			BNE .NoJump			;/ branch
			
			JSR Jump

			LDA !RAM_ExtraByte1,x		;\ if first bit of extra byte 1 set,
			AND #$01			; |
			BNE .NoJump			;/ branch

			LDA !RAM_Tweaker1686,x		;\ set "don't interact with objects" flag
			ORA #$80			; |
			STA !RAM_Tweaker1686,x		;/

.NoJump			LDA !RAM_FrameCounterB		;\ if not time to switch directions,
			AND #$3F			; |
			BNE .DontChangeDir1		;/ branch
			LDA $151C,x			;\ switch sprite direction
			EOR #$01			; |
			STA $151C,x			;/

.DontChangeDir1		LDA $151C,x			;\ set sprite x speed
			AND #$01			; |
			TAY				; |
			LDA SpeedX,y			; |
			STA !RAM_SpriteSpeedX,x		;/

			JSL $01802A			; update sprite position

			LDA !RAM_SprObjStatus,x		;\ if sprite isn't touching wall,
			AND #$03			; |
			BEQ .DontChangeDir2		;/ branch
			LDA $151C,x			;\ switch sprite direction
			EOR #$01			; |
			STA $151C,x			;/

.DontChangeDir2		JSL $018032			; interact with other sprites
			JSL $01A7DC			; interact with Mario

			RTS



JumpSpeed:		db $B0,$D0
DisableTime:		db $24,$28

Jump:			LDA !RAM_SpriteYHi,x		;\ load sprite position (16-bit) into accumulator
			XBA				; |
			LDA !RAM_SpriteYLo,x		;/
			PHP
			REP #$30			
			CMP #$0138			;\ if sprite too low to jump down,
			BCS .JumpUp			;/ branch
			CMP #$00F8			;\ if sprite too high to jump up,
			BCC .JumpDown			;/ branch

			PLP
			JSL $01ACF9			;\ get random number (0 or 1)
			AND #$01			;/
			PHY
			TAY

.SetHeight		LDA DisableTime,y		;\ set time to disable interaction with objects
			STA $1540,x			;/
			LDA JumpSpeed,y			;\ set sprite y speed
			STA !RAM_SpriteSpeedY,x		;/
			PLY
			RTS
			
.JumpUp			PLP
			PHY
			LDY #$00
			BRA .SetHeight

.JumpDown		PLP
			PHY
			LDY #$01
			BRA .SetHeight

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

HammerX:		db $F6,$0A
HammerX2:		db $FF,$00

			!HammerSpeedY		= $C8

SubHammerThrow:		LDA !RAM_OffscreenHorz,x	;\ if sprite offscreen or getting eaten,
			ORA !RAM_OffscreenVert,x	; |
			ORA $15D0,x			; |
			BNE .Return			;/ branch

			LDY #$EE			;\ $00 = x speed to use for hammer
			LDA !RAM_SpriteDir,x		; |
			BNE .Label9			; |
			LDY #$12			; |
.Label9			STY $00				;/

			LDY #$07			; setup loop
.LoopStart		LDA !RAM_ExSpriteNum,y		;\ if current slot is empty,
			BEQ .FoundSlot			;/ branch
			DEY				;\ if still slots left to check,
			BPL .LoopStart			;/ branch
.Return			RTS

.FoundSlot		LDA #$04			;\ set new extended sprite number (hammer)
			STA !RAM_ExSpriteNum,y		;/
			
			PHY				;\ set new extended sprite x position
			LDA !RAM_SpriteDir,x		; |
			TAY				; |
			LDA !RAM_SpriteXLo,x		; |
			CLC				; |
			ADC HammerX,y			; |
			PLY				; |
			STA !RAM_ExSpriteXLo,y		; |
			PHY				; |
			LDA !RAM_SpriteDir,x		; |
			TAY				; |
			LDA !RAM_SpriteXHi,x		; |
			ADC HammerX2,y			; |
			PLY				; |
			STA !RAM_ExSpriteXHi,y		;/
			
			LDA !RAM_SpriteYLo,x		;\ set new extended sprite y position
			SEC				; |
			SBC #$09			; |
			STA !RAM_ExSpriteYLo,y		; |
			LDA !RAM_SpriteYHi,x		; |
			SBC #$00			; |
			STA !RAM_ExSpriteYHi,y		;/

			LDA #!HammerSpeedY		;\ set new extended sprite speed
			STA !RAM_ExSprSpeedY,y		; |
			LDA $00				; |
			STA !RAM_ExSprSpeedX,y		;/

			LDA !RAM_ExtraByte1,x		;\ if second bit of extra byte 1 set,
			AND #$02			; |
			BEQ .Return			;/ branch

			LDA #!SFX			;\ play sound effect
			STA !SFXPort			;/ 

			RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteGraphics:		JSL !GetDrawInfo
			LDA $1602,x			;\ $03 = sprite graphics frame to use
			ASL A				; |
			ASL A				; |
			STA $03				;/
			LDA !RAM_SpriteDir,x		;\ $02 = sprite direction
			STA $02				;/
			PHX

			LDX #$02			; setup loop

.LoopStart		PHX
			PHX
			TXA
			LDX $02				;\ if sprite is facing right,
			BNE .NoAdj			;/ branch
			CLC				;\ adjust index for x offset table
			ADC #$03			;/
.NoAdj			TAX
			
			LDA $00				;\ set tile x position
			CLC				; |
			ADC OffsetX,x			; |
			STA !OAM_DispX,y		;/
			PLX

			LDA $01				;\ set tile y position
			CLC				; |
			ADC OffsetY,x			; |
			STA !OAM_DispY,y		;/
			
			LDA TileSize,x			;\ set tile size
			PHX				; |
			PHA				; |
			TYA				; |
			LSR A				; |
			LSR A				; |
			TAX				; |
			PLA				; |
			STA !OAM_TileSize,x		;/
			PLX

			TXA				;\ adjust index for tilemap table depending on graphics frame
			ORA $03				; |
			TAX				;/

			LDA Tilemap,x			;\
			STA !OAM_Tile,y			;/

			LDX $02				;\ set tile properties
			LDA Properties,x		; |
			LDX $15E9			; |
			ORA !RAM_SpritePal,x		; |
			ORA $64				; |
			STA !OAM_Prop,y			;/
			
			PLX

			INY				;\ increase OAM index by four, so that the next tile can be drawn
			INY				; |
			INY				; |
			INY				;/

			DEX				;\ if still tiles left to draw,
			BPL .LoopStart			;/ branch

			PLX

			LDA !RAM_ThrowTimer,x		;\ if not time to show hammer,
			CMP #$02			; |
			BCC .DontShowHammer		; |
			CMP #30				; |
			BCS .DontShowHammer		;/ branch
			LDA $1602,x			;\ if hammer bro is in "holding hammer" pose,
			CMP #$02			; |
			BCS .ShowHammer			;/ branch
			
.DontShowHammer		LDA #$02			; three tiles were drawn
			LDY #$FF			; the tiles use $0460 for size data
			JSL $01B7B3			; finish OAM write
			RTS

.ShowHammer		PHX
			
			LDA $00				;\ set tile x position
			LDX $02				; |
			CLC				; |
			ADC HammerOffset,x		; |
			STA !OAM_DispX,y		;/
			
			LDA $01				;\ set tile y position
			CLC				; |
			ADC #$F2			; |
			STA !OAM_DispY,y		;/
			
			LDA #!HammerTile		;\ set tile number
			STA !OAM_Tile,y			;/

			PHX
			TYA
			LSR A
			LSR A
			TAX
			LDA #$02			;\ set tile size
			STA !OAM_TileSize,x		;/
			PLX

			LDA #$07			;\ set tile properties
			CPX #$00			; |
			BNE .DontFlipHammer		; |
			ORA #$40			; |
.DontFlipHammer		ORA $64				; |
			STA !OAM_Prop,y			;/
			
			PLX

			INY				;\ increase OAM index by four, so that the next tile can be drawn
			INY				; |
			INY				; |
			INY				;/

			LDA #$03			; three tiles were drawn
			LDY #$FF			; the tiles use $0460 for size data
			JSL $01B7B3			; finish OAM write
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dl Init,Main