;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Invisible Sprite, by imamelia and Sonikku
;;
;; This sprite will spawn another sprite (flying up into the air) when
;; the player touches it.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, the sprite will use the extra property
;; byte 1 for some properties; if it is set, the sprite will use the
;; extra property byte 2 instead.
;;
;; Uses extra property bytes: YES
;;
;; Bits 0-4: Table index for the sprite number and state.  With this,
;; you can spawn up to 32 different sprites using the same .asm file.
;; Bit 5: If this is set, the invisible sprite will spawn five new sprites
;; instead of just one.
;; Bit 6: If bit 5 is clear, setting this will make the sprite spawn with
;; no X speed.  If bit 5 is set, setting this will make the five sprites
;; spread out evenly instead of having random speeds.  (It might be
;; a good idea to set this when spawning 5 sprites, because the random
;; speed routine kind of sucks.)
;; Bit 7: If this is set, the sprite that is spawned will be a custom one.
;;
;; Full credit to Sonikku for the Invisible Mushroom disassembly.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;incsrc subroutinedefsx.asm		; shared subroutine definitions

SpriteNumber:
db $00,$00,$00,$00,$00,$00,$00,$00	; indexes 00-07
db $00,$00,$00,$00,$00,$00,$00,$00	; indexes 08-0F
db $00,$00,$00,$00,$00,$00,$00,$00	; indexes 10-17
db $00,$00,$00,$00,$00,$00,$00,$00	; indexes 18-1F

SpriteStatus:
db $01,$01,$01,$01,$01,$01,$01,$01	; indexes 00-07
db $01,$01,$01,$01,$01,$01,$01,$01	; indexes 08-0F
db $01,$01,$01,$01,$01,$01,$01,$01	; indexes 10-17
db $01,$01,$01,$01,$01,$01,$01,$01	; indexes 18-1F

ExtraBits:				; only relevant when spawning a custom sprite
db $08,$08,$08,$08,$08,$08,$08,$08	; indexes 00-07
db $08,$08,$08,$08,$08,$08,$08,$08	; indexes 08-0F
db $08,$08,$08,$08,$08,$08,$08,$08	; indexes 10-17
db $08,$08,$08,$08,$08,$08,$08,$08	; indexes 18-1F

XSpeedTable:		; the X speeds when spawning 5 sprite
db $E0,$F0,$00,$10,$20	; with non-random speeds
!YSpeed = $C0		; the normal Y speed
!XSpeed = $10		; the normal X speed
!UpperBoundX = $29	; one more than the maximum random X speed
!LowerBoundX = $04	; the minimum random X speed
!UpperBoundY = $38	; the maximum random Y speed
!LowerBoundY = $59	; one more than the minimum random Y speed
; Note on the Y speeds: These are negative values, so they are INVERSED.
; Therefore, the lower boundary should actually be HIGHER then the upper one.
; Think of it this way: What would the values be if you set bit 7 of them both,
; i.e. ORA #$80?
!Sound = $02		; the sound effect to play when the sprite spawns
!SoundBank = $1DFC	; the sound effect to use

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	print "INIT ",pc
	LDA $7FAB10,x
	AND #$04
	BNE UseProp2
	LDA $7FAB28,x
	BRA StoreProp
	UseProp2:
	LDA $7FAB34,x
	StoreProp:
	STA $1510,x
	RTL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite main routine JSR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR InvisibleSpriteMain
	PLB
	RTL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite main routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

InvisibleSpriteMain:

JSR GetDrawInfo	; not sure what the purpose of this is...offscreen checking?
JSL $01A7DC	; is the sprite in contact with the player?
BCC Return0	; if not, return

LDA $1510,x	; check our property byte
AND #$1F	; get the index bits
STA $06		;
TAY		; index into Y
LDA $1510,x	; reload the property byte
BMI SpawnCustom	; if bit 7 is set, then spawn a custom sprite

SpawnNormal:	;

AND #$20	; if bit 5 is set, spawn five sprites
BNE Spawn5Normal	;

LDA SpriteNumber,y	; set the sprite number
STA $9E,x		;
LDA SpriteStatus,y	; set the sprite status
STA $14C8,x	;

JSL $07F7D2	; reset sprite tables

JSR Shared1	; some shared code

Return0:
RTS

Spawn5Normal:	;

LDA #$04		; 5 sprites to spawn
STA $1594,x	;

SpawnLoop1:

JSL $02A9E4
BMI ReturnLoop1

LDA #$20
STA $154C,y

PHX
TYX
LDY $06
LDA SpriteNumber,y
STA $9E,x
LDA SpriteStatus,y
STA $14C8,x
TXY
PLX

JSR Shared3

PHX
TYX
JSL $07F7D2
TXY		;
JSR Shared2	;
PLX
DEC $1594,x

BPL SpawnLoop1

LDA #!Sound
STA !SoundBank

ReturnLoop1:
STZ $14C8,x
Return1:
RTS

SpawnCustom:	;

STA $7FB000

AND #$20	; if bit 5 is set, spawn five sprites
BNE Spawn5Custom	;

STZ $14C8,x

JSL $02A9E4
BMI Return1

TYX

STA $7FB001

LDY $06
LDA SpriteNumber,y	; set the sprite number
STA $7FAB9E,x	;
LDA SpriteStatus,y	; set the sprite status
STA $14C8,x	;

JSL $07F7D2	; reset sprite tables
JSL $0187A7	; get table values for a custom sprite
LDY $06		;
LDA ExtraBits,y	;
STA $7FAB10,x	;

JSR Shared1	; some shared code

LDX $15E9

RTS

Spawn5Custom:	;

STA $7FB002

LDA #$04		; 5 sprites to spawn
STA $1594,x	;

SpawnLoop2:

JSL $02A9E4
BMI ReturnLoop2

LDA #$20
STA $154C,y

PHX
TYX
LDY $06
LDA SpriteNumber,y
STA $7FAB9E,x
LDA SpriteStatus,y
STA $14C8,x
TXY
PLX

JSR Shared3

PHX
TYX
JSL $07F7D2
JSL $0187A7
LDY $06
LDA ExtraBits,y
STA $7FAB10,x

TXY		;
JSR Shared2
PLX
DEC $1594,x
BPL SpawnLoop2

LDA #!Sound
STA !SoundBank

ReturnLoop2:

STZ $14C8,x
RTS

Shared1:

LDA #$20		; set the timer
STA $154C,x	; to disable interaction with the player

LDA $D8,x	; base Y position
SEC		;
SBC #$0F		; move the new sprite up a few pixels
STA $D8,x	;
LDA $14D4,x	;
SBC #$00		; handle the high byte to prevent overflow
STA $14D4,x	;

LDA #$00		;
LDY $7B		; set the new sprite's direction depending on the player's speed
BPL SpawnRight	; if the player is moving left...
INC		; then the sprite direction is 01,
SpawnRight:	; else, the sprite direction is 00
STA $157C,x	;

STZ $B6,x		; zero out the X speed first
LDA $1510,x	; check the properties
AND #$40	; if bit 6 is set...
BNE NoXSpeed1	; don't give the sprite any X speed
LDA #!XSpeed	; load the base X speed
LDY $157C,x	; if the sprite is facing left...
BEQ NoInvertX1	;
EOR #$FF		; invert the X speed value
INC		;
NoInvertX1:	;
STA $B6,x		;
NoXSpeed1:	;
LDA #!YSpeed	;
STA $AA,x	; set the new sprite Y speed
LDA #!Sound	;
STA !SoundBank	; play a sound effect

RTS

Shared2:

LDX $15E9
LDA $1510,x
AND #$40
BEQ RandomGen1

LDA $1594,x
TYX
TAY
LDA XSpeedTable,y
STA $B6,x
LDA #!YSpeed
STA $AA,x
TXY
LDX $15E9
BRA End2

RandomGen1:	;
JSL $01ACF9	; random number generator
AND #$7F	;
CMP #!UpperBoundX	; if the result is too high...
BCS RandomGen1	;
CMP #!LowerBoundX	; or too low...
BCC RandomGen1	; try again
STA $00B6,y	; if not, store this to the sprite's X speed table

RandomGen2:	;
JSL $01ACF9	; random number generator
AND #$7F	;
CMP #!UpperBoundY	; if the result is too high...
BCC RandomGen2	;
CMP #!LowerBoundY	; or too low...
BCS RandomGen2	; try again
ORA #$80		; set bit 7 so that the value is always negative (and the sprite goes up)
STA $00AA,y	; if not, store this to the sprite's Y speed table

End2:
RTS

Shared3:

LDA $E4,x
STA $00E4,y
LDA $14E0,x
STA $14E0,y
LDA $D8,x
SEC
SBC #$0F
STA $00D8,y
LDA $14D4,x
SBC #$00
STA $14D4,y

RTS

Table1:              db $0C,$1C
Table2:              db $01,$02

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS
