;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Punchy
;  Punchy will shoot Mario up and to the desired direction.
;  This sprite uses the Extra bit to determine in which direction
;  it will Punch Mario.
;  To make the sprite behave in different ways, tiles are used.
;
; Made by Mandew.
;  Questions? suggestions? Drop me a Private Message on
;   on Super Mario World Central or raocow's talkhaus.
;	http://talkhaus.raocow.com/memberlist.php?mode=viewprofile&u=1092
;	http://www.smwcentral.net/?p=profile&id=565
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;	Defines
;	  When programming, I don't want to worry about RAM addresses or values as I am doing so.
;         This is why all these defines are here.
; 	  I just fill them in later.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; [Customizable Defines] (Change these if you need to)

   ; Graphics:

     ; Pose 0
	!Tile0 = #$00
	!Tile1 = #$00
	!Property0 = #$01
	!Property1 = #$41

   ; Behavior
	!DefaultBehavior = #$01		; Change this to select a default behavior.
				; By default:
				; #$00 - don't use this
				; #$01 - standing still
				; #$02 - going up and down
				; #$03 - following Mario up and down
				; The rest of the values crash the game.
				; Any suggestion on behaviors to add?
				;  Drop me (Mandew) a Personal Message on Super Mario World Central
				;  or on raocow's talkhaus.

				; TO-DO - add Phanto behavior in future update.

; [Used with other components] (AKA probably don't change these.

   ; Behavior
	!Behavior = $C2
	!SpriteStage = $1528
   ; Misc
	!ShootMarioYSpeed = #$B0
	!ShootSpriteYSpeed = #$C0

; [Predetermined by the engine] (AKA definitely don't change these)

   ; Sprite Number defines
	!Springboard = #$2F

   ; Position defines.
	!XPosLo = $E4
	!XPosHi = $14E0
	!YPosLo = $D8
	!YPosHi = $14D4
	!XSpeed = $B6
	!YSpeed = $AA
	!XDirection = $157C
	!YDirection = $151C
	!MarioY = $D3
	!MarioXSpeed = $7B
	!MarioYSpeed = $7D

   ; Sprite Status defines
	!SprObj = $1588
	!Status = $14C8
	!ExtraBits = $7FAB10
	!SpriteNumber = $9E
   ; OAM
	!OAM_X = $0300 
	!OAM_Y = $0301
	!OAM_Tile = $0302
	!OAM_Properties = $0303
	!TableProperties = $15F6

   ; Subroutines from the game.
	!GoToPointer = $0086DF
	!FinishOAMWrite = $01B7B3
	!MarioInteractions = $01A7DC
	!SpriteClippingA = $03B69F
	!SpriteClippingB = $03B6E5
	!CheckSpriteContact = $03B72B
	!UpdateYPosNoGrav = $01801A
	!ObjectInteract = $019138

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	;INC !YPosLo,x

	LDA #$01
	STA !YDirection,x

	STZ !XDirection,x
	STZ !Behavior,x
	LDA !ExtraBits,x
	AND #$04
	BEQ ReturnInit

	LDA #$01
	STA !XDirection,x

ReturnInit:
	RTL

	print "MAIN ",pc
		PHB
		PHK
		PLB
		JSR MainCode	; Jump to Main Code.
		PLB
	RTL




; 
;------------------------------------------------------------------------------------------
; MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE
;------------------------------------------------------------------------------------------


; Main code 
;  This section handles sending Punchy through the appropriate routines:
;  - Sends the sprite to the Graphics drawing routine.
;  - Sends the sprite to the Mario/Sprite interaction routine.
;  - Sends the sprite to the appropriate behavior-specific routines.
;				[MAIN CODE]

MainCode:
	JSR DrawGraphics
	JSR Interactions 	; How the sprite interacts with Mario and other sprites.
				; Go to [Help: Interactions] if you want more details.

	LDA !Behavior,x		; This will make the sprite go to a behavior specified below.
	JSL !GoToPointer	; 

BehaviorPointers:
	dw Behavior_Undefined
	dw Behavior_StandStill
	dw Behavior_UpAndDown	
	dw Behavior_FollowMario

		; [Fun Fact: BehaviorPointers]
		;  BehaviorPointers is actually a dummy label.
		;   It doesn't actually do anything except make the code more readable,
		;   so don't go looking for that label elsewhere in the code!


;------------------------------------------------------------------------------------------
; MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE MAIN CODE
;------------------------------------------------------------------------------------------












;------------------------------------------------------------------------------------------
; BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS
;------------------------------------------------------------------------------------------

; The specific behaviors are below.
;  The behavior that the sprite will use is determined by !Behavior,x ($C4,x by default)
;   The sprite does not decide what $C4,x is on its own. Instead, you may place one of the provided blocks
;   under the sprite.
;
;	--- Behavior_Undefined
;		On the first frame Punchy is active, it will make contact with the block it's placed on.
;		If it does not find anything, it will default to !DefaultBehavior
;		Feel free to redefine !DefaultBehavior to what suits your needs.
;
; 	--- Behavior_StandStill
;		This is the most complex behavior ever. You can't even understand.
;
;	--- Behavior_UpAndDown
;		The Sprite will move up until it comes in contact with a solid object,
;		and then move down until it comes in contact with a solid object,
;		and then move up until it comes in contact with a solid object,
;		and then move down until [... ... ...]
;
;
;  You may add your own if you so desire.
;	[If you have a suggestion on how to improve this sprite, please contact me (Mandew)	]
;	[through Private Messages on Super Mario World Central or raocow's talkhaus.		]
;				[BEHAVIORS]


;++++++++++++++++++++++
; Behavior_Undefined
;++++++++++++++++++++++

Behavior_Undefined:
	LDA !DefaultBehavior
	STA !Behavior,x

	LDA !YPosLo,x
	SEC
	SBC #$01
	STA !YPosLo,x
	LDA !YPosHi,x
	SBC #$00
	STA !YPosHi,x
	JSL !ObjectInteract
	RTS


;++++++++++++++++++++++
; Behavior_StandStill
;++++++++++++++++++++++

Behavior_StandStill:
	RTS

;++++++++++++++++++++++
; Behavior_UpAndDown
;++++++++++++++++++++++

UpAndDown_Speed: 		
 db $10,$F0			; These are the speeds at which Punchy goes up and down on this behavior.	

Behavior_UpAndDown:
	JSL !ObjectInteract	;
	LDY !YDirection,x	; Load the Vertical Direction into the Y Register.
	BNE GoingUp		; 

 GoingDown:			; Detects solid objects according to vertical directions.
	LDA !SprObj,x		; Down = Detect floor.
	AND #$04		;
	BEQ SetSpeed		;
	BRA InvertDirection	;
				;
 GoingUp:			; Up = Detect ceiling
	LDA !SprObj,x		;
	AND #$08		;  
	BEQ SetSpeed		;  
 
 InvertDirection:		; If a floor/ceiling was detected,
	TYA			;  this inverts the sprite's direction.
  	EOR #$01		;
	STA !YDirection,x	;
	TAY			;

 SetSpeed:			; The speed is set accordingly to the sprite's vertical direction.
	LDA UpAndDown_Speed,y	;  
	STA !YSpeed,x		;
	JSL !UpdateYPosNoGrav ; $01801A

	RTS

;++++++++++++++++++++++++
; Behavior_FollowMario
;++++++++++++++++++++++++


FollowMarioSpeed:
	db $00,$00,$04,$FC,$08,$F8,$0C,$F4
	db $10,$F0,$14,$EC,$18,$E8,$1C,$E4

Behavior_FollowMario:			;
	JSR GetYMarioDifference		; > Get the difference between Mario and the sprite.
	LDA $01
	BPL NotNegative
	
	EOR #$FF
	INC
NotNegative:
	CMP #$07
	BMI NotCap
	LDA #$07
NotCap:	
	ASL
	CLC
	ADC $00
	TAY
	LDA FollowMarioSpeed,y
	STA !YSpeed,x

	JSL !ObjectInteract
	LDA !SprObj,x
	BIT #$08
	BEQ TestDown
	STZ !YSpeed,x
TestDown:
	BIT #$04
	BEQ OK
	STZ !YSpeed,x
OK:
	JSL !UpdateYPosNoGrav


ReturnFollow:
	RTS

		; Fun Fact: The Carry flag.
			; When the result of an addition is superior to #$FF, the carry flag is set.
			; When the result of a substraction is inferior to #$00, the carry flag is cleared.
			; When the carry flag is set, one more is added to additions.
			; When the carry flag is clear, one more is subtracted to substractions.

			; The carry flag carries over to the next math operation.
			; This allows to effectively use the carry to handle a Hi byte
			; after handling a low byte. This is why ADC #$00 is necessary.



;------------------------------------------------------------------------------------------
; BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS BEHAVIORS
;------------------------------------------------------------------------------------------













;hi there







;------------------------------------------------------------------------------------------
; SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES
;------------------------------------------------------------------------------------------

	; Not to be confused with Generics.
	; 
	; List of subroutines:
	; --- GetYMarioDifference
	;	This subroutine detects the difference in height between Mario
	;	and this sprite.
	; --- Interactions:
	; 	How the sprite interacts with other sprites.
	;	This is the bulk of this sprite's code.
	; --- CheckElligible:
	;	Checks if a sprite is eligible to get punched.
	; --- DrawGraphics:
	; 	Handles drawing graphics.
	;			[SUBROUTINES]

;++++++++++++++++++++++
; GetYMarioDifference
;++++++++++++++++++++++

GetYMarioDifference: 		;
	LDA !YPosLo,x		; Using Scratch RAM addresses,
	STA $01			;  calculate the difference between Mario and this sprite
	LDA !YPosHi,x		;  in 16-bit mode.
	STA $02			;
	REP #$20		;
	LDA #$0010
	CLC
	ADC !MarioY		;
	SEC			;
	SBC $01			;
	STA $01			;

	BMI Negative
Positive:
	SEP #$20
	STZ $00
	RTS

Negative:
	SEP #$20
	LDA #$01
	STA $00
	RTS

;++++++++++++++++++++++
; Interactions
;++++++++++++++++++++++

ShootMarioXSpeed:
db 	$CF,$30
ShootSpriteXSpeed:
db 	$D7,$28


Interactions:
	LDA !SpriteStage,x	;
	BEQ Stage_idle		; Depending on which stage the sprite is currently through,
	CMP #$02		; The interactions are handled differently.
	BMI Stage_WindUp	;
	BRA Stage_Punch		;

;***************
; Stage_idle
;***************

; When the sprite is idle, it will try and detect if something it can punch is close by.
; Doing that will trigger its animation.
; The subroutine "CheckElligible" is used to determine what can trigger it.

Stage_idle:
	JSL !MarioInteractions
	BCC EnemyTrigger
	INC !SpriteStage,x
	RTS

EnemyTrigger:
	JSL !SpriteClippingA
	PHX

	LDX #$0C
	What:
	 DEX
	 JSR CheckElligible
	 BCC NoTrigger
	IsTriggerable:
	 JSL !SpriteClippingB 	
	 JSL !CheckSpriteContact
	 BCC NoTrigger
	 PLX	
	 INC !SpriteStage,x
	 RTS

	NoTrigger:
	 CPX #$00
	 BNE What

	 PLX	
	 RTS

;****************
; Stage_WindUp
;****************

	; For a couple of frames after getting triggered,
	; Punchy winds up.
	; It does nothing except progress the animation.

Stage_WindUp:
	INC !SpriteStage,x
	RTS

;***************
; Stage_Punch
;***************

	; Punchy will detect and interact with things it can punch.
	; The subroutine "CheckElligible" is used to determine what it can punch.

Stage_Punch:
	JSL !MarioInteractions
	BCC EnemyPunch
	LDA !XDirection,x
	TAY
	LDA ShootMarioXSpeed,y
	STA !MarioXSpeed
	LDA !ShootMarioYSpeed
	STA !MarioYSpeed
	BRA EnemyPunch

EnemyPunch:

	JSL !SpriteClippingA	; $03B69F
	LDA !XDirection,x
	TAY
	PHX

	LDX #$0C
	XLoop:
	 DEX
	 JSR CheckElligible
	 BCC NoContact
	IsViable:
	 PHY
	 JSL !SpriteClippingB 	 ;$03B6E5
	 JSL !CheckSpriteContact ;$03B72B
	 PLY
	 BCC NoContact
	
 	 LDA.l ShootSpriteXSpeed,y
	 STA !XSpeed,x

	 LDA !ShootSpriteYSpeed
	 STA !YSpeed,x

	 LDA !Status,x
	 CMP #$09
	 BNE DoneShoving
	 LDA #$0A
	 STA !Status,x

	DoneShoving:
	NoContact:
	 CPX #$00
	 BNE XLoop
	 PLX
	
CheckTimer:
	INC !SpriteStage,x 
	LDA !SpriteStage,x
	CMP #$07
	BMI NoTimerReset
	STZ !SpriteStage,x
NoTimerReset:
	RTS


;++++++++++++++++++++++
; CheckElligible
;++++++++++++++++++++++

	; Checks sprites to see if they are elligible.

CheckElligible:
	LDA !Status,x		; Section 1 : Check status.
	CMP #$09		; The sprite is considered elligible
	BEQ ElligibleOK		; if it is stunned, thrown,
	CMP #$0A		; or a sprite that passed the goal tape.
	BEQ ElligibleOK
	CMP #$0C
	BEQ ElligibleOK

	LDA !SpriteNumber,x	; Section 2: Vanilla sprites.
	CMP !Springboard	; Since the springobard is not a 'stunned sprite'
	BEQ ElligibleOK		; unlike many others, it gets an exception here
	CLC			; to be considered elligible.
	RTS

ElligibleOK:
	SEC
	RTS

;++++++++++++++++++++++
; DrawGraphics
;++++++++++++++++++++++

	; Just your standard graphics drawing routine.

TileMap: 
db	!Tile0,!Tile0
Properties: 
db	!Property0,!Property1
XDisplacement:
db	$10,$F0,$14,$EC,$18,$E8,$14,$EC,$10,$F0
db	$08,$F8,$00,$00,$04,$FC,$08,$F8,$0C,$FC

DrawGraphics:
	PHX
	LDA !SpriteStage,x
	ASL
	CLC
	ADC !XDirection,x
	TAX
	LDA XDisplacement,x
	STA $0A
	PLX

	JSR SubOffscreen0Bnk2
	JSR GET_DRAW_INFO

	LDA !TableProperties,x
	STA $0B

	PHX
	LDA !XDirection,x
	TAX

	LDA $00
	CLC
	ADC $0A
	STA !OAM_X,y
	
	LDA $01
	STA !OAM_Y,y

	LDA TileMap,x
	STA !OAM_Tile,y

	LDA Properties,x
	ORA $0B
	ORA $64
	STA !OAM_Properties,y

	PLX
        LDA #$01                
        LDY #$02             
        JSL !FinishOAMWrite 	
	RTS


;------------------------------------------------------------------------------------------
; SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES SUBROUTINES
;------------------------------------------------------------------------------------------








;------------------------------------------------------------------------------------------
; Below are generic subroutines.
;------------------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get Draw Info
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TABLE1:             db $0C,$1C
TABLE2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC #$0040            ;  |
                    CMP #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite Graphics Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DATA_02D003:        db $40,$B0
DATA_02D005:        db $01,$FF
DATA_02D007:        db $30,$C0,$A0,$C0,$A0,$70,$60,$B0
DATA_02D00F:        db $01,$FF,$01,$FF,$01,$FF,$01,$FF


SubOffscreen0Bnk2:  STZ $03                   ; / 
                    JSR IsSprOffScreenBnk2  ; \ if sprite is not off screen, return 
                    BEQ Return02D090          ; / 
                    LDA $5B     ; \  vertical level 
                    AND #$01                ;  | 
                    BNE VerticalLevelBnk2     ; / 
                    LDA $03                   
                    CMP #$04                
                    BEQ CODE_02D04D           
                    LDA $D8,X       ; \ 
                    CLC                       ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level... 
                    LDA $14D4,X     ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2) 
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL OffScrEraseSprBnk2    ; /    ...erase the sprite 
                    LDA $167A,X   ; \ if "process offscreen" flag is set, return 
                    AND #$04                ;  | 
                    BNE Return02D090          ; / 
CODE_02D04D:        LDA $13      
                    AND #$01                
                    ORA $03                   
                    STA $01                   
                    TAY                       
                    LDA $1A    
                    CLC                       
                    ADC DATA_02D007,Y       
                    ROL $00                   
                    CMP $E4,X       
                    PHP                       
                    LDA $1B    
                    LSR $00                   
                    ADC DATA_02D00F,Y       
                    PLP                       
                    SBC $14E0,X     
                    STA $00                   
                    LSR $01                   
                    BCC CODE_02D076           
                    EOR #$80                
                    STA $00                   
CODE_02D076:        LDA $00                   
                    BPL Return02D090          
OffScrEraseSprBnk2: LDA $14C8,X             ; \ If sprite status < 8, permanently erase sprite 
                    CMP #$08                ;  | 
                    BCC OffScrKillSprBnk2     ; / 
                    LDY $161A,X ; \ Branch if should permanently erase sprite 
                    CPY #$FF                ;  | 
                    BEQ OffScrKillSprBnk2     ; / 
                    LDA #$00                ; \ Allow sprite to be reloaded by level loading routine 
                    STA $1938,Y ; / 
OffScrKillSprBnk2:  STZ $14C8,X             ; Erase sprite 
Return02D090:       RTS                       ; Return 

VerticalLevelBnk2:  LDA $167A,X   ; \ If "process offscreen" flag is set, return 
                    AND #$04                ;  | 
                    BNE Return02D090          ; / 
                    LDA $13      ; \ Return every other frame 
                    LSR                       ;  | 
                    BCS Return02D090          ; / 
                    AND #$01                
                    STA $01                   
                    TAY                       
                    LDA $1C    
                    CLC                       
                    ADC DATA_02D003,Y       
                    ROL $00                   
                    CMP $D8,X       
                    PHP                       
                    LDA $1D  
                    LSR $00                   
                    ADC DATA_02D005,Y       
                    PLP                       
                    SBC $14D4,X     
                    STA $00                   
                    LDY $01                   
                    BEQ CODE_02D0C3           
                    EOR #$80                
                    STA $00                   
CODE_02D0C3:        LDA $00                   
                    BPL Return02D090          
                    BMI OffScrEraseSprBnk2    
IsSprOffScreenBnk2: LDA $15A0,X 
                    ORA $186C,X 
                    RTS                       ; Return 
