;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  
;; A Robot that ejects a small version of itself
;;   When Mario gets within the range specified below, it will stop moving and start flashing
;;   red, and explodes after a little while.
;;   Before doing so however, it ejects a small copy of itself that turns back into a full-sized
;;   robot upon landing.
;;
;;  by leod
;;  
;;  
;;  I highly recommend editing the defines below, as the sprite doesn't pose a big threat with its
;;  current fuse timers etc., though that may actually be what you want.
;;
;;  Included graphics replace unused tiles in the forest sprite file, the one with the wigglers,
;;  Lakitu, Spinies etc.
;;  All the tiles it uses can be adjusted at the top of the "SubGFX" graphics routine,
;;  fairly far down the file.
;;
;;  Uses 3 (vanilla) palettes: 9 (grey), A (yellow) and C (red)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!EjectX = $14
!EjectY = $B8
;X and Y speeds that the mini is shot out at

!Range = $0040
;how near mario has to be to the sprite for it to switch to blow-up mode (in pixels)

!Countdown = $60
;how long to wait and blink before actually exploding (in frames)
;note: max is $60

!ShootFrame = $20
;how many frames before the explosion the new robot should be should out
;note: obviously has to be less than !Countdown

!CooldownTime = $20
;how long ejected robots wait before checking for mario again (in frames)



!SP = $01
;which sprite page the graphics use
;SP1/2 = $00,  SP3/4 = $01

!HelmetPalette = $0A
;yellow by default

!BodyPalette = $09
;grey by default

!WheelPalette = $0A
;yellow by default

!TinyWheelPalette = $0C
;red by default
;this is only used in the mini version, so the wheels dont just look like yellow specks




;don't change these defines. or do, but it won't help much, they're just random sprite tables
!Size    = $1594       ;0 = sprite is normal size, 1 = sprite is small

!BlowCap = $151C       ;0 = helmet is closed, 1 = helmet open

!Dir     = $1528       ;direction the sprite is facing
                       ;00 = right
                       ;01 = left
                       
!GoalSpeed = $C2       ;speed sprite should aim towards

!Stop = $1602          ;if the sprite should stop or not

!Color = $1510         ;current body color
                       ;00 = grey
                       ;01 = red

!Cooldown  = $163E     ;decreases, how long to wait till the freshly made robot checks for mario again


!Timer  = $15AC        ;timer used in a lot of states (decreases itself)
!State  = $187B        ;sprite state index, values are as follows:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
STZ !Color,x
STZ !BlowCap,x
STZ !Cooldown,x

JSR SubHorzPos
TYA
EOR #$01
STA !Dir,x
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
print "MAIN ",pc
PHB                     ; \
PHK                     ;  | The same old
PLB                     ; /
JSR MainSprtRt          ;  Jump to sprite code
PLB                     ; Yep
RTL                     ; Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Interact:
db $02,$04
;x pos
db $02,$04
;y pos
db $0C,$08
;width
db $0E,$08
;height


  Return1:
RTS

  MainSprtRt:
JSR SubGFX          ;render graphics

LDA $9D             ;if sprites are locked, return
BNE Return1

JSR SubOffScreen_X3	;handle off screen situation
JSL $019138         ;interact with objects

LDA $14C8,x
CMP #$08
BEQ .DoStuff

RTS                ;if sprite isn't in normal state, don't even bother with any of this

  .DoStuff
;; interaction with mario
JSL $03B664           ;fill in mario clipping into slot B

;fill robot into slot A
LDA !Size,x
STA $0E

LDX $0E
LDA Interact,x  ;load x offset
LDX $15E9
CLC
ADC $E4,x       ;x low
STA $04
LDA $14E0,x     ;x hi
ADC #$00
STA $0A


LDX $0E
LDA Interact+2,x;load y offset
LDX $15E9
CLC
ADC $D8,x       ;y low
STA $05
LDA $14D4,x     ;y hi
ADC #$00
STA $0B


LDX $0E
LDA Interact+4,x
STA $06
LDA Interact+6,x
STA $07
LDX $15E9


JSL $03B72B     ;check if the filled in objects intersect
BCS +
JMP .NoContact
+

LDA $1490
ORA $13ED       ;if mario has a star or is sliding
BEQ .NoStar

JSR Score3
  
  .Die
;smoke effect
LDA #$01
STA $0D
LDA #$1C
STA $0C
STZ $0B
JSR Smoke

STZ $14C8,x           ;kill the sprite for real
BRA .NoContact        ;return after killing the sprite either way

  .NoStar
LDA $7D               ;check if hes falling
BPL +
BRA .HurtMario
+

;move up mario's hitbox by his speed to check for top contact
LDA $01
SEC
SBC $7D
STA $01
LDA $09
SBC #$00
STA $09
JSL $03B72B           ;check if the filled in objects intersect
BCC +
BRA .HurtMario        ;if yes, mario isnt on top, so hurt him
+

STZ $7D

LDA $140D
BNE .SpinJump         ;if spinjumping, go do that

JSL $01AA33           ;set mario speed
LDA $187A             ;if on yoshi, also go spinjumping but use the normal mario jump routine
BEQ .NormalJump


  .SpinJump
JSR Score             ;display score image and play sound
LDA #$1F              ;set spin jump animation timer
STA $1540,x
JSL $07FC3B           ;show star animation
LDA #$08              ;play sound effect
STA $1DF9             ;(overwrites score sound)

BRA .Die              ;kill sprite too


  .NormalJump
JSL $01AB99           ;display contact graphic
JSR Score             ;display score image and play sound


LDA !Size,x
BNE .Die              ;if it's small, disappear in smoke

LDA !State,x
CMP #$02
BNE .FallNormal

JMP Flash_explode

  .FallNormal
;otherwise fall like normal
LDA #$02
STA $14C8,x

STZ $B6,x
STZ $AA,x
BRA .NoContact



  .HurtMario
LDA $1497           ;if mario is invincible
BNE .NoContact      ;then skip

LDA $187A
BEQ .DoHurt         ;skip this if mario isnt riding yoshi

JSR LoseYoshi

BRA .NoContact

  .DoHurt
JSL $00F5B7         ;hurt mario


  .NoContact
;; KICKED SPRITE INTERACTION
LDY #$0B          ;initiate loop
  .KickLoop

LDA $14C8,y
CMP #$0A          ;if state is not kicked or carried, skip
BCC .next

LDA $009E,y       ;dont interact with springs etc (blacklist because of custom sprites)
CMP #$2F          ;spring
BEQ .next
CMP #$3E          ;switch
BEQ .next
CMP #$2D          ;baby yoshi
BEQ .next
CMP #$80          ;key
BEQ .next

TYX
JSL $03B6E5       ;get that sprite's clip values
LDX $15E9              
JSL $03B72B       ;and check if they intersect (mole is in slot A from mario interaction)
BCC .next         ;if they dont, skip

LDA $14C8,y       ;seems redundant, but the former check was so we dont constantly do the clipping routine
CMP #$0A
BEQ .NoKillSprite

LDA #$02          ;kill sprite
STA $14C8,y

LDA #$DE          ;throw it up a tad
STA $00AA,y

  .NoKillSprite
;we dont kill the kicked sprite, but we kill mole
JSR Score2        ;shell kill score

LDX $15E9
LDA #$02          ;kill sprite
STA $14C8,x

LDA #$DE          ;throw it up a tad
STA $AA,x
RTS               ;break loop and stop the sprite code, sprite only needs to die once after all

  .next
DEY
BPL .KickLoop
  .NoKicked

;; actual sprite behaviour
LDA !Size,x
BEQ BigBehaviour


SmallBehaviour:
LDA $1588,x
AND #$04
BNE .SpawnBig
LDA $1588,x
AND #$08
BEQ .NoCeiling

LDA #$08            ;move down if head is hit
STA $AA,x
LDA #$01            ;hit head sound
STA $1DF9

  .NoCeiling
LDA $1588,x
AND #$03
BEQ .NoWall

LDA $B6,x
EOR #$FF            ;invert x speed if wall is hit
INC
STA $B6,x
LDA #$01            ;hit head sound
STA $1DF9

  .NoWall
JSL $01802A         ;update Y with gravity
RTS


  .SpawnBig   ;spawn the big version of this sprite
LDA #$01
STA $0D
LDA #$1C
STA $0C
STZ $0B
JSR Smoke

LDA #$03            ;shellkick sound for "plopping" out of the mini
STA $1DF9

LDA #$10            ;make edible
STA $1686,x

LDA #!CooldownTime
STA !Cooldown,x

STZ !Size,x
STZ $AA,x           ;no y speed
STZ $B6,x           ;no x speed
STZ !Color,x
RTS







BigBehaviour:
;; turning around when level demands it
LDA $1588,x
AND #$03
BEQ .NoTurn

  .Turn
LDA !Dir,x
EOR #$01    ;switch direction
STA !Dir,x

LDA $B6,x   ;switch speeds
EOR #$FF
INC
STA $B6,x

  .NoTurn
STZ !GoalSpeed,x
LDA !Stop,x
BNE .NoMoving

LDA !Dir,x
TAX
LDA GoalSpeeds,x
LDX $15E9
STA !GoalSpeed,x
  .NoMoving

;; move towards goal speed
LDA $14
AND #$03
BNE .XSpeedAchieved

LDA $B6,x
SEC
SBC !GoalSpeed,x      ;substract goal from current
BEQ .XSpeedAchieved    ;if equal, gottem
BMI .INCXSpeed         ;if not, adjust

DEC $B6,x
BRA .XSpeedAchieved

  .INCXSpeed
INC $B6,x

  .XSpeedAchieved
;; slope and hole behaviour 
LDA $1588,x
AND #$04
BEQ .Fall

STZ $AA,x
BRA .YSpeedAchieved

  .Fall
LDA $AA,x
SEC
SBC #$40               ;substract goal from current
BEQ .YSpeedAchieved    ;if equal, gottem
BMI .INCYSpeed         ;if not, adjust

DEC $AA,x
DEC $AA,x
BRA .YSpeedAchieved

  .INCYSpeed
INC $AA,x
INC $AA,x

  .YSpeedAchieved
JSL $01801A         ;update x
JSL $018022         ;update y

LDA !State,x
ASL
TAX
JMP.w (BigStates,x)

GoalSpeeds:
db $FB,$05

BigStates:
dw Drive,Wait,Flash


Drive:
LDX $15E9

LDA !Cooldown,x       ;dont check shit while cooldown isnt up
BNE .MarioIsBelow

LDA $14D4,x
XBA
LDA $D8,x
REP #$20
CMP $96
SEP #$20
BCC .MarioIsBelow     ;if mario is below the sprite, dont ever flash

REP #$20
SEC
SBC $96             ;sprite y - mario y
BPL .NoInvertY

EOR #$FFFF          ;invert if needed
INC

  .NoInvertY
LSR                 ;double the range upwards

CMP #!Range         ;check just how close mario is
SEP #$20
BCS .MarioIsBelow

LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $94             ;sprite x - mario x
BPL .NoInvertX

EOR #$FFFF          ;invert if needed
INC

  .NoInvertX
CMP #!Range         ;check just how close mario is
SEP #$20
BCC .next

  .MarioIsBelow
RTS



.next
INC !State,x        ;start flashing
INC !Stop,x         ;make it stop moving

LDA #$20
STA !Timer,x

;also turn around to face mario
JSR SubHorzPos
TYA
EOR #$01
STA !Dir,x
RTS



Wait:
LDX $15E9

LDA !Timer,x
BEQ .next
RTS

.next
INC !State,x

LDA.b #!Countdown
STA !Timer,x
RTS




Flash:
LDX $15E9

LDA $14
AND #$0F
BNE .NoTurn

LDA !Timer,x
CMP #!ShootFrame
BCC .NoTurn

JSR SubHorzPos
TYA
EOR #$01
STA !Dir,x

  .NoTurn
LDA !Timer,x
BEQ .explode
CMP #!ShootFrame
BNE .NoShoot
JSR ShootMini
  .NoShoot
LSR : LSR
TAX
LDA .FlashTable,x
LDX $15E9
STA !Color,x
RTS

.FlashTable
db $01,$01,$00,$01 : db $00,$01,$00,$01
db $00,$01,$00,$00 : db $01,$00,$00,$01
db $00,$00,$01,$01 : db $00,$00,$01,$01


.explode      ;explode
LDA #$15
STA $1887	        ;shake the ground

LDA #$0D
STA $9E,x	        ;set sprite to bob-omb

LDA #$08	        ;active state
STA $14C8,x

JSL $07F7D2	      ;reset sprite tables

LDA #$01		      ;explosion flag
STA $1534,x

LDA #$40		      ;timer
STA $1540,x

LDA #$1B          ;make invincible to mostly everything mario could kill this with
STA $167A,x

LDA #$21          ;make inedible
STA $1686,x

LDA #$09          ;sound effect
STA $1DFC
RTS



SmokeOffs:
db $02,$06

EjectXSpeeds:
db !EjectX^$FF+1,!EjectX


ShootMini:
JSL $02A9DE     ;get an index to an unused sprite slot, return if all slots full
BPL +
JMP .NoSlot     ;after: Y has index of sprite being generated
+

LDA !Dir,x
TAX
LDA SmokeOffs,x
LDX $15E9
STA $0B

LDA #$01            ;init state
STA $14C8,y

LDA $7FAB9E,x
TYX                 ;same sprite number
STA $7FAB9E,x

JSL $07F7D2         ;reset sprite properties

LDA $7FAB10,x
ORA #$08            ;mark the sprite as custom
STA $7FAB10,x

LDA #$01            ;make it small
STA !Size,x

LDA #$11            ;make inedible
STA $1686,x

;positioning
LDX $15E9

LDA $E4,x
CLC
ADC $0B
STA $00E4,y
LDA $14E0,x
ADC #$00
STA $14E0,y

LDA $D8,x
SEC
SBC #$06        ;a little bit above the sprite
STA $00D8,y
LDA $14D4,x
SBC #$00
STA $14D4,y

;speeds
LDA !Dir,x
TAX
LDA EjectXSpeeds,x
LDX $15E9
STA $00B6,y

LDA #!EjectY
STA $00AA,y


;display smoke effect
LDA #$03
STA $0D
LDA #$10
STA $0C
JSR Smoke

LDA #$09           ;bullet bill sound
STA $1DFC

INC !BlowCap,x

  .NoSlot
LDA #!ShootFrame   ;restore frame counter
RTS




;; smoke effect at sprite position
Smoke:
LDA $7F1337
LDA $14E0,x
XBA
LDA $E4,x

REP #$20
CLC
ADC #$0008
CMP $1462
SEP #$20

BCC .Return

REP #$20
SEC
SBC #$0108
BCC .SEPSpawn
CMP $1462
SEP #$20

BCS .Return
;above code just makes sure the sprite is on the screen when this runs

  .Spawn
LDY #$03
  .SmokeLoop
LDA $17C0,y
BEQ .FoundSmoke     ;check if this slot is empty
DEY
BPL .SmokeLoop
RTS

  .FoundSmoke
LDA $0D             ;puff of smoke effect from 0D
STA $17C0,y

LDA $0C             ;smoke timer
STA $17CC,y

LDA $D8,x           ;y pos
STA $17C4,y
LDA $E4,x           ;x pos
CLC
ADC $0B
STA $17C8,y
  .Return
RTS

  .SEPSpawn
SEP #$20
BRA .Spawn


;score and sound routine
ScoreSounds:
db $13,$14,$15,$16,$17,$18,$19
;all the stompy sounds with increasingly higher pitch

Score:
;JSR here for mario-stomp chain
PHY
LDA $1697               ;load number of enemies stomped, THEN
INC $1697               ;increase consecutive enemies stomped
                        ;this is stupid but it's how SMW sprite expect it to be
BRA Score3_Apply

Score2:
;JSR here for kicked-sprite chain
LDA $1626,y             ;load number of enemies kicked by sprite in y, THEN
PHA
INC
STA $1626,y             ;increase that number
PLA
PHY

BRA Score3_Apply

Score3:
;JSR here for star chain
PHY
LDA $18D2               ;load number of enemies starred, THEN
INC $18D2               ;increase that number

  .Apply
TAY
INY
CPY #$08                ;if consecutive enemies stomped >= 8
BCS .1UPSound           ;don't play sound so 1UP sound can play

LDA ScoreSounds-$1,y    ;play according sound effect
STA $1DF9

  .1UPSound
TYA
CMP #$08                ;if consecutive enemies stomped >= 8, reset to 8
BCC .NoOverflow
LDA #$08

  .NoOverflow
JSL $02ACE5             ;give mario score according to number of enemies
PLY
RTS                     ;return



;;lose yoshi routine (originally by chdata, unstupified by leod)
LoseYoshi:
PHX
LDX $18E2		;Yoshi sprite index +1, would be zero if no Yoshi

LDA #$10
STA $163D,x	;

LDA #$03		;turn off Yoshi drums
STA $1DFA
LDA #$13		;play lose-Yoshi
STA $1DFC

LDA #$02		;set Yoshi to running-away state
STA $C1,x

STZ $187A		;reset riding Yoshi flag
STZ $0DC1		;reset "Mario has Yoshi on OW" flag so he can't keep it anyway
            ;(this is how castle sequences memorize that Yoshi exists)

LDA #$C0		;make Mario jump upwards
STA $7D
STZ $7B			;no x movement

JSR SubHorzPos
LDA .SPEED,y;load yoshi speed indexed by direction
STA $B5,x   ;and make him run

STZ $1593,x		;
STZ $151B,x		;
STZ $18AE		  ;

LDA #$30		;Mario invincibility timer
STA $1497

PLX			;restore X (sprite index)
RTS

.SPEED
db $E8,$18	;This is Yoshi's X speed table.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GFX Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;tile defines and tables
BodyTiles:
db $C0,$C1
;the grey body
HatTiles:
db $D0,$D1
;2 opened helmet tiles

!HeadTile  = $EA
!WheelTile = $E0

!TinyShape = $E1
!TinyFeet  = $F0
!TinyFill  = $F1

;;positioning
XOffs:
db $00,$08
;left, right
Flips:
db $00,$40
;no flip, flip

WheelX:
db $07,$01

BodyColors:
db #!BodyPalette-$08<<1|$20|!SP,$29
;specified body palette, red


  SubGFX:
JSR GET_DRAW_INFO
  PHX

LDA !Size,x
BEQ +
JMP SmallTiles
+

;little bobbing motion when "driving"
STZ $0D
LDA $B6,x
BEQ .NoBob

LDA $14
AND #$20
BNE .NoBob
DEC $0D

  .NoBob

;;; front-wheel tile start
LDA !Dir,x
TAX
LDA WheelX,x          ;depending on direction, wheel is left or right of thing
CLC
ADC $00
STA $0300,y

LDA $01
CLC
ADC #$09              ;wheel is at bottom
STA $0301,y

LDA #!WheelTile
STA $0302,y


LDX $15E9
LDA !GoalSpeed,x
LDX #$00
CMP #$00              ;if there's no horizontal movement, don't bother turning the wheels
BEQ .NoWheelMove

LDA $14
LSR : LSR : LSR : LSR
AND #$01
TAX

  .NoWheelMove
LDA Flips,x
ORA.b #!WheelPalette-$08<<1|$20|!SP  ;yellow
STA $0F
STA $0303,y


TYA
LSR A
LSR A
PHY
TAY
LDA #$00            ;8x8
STA $0460,y         ;store to size table
PLY

INY : INY : INY : INY
;;; front-wheel tile end

;;; helmet hole tiles start
LDX #$01
  .HelmetLoop
PHX

LDA $00
CLC
ADC XOffs,x
STA $0300,y

TXA
LDX $15E9
EOR !Dir,x
TAX
LDA HatTiles,x
STA $0302,y

LDX $15E9
LDA !BlowCap,x
BNE .DrawCap

LDA #$E0
BRA .StoreCapY

  .DrawCap
LDA $01
SEC
SBC #$02            ;move up slightly to overlay the helmet
CLC
ADC $0D             ;add bobbing
  .StoreCapY
STA $0301,y


LDA !Dir,x
TAX
LDA Flips,x
ORA.b #!HelmetPalette-$08<<1|$20|!SP            ;yellow
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00            ;8x8
STA $0460,y         ;store to size table
PLY

INY : INY : INY : INY

PLX
DEX
BPL .HelmetLoop
;;; helmet hole tiles end

;;; helmet and face tile start
LDA $00
STA $0300,y

LDA $01
CLC
ADC $0D             ;add bobbing
INC
STA $0301,y

LDA #!HeadTile
STA $0302,y

LDX $15E9
LDA !Dir,x
TAX
LDA Flips,x
ORA.b #!HelmetPalette-$08<<1|$20|!SP            ;yellow
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$02            ;16x16
STA $0460,y         ;store to size table
PLY

INY : INY : INY : INY
;;; helmet and face tile end

;;; body tiles start
LDX #$01
  .BodyLoop
PHX

LDA $00
CLC
ADC XOffs,x
STA $0300,y

LDA $01
CLC
ADC #$09            ;move below the helmet
CLC
ADC $0D             ;add bobbing
STA $0301,y

TXA
LDX $15E9
EOR !Dir,x
TAX
LDA BodyTiles,x
STA $0302,y


LDX $15E9
LDA !Dir,x
TAX
LDA Flips,x
STA $0E

LDX $15E9
LDA !Color,x
TAX
LDA BodyColors,x   ;load current color (red or grey)
ORA $0E            ;add flipping
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00            ;8x8
STA $0460,y         ;store to size table
PLY

INY : INY : INY : INY

PLX
DEX
BPL .BodyLoop
;;; body tiles end

;;; hind-wheel tile start
LDX $15E9
LDA !Dir,x
EOR #$01
TAX
LDA WheelX,x        ;depending on direction, wheel is left or right of thing
CLC
ADC $00
STA $0300,y

LDA $01
CLC
ADC #$09            ;wheel is at bottom
STA $0301,y

LDA #!WheelTile
STA $0302,y

LDA $0F             ;load the props we had from earlier
STA $0303,y


TYA
LSR A
LSR A
PHY
TAY
LDA #$00            ;8x8
STA $0460,y         ;store to size table
PLY

INY : INY : INY : INY
;;; hind-wheel tile end

  PLX

LDY #$FF        ; #$00 = 8x8, #$02 = 16x16
LDA #$06        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam
RTS








SmallTiles:
LDA $00
CLC
ADC #$04
STA $0300,y
STA $0304,y
STA $0308,y
;store x position for all 3 tiles

LDA $01
CLC
ADC #$04
STA $0301,y
STA $0305,y
STA $0309,y
;store y position for all 3 tiles

LDA #!TinyShape
STA $0302,y
LDA #!TinyFeet
STA $0306,y
LDA #!TinyFill
STA $030A,y
;fill in all the tiles


LDA $14
LSR : LSR
AND #$03
TAX
LDA .Rotations,x
STA $0E

LDA.b #!HelmetPalette-$08<<1|$20|!SP
ORA $0E
STA $0303,y
;yellow + flips
LDA.b #!TinyWheelPalette-$08<<1|$20|!SP
ORA $0E
STA $0307,y
;red + flips
LDA.b #!BodyPalette-$08<<1|$20|!SP
STA $030B,y
;grey

  PLX

LDY #$00        ; #$00 = 8x8, #$02 = 16x16
LDA #$02        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam
RTS

.Rotations
db $00,$80,$C0,$40

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
        ;org $03B75C
 
TABLE1:              db $0C,$1C
TABLE2:              db $01,$02
 
GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
        STZ $15A0,x             ; reset sprite offscreen flag, horizontal
        LDA $E4,x               ; \
        CMP $1A                 ;  | set horizontal offscreen if necessary
        LDA $14E0,x             ;  |
        SBC $1B                 ;  |
        BEQ ON_SCREEN_X         ;  |
        INC $15A0,x             ; /
 
ON_SCREEN_X:         LDA $14E0,x             ; \
        XBA      ;  |
        LDA $E4,x               ;  |
        REP #$20                ;  |
        SEC      ;  |
        SBC $1A                 ;  | mark sprite INVALID if far enough off screen
        CLC      ;  |
        ADC.w #$0040            ;  |
        CMP.w #$0180            ;  |
        SEP #$20                ;  |
        ROL A                   ;  |
        AND #$01                ;  |
        STA $15C4,x             ; /
        BNE INVALID             ;
       
        LDY #$00                ; \ set up LOOP:
        LDA $1662,x             ;  |
        AND #$20                ;  | if not smushed (1662 & 0x20), go through LOOP twice
        BEQ ON_SCREEN_LOOP      ;  | else, go through LOOP once
        INY      ; /
ON_SCREEN_LOOP:      LDA $D8,x               ; \
        CLC      ;  | set vertical offscreen if necessary
        ADC TABLE1,y            ;  |
        PHP      ;  |
        CMP $1C                 ;  | (vert screen boundry)
        ROL $00                 ;  |
        PLP      ;  |
        LDA $14D4,x             ;  |
        ADC #$00                ;  |
        LSR $00                 ;  |
        SBC $1D                 ;  |
        BEQ ON_SCREEN_Y         ;  |
        LDA $186C,x             ;  | (vert offscreen)
        ORA TABLE2,y            ;  |
        STA $186C,x             ;  |
ON_SCREEN_Y:         DEY         ;  |
        BPL ON_SCREEN_LOOP      ; /
 
        LDY $15EA,x             ; get offset to sprite OAM
        LDA $E4,x               ; \
        SEC      ;  |
        SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
        STA $00                 ; /
        LDA $D8,x               ; \
        SEC      ;  |
        SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
        STA $01                 ; /
        RTS      ; RETURN
 
INVALID:             PLA         ; \ RETURN from *main gfx routine* subroutine...
        PLA      ;  |    ...(not just this subroutine)
        RTS      ; /
		    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS RETURN_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / return
                   
                   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubHorzPos
; Stores direction Sprite is in from Mario in Y
; Stores proximity in $0F
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL HorzIncY
	INY 
HorzIncY:
	RTS 