;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Random Sprite Generator, by imamelia
;;
;; This generator generates random sprites from both sides
;; of the screen.  This one does not have fixed settings for
;; either the sprite type or number.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, this generator will generate normal sprites.
;; If the first extra bit is set, this generator will generate custom sprites.
;;
;; Based on the following sprite:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite Generator, by mikeyk
;;
;; Description: This will generate a normal sprite or a custom sprite depending on the
;; first extra bit.  Specify the actual sprite that is generated below
;;
;; NOTE: Trying to generate a sprite that doesn't exist will crash your game
;;
;; Uses first extra bit: YES
;; if the first extra bit is set a custom sprite will be generated
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    NormalSprites: dcb $04,$05,$06,$07,$0F,$10         ;only used if first extra bit is clear
                    CustomSprites: dcb $00,$01,$02,$03,$04,$05    ;only used if first extra bit is set
                    NormalStatus: dcb $01,$01,$01,$01,$01,$01         ;only used if first extra bit is clear
                    CustomStatus: dcb $01,$01,$01,$01,$01,$01    ;only used if first extra bit is set
	    NumberToGen = $06	; the number of different sprites you want to generate
	    RoundedValue = $07	; that number rounded up to the nearest power of 2, minus one

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"              ;generators don't have an init routine
                    dcb "MAIN"                                    
                    PHB                     
                    PHK                     
                    PLB                     
                    JSR SPRITE_CODE_START   
                    PLB                     
                    RTL      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main sprite code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    EXTRA_BITS = $7FAB10
                    NEW_SPRITE_NUM = $7FAB9E

TBL_B2D0            dcb $F0,$FF
TBL_B2D2            dcb $FF,$00
TBL_B2D4            dcb $10,$F0

SPRITE_CODE_START   LDA $14    
                    AND #$3F
                    ORA $9D   
                    BNE RETURN
                    JSL $02A9DE
                    BMI RETURN
                    TYX        
                
	Retry:
	JSL $01ACF9		; random value routine
	AND #RoundedValue	; maximum number of values in powers of 2...
	CMP #NumberToGen	; but if the result is greater than the number we want...
	BCS Retry			; generate a new random number
	STA $00			; store to scratch RAM

	PHY			;
	LDY $00			; get sprite table index into Y

                    LDA $18B9               ; check if first extra bit is set
                    AND #$40
                    BNE CUST
                    
NORMAL              LDA NormalSprites,y      ; store sprite number
                    STA $9E,x
	LDA NormalStatus,y
	STA $14C8,x
	PLY
                    JSL $07F7D2             ; reset sprite properties
                    BRA SHARED                    
                    
CUST                LDA CustomSprites,y ; store custom sprite number
                    STA NEW_SPRITE_NUM,x
	LDA CustomStatus,y
	STA $14C8,x
	PLY
                    JSL $07F7D2             ; reset sprite properties
                    JSL $0187A7             ; get table values for custom sprite
	LDA #$08
                    STA EXTRA_BITS,x

SHARED       JSL $01ACF9
                    AND #$7F   
                    ADC #$40   
                    ADC $1C    
                    STA $D8,x  
                    LDA $1D    
                    ADC #$00   
                    STA $14D4,x
                    LDA $148E  
                    AND #$01   
                    TAY        
                    LDA TBL_B2D0,y
                    CLC        
                    ADC $1A    
                    STA $E4,x  
                    LDA $1B    
                    ADC TBL_B2D2,y
                    STA $14E0,x
                    LDA TBL_B2D4,y
                    STA $B6,x  
RETURN              RTS        
