;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Skeeter (from Mario 64), by Dispari Scuro
;;
;; Description: A simple stompable sprite which "skates" across the top of water
;; in short bursts. Skeeter will also follow water that's rising or falling, and
;; if placed undewater, will float to the top. Once on land, skeeter will walk
;; randomly back and forth, stopping occasionally.
;;
;; Note: Small bug in which skeeter can skate off the first or last screen in a
;; tide level and end up floating to the top of the screen for no reason. Didn't
;; feel like fixing it, so just make sure you don't allow them to do it.
;;
;; Based on the following sprites:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shy Guy, by mikeyk
;;
;; Description: A Shy Guy with many available configurations.
;;
;; Note: When rideable, clipping tables values should be: 03 0A FE 0E
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Uses first extra bit: NO
;;
;; Extra Property Byte 1			Green	Red	Blue	Yellow
;;    bit 0 - move fast				0	0	1	1
;;    bit 1 - stay on ledges			0	1	1	0
;;    bit 2 - follow mario			0	0	0	1
;;    bit 3 - jump over shells			0	0	0	0
;;    bit 4 - enable spin killing (if rideable)	1	1	1	1
;;    bit 5 - can be carried (if rideable)	1 (30)	1 (32)	1 (33)	1 (35)
;;
;; Extra Property Byte 2
;;    bit 0 - use turning image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	UpdateSpritePos = $01802A  
	SprSprInteract = $018032   
	MarioSprInteract = $01A7DC
	GetSpriteClippingA = $03B69F
	CheckForContact = $03B72B
	DisplayContactGfx = $01AB99 
	FinishOAMWrite = $01B7B3
	ShowSpinJumpStars = $07FC3B
	GivePoints = $02ACE5
	HurtMario = $00F5B7
	
	ExtraProperty1 = $7FAB28
	ExtraProperty2 = $7FAB34
        RAM_SpriteDir = $157C
	RAM_SprTurnTimer = $15AC

	SkeeterInWater = $1528
	CurrentSpeed = $1504
	StopTimer = $1510		; 163E / 1564
	RAM_SprStopTimer = $1540	; 1540 
	GetRand = $01ACF9
	SpriteStop = $41		; How long a sprite stops before it turns around.
					; Set this to 00 if you don't want NPCs to stop
					; before turning around.
					; DEFAULT: $41

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "INIT"
	LDA $167A,x
	STA $1528,x
	
	STZ SkeeterInWater,x	; Assume skeeter isn't in water when spawned
	STZ CurrentSpeed,x	; Skating speed set to 0
	LDA #$08		; \
	STA StopTimer,x		; / Stop timer set to 08

	;LDA #$01		
	;STA $151C,x
	
        JSR SubHorzPos		; Face Mario
        TYA
        STA $157C,x

	JSL GetRand
	STA RAM_SprTurnTimer,x	; Issue random walking duration
        RTL                 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "MAIN"                                    
        PHB                  
        PHK                  
        PLB                  
        JSR SpriteMainSub    
        PLB                  
        RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpeedX:
	dcb $08,$F8
SpeedSkating:
	dcb $30,$D0
BounceSpeed:
	dcb $0C,$F4
KILLED_X_SPEED
	dcb $F0,$10

Return:
	RTS
SpriteMainSub:
	JSR SubGfx

        LDA $9D                 ; \ if sprites locked, return
        BNE Return              ; /
	LDA $14C8,x
	CMP #$08
	BNE Return

        JSR SubOffScreen	; handle off screen situation
	INC $1570,x

	LDA SkeeterInWater,x	; \
	BEQ NotInWater		; | Decrement the in water timer if it's not 0
	DEC SkeeterInWater,x	; /

NotInWater:
	LDA $164A,x		; \
	BEQ NormalSpeed		; | If in water...
	STZ $B6,x		; | Stop moving and float
	LDA #$FC		; |
	STA $AA,x		; |
	LDA #$08		; | Set in water timer to 08
	STA SkeeterInWater,x	; /
	BRA UpdatePosition
NormalSpeed:
	LDA SkeeterInWater,x	; \ If skeeter is in the water...
	BEQ WalkingSkeeter	; /
	LDA CurrentSpeed,x	; \
	STA $B6,x		; | Check stop status
	BNE SlowSkeeter		; /
	LDA StopTimer,x		; \
	BEQ NoStopTimer		; | Decrement the stop timer if it's not 0
	DEC StopTimer,x		; /
NoStopTimer:
	LDA StopTimer,x
	BEQ ResetSpeed

SlowSkeeter:
	LDA CurrentSpeed,x	; \ Left or right facing
	CMP #$80		; / 
	BCC GoDown		; \ Slow down sprite
	INC CurrentSpeed,x	; /
	BRA UpdatePosition
GoDown:
	DEC CurrentSpeed,x	; \ Slow down sprite
	BRA UpdatePosition	; /
ResetSpeed:
	LDY $157C,x		; \
	LDA SpeedSkating,y	; | Set to skating speed
	STA $B6,x		; |
	STA CurrentSpeed,x	; /
	LDA #$08		; \ Reset stop timer
	STA StopTimer,x		; /
	BRA UpdatePosition

WalkingSkeeter:
	LDA SkeeterInWater,x	; \ If skeeter has been in contact with water recently...
	BEQ JustWalking		; |
	STZ $B6,x		; / ...don't move
	BRA UpdatePosition
JustWalking:
	JSR WanderRoutine	; Go to wander routine
UpdatePosition:
	JSL UpdateSpritePos     ; Update position based on speed values
	
	LDA $1588,x             ; If sprite is in contact with an object...
        AND #$03                  
        BEQ NoObjContact	
        JSR SetSpriteTurning    ;    ...change direction
	LDY $157C,x             ; \ Set walking x speed based on direction
        LDA BounceSpeed,y       ; |
        STA $B6,x		; /
	STA CurrentSpeed,x
	LDA #$08
	STA StopTimer,x

	STZ RAM_SprStopTimer,x	; Halt the stop timer if it's going for some reason.
    
    	JSL GetRand		; \
	STA RAM_SprTurnTimer,x	; | ...issue random walking duration.
	CMP #$20		; |
	BCS NoObjContact	; |
	LDA #$20		; | Make sure turn time is at least 20.
	STA RAM_SprTurnTimer,x	; /

NoObjContact:
	;JSR MaybeStayOnLedges

	LDA $1588,x             ; if on the ground, reset the turn counter
        AND #$04
        BEQ NotOnGround
	STZ $AA,x
	;STZ $151C,x		; Reset turning flag (used if sprite stays on ledges)
	;JSR MaybeFaceMario
	;JSR MaybeJumpShells
NotOnGround:

	LDA $1528,x
	STA $167A,x

        JSL SprSprInteract	; Interact with other sprites

	JSL MarioSprInteract	; Check for mario/sprite contact (carry set = contact)
        BCC Return1             ; return if no contact
	BRA SpinKill
	
        JSR SubVertPos          ; \
        LDA $0E                 ;  | if mario isn't above sprite, and there's contact...
        CMP #$E6                ;  |     ... sprite wins
        BPL SpriteWins          ; /
        LDA $7D                 ; \ if mario speed is upward, return
        BMI Return1             ; /
	BRA SpinKill
SpriteWins:
        LDA $1490               ; Branch if Mario has a star
        BNE MarioHasStar        
        JSL HurtMario
Return1:
	RTS                    

SpinKill:
	JSR SUB_STOMP_PTS       ; give mario points
        JSL DisplayContactGfx   ; display contact graphic
        LDA #$04                ; \ status = 4 (being killed by spin jump)
        STA $14C8,x             ; /   
        LDA #$1F                ; \ set spin jump animation timer
        STA $1540,x             ; /
        JSL ShowSpinJumpStars
        LDA #$08                ; \ play sound effect
        STA $1DF9               ; /
        RTS                     ; return

MarioHasStar:
        LDA #$02                ; \ status = 2 (being killed by star)
        STA $14C8,x             ; /
        LDA #$D0                ; \ set y speed
        STA $AA,x               ; /
        JSR SubHorzPos          ; get new direction
        LDA KILLED_X_SPEED,y    ; \ set x speed based on direction
        STA $B6,x               ; /
        INC $18D2               ; increment number consecutive enemies killed
        LDA $18D2               ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NoReset             ;  |
        LDA #$08                ;  |
        STA $18D2               ; /   
NoReset:
	JSL GivePoints
        LDY $18D2               ; \ 
        CPY #$08                ;  | if consecutive enemies stomped < 8 ...
        BCS NO_SOUND2           ;  |
        LDA STAR_SOUNDS,y       ;  |    ... play sound effect
        STA $1DF9               ; /
NO_SOUND2:
	RTS                     ; final return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MaybeStayOnLedges:	
	LDA ExtraProperty1,x	; Stay on ledges if bit 1 is set
	AND #$02                
	BEQ NoFlipDirection
	LDA $1588,x             ; If the sprite is in the air
	ORA $151C,x             ;   and not already turning
	BNE NoFlipDirection
	JSR SetSpriteTurning 	;   flip direction
        LDA #$01                ;   set turning flag
	STA $151C,x    
NoFlipDirection:
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
MaybeFaceMario:
	LDA $1570,x
	AND #$7F
	BNE Return4
	LDA RAM_SpriteDir,x
	PHA
	
	JSR SubHorzPos         	; Face Mario
        TYA                       
	STA RAM_SpriteDir,X
	
	PLA
	CMP RAM_SpriteDir,x
	BEQ Return4
	LDA #$08
	STA RAM_SprTurnTimer,x
Return4:	
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
MaybeJumpShells:
	LDA ExtraProperty1,x    ; Face Mario if bit 3 is set
	AND #$08
	BEQ Return4
	TXA                     ; \ Process every 4 frames 
        EOR $13                 ;  | 
        AND #$03		;  | 
        BNE Return0188AB        ; / 
        LDY #$09		; \ Loop over sprites: 
JumpLoopStart:
	LDA $14C8,Y             ;  | 
        CMP #$0A       		;  | If sprite status = kicked, try to jump it 
        BEQ HandleJumpOver	;  | 
JumpLoopNext:
	DEY                     ;  | 
        BPL JumpLoopStart       ; / 
Return0188AB:
	RTS                     ; Return 

HandleJumpOver:
	LDA $00E4,Y             
        SEC                       
        SBC #$1A                
        STA $00                   
        LDA $14E0,Y             
        SBC #$00                
        STA $08                   
        LDA #$44                
        STA $02                   
        LDA $00D8,Y             
        STA $01                   
        LDA $14D4,Y             
        STA $09                   
        LDA #$10                
        STA $03                   
        JSL GetSpriteClippingA  
        JSL CheckForContact     
        BCC JumpLoopNext        ; If not close to shell, go back to main loop
	LDA $1588,x 		; \ If sprite not on ground, go back to main loop 
	AND #$04		;  |
        BEQ JumpLoopNext        ; / 
        LDA $157C,Y             ; \ If sprite not facing shell, don't jump 
        CMP $157C,X             ;  | 
        BEQ Return0188EB        ; / 
        LDA #$C0                ; \ Finally set jump speed 
        STA $AA,X               ; / 
Return0188EB:
	RTS                     ; Return
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetSpriteTurning:
	LDA #$08                ; Set turning timer 
	STA RAM_SprTurnTimer,X   
        LDA $157C,x
        EOR #$01
        STA $157C,x
Return0190B1:	
        RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Some mikey routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteTurning:
		LDA $157C,x		; \ If sprite is going right...
		BNE GoLeft		; / ...go left.
		LDA #$01		; \ Otherwise go right.
		STA $157C,x		; /
		RTS
		
GoLeft:
		LDA #$00
		STA $157C,x
        RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; NPC Wandering Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

WanderRoutine:
	LDA RAM_SprStopTimer,x	; \ If sprite has stopped long enough...
    	CMP #$01		; |
    	BEQ RestartSpriteMove	; / ...restart its movement
    
    	LDA RAM_SprTurnTimer,x	; \ If turn timer is zero...
    	BNE SetSpeed		; /
	;LDA #SpriteStop		; \ ...or sprite doesn't stop...
	;BEQ RestartSpriteMove	; / ...just turn around.
    
    	LDA RAM_SprStopTimer,x	; \ ...and sprite isn't already stopped...
    	BNE SetSpeed		; /
    	LDA #SpriteStop		; \ ...start the stop timer.
    	STA RAM_SprStopTimer,x	; /
    	BRA SetSpeed
    
RestartSpriteMove:
	STZ RAM_SprStopTimer,x
    	JSR SpriteTurning	; Turn the NPC around.
    
    	JSL GetRand		; \
	STA RAM_SprTurnTimer,x	; | Issue random walking duration.
	BRA SetSpeed		; /
    
    	LDA ExtraProperty1,x 	; \
    	STA RAM_SprTurnTimer,x	; / ...and reset turn timer
SetSpeed:
	LDY $157C,x             ; Set x speed based on direction
        LDA SpeedX,y           
        STA $B6,x

CheckIfStopped:
    	LDA RAM_SprStopTimer,x	; \
    	CMP #$02		; | Reset speed to zero...
    	BCC DoneWithSpeed	; | ...if Stop Timer is going.
    	STZ $B6,x		; /
DoneWithSpeed:
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Tilemap:
	dcb $E2,$E4		; Skating (left, right) -- facing left
	dcb $E6,$E8		; Walking (left, right) -- facing left
	dcb $E4,$E2		; Skating (left, right) -- facing right
	dcb $E8,$E6		; Walking (left, right) -- facing right

	dcb $E2,$E4
	dcb $E6,$E8
	dcb $E4,$E2
	dcb $E8,$E6

HorizDisp:
	dcb $F8,$08
	dcb $F8,$08
	dcb $F8,$08
	dcb $F8,$08

	dcb $F8,$08
	dcb $F8,$08
	dcb $F8,$08
	dcb $F8,$08

VertiDisp:
	dcb $FF,$FF
	dcb $FF,$FF
	dcb $FF,$FF
	dcb $FF,$FF

	dcb $FE,$FE
	dcb $FE,$FE
	dcb $FE,$FE
	dcb $FE,$FE

TileFlip:
	dcb $00,$00
	dcb $00,$00
	dcb $40,$40
	dcb $40,$40

	dcb $00,$00
	dcb $00,$00
	dcb $40,$40
	dcb $40,$40

SubGfx:
	JSR GetDrawInfo   

	STZ $03               	; Static sprite

	LDA $157C,x             ; $02 = direction
        STA $02

	LDA StopTimer,x		; \
	CMP #$06		; | If the stop timer is about to end...
	BCS WaterCheck		; |
	LDA $03			; |
	CLC			; | ...skip some tiles to kick feet up
	ADC #$02		; |
	STA $03			; /

WaterCheck:
	LDA $164A,x		; \ Sprite is in water check
	BNE IsInWater		; /
	LDA SkeeterInWater,x	; \ Been near water recently check
	BNE CheckDirection	; /
	LDA $B6,x		; \ Don't animate if not moving
	BEQ CheckDirection	; /
	LDA $14			; \
        LSR A                   ; | Tile animations for walking
        LSR A			; |
	LSR A			; |
	AND #$01		; |
	ASL A			; |
        STA $03			; /
	BRA CheckDirection
	
IsInWater:
	LDA $03			; \
	CLC			; |
	ADC #$08		; | Skip some tiles if in the water
	STA $03			; /

CheckDirection:
        LDA $157C,x             ; \ $02 = sprite direction
        STA $02                 ; /
        BNE LOOP_START
        LDA $03			; \
        CLC			; |
        ADC #$04		; | Skip some tiles if facing the other way
        STA $03			; /

LOOP_START:
	PHX                     ; push sprite index
        LDX #$01                ; loop counter = (number of tiles per frame) - 1

LOOP_START_2:
        PHX			; push current tile number
        
        TXA                     ; \ X = index of frame start + current tile
        CLC                     ;  |
        ADC $03                 ;  |
        TAX                     ; /
        
        PHY                     ; \
        TYA                     ; |
        LSR A                   ; |
        LSR A                   ; |
        TAY                     ; | Set tile to be either 8x8 ($00) or 16x16 ($02)
        LDA #$02	        ; |
        STA $0460,y             ; |
        PLY                     ; /

        LDA $00                 ; \ tile x position = sprite x location ($00)
        CLC                     ;  |
        ADC HorizDisp,x         ;  |
        STA $0300,y             ; /
        
        LDA $01                 ; \ tile y position
	CLC			; |
	ADC VertiDisp,x		; |
        STA $0301,y             ; /
        
        LDA Tilemap,x           ; \ store tile
        STA $0302,y             ; / 
        
        PHX
        LDX $15E9
        LDA $15F6,x             ; get palette info
        PLX
        ORA TileFlip,x		; flip tile if necessary
        ORA $64                 ; add in tile priority of level
        STA $0303,y             ; store tile properties

	PLX                     ; \ pull, X = current tile of the frame we're drawing
        INY                     ;  | increase index to sprite tile map ($300)...
        INY                     ;  |    ...we wrote 1 16x16 tile...
        INY                     ;  |    ...sprite OAM is 8x8...
        INY                     ;  |    ...so increment 4 times
        DEX                     ;  | go to next tile of frame and loop
        BPL LOOP_START_2        ; /

        PLX                     ; pull, X = sprite index
        LDY #$FF                ; \ We've already set 460, so use FF!
        LDA #$01                ; | A = number of tiles drawn - 1
        JSL $01B7B3             ; / don't draw if offscreen
        RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from SMW
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS
	dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS:
	PHY                      
        LDA $1697               ; \
        CLC                     ;  | 
        ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
        INC $1697               ; increase consecutive enemies stomped
        TAY                     ;
        INY                     ;
        CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
        BCS NO_SOUND            ; /    ... don't play sound 
        LDA STAR_SOUNDS,y       ; \ play sound effect
        STA $1DF9               ; /   
NO_SOUND:
	TYA                     ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NO_RESET            ;  |
        LDA #$08                ; /
NO_RESET:
	JSL GivePoints
        PLY                     
        RTS                     ; return
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubVertPos:
	LDY #$00
        LDA $96
        SEC
        SBC $D8,x
        STA $0F
        LDA $97
        SBC $14D4,x
        BPL VertIncY
        INY
VertIncY:
        RTS   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL HorzIncY
	INY 
HorzIncY:
	RTS 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 	
DATA_03B75C:
	dcb $0C,$1C
DATA_03B75E:
	dcb $01,$02

GetDrawInfo:
	STZ $186C,X             ; Reset sprite offscreen flag, vertical 
        STZ $15A0,X             ; Reset sprite offscreen flag, horizontal 
        LDA $E4,X               ; \ 
        CMP $1A                 ;  | Set horizontal offscreen if necessary 
        LDA $14E0,X             ;  | 
        SBC $1B                 ;  | 
        BEQ ADDR_03B774         ;  | 
        INC $15A0,X             ; / 
ADDR_03B774:
        LDA $14E0,X             ; \ 
        XBA                     ;  | Mark sprite invalid if far enough off screen 
        LDA $E4,X               ;  | 
        REP #$20                ; Accum (16 bit) 
        SEC                     ;  | 
        SBC $1A                 ;  | 
        CLC                     ;  | 
        ADC.W #$0040            ;  | 
        CMP.W #$0180            ;  | 
        SEP #$20                ; Accum (8 bit) 
        ROL                     ;  | 
        AND.B #$01              ;  | 
        STA $15C4,X             ;  | 
        BNE ADDR_03B7CF         ; /  
        LDY.B #$00              ; \ set up loop: 
        LDA $1662,X             ;  |  
        AND.B #$20              ;  | if not smushed (1662 & 0x20), go through loop twice 
        BEQ ADDR_03B79A         ;  | else, go through loop once 
        INY                     ; /                        
ADDR_03B79A:
        LDA $D8,X               ; \                        
        CLC                     ;  | set vertical offscree 
        ADC DATA_03B75C,Y       ;  |                       
        PHP                     ;  |                       
        CMP $1C                 ;  | (vert screen boundry) 
        ROL $00                 ;  |                       
        PLP                     ;  |                       
        LDA $14D4,X             ;  |                       
        ADC.B #$00              ;  |                       
        LSR $00                 ;  |                       
        SBC $1D                 ;  |                       
        BEQ ADDR_03B7BA         ;  |                       
        LDA $186C,X             ;  | (vert offscreen)      
        ORA DATA_03B75E,Y       ;  |                       
        STA $186C,X             ;  |                       
ADDR_03B7BA:
        DEY                     ;  |                       
        BPL ADDR_03B79A         ; /                        
        LDY $15EA,X             ; get offset to sprite OAM                           
        LDA $E4,X               ; \ 
        SEC                     ;  |                                                     
        SBC $1A                 ;  |                                                    
        STA $00                 ; / $00 = sprite x position relative to screen boarder 
        LDA $D8,X               ; \                                                     
        SEC                     ;  |                                                     
        SBC $1C                 ;  |                                                    
        STA $01                 ; / $01 = sprite y position relative to screen boarder 
        RTS                     ; Return 

ADDR_03B7CF:
        PLA                     ; \ Return from *main gfx routine* subroutine... 
        PLA                     ;  |    ...(not just this subroutine) 
        RTS                     ; / 

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
DATA_01AC0D:
	dcb $40,$B0
DATA_01AC0F:
	dcb $01,$FF
DATA_01AC11:
        dcb $30,$C0
DATA_01AC19:
        dcb $01,$FF

SubOffScreen:
	JSR IsSprOnScreen       ; \ if sprite is not off screen, return                                       
        BEQ Return01ACA4        ; /                                                                           
        LDA $5B                 ; \  vertical level                                    
        AND #$01                ;  |                                                                           
        BNE VerticalLevel       ; /                                                                           
        LDA $D8,X               ; \                                                                           
        CLC                     ;  |                                                                           
        ADC #$50                ;  | if the sprite has gone off the bottom of the level...                     
        LDA $14D4,X             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)   
        ADC #$00                ;  |                                                                           
        CMP #$02                ;  |                                                                           
        BPL OffScrEraseSprite   ; /    ...erase the sprite                                                    
        LDA $167A,X             ; \ if "process offscreen" flag is set, return                                
        AND #$04                ;  |                                                                           
        BNE Return01ACA4        ; /                                                                           
        LDA $13                   
        AND #$01                
        STA $01                   
        TAY                       
        LDA $1A                   
        CLC                       
        ADC DATA_01AC11,Y       
        ROL $00                   
        CMP $E4,X                 
        PHP                       
        LDA $1B                   
        LSR $00                   
        ADC DATA_01AC19,Y       
        PLP                       
        SBC $14E0,X             
        STA $00                   
        LSR $01                   
        BCC ADDR_01AC7C           
        EOR #$80                
        STA $00                   
ADDR_01AC7C:
        LDA $00                   
        BPL Return01ACA4          
OffScrEraseSprite:
	LDA $14C8,X             ; \ If sprite status < 8, permanently erase sprite 
        CMP #$08                ;  | 
        BCC OffScrKillSprite    ; / 
        LDY $161A,X             
        CPY #$FF                
        BEQ OffScrKillSprite      
        LDA #$00                
        STA $1938,Y             
OffScrKillSprite:
	STZ $14C8,X             ; Erase sprite 
Return01ACA4:
	RTS                       

VerticalLevel:
	LDA $167A,X             ; \ If "process offscreen" flag is set, return                
        AND #$04                ;  |                                                           
        BNE Return01ACA4        ; /                                                           
        LDA $13                 ; \                                                           
        LSR                     ;  |                                                           
        BCS Return01ACA4        ; /                                                           
        LDA $E4,X               ; \                                                           
        CMP #$00                ;  | If the sprite has gone off the side of the level...      
        LDA $14E0,X             ;  |                                                          
        SBC #$00                ;  |                                                          
        CMP #$02                ;  |                                                          
        BCS OffScrEraseSprite   ; /  ...erase the sprite      
        LDA $13                   
        LSR                       
        AND #$01                
        STA $01                   
        TAY                       
	LDA $1C                   
        CLC                       
        ADC DATA_01AC0D,Y       
        ROL $00                   
        CMP $D8,X                 
        PHP                       
        LDA $001D               
        LSR $00                   
        ADC DATA_01AC0F,Y       
        PLP                       
        SBC $14D4,X             
        STA $00                   
        LDY $01                   
        BEQ ADDR_01ACF3           
        EOR #$80                
        STA $00                   
ADDR_01ACF3:
        LDA $00                   
        BPL Return01ACA4          
        BMI OffScrEraseSprite  

IsSprOnScreen:
	LDA $15A0,X             ; \ A = Current sprite is offscreen 
        ORA $186C,X             ; /  
        RTS                     ; Return 
		
