; ========================
; KICKCHON's SPRITE ENGINE
; ========================

; TODO:
; (HDMA Macros)
; (Mode 7 Macros)
; (Life bar Macros)

; This is the header file, which should be included at the top of your sprite
; It contains all global constants and macros to use the animation and state engine

; This library uses the following sprite tables:


;	$1534,x		used for x acceleration
;	$1570,x		used for y acceleration
;	$157C,x		used for direction bit
; 	$1594,x		used for animation tilemap pointer highbyte
;	$15AC,x		used for sprite interaction	
; 	$1602,x		used for animation tilemap pointer lowbyte
; 	$160E,x		used for next sprite state and as external death marker
; 	$163E,x		used for sprite state timer
;	$187B,x		used for current sprite state
;	$1FD6,x		used for current sprite animation index

; The other sprite tables can be used freely!

; It should not be necessary to modify this file, but it is recommended to use this file as a reference. ;)

; ===============
; PROVIDED MACROS
; ===============

; ===== Accumulator Macros =====
;	macro	set8bit()			; Changes to 8 bit
;	macro	set16bit()			; Changes to 16 bit

; ===== Jump Macros =====
;	macro	jumpBitClear(bit, target)	; Jumps to target if the bit in the accumulator is clear
;	macro	jumpBitSet(bit, target)		; Jumps to target if the bit in the accumulator is set
;	macro	jumpExtraBitSet(target)		; Jumps to target if the extra bit is set
;	macro	jumpExtraBitClear(target)	; Jumps to target if the extra bit is clear
;	macro	jumpDirectionBitSet(target)	; Jumps to target if the direction bit is set (Mario is on the right)
;	macro	jumpDirectionBitClear(target)	; Jumps to target if the direction bit is clear (Mario is on the left)
;	macro	jumpMarioLeftSide(target)	; Jumps to target if Mario is on the left side (without updateing the direction bit)
;	macro	jumpMarioRightSide(target)	; Jumps to target if Mario is on the right side (without updateing the direction bit)
;	macro	jumpTouchingWall(target)	; Jumps to target if sprite is touching a wall
;	macro	jumpNotTouchingWall(target)	; Jumps to target if sprite is not touching a wall
;	macro	jumpInAir(target)		; Jumps to target if sprite is in air
;	macro	jumpNotInAir(target)		; Jumps to target if sprite is not in air
;	macro	jumpSpriteContact(target)	; Jumps to target if sprite is touching another sprite
;	macro	jumpNoSpriteContact(target)	; Jumps to target if sprite is not touching another sprite
;	macro	jumpMarioContact(target)	; Jumps to target if sprite is touching Mario
;	macro	jumpNoMarioContact(target)	; Jumps to target if sprite is not touching Mario
;	macro	jumpMarioSliding(target)	; Jumps to target if Mario is sliding
;	macro	jumpMarioNotSliding(target)	; Jumps to target if Mario is not sliding
;	macro	jumpMarioHittingAbove(target)	; Jumps to target if Mario is hitting the sprite from above (req: prev. call to a MarioContact jump)
;	macro	jumpMarioHittingSide(target)	; Jumps to target if Mario is being hit by the sprite from the side (req: prev. call to a MarioContact jump)
;	macro	jumpMarioHasStar(target)	; Jumps to target if Mario has a star
;	macro	jumpMarioNoStar(target)		; Jumps to target if Mario doesn't have a star
;	macro	jumpMarioSpinJump(target)	; Jumps to target if Mario is spin jumping
;	macro	jumpMarioNoSpinJump(target)	; Jumps to target if Mario is not spin jumping
;	macro	jumpMarioOnYoshi(target)	; Jumps to target if Mario is riding yoshi
;	macro	jumpMarioNotOnYoshi(target)	; Jumps to target if Mario is not riding yoshi
;	macro	jumpSpinJumpOrYoshi(target)	; Jumps to target if Mario is spin jumping or riding Yoshi
;	macro	jumpNoSpinJumpOrYoshi(target)	; Jumps to target if Mario is neither spin jumping nor riding yoshi
;	macro	jumpExternalDeath(target)	; Jumps to target if the sprite died from an external interaction (to be used in the death state)
;	macro	jumpNoExternalDeath(target)	; Jumps to target if the sprite did not die from an external interaction (to be used in the death state)

; ===== State machine macros =====
;	macro	changeSMWSpriteStatus(status)	; Changes to the given sprite state (internal state of SMW not the engine state machine)
;	macro	initialState(init)		; Changes to the initial state
;	macro	setNextState(next)		; Sets which state the sprite will change to on next change
;	macro	setNextStateA()			; Sets which state the sprite will change to on next change from the accumulator
;	macro	changeToNextState()		; Changes to the state which was last set
;	macro	changeToDeathState()		; Changes to the death state

; ===== Animation macros =====
;	macro	changeAnimation(newanimation)	; Makes the sprites use the selected animation
;	macro	changeAnimationA()		; Makes the sprite use the animation stored in the accumulator
;	macro	updateAnimationDirection()	; Updates the sprite's animation to reflect the direction bit (aka, animation faces Mario)
;	macro	updateDirectionBit()		; Update direction bit to face Mario, (0 = Mario on left side, 1 = Mario on right side)
;	macro	flipDirectionBit()		; Flips the direction bit

; ===== Movement macros =====
;	macro	setSpeedX(newval)		; Sets the sprite's x speed (newspeed = newval)
;	macro	setSpeedXA()			; Sets the sprite's x speed (newspeed = accumulator)
;	macro	setSpeedY(newval)		; Sets the sprite's y speed (newspeed = newval)
;	macro	setSpeedYA()			; Sets the sprite's y speed (newspeed = accumulator)
;	macro	setSpeeds(x, y)			; Sets the sprite's speed to the given values
;	macro	setSpeedXRelative(newval)	; Modifies the sprite's x speed relative to its current speed (newspeed = oldspeed + newval)
;	macro	setSpeedXRelativeA()		; Modifies the sprite's x speed relativ to its current speed (newspeed = oldspeed + accumulator value)
;	macro	setSpeedYRelative(newval)	; Modifies the sprite's y speed relative to its current speed (newspeed = oldspeed + newval)
;	macro	setSpeedYRelativeA()		; Modifies the sprite's x speed relativ to its current speed (newspeed = oldspeed + accumulator value)
;	macro	setSpeedsRelative(x, y)		; Modifies the sprite's speed relative to its current speed
;	macro	incSpeedX()			; Increases sprite's x speed by one
;	macro	incSpeedY()			; Increases sprite's y speed by one
;	macro	decSpeedX()			; Decreases sprite's x speed by one
;	macro	decSpeedY()			; Decreases sprite's y speed by one
;	macro	setAccelX(newval)		; Sets the sprite's x acceleration (newaccel = newval)
;	macro	setAccelXA()			; Sets the sprite's x acceleration (newaccel = accumulator)
;	macro	setAccelY(newval)		; Sets the sprite's y acceleration (newaccel = newval)
;	macro	setAccelYA()			; Sets the sprite's y acceleration (newaccel = accumulator)
;	macro	setAccels(x, y)			; Sets the sprite's acceleration to the given values
;	macro	setAccelXRelative(newval)	; Modifies the sprite's x acceleration relative to its current one (newaccel = oldaccel + newval)
;	macro	setAccelXRelativeA()		; Modifies the sprite's x acceleration relative to its current one (newaccel = oldaccel + accumulator value)
;	macro	setAccelYRelative(newval)	; Modifies the sprite's y acceleration relative to its current one (newaccel = oldaccel + newval)
;	macro	setAccelYRelativeA()		; Modifies the sprite's y acceleration relative to its current one (newaccel = oldaccel + accumulator value)
;	macro	setAccelsRelative(x, y)		; Modifies the sprite's acceleration relative to its current acceleration
;	macro	incAccelX()			; Increases sprite's x acceleration by one
;	macro	incAccelY()			; Increases sprite's y acceleration by one
;	macro	decAccelX()			; Decreases sprite's x acceleration by one
;	macro	decAccelY()			; Decreases sprite's y acceleration by one
;	macro	flipSpeedX()			; Flips the x speed
;	macro	flipSpeedY()			; Flips the y speed
;	macro	flipAccelX()			; Flips the x acceleration
;	macro	flipAccelY()			; Flips the y acceleration
;	macro	clearSpeedX()			; Resets the x speed
;	macro	clearSpeedY()			; Resets the y speed
;	macro	clearSpeeds()			; Resets both speeds
;	macro	clearAccelX()			; Resets the x acceleration
;	macro	clearAccelY()			; Resets the y acceleration
;	macro	clearAccels()			; Resets both accelerations
;	macro	updateSpeedX()			; Updates the sprites x speed according to the current x acceleration
;	macro	updateSpeedY()			; Updates the sprites y speed according to the current y acceleration
;	macro	updateSpeeds()			; Updates the sprites speed according to the current acceleration
;	macro	moveSpriteGravity()		; Moves the sprite according to the current speed and gravity
;	macro	moveSpriteNoGravity()		; Moves the sprite according to the current speed ignoring gravity

; ===== Sprite Interaction Macros =====
;	macro	hurtMario()			; Hurts Mario
;	macro	killMario()			; Kills Mario directly
;	macro	givePoints()			; Gives Mario points
;	macro	giveStarPoints()		; Gives Mario points, when he has a star
;	macro	handleMarioContact()		; Displays contact graphics and prevents Mario from hitting the sprite again
;	macro	handleSpriteContact()		; Needs to be called to update other sprites which hit this sprite
;	macro	showContactStar()		; Displays the star contact graphics
;	macro	showContactSmoke()		; Displays the smoke contact graphics
;	macro	stayDead()			; Marks the sprite dead, will not respawn once offscreen
;	macro	callForInteractSprite(target)	; Calls the subroutine at target for ever interacting sprite, subroutine needs to preserve X, Y and $0E

; ===== Miscellaneous macros =====
;	macro	playSoundEffect(cha, eff)	; Plays a sound effect on the specified sound channel
;	macro	useDefaultRoutine()		; Makes the sprite use the default sprite routine

; ===== Sprite Morphing/Spawning Macros =====
;	macro	changeToSprite(num)				; Changes the current sprite to the new one
;	macro	changeToSpriteA()				; Changes the current sprite to the new one in the accumulator
;	macro	changeToCustomSprite(num)			; Changes the current custom sprite to the new one
;	macro	changeToCustomSpriteA()				; Changes the current custom sprite to the new one in the accumulator
;	macro	spawnSprite(n, xl, xh, yl, yh)			; Spawns the new sprite n, at coordinates (xl,xh) (yl,yh)
;	macro	spawnSpriteA(xl, xh, yl, yh)			; Spawns the new sprite in the accumulator, at coordinates (xl,xh) (yl,yh)
;	macro	spawnSpriteRelative(n, x, y)			; Spawns a new sprite n, at coordinates relative to the current sprite's position
;	macro	spawnSpriteRelativeA(x, y)			; Spawns a new sprite in A, at coordinates relative to the current sprite's position
;	macro	spawnCustomSprite(n, xl, xh, yl, yh)		; Spawns a custom sprite n at the position (xl,xh) (yl,yh)
;	macro	spawnCustomSpriteA(xl, xh, yl, yh)		; Spawns a custom sprite in A at the position (xl,xh) (yl,yh)
;	macro	spawnCustomSpriteRelative(n, x, y)		; Spawns a new custom sprite n, at coordinates relative to the current sprite's position
;	macro	spawnCustomSpriteRelativeA(x, y)		; Spawns a new custom sprite in A, at coordinates relative to the current sprite's position
;	macro	spawnExtendedSprite(n, xl, xh, yl, yh, xs, ys)	; Spawns an extended sprite n at position (xl,xh) (yl,yh), with speed (xs,ys)
;	macro	spawnExtendedSpriteA(xl, xh, yl, yh, xs, ys)	; Spawns an extended sprite in A at position (xl,xh) (yl,yh), with speed (xs,ys)
;	macro	spawnExtendedSpriteRelative(n, x, y, xs, ys)	; Spawns an extended sprite n relative to the current sprites position
;	macro	spawnExtendedSpriteRelativeA(x, y, xs, ys)	; Spawns an extended sprite in A relative to the current sprites position

; =================
; GENERAL CONSTANTS
; =================

!SPRITE_CINDEX	=	$15E9
!SPRITE_LOCKED	=	$9D
!SPRITE_FRAMET	=	$14

!SPRITE_XSPEED 	=	$B6
!SPRITE_YSPEED 	=	$AA
!SPRITE_XACCEL	=	$1534
!SPRITE_YACCEL	=	$1570

!SPRITE_XPOSHI	=	$14E0
!SPRITE_XPOSLO	=	$E4
!SPRITE_YPOSHI	=	$14D4
!SPRITE_YPOSLO	=	$D8

!MARIO_XPOSHI	=	$95
!MARIO_XPOSLO	=	$94
!MARIO_YPOSHI	=	$97
!MARIO_YPOSLO	=	$96

!MARIO_STARTME	=	$1490
!MARIO_SPINJMP	=	$140D
!MARIO_HASYSHI	=	$187A
!MARIO_ISSLIDE	=	$13ED

!SPRITE_STATUS	=	$14C8
!SPRITE_BLCKED	=	$1588
!SPRITE_INTERT	=	$15AC
!INTERACT_TIME	=	$08

!SPRITE_NUMCUS	=	$7FAB9E
!SPRITE_NUMSTD	=	$9E

!SPRITE_OAMINX	=	$15EA
!SPRITE_PALGFX	=	$15F6

!SPRITE_EXTRA1	=	$7FAB28
!SPRITE_EXTRA2	=	$7FAB34

!SPRITE_TWEAK1	=	$1656
!SPRITE_TWEAK2	=	$1662
!SPRITE_TWEAK3	=	$166E
!SPRITE_TWEAK4	=	$167A
!SPRITE_TWEAK5	=	$1686
!SPRITE_TWEAK6	=	$190F

!OAM_XOFFSET	=	$0300
!OAM_YOFFSET	=	$0301
!OAM_TILENBR	=	$0302
!OAM_PROPBYT	=	$0303

!SPRITE_STATEN	=	$160E
!SPRITE_STATEC	=	$187B
!SPRITE_STATET	=	$163E
!SPRITE_ANIMID	=	$1FD6
!SPRITE_ANIMLO	=	$1602
!SPRITE_ANIMHI	=	$1594
!SPRITE_DIRECT	=	$157C

!PLAYER_COINS	=	$0DBF
!PLAYER_LIVES	=	$0DBE

!SOUND_CHAN0	=	$1DF9
!SOUND_CHAN1	=	$1DFA
!SOUND_CHAN2	=	$1DFB
!SOUND_CHAN3	=	$1DFC

!STATUS_NOSPRT	=	$00
!STATUS_INIT	=	$01
!STATUS_KILLED	=	$02
!STATUS_SMUSHD	=	$03
!STATUS_SPNJMP	=	$04
!STATUS_SINKNG	=	$05
!STATUS_TOCOIN	=	$06
!STATUS_INYSHI	=	$07
!STATUS_NORMAL	=	$08
!STATUS_STATIC	=	$09
!STATUS_KICKED	=	$0A
!STATUS_CRRIED	=	$0B
!STATUS_POWRUP	=	$0C

!POINTS_STAR	=	$18D2
!POINTS_NORMAL	=	$1697
!POINTS_INC	=	$1626

!EXTEND_NONE	=	$00
!EXTEND_SMOKE	=	$01
!EXTEND_RZFIRE	=	$02
!EXTEND_FLAME	=	$03
!EXTEND_HAMMER	=	$04
!EXTEND_MRFIRE	=	$05
!EXTEND_BONE	=	$06
!EXTEND_LAVA	=	$07
!EXTEND_TEDARM	=	$08
!EXTEND_FLICK	=	$09
!EXTEND_COIN	=	$0A
!EXTEND_PPFIRE	=	$0B
!EXTEND_LOTUS	=	$0C
!EXTEND_BASEBL	=	$0D
!EXTEND_FLOWER	=	$0E
!EXTEND_SMKTRL	=	$0F
!EXTEND_STAR	=	$10
!EXTEND_YSFIRE	=	$11
!EXTEND_BUBBLE	=	$12

!EXTEND_NUMBER	=	$170B
!EXTEND_XPOSLO	=	$171F
!EXTEND_XPOSHI	=	$1733
!EXTEND_YPOSLO	=	$1715
!EXTEND_YPOSHI	=	$1729
!EXTEND_XSPEED	=	$1747
!EXTEND_YSPEED	=	$173D
!EXTEND_STABLE	=	$1765

; ==============
; GENERAL MACROS
; ==============

macro	set8bit()			; Changes to 8 bit
		SEP #$30
endmacro
macro	set16bit()			; Changes to 16 bit
		REP #$30
endmacro

macro	jumpBitClear(bit, target)	; Jumps to target if the bit in the accumulator is clear
		AND #(1 << <bit>)
		BEQ <target>
endmacro
macro	jumpBitSet(bit, target)		; Jumps to target if the bit in the accumulator is set
		AND #(1 << <bit>)
		BNE <target>
endmacro
macro	jumpExtraBitClear(target)	; Jumps to target if the extra bit is clear
		LDA $7FAB10,x
		AND #$04
		BEQ <target>
endmacro
macro	jumpExtraBitSet(target)		; Jumps to target if the extra bit is set
		LDA $7FAB10,x
		AND #$04
		BNE <target>
endmacro
macro	jumpDirectionBitSet(target)	; Jumps to target if the direction bit is set (Mario is on the right)
		LDA !SPRITE_DIRECT,x
		BNE <target>
endmacro
macro	jumpDirectionBitClear(target)	; Jumps to target if the direction bit is clear (Mario is on the left)
		LDA !SPRITE_DIRECT,x
		BEQ <target>
endmacro
macro	jumpMarioLeftSide(target)	; Jumps to target if Mario is on the left side (without updateing the direction bit)
		LDA !MARIO_XPOSLO
		SEC
		SBC !SPRITE_XPOSLO,x
		LDA !MARIO_XPOSHI
		SBC !SPRITE_XPOSHI,x
		BMI <target>
endmacro
macro	jumpMarioRightSide(target)	; Jumps to target if Mario is on the right side (without updateing the direction bit)
		LDA !MARIO_XPOSLO
		SEC
		SBC !SPRITE_XPOSLO,x
		LDA !MARIO_XPOSHI
		SBC !SPRITE_XPOSHI,x
		BPL <target>
endmacro
macro	jumpTouchingWall(target)	; Jumps to target if sprite is touching a wall
		LDA !SPRITE_BLCKED,x
		AND #$03
		BNE <target>
endmacro
macro	jumpNotTouchingWall(target)	; Jumps to target if sprite is not touching a wall
		LDA !SPRITE_BLCKED,x
		AND #$03
		BEQ <target>
endmacro
macro	jumpInAir(target)		; Jumps to target if sprite is in air
		LDA !SPRITE_BLCKED,x
		AND #$04
		BEQ <target>
endmacro
macro	jumpNotInAir(target)		; Jumps to target if sprite is not in air
		LDA !SPRITE_BLCKED,x
		AND #$04
		BNE <target>
endmacro
macro	jumpMarioSliding(target)	; Jumps to target if Mario is sliding
		LDA !MARIO_ISSLIDE
		BNE <target>
endmacro
macro	jumpMarioNotSliding(target)	; Jumps to target if Mario is not sliding
		LDA !MARIO_ISSLIDE
		BEQ <target>
endmacro
macro	jumpSpriteContact(target)	; Jumps to target if sprite is touching another sprite, (other sprite index is in y)
		LDA !SPRITE_INTERT,x
		BNE ?return
		STX $0E
		LDY #$0B
?loop:		LDA !SPRITE_STATUS,y
		BEQ ?next
		CPY $0E
		BEQ ?next
		LDA !SPRITE_TWEAK5,y
		AND #$08
		BNE ?next
		TYX
		JSL $03B6E5
		LDX $0E
		JSL $03B69F
		JSL $03B72B
		BCS ?jmp
?next:		DEY
		BPL ?loop
		BRA ?return
?jump:		LDA #!INTERACT_TIME
		STA !SPRITE_INTERT,x
		BRA <target>
?return:
endmacro
macro	jumpNoSpriteContact(target)	; Jumps to target if sprite is not touching another sprite
		LDA !SPRITE_INTERT,x
		BNE <target>
		STX $0E
		LDY #$0B
?loop:		LDA !SPRITE_STATUS,y
		BEQ ?next
		CPY $0E
		BEQ ?next
		LDA !SPRITE_TWEAK5,y
		AND #$08
		BNE ?next
		TYX
		JSL $03B6E5
		LDX $0E
		JSL $03B69F
		JSL $03B72B
		BCS ?return
?next:		DEY
		BPL ?loop
		BRA <target>
?return:	LDA #!INTERACT_TIME
		STA !SPRITE_INTERT,x
endmacro
macro	jumpMarioContact(target)	; Jumps to target if sprite is touching Mario
		JSL $01A7DC
		BCS <target>
endmacro
macro	jumpNoMarioContact(target)	; Jumps to target if sprite is not touching Mario
		JSL $01A7DC
		BCC <target>
endmacro
macro	jumpMarioHittingAbove(target)	; Jumps to target if Mario is hitting the sprite from above (requires previous call to a MarioContact jump)
		LDA $0E
		CMP #$E6
		BMI <target>
endmacro
macro	jumpMarioHittingSide(target)	; Jumps to target if Mario is being hit by the sprite from the side (requires previous call to a MarioContact jump)
		LDA $0E
		CMP #$E6
		BPL <target>
endmacro
macro	jumpMarioHasStar(target)	; Jumps to target if Mario has a star
		LDA !MARIO_STARTME
		BNE <target>
endmacro
macro	jumpMarioNoStar(target)		; Jumps to target if Mario doesn't have a star
		LDA !MARIO_STARTME
		BEQ <target>
endmacro
macro	jumpMarioSpinJump(target)	; Jumps to target if Mario is spin jumping
		LDA !MARIO_SPINJMP
		BNE <target>
endmacro
macro	jumpMarioNoSpinJump(target)	; Jumps to target if Mario is not spin jumping
		LDA !MARIO_SPINJMP
		BEQ <target>
endmacro
macro	jumpMarioOnYoshi(target)	; Jumps to target if Mario is riding yoshi
		LDA !MARIO_HASYSHI
		BNE <target>
endmacro
macro	jumpMarioNotOnYoshi(target)	; Jumps to target if Mario is not riding yoshi
		LDA !MARIO_HASYSHI
		BEQ <target>
endmacro
macro	jumpSpinJumpOrYoshi(target)	; Jumps to target if Mario is spin jumping or riding Yoshi
		LDA !MARIO_SPINJMP
		ORA !MARIO_HASYSHI
		BNE <target>
endmacro
macro	jumpNoSpinJumpOrYoshi(target)	; Jumps to target if Mario is neither spin jumping nor riding yoshi
		LDA !MARIO_SPINJMP
		ORA !MARIO_HASYSHI
		BEQ <target>
endmacro
macro	jumpExternalDeath(target)	; Jumps to target if the sprite died from an external interaction (to be used in the death state)
		LDA !SPRITE_STATEN,x
		CMP #$FF
		BEQ <target>
endmacro
macro	jumpNoExternalDeath(target)	; Jumps to target if the sprite did not die from an external interaction (to be used in the death state)
		LDA !SPRITE_STATEN,x
		CMP #$FF
		BNE <target>
endmacro

macro	changeSMWSpriteStatus(status)	; Changes to the given sprite state
		LDA <status>
		STA !SPRITE_STATUS,x
endmacro
macro	initialState(init)		; Changes to the initial state
		LDA <init>
		STA !SPRITE_STATEC,x
		PHX
		TAX
		LDA STATE_TIMES,x
		PLX
		STA !SPRITE_STATET,x
		JSR _PROCESS_INIT
endmacro
macro	setNextState(next)		; Sets which state the sprite will change to on next change
		LDA <next>
		STA !SPRITE_STATEN,x
endmacro
macro	setNextStateA()			; Sets which state the sprite will change to on next change from the accumulator
		STA !SPRITE_STATEN,x
endmacro
macro	changeToNextState()		; Changes to the state which was last set
		JSR _PROCESS_FINAL
		LDA !SPRITE_STATEN,x
		STA !SPRITE_STATEC,x
		PHX
		TAX
		LDA STATE_TIMES,x
		PLX
		STA !SPRITE_STATET,x
		JSR _PROCESS_INIT
endmacro
macro	changeToDeathState()		; Changes to the death state
		%changeSMWSpriteStatus(#!STATUS_KILLED)
		JSR _PROCESS_FINAL
		LDA #!STATE_DEADFNC
		STA !SPRITE_STATEC,x
		JSR _PROCESS_INIT
endmacro

macro	changeAnimation(newanimation)	; Makes the sprites use the selected animation
		LDA <newanimation>
		STA !SPRITE_ANIMID,x
		%updateAnimationDirection()
endmacro
macro	changeAnimationA()		; Makes the sprite use the animation stored in the accumulator
		STA !SPRITE_ANIMID,x
		%updateAnimationDirection()		
endmacro
macro	updateAnimationDirection()	; Updates the sprite's animation to reflect the direction bit (aka, animation faces Mario)
		PHY
		CLC
		LDA !SPRITE_DIRECT,x	; Get direction bit
		BEQ ?left
?right:		LDA !SPRITE_ANIMID,x
		PHX
		TAX
		ASL
		TAY			; Get tile count index to X
		LDA GFX_ANIMATIONS,y	; Load animation pointer
		ADC GFX_TILECNT,x	; Add tile count offset
		PLX
		STA !SPRITE_ANIMLO,x	; Store to animation pointer lowbyte
		BRA ?highByte
?left:		LDA !SPRITE_ANIMID,x
		ASL
		TAY			; Get animation pointer index to Y
		LDA GFX_ANIMATIONS,y	; Load animation pointer
		STA !SPRITE_ANIMLO,x
?highByte:	INY
		LDA GFX_ANIMATIONS,y	; Get animation pointer highbyte
		ADC #$00		; Might need to add in carry
		STA !SPRITE_ANIMHI,x
		PLY
endmacro
macro	updateDirectionBit()		; Update direction bit to face Mario, (0 = Mario on left side, 1 = Mario on right side)
		STZ !SPRITE_DIRECT,x
		LDA !MARIO_XPOSLO
		SEC
		SBC !SPRITE_XPOSLO,x
		LDA !MARIO_XPOSHI
		SBC !SPRITE_XPOSHI,x
		BMI ?keepNoBit
		LDA #$01
		STA !SPRITE_DIRECT,x
?keepNoBit:
endmacro
macro	flipDirectionBit()		; Flips the direction bit
		LDA !SPRITE_DIRECT,x
		EOR #$01
		STA !SPRITE_DIRECT,x
endmacro

macro	setSpeedX(newval)		; Sets the sprite's x speed (newspeed = newval)
		LDA <newval>
		STA !SPRITE_XSPEED,x
endmacro
macro	setSpeedY(newval)		; Sets the sprite's y speed (newspeed = newval)
		LDA <newval>
		STA !SPRITE_YSPEED,x
endmacro
macro	setSpeedXA()			; Sets the sprite's x speed (newspeed = accumulator)
		STA !SPRITE_XSPEED,x
endmacro
macro	setSpeedYA()			; Sets the sprite's y speed (newspeed = accumulator)
		STA !SPRITE_YSPEED,x
endmacro
macro	setSpeeds(x, y)			; Sets the sprite's speed to the given values
		%setSpeedX(<x>)
		%setSpeedY(<y>)
endmacro
macro	setSpeedXRelative(newval)	; Modifies the sprite's x speed relative to its current speed (newspeed = oldspeed + newval)
		LDA !SPRITE_XSPEED,x
		CLC
		ADC <newval>
		STA !SPRITE_XSPEED,x
endmacro
macro	setSpeedYRelative(newval)	; Modifies the sprite's y speed relative to its current speed (newspeed = oldspeed + newval)
		LDA !SPRITE_YSPEED,x
		CLC
		ADC <newval>
		STA !SPRITE_YSPEED,x
endmacro
macro	setSpeedXRelativeA()		; Modifies the sprite's x speed relativ to its current speed (newspeed = oldspeed + accumulator value)
		CLC
		ADC !SPRITE_XSPEED,x
		STA !SPRITE_XSPEED,x
endmacro
macro	setSpeedYRelativeA()		; Modifies the sprite's x speed relativ to its current speed (newspeed = oldspeed + accumulator value)
		CLC
		ADC !SPRITE_YSPEED,x
		STA !SPRITE_YSPEED,x
endmacro
macro	setSpeedsRelative(x, y)		; Modifies the sprite's speed relative to its current speed
		%setSpeedXRelative(<x>)
		%setSpeedYRelative(<y>)
endmacro
macro	incSpeedX()			; Increases sprite's x speed by one
		LDA !SPRITE_XSPEED,x
		INC A
		STA !SPRITE_XSPEED,x
endmacro
macro	incSpeedY()			; Increases sprite's y speed by one
		LDA !SPRITE_YSPEED,x
		INC A
		STA !SPRITE_YSPEED,x
endmacro
macro	decSpeedX()			; Decreases sprite's x speed by one
		LDA !SPRITE_XSPEED,x
		DEC A
		STA !SPRITE_XSPEED,x
endmacro
macro	decSpeedY()			; Decreases sprite's y speed by one
		LDA !SPRITE_YSPEED,x
		DEC A
		STA !SPRITE_YSPEED,x
endmacro
macro	setAccelX(newval)		; Sets the sprite's x acceleration (newaccel = newval)
		LDA <newval>
		STA !SPRITE_XACCEL,x
endmacro
macro	setAccelY(newval)		; Sets the sprite's y acceleration (newaccel = newval)
		LDA <newval>
		STA !SPRITE_YACCEL,x
endmacro
macro	setAccelXA()			; Sets the sprite's x acceleration (newaccel = accumulator)
		STA !SPRITE_XACCEL,x
endmacro
macro	setAccelYA()			; Sets the sprite's y acceleration (newaccel = accumulator)
		STA !SPRITE_YACCEL,x
endmacro
macro	setAccels(x, y)			; Sets the sprite's acceleration to the given values
		%setAccelX(<x>)
		%setAccelY(<y>)
endmacro
macro	setAccelXRelative(newval)	; Modifies the sprite's x acceleration relative to its current one (newaccel = oldaccel + newval)
		LDA !SPRITE_XACCEL,x
		CLC
		ADC <newval>
		STA !SPRITE_XACCEL,x
endmacro
macro	setAccelYRelative(newval)	; Modifies the sprite's y acceleration relative to its current one (newaccel = oldaccel + newval)
		LDA !SPRITE_YACCEL,x
		CLC
		ADC <newval>
		STA !SPRITE_YACCEL,x
endmacro
macro	setAccelXRelativeA()		; Modifies the sprite's x acceleration relative to its current one (newaccel = oldaccel + accumulator value)
		CLC
		ADC !SPRITE_XACCEL,x
		STA !SPRITE_XACCEL,x
endmacro
macro	setAccelYRelativeA()		; Modifies the sprite's y acceleration relative to its current one (newaccel = oldaccel + accumulator value)
		CLC
		ADC !SPRITE_YACCEL,x
		STA !SPRITE_YACCEL,x
endmacro
macro	setAccelsRelative(x, y)		; Modifies the sprite's acceleration relative to its current acceleration
		%setAccelXRelative(<x>)
		%setAccelYRelative(<y>)
endmacro

macro	incAccelX()			; Increases sprite's x acceleration by one
		LDA !SPRITE_XACCEL,x
		INC A
		STA !SPRITE_XACCEL,x
endmacro
macro	incAccelY()			; Increases sprite's y acceleration by one
		LDA !SPRITE_YACCEL,x
		INC A
		STA !SPRITE_YACCEL,x
endmacro
macro	decAccelX()			; Decreases sprite's x acceleration by one
		LDA !SPRITE_XACCEL,x
		DEC A
		STA !SPRITE_XACCEL,x
endmacro
macro	decAccelY()			; Decreases sprite's y acceleration by one
		LDA !SPRITE_YACCEL,x
		DEC A
		STA !SPRITE_YACCEL,x
endmacro
macro	flipSpeedX()			; Flips the x speed
		LDA !SPRITE_XSPEED,x
		EOR #$FF
		INC A
		STA !SPRITE_XSPEED,x
endmacro
macro	flipSpeedY()			; Flips the y speed
		LDA !SPRITE_YSPEED,x
		EOR #$FF
		INC A
		STA !SPRITE_YSPEED,x
endmacro
macro	flipAccelX()			; Flips the x acceleration
		LDA !SPRITE_XACCEL,x
		EOR #$FF
		INC A
		STA !SPRITE_XACCEL,x
endmacro
macro	flipAccelY()			; Flips the y acceleration
		LDA !SPRITE_YACCEL,x
		EOR #$FF
		INC A
		STA !SPRITE_YACCEL,x
endmacro
macro	clearSpeedX()			; Resets the x speed
		STZ !SPRITE_XSPEED,x
endmacro
macro	clearSpeedY()			; Resets the y speed
		STZ !SPRITE_YSPEED,x
endmacro
macro	clearSpeeds()			; Resets both speeds
		STZ !SPRITE_XSPEED,x
		STZ !SPRITE_YSPEED,x
endmacro
macro	clearAccelX()			; Resets the x acceleration
		STZ !SPRITE_XACCEL,x
endmacro
macro	clearAccelY()			; Resets the y acceleration
		STZ !SPRITE_YACCEL,x
endmacro
macro	clearAccels()			; Resets both accelerations
		STZ !SPRITE_XACCEL,x
		STZ !SPRITE_YACCEL,x
endmacro
macro	updateSpeedX()			; Updates the sprites x speed according to the current x acceleration
		LDA !SPRITE_XSPEED,x
		CLC
		ADC !SPRITE_XACCEL,x
		STA !SPRITE_XSPEED,x
endmacro
macro	updateSpeedY()			; Updates the sprites y speed according to the current y acceleration
		LDA !SPRITE_YSPEED,x
		CLC
		ADC !SPRITE_YACCEL,x
		STA !SPRITE_YSPEED,x
endmacro
macro	updateSpeeds()			; Updates the sprites speed according to the current acceleration
		%updateSpeedX()
		%updateSpeedY()
endmacro

macro	moveSpriteGravity()		; Moves the sprite according to the current speed and gravity
		JSL $01802A
endmacro
macro	moveSpriteNoGravity()		; Moves the sprite according to the current speed
		JSL $01801A
		JSL $018022
endmacro

macro	hurtMario()			; Hurts Mario
		JSL $00F5B7
endmacro
macro	killMario()			; Kills Mario directly
		JSL $00F606
endmacro
macro	givePoints()			; Gives Mario points
		LDA !POINTS_NORMAL
		INC A
		CLC
		ADC !POINTS_INC,x
		CMP #$08
		BCC ?noLimit
		LDA #$08
		STA !POINTS_NORMAL
		BRA ?noSound
?noLimit:	STA !POINTS_NORMAL
		CLC
		ADC #$12
		STA $1DF9
?noSound:	LDA !POINTS_NORMAL
		JSL $02ACE5
endmacro
macro	giveStarPoints()		; Gives Mario points, when he has a star
		LDA !POINTS_STAR
		INC A
		CMP #$08
		BCC ?noLimit
		LDA #$08
		STA !POINTS_STAR
		BRA ?noSound
?noLimit:	STA !POINTS_STAR
		CLC
		ADC #$12
		STA $1DF9
?noSound:	LDA !POINTS_STAR
		JSL $02ACE5
endmacro
macro	handleMarioContact()		; Displays contact graphics and prevents Mario from hitting the sprite again
		JSL $01AB99
		JSL $01AA33
endmacro
macro	handleSpriteContact()		; Needs to be called to update other sprites which hit this sprite
		JSL $018032
endmacro
macro	showContactStar()		; Displays the star contact graphics
		JSL $07FC3B
endmacro
macro	showContactSmoke()		; Displays the smoke contact graphics
		LDY #$03
?loop:		LDA $17C0,y
		BEQ ?found
		DEY
		BPL ?loop
		BRA ?end
?found:		LDA #$01
		STA $17C0,y
		LDA #$1F
		STA $17CC,y
		LDA !SPRITE_YPOSLO,x
		STA $17C4,y
		LDA !SPRITE_XPOSLO,x
		STA $17C8,y
?end:
endmacro
macro	stayDead()			; Marks the sprite dead, will not respawn once offscreen
		LDA #$FF
		STA $161A,x
endmacro
macro	callForInteractSprite(target)	; Calls the subroutine at target for ever interacting sprite, subroutine needs to preserve X, Y and $0E
		LDA !SPRITE_INTERT,x
		BNE ?return
		STX $0E
		LDY #$0B
?loop:		LDA !SPRITE_STATUS,y
		BEQ ?next
		CPY $0E
		BEQ ?next
		LDA !SPRITE_TWEAK5,y
		AND #$08
		BNE ?next
		TYX
		JSL $03B6E5
		LDX $0E
		JSL $03B69F
		JSL $03B72B
		BCS ?call
?next:		DEY
		BPL ?loop
		BRA ?return
?call:		LDA #!INTERACT_TIME
		STA !SPRITE_INTERT,x
		JSR <target>
		BRA ?next
?return:
endmacro

macro	playSoundEffect(cha, eff)	; Plays a sound effect on the specified sound channel
		LDA <eff>
		STA <cha>
endmacro
macro	useDefaultRoutine()		; Makes the sprite use the default sprite routine
		LDA !SPRITE_EXTRA2,x
		AND #$7F
		STA !SPRITE_EXTRA2,x
endmacro

macro	changeToSprite(num)		; Changes the current sprite to the new one
		LDA <num>
		%changeToSpriteA()
endmacro
macro	changeToSpriteA()		; Changes the current sprite to the new one in the accumulator
		STA !SPRITE_NUMSTD,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		JSL $07F7D2
endmacro
macro	changeToCustomSprite(num)	; Changes the current custom sprite to the new one
		LDA <num>
		%changeToCustomSpriteA()
endmacro
macro	changeToCustomSpriteA()		; Changes the current custom sprite to the new one in the accumulator
		STA !SPRITE_NUMCUS,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
endmacro
macro	spawnSprite(n, xl, xh, yl, yh)	; Spawns the new sprite n, at coordinates (xl,xh) (yl,yh)
		JSL $02A9DE
		BMI ?return
		PHX
		TYX
		LDA <n>
		STA !SPRITE_NUMSTD,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA <xl>
		STA !SPRITE_XPOSLO,x
		LDA <xh>
		STA !SPRITE_XPOSHI,x
		LDA <yl>
		STA !SPRITE_YPOSLO,x
		LDA <yh>
		STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		PLX
endmacro
macro	spawnSpriteA(xl, xh, yl, yh)	; Spawns the new sprite in the accumulator, at coordinates (xl,xh) (yl,yh)
		STA $00
		JSL $02A9DE
		BMI ?return
		PHX
		TYX
		LDA $00
		STA !SPRITE_NUMSTD,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA <xl>
		STA !SPRITE_XPOSLO,x
		LDA <xh>
		STA !SPRITE_XPOSHI,x
		LDA <yl>
		STA !SPRITE_YPOSLO,x
		LDA <yh>
		STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		PLX
?return:
endmacro
macro	spawnSpriteRelative(n, x, y)	; Spawns a new sprite n, at coordinates relative to the current sprite's position
		JSL $02A9DE
		BMI ?return
		PHX
		PHX
		TYX
		PLY
		LDA <n>
		STA !SPRITE_NUMSTD,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA.w !SPRITE_XPOSLO,y
		CLC
		ADC <x>
		STA !SPRITE_XPOSLO,x
?calculateX:	LDA <x>
		BMI ?negativeX
?positiveX:	LDA !SPRITE_XPOSHI,y
		ADC #$00
		BRA ?calculateY
?negativeX:	LDA !SPRITE_XPOSHI,y
		SBC #$00
?calculateY:	STA !SPRITE_XPOSHI,x
		LDA.w !SPRITE_YPOSLO,y
		CLC
		ADC <y>
		STA !SPRITE_YPOSLO,x
		LDA <y>
		BMI ?negativeY
?positiveY:	LDA !SPRITE_YPOSHI,y
		ADC #$00
		BRA ?createSprite
?negativeY:	LDA !SPRITE_YPOSHI,y
		SBC #$00
?createSprite:	STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		PLX
?return:
endmacro
macro	spawnSpriteRelativeA(x, y)	; Spawns a new sprite in A, at coordinates relative to the current sprite's position
		STA $00
		JSL $02A9DE
		BMI ?return
		PHX
		PHX
		TYX
		PLY
		LDA $00
		STA !SPRITE_NUMSTD,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA.w !SPRITE_XPOSLO,y
		CLC
		ADC <x>
		STA !SPRITE_XPOSLO,x
?calculateX:	LDA <x>
		BMI ?negativeX
?positiveX:	LDA !SPRITE_XPOSHI,y
		ADC #$00
		BRA ?calculateY
?negativeX:	LDA !SPRITE_XPOSHI,y
		SBC #$00
?calculateY:	STA !SPRITE_XPOSHI,x
		LDA.w !SPRITE_YPOSLO,y
		CLC
		ADC <y>
		STA !SPRITE_YPOSLO,x
		LDA <y>
		BMI ?negativeY
?positiveY:	LDA !SPRITE_YPOSHI,y
		ADC #$00
		BRA ?createSprite
?negativeY:	LDA !SPRITE_YPOSHI,y
		SBC #$00
?createSprite:	STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		PLX
?return:
endmacro
macro	spawnCustomSprite(n, xl, xh, yl, yh)	; Spawns a custom sprite n at the position (xl,xh) (yl,yh)
		JSL $02A9DE
		BMI ?return
		PHX
		TYX
		LDA <n>
		STA !SPRITE_NUMCUS,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA <xl>
		STA !SPRITE_XPOSLO,x
		LDA <xh>
		STA !SPRITE_XPOSHI,x
		LDA <yl>
		STA !SPRITE_YPOSLO,x
		LDA <yh>
		STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX
?return:
endmacro
macro	spawnCustomSpriteA(xl, xh, yl, yh)		; Spawns a custom sprite in A at the position (xl,xh) (yl,yh)
		STA $00
		JSL $02A9DE
		BMI ?return
		PHX
		TYX
		LDA $00
		STA !SPRITE_NUMCUS,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA <xl>
		STA !SPRITE_XPOSLO,x
		LDA <xh>
		STA !SPRITE_XPOSHI,x
		LDA <yl>
		STA !SPRITE_YPOSLO,x
		LDA <yh>
		STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX
?return:
endmacro
macro	spawnCustomSpriteRelative(n, x, y)	; Spawns a new custom sprite n, at coordinates relative to the current sprite's position
		JSL $02A9DE
		BMI ?return
		PHX
		PHX
		TYX
		PLY
		LDA <n>
		STA !SPRITE_NUMCUS,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA.w !SPRITE_XPOSLO,y
		CLC
		ADC <x>
		STA !SPRITE_XPOSLO,x
?calculateX:	LDA <x>
		BMI ?negativeX
?positiveX:	LDA !SPRITE_XPOSHI,y
		ADC #$00
		BRA ?calculateY
?negativeX:	LDA !SPRITE_XPOSHI,y
		SBC #$00
?calculateY:	STA !SPRITE_XPOSHI,x
		LDA.w !SPRITE_YPOSLO,y
		CLC
		ADC <y>
		STA !SPRITE_YPOSLO,x
		LDA <y>
		BMI ?negativeY
?positiveY:	LDA !SPRITE_YPOSHI,y
		ADC #$00
		BRA ?createSprite
?negativeY:	LDA !SPRITE_YPOSHI,y
		SBC #$00
?createSprite:	STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX
?return:
endmacro
macro	spawnCustomSpriteRelativeA(x, y)		; Spawns a new custom sprite in A, at coordinates relative to the current sprite's position
		STA $00
		JSL $02A9DE
		BMI ?return
		PHX
		PHX
		TYX
		PLY
		LDA $00
		STA !SPRITE_NUMCUS,x
		LDA #!STATUS_INIT
		STA !SPRITE_STATUS,x
		LDA.w !SPRITE_XPOSLO,y
		CLC
		ADC <x>
		STA !SPRITE_XPOSLO,x
?calculateX:	LDA <x>
		BMI ?negativeX
?positiveX:	LDA !SPRITE_XPOSHI,y
		ADC #$00
		BRA ?calculateY
?negativeX:	LDA !SPRITE_XPOSHI,y
		SBC #$00
?calculateY:	STA !SPRITE_XPOSHI,x
		LDA.w !SPRITE_YPOSLO,y
		CLC
		ADC <y>
		STA !SPRITE_YPOSLO,x
		LDA <y>
		BMI ?negativeY
?positiveY:	LDA !SPRITE_YPOSHI,y
		ADC #$00
		BRA ?createSprite
?negativeY:	LDA !SPRITE_YPOSHI,y
		SBC #$00
?createSprite:	STA !SPRITE_YPOSHI,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX
?return:
endmacro
macro	spawnExtendedSprite(n, xl, xh, yl, yh, xs, ys)	; Spawns an extended sprite n at position (xl,xh) (yl,yh), with speed (xs,ys)
		LDY #$07
?loop:		LDA !EXTEND_NUMBER,y
		BEQ ?foundSlot
		DEY
		BPL ?loop
		BRA ?return
?foundSlot:	LDA <n>
		STA !EXTEND_NUMBER,y
		LDA <xs>
		STA !EXTEND_XSPEED,y
		LDA <ys>
		STA !EXTEND_YSPEED,y
		LDA <xl>
		STA !EXTEND_XPOSLO,y
		LDA <xh>
		STA !EXTEND_XPOSHI,y
		LDA <yl>
		STA !EXTEND_YPOSLO,y
		LDA <yh>
		STA !EXTEND_YPOSHI,y
?return:	
endmacro
macro	spawnExtendedSpriteA(xl, xh, yl, yh, xs, ys)	; Spawns an extended sprite in A at position (xl,xh) (yl,yh), with speed (xs,ys)
		STA $00
		LDY #$07
?loop:		LDA !EXTEND_NUMBER,y
		BEQ ?foundSlot
		DEY
		BPL ?loop
		BRA ?return
?foundSlot:	LDA $00
		STA !EXTEND_NUMBER,y
		LDA <xs>
		STA !EXTEND_XSPEED,y
		LDA <ys>
		STA !EXTEND_YSPEED,y
		LDA <xl>
		STA !EXTEND_XPOSLO,y
		LDA <xh>
		STA !EXTEND_XPOSHI,y
		LDA <yl>
		STA !EXTEND_YPOSLO,y
		LDA <yh>
		STA !EXTEND_YPOSHI,y
?return:
endmacro
macro	spawnExtendedSpriteRelative(n, x, y, xs, ys)	; Spawns an extended sprite n relative to the current sprites position, extended sprite index in y
		LDY #$07
?loop:		LDA !EXTEND_NUMBER,y
		BEQ ?foundSlot
		DEY
		BPL ?loop
		BRA ?return
?foundSlot:	LDA <n>
		STA !EXTEND_NUMBER,y
		LDA <xs>
		STA !EXTEND_XSPEED,y
		LDA <ys>
		STA !EXTEND_YSPEED,y
		LDA.w !SPRITE_XPOSLO,x
		CLC
		ADC <x>
		STA !EXTEND_XPOSLO,y
?calculateX:	LDA <x>
		BMI ?negativeX
?positiveX:	LDA !SPRITE_XPOSHI,x
		ADC #$00
		BRA ?calculateY
?negativeX:	LDA !SPRITE_XPOSHI,x
		SBC #$00
?calculateY:	STA !EXTEND_XPOSHI,y
		LDA.w !SPRITE_YPOSLO,x
		CLC
		ADC <y>
		STA !EXTEND_YPOSLO,y
		LDA <y>
		BMI ?negativeY
?positiveY:	LDA !SPRITE_YPOSHI,x
		ADC #$00
		BRA ?createSprite
?negativeY:	LDA !SPRITE_YPOSHI,x
		SBC #$00
?createSprite:	STA !EXTEND_YPOSHI,y
?return:	
endmacro
macro	spawnExtendedSpriteRelativeA(x, y, xs, ys)	; Spawns an extended sprite in A relative to the current sprites position
		STA $00
		LDY #$07
?loop:		LDA !EXTEND_NUMBER,y
		BEQ ?foundSlot
		DEY
		BPL ?loop
		BRA ?return
?foundSlot:	LDA $00
		STA !EXTEND_NUMBER,y
		LDA <xs>
		STA !EXTEND_XSPEED,y
		LDA <ys>
		STA !EXTEND_YSPEED,y
		LDA.w !SPRITE_XPOSLO,x
		CLC
		ADC <x>
		STA !EXTEND_XPOSLO,y
?calculateX:	LDA <x>
		BMI ?negativeX
?positiveX:	LDA !SPRITE_XPOSHI,x
		ADC #$00
		BRA ?calculateY
?negativeX:	LDA !SPRITE_XPOSHI,x
		SBC #$00
?calculateY:	STA !EXTEND_XPOSHI,y
		LDA.w !SPRITE_YPOSLO,x
		CLC
		ADC <y>
		STA !EXTEND_YPOSLO,y
		LDA <y>
		BMI ?negativeY
?positiveY:	LDA !SPRITE_YPOSHI,x
		ADC #$00
		BRA ?createSprite
?negativeY:	LDA !SPRITE_YPOSHI,x
		SBC #$00
?createSprite:	STA !EXTEND_YPOSHI,y
?return:
endmacro
