; ==========================
; SECRET OF MANA CRAB SPRITE
;     PINCER PROJECTILE
;	by Kickchon
; ==========================

; ==================
; SPRITE TABLE USAGE
; ==================

; $C2,x				used for x center lo
; $1504,x			used for y center lo
; $1510,x			used for x center hi
; $151C,x			used for y center hi
; $1528,x			used to limit dead speed
; $1540,x	autodec		used to kill the pincer
; $154C,x	autodec
; $1558,x	autodec

; $1534,x			used by kick_engine
; $1570,x			used by kick_engine
; $157C,x			used by kick_engine
; $1594,x			used by kick_engine
; $15AC,x	autodec		used by kick_engine
; $1602,x			used by kick_engine
; $160E,x			used by kick_engine
; $163E,x	autodec		used by kick_engine
; $187B,x			used by kick_engine
; $1FD6,x			used by kick_engine

; ========================
; KICKCHON's SPRITE ENGINE
; ========================

incsrc sprites/kick_engine_top.asm

; ================
; SPRITE CONSTANTS
; ================

!PINCER_TIME	=	$32
!PINCER_RADIUS	=	$20
!PINCER_INITSP	=	$80

!PINCER_DEADT	=	$1540
!PINCER_DEADS	=	$1528
!PINCER_XCENLO	=	$C2
!PINCER_YCENLO	=	$1504
!PINCER_XCENHI	=	$1510
!PINCER_YCENHI	=	$151C

; ==================================
; KICKCHON's SPRITE ENGINE CONSTANTS
; ==================================

; The states the sprite may be in, should be an enumeration starting from $00

!STATE_MOVE0	=	$00
!STATE_MOVE1	=	$01
!STATE_DEAD	=	$02

; The table for the state durations, only relevant if automatic state change is activated for the state

STATE_TIMES:	db	$08, $08, $FF

; The table which defines if the automatic state change should be used, $00 means no automatic change, $01 equals automatic change after time runs out

STATE_USETIMES:	db	$01, $01, $00

; The table for the sprite's state processing functions, (main, init, finalize and death state)

STATE_FUNCS:	dw	SPRITE_MOVE, SPRITE_MOVE, SPRITE_DEAD
STATE_INITFNCS:	dw	SPRITE_MOVE0I, SPRITE_MOVE1I, SPRITE_DEADI
STATE_FINALFNC:	dw	SPRITE_FINISH, SPRITE_FINISH, SPRITE_FINISH
!STATE_DEADFNC	=	!STATE_DEAD

; The Animations the sprite has, should be an enumeration starting from $00

!ANIM_HORZ	=	$00
!ANIM_VERT	=	$01

; Tile render mode, set to $02 for 16x16 tiles, $00 for 8x8 tiles

!GFX_TILEFMT	=	$00

; Table for the amount of tiles in the tilemaps, needs to be amount of tiles * 3

GFX_TILECNT:	db	$06, $06

; The tilemaps for the sprite, format: <tile>, <xoffset>, <yoffset>, [<tile>, <xoffset>, <yoffset>, ...][, optional flipped values (those are not counted)]

GFX_HORZ:	db	$EF,$00,$04, $FF,$08,$04
GFX_VERT:	db	$EE,$04,$00, $FE,$04,$08

; The animation table

GFX_ANIMATIONS:	dw	GFX_HORZ, GFX_VERT

; =============================
; GLOBAL SPRITE INITIALIZIATION
; =============================

GLOBAL_INIT:	%initialState(#!STATE_MOVE0)
		LDA #!PINCER_TIME
		STA !PINCER_DEADT,x

		LDA !SPRITE_XPOSLO,x
		STA !PINCER_XCENLO,x
		LDA !SPRITE_XPOSHI,x
		STA !PINCER_XCENHI,x

		LDA !SPRITE_YPOSLO,x
		SEC
		SBC #!PINCER_RADIUS
		STA !PINCER_YCENLO,x
		LDA !SPRITE_YPOSHI,x
		SBC #$00
		STA !PINCER_YCENHI,x

		%setSpeedX(#!PINCER_INITSP)
		%jumpMarioRightSide(INIT_FLIP)
		RTS
INIT_FLIP:	%flipSpeedX()
		RTS

; =====================
; COMMON STATE FUNCTION
; =====================

COMMON_MAIN:	LDA !PINCER_DEADT,x
		BNE MAIN_CONTACT
		%changeSMWSpriteStatus(#!STATUS_NOSPRT)
		RTS
MAIN_CONTACT:	%jumpNoMarioContact(RETURN)
		%jumpMarioHittingAbove(MAIN_SPIN)
		%jumpMarioHasStar(MAIN_STARLOSE)
		%jumpMarioSliding(MAIN_SLIDLOSE)
MAIN_HURT:	%hurtMario()
		RTS
MAIN_SPIN:	%jumpMarioSpinJump(MAIN_BOUNCE)
		BRA MAIN_HURT
MAIN_BOUNCE:	%handleMarioContact()
		RTS
MAIN_STARLOSE:	%giveStarPoints()
		BRA MAIN_LOSE
MAIN_SLIDLOSE:	%givePoints()
MAIN_LOSE:	%changeToDeathState()
RETURN:		RTS

; =====================================
; SPRITE STATE INITIALIZATION FUNCTIONS
; =====================================

SPRITE_MOVE0I:	LDX !SPRITE_CINDEX
		%changeAnimation(#!ANIM_VERT)
		%setNextState(#!STATE_MOVE1)
		RTS

SPRITE_MOVE1I:	LDX !SPRITE_CINDEX
		%changeAnimation(#!ANIM_HORZ)
		%setNextState(#!STATE_MOVE0)
		RTS

SPRITE_DEADI:	LDX !SPRITE_CINDEX
		%stayDead()
		%setAccelY(#$04)
		%setSpeedY(#$D0)
		%setSpeedX(#$10)
		%jumpMarioRightSide(DEADI_FLIP)
		RTS
DEADI_FLIP:	%flipSpeedX()
		RTS

; ===================================
; SPRITE STATE FINALIZATION FUNCTIONS
; ===================================

SPRITE_FINISH:	LDX !SPRITE_CINDEX
		RTS

; ======================
; SPRITE STATE FUNCTIONS
; ======================

SPRITE_MOVE:	LDX !SPRITE_CINDEX
		LDA !PINCER_XCENHI,x
		SEC
		SBC !SPRITE_XPOSHI,x
		LDA !PINCER_XCENLO,x
		SBC !SPRITE_XPOSLO,x
		BMI MOVE_NEGX
		LSR
		LSR
		BRA MOVE_SETX
MOVE_NEGX:	LSR
		LSR
		ORA #$C0
MOVE_SETX:	%setAccelXA()
		LDA !PINCER_YCENHI,x
		SEC 
		SBC !SPRITE_YPOSHI,x
		LDA !PINCER_YCENLO,x
		SBC !SPRITE_YPOSLO,x
		BMI MOVE_NEGY
		LSR
		LSR
		BRA MOVE_SETY
MOVE_NEGY:	LSR
		LSR
		ORA #$C0
MOVE_SETY:	%setAccelYA()
		%updateSpeeds()
		%moveSpriteNoGravity()
		RTS

SPRITE_DEAD:	LDX !SPRITE_CINDEX
		LDA !SPRITE_YSPEED,x
		EOR #$80
		STA !PINCER_DEADS,x
		%updateSpeedY()
		AND !PINCER_DEADS,x
		AND #$80
		BEQ DEAD_MOVE
		%flipAccelY()
		%updateSpeedY()
		%clearAccelY()
DEAD_MOVE:	%moveSpriteNoGravity()
		RTS


; ========================
; KICKCHON's SPRITE ENGINE
; ========================

incsrc sprites/kick_engine_bottom.asm