;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Hopping Snift
;; Exactly like MikeyK's version, but hops a little bit. It has 
;; most of the same settings. 
;; The extra bit determines if it should face Mario.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	UpdateSpritePos = $01802A  
	SprSprInteract = $018032   
	MarioSprInteract = $01A7DC
	GetSpriteClippingA = $03B69F
	CheckForContact = $03B72B
	DisplayContactGfx = $01AB99 
	FinishOAMWrite = $01B7B3
	ShowSpinJumpStars = $07FC3B
	GivePoints = $02ACE5
	HurtMario = $00F5B7

	EXTRA_BITS 	= $7FAB10	
	ExtraProperty1	= $7FAB28
	ExtraProperty2	= $7FAB34
	
	RAM_ThrowTimer = $1504
	RAM_SpriteDir = $157C
	RAM_SprTurnTimer = $15AC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "INIT"
	LDA $167A,x
	STA $1528,x
	LDA #$40
	STA $C2,x
	LDA #$01		
	STA $151C,x

        JSR SubHorzPos
        TYA
        STA $157C,x

        TXA
        AND #$03
        ASL A
        ASL A
        ASL A
        ASL A
        ASL A
        ASL A
        STA RAM_ThrowTimer,x
                    
        RTL                 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "MAIN"                                    
        PHB                  
        PHK                  
        PLB                  
	CMP #09
	BCS HandleStunnd
        JSR SpriteMainSub    
        PLB                  
        RTL

HandleStunnd:
	LDA $167A,x		; Set to interact with Mario
	AND #$7F
	STA $167A,x
	
	LDY $15EA,X		; Replace Goomba's tile
	PHX
	LDX Tilemap
	LDA $302,Y
	CMP #$A8
	BEQ SetTile
	LDX Tilemap+1
SetTile:
	TXA
 	STA $302,Y
	PLX	
	PLB
	RTL
	
DecrementTimers:
	LDA RAM_ThrowTimer,x
	BEQ DoneDec
	DEC RAM_ThrowTimer,x
DoneDec:
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	X_SPEED             dcb $08,$F8	
        SPRITE_GRAVITY = $20
        TIME_TO_SHOW = $28      ;time to display the boomerang before it is thrown
        TIME_TILL_THROW = $A8
Return:
	RTS
SpriteMainSub:
	LDA EXTRA_BITS,x
	AND #$04
	BEQ CLEAR
	JSR SUB_GFX2	; graphics routine when extra bit is set
	BRA CONTINUE
CLEAR	JSR SUB_GFX	; graphics routine when extra bit is clear
CONTINUE

        LDA $9D                 ; \ if sprites locked, return
        BNE Return              ; /
	LDA $14C8,x
	CMP #$08
	BNE Return
        JSR SubOffScreen	; handle off screen situation
	INC $1570,x

	LDA RAM_ThrowTimer,x             
        CMP #TIME_TO_SHOW  
	BCS WalkingState
	
WalkingState:	
        LDA $14                 ; Set walking frame based on frame counter
        LSR A                   
        LSR A                   
        LSR A                   
        CLC                    
        ADC $15E9              
        AND #$01               
        STA $1602,x                   

	LDA EXTRA_BITS,x
	AND #$04
	BEQ CONTINUE2
X_TIME
	LDY $157C,x             ; \ set x speed based on direction
	LDA X_SPEED,y           ;  |
	STA $B6,x               ; /
	BRA CONTINUE2
CONTINUE2
	JSR Hop		; Jump to custom code

	JSL UpdateSpritePos     ; Update position based on speed values

	LDA $1588,x             ; if on the ground, reset the turn counter
        AND #$04
        BEQ NotOnGround
	JSR DecrementTimers	
	STZ $AA,x
	STZ $151C,x		; Reset turning flag (used if sprite stays on ledges)

NotOnGround:	
	
 	
SharedCode:	
	LDA $1528,x
	STA $167A,x
	
        JSL SprSprInteract	; Interact with other sprites
	JSL MarioSprInteract	; Check for mario/sprite contact (carry set = contact)
        BCC Return1             ; return if no contact
	
        JSR SubVertPos          ; \
        LDA $0E                 ;  | if mario isn't above sprite, and there's vertical contact...
        CMP #$E6                ;  |     ... sprite wins
        BPL SpriteWins          ; /
        LDA $7D                 ; \ if mario speed is upward, return
        BMI Return1             ; /

        LDA ExtraProperty1,x 	; Check property byte to see if sprite can be spin jumped
        AND #$10
        BEQ SpinKillDisabled    
        LDA $140D               ; Branch if mario is spin jumping
        BNE SpinKill
SpinKillDisabled:	
	LDA $187A
	BNE RideSprite
	LDA ExtraProperty1,x
	AND #$20
	BEQ RideSprite
	BIT $16		        ; Don't pick up sprite if not pressing button
        BVC RideSprite
	LDA #$0B		; Sprite status = Carried
	STA $14C8,x
	LDA #$FF		; Set time until recovery
	STA $1540,x
	RTS

RideSprite:	
	LDA #$01                ; \ set "on sprite" flag
        STA $1471               ; /
        LDA #$06                ; Disable interactions for a few frames
        STA $154C,x             
        STZ $7D                 ; Y speed = 0
        LDA #$E1                ; \
        LDY $187A               ;  | mario's y position += E1 or D1 depending if on yoshi
        BEQ NO_YOSHI            ;  |
        LDA #$D1                ;  |
NO_YOSHI:
	CLC                     ;  |
        ADC $D8,x               ;  |
        STA $96                 ;  |
        LDA $14D4,x             ;  |
        ADC #$FF                ;  |
        STA $97                 ; /
        LDY #$00                ; \ 
        LDA $1491               ;  | $1491 == 01 or FF, depending on direction
        BPL LABEL9              ;  | set mario's new x position
        DEY                     ;  |
LABEL9:
	CLC                     ;  |
        ADC $94                 ;  |
        STA $94                 ;  |
        TYA                     ;  |
        ADC $95                 ;  |
        STA $95                 ; /
        RTS                     

SpriteWins:
	LDA $154C,x             ; \ if disable interaction set...
        ORA $15D0,x             ;  |   ...or sprite being eaten...
        BNE Return1             ; /   ...return
        LDA $1490               ; Branch if Mario has a star
        BNE MarioHasStar        
        JSL HurtMario	
Return1:
	RTS                    

SpinKill:
	JSR SUB_STOMP_PTS       ; give mario points
	LDA #$F8	        ; Set Mario Y speed
	STA $7D
        JSL DisplayContactGfx   ; display contact graphic
        LDA #$04                ; \ status = 4 (being killed by spin jump)
        STA $14C8,x             ; /   
        LDA #$1F                ; \ set spin jump animation timer
        STA $1540,x             ; /
        JSL ShowSpinJumpStars
        LDA #$08                ; \ play sound effect
        STA $1DF9               ; /
        RTS                     ; return

MarioHasStar:
        LDA #$02                ; \ status = 2 (being killed by star)
        STA $14C8,x             ; /
        LDA #$D0                ; \ set y speed
        STA $AA,x               ; /
        ;JSR SubHorzPos          ; get new direction
        LDA KILLED_X_SPEED,y    ; \ set x speed based on direction
        STA $B6,x               ; /
        INC $18D2               ; increment number consecutive enemies killed
        LDA $18D2               ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NoReset             ;  |
        LDA #$08                ;  |
        STA $18D2               ; /   
NoReset:
	JSL GivePoints
        LDY $18D2               ; \ 
        CPY #$08                ;  | if consecutive enemies stomped < 8 ...
        BCS NO_SOUND2           ;  |
        LDA STAR_SOUNDS,y       ;  |    ... play sound effect
        STA $1DF9               ; /
NO_SOUND2:
	RTS                     ; final return

KILLED_X_SPEED:
	dcb $F0,$10
Hop:
        LDA #$40		; If timer isn't 40..
        CMP $C2,x		; 
        BNE IncreaseHop	; Increase it.
        LDA $1588,x		; Don't jump if already on ground.
        AND #$04		; 
        BEQ RETURN3	; 
        LDA #$C0		; Set jump height.
        STA $AA,x		; Store it too. 
        STZ $C2,x		; Reset timer.
        RTS		; Return
IncreaseHop:
        INC $C2,x		; Increase timer.
        RETURN3
        RTS		; Return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PROPERTIES: dcb $7F,$3F

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAP:
	dcb $C0,$C2,$E0,$E2 ; ATTACK 1 	;\ RIGHT
	dcb $C0,$C2,$E0,$E2 ; ATTACK 2 	;/ RIGHT
	
	dcb $C0,$C2,$E0,$E2 ; ATTACK 1 	;\ LEFT
	dcb $C0,$C2,$E0,$E2 ; ATTACK 2 	;/ LEFT

YDISP: 	
	dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ RIGHT

	dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ LEFT
	dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ LEFT

XDISP: 	
	dcb $00,$10,$00,$10 ; ATTACK 1 	;\ RIGHT
	dcb $00,$10,$00,$10 ; ATTACK 2 	;/ RIGHT

	dcb $10,$00,$10,$00 ; ATTACK 1 	;\ LEFT
	dcb $10,$00,$10,$00 ; ATTACK 2 	;/ LEFT\

SUB_GFX:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	AND #$01		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

DONE_WALKING:
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE NoAdd
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$08		; | Adding 16 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAdd:
	PHX			;\ Push sprite index ..
	LDX #$03		;/ And load X with number of tiles to loop through.
Loop:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			;\ Get it back into X for an index.

	LDA $00			;\
	CLC			; | Apply X displacement of the sprite.
	ADC XDISP,x		; |
	STA $0300,y		;/ 

	LDA $01			;\
	CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
	ADC YDISP,x		; | Otherwise, both tiles would have been drawn to the same position!
	STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
				;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

	LDA TILEMAP,x
	STA $0302,y

	PHX			; Push number of times to go through loop + "left" displacement if necessary.
	LDX $02			;\
	LDA PROPERTIES,x	; | Set properties based on direction.
	STA $0303,y		;/
	PLX			; Pull number of times to go through loop.

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
		
	PLX			; Pull current tile back.
	DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
				; is drawn, then loop again to draw the first tile.

	BPL Loop		; Loop until X becomes negative (FF).
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
	LDA #$03		; A -> number of tiles drawn - 1.
				; I drew 2 tiles, so 2-1 = 1. A = 01.

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			; Never forget this!       

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PROPERTIES2: dcb $7D,$3D

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAP2:
	dcb $C0,$C2,$E0,$E2 ; ATTACK 1 	;\ RIGHT
	dcb $C0,$C2,$E0,$E2 ; ATTACK 2 	;/ RIGHT

	dcb $C0,$C2,$E0,$E2 ; ATTACK 1 	;\ LEFT
	dcb $C0,$C2,$E0,$E2 ; ATTACK 2 	;/ LEFT

YDISP2: 	
	dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ RIGHT

	dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ LEFT
	dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ LEFT

XDISP2: 	
	dcb $00,$10,$00,$10 ; ATTACK 1 	;\ RIGHT
	dcb $00,$10,$00,$10 ; ATTACK 2 	;/ RIGHT

	dcb $10,$00,$10,$00 ; ATTACK 1 	;\ LEFT
	dcb $10,$00,$10,$00 ; ATTACK 2 	;/ LEFT

SUB_GFX2:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	AND #$01		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

DONE_WALKING2:
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE NoAdd2
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$08		; | Adding 16 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAdd2:
	PHX			;\ Push sprite index ..
	LDX #$03		;/ And load X with number of tiles to loop through.
Loop2:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			;\ Get it back into X for an index.

	LDA $00			;\
	CLC			; | Apply X displacement of the sprite.
	ADC XDISP2,x		; |
	STA $0300,y		;/ 

	LDA $01			;\
	CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
	ADC YDISP2,x		; | Otherwise, both tiles would have been drawn to the same position!
	STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
				;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

	LDA TILEMAP2,x
	STA $0302,y

	PHX			; Push number of times to go through loop + "left" displacement if necessary.
	LDX $02			;\
	LDA PROPERTIES2,x	; | Set properties based on direction.
	STA $0303,y		;/
	PLX			; Pull number of times to go through loop.

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
		
	PLX			; Pull current tile back.
	DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
				; is drawn, then loop again to draw the first tile.

	BPL Loop2		; Loop until X becomes negative (FF).
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
	LDA #$03		; A -> number of tiles drawn - 1.
				; I drew 2 tiles, so 2-1 = 1. A = 01.

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			; Never forget this!            

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubVertPos:
	LDY #$00
        LDA $96
        SEC
        SBC $D8,x
        STA $0F
        LDA $97
        SBC $14D4,x
        BPL VertIncY
        INY
VertIncY:
        RTS   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL HorzIncY
	INY 
HorzIncY:
	RTS 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19
             
SUB_STOMP_PTS:
	PHY                      
        LDA $1697               ; \
        CLC                     ;  | 
        ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
        INC $1697               ; increase consecutive enemies stomped
        TAY                     ;
        INY                     ;
        CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
        BCS NO_SOUND            ; /    ... don't play sound 
        LDA STAR_SOUNDS,y       ; \ play sound effect
        STA $1DF9               ; /   
NO_SOUND:
	TYA                     ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NO_RESET            ;  |
        LDA #$08                ; /
NO_RESET:
	JSL GivePoints
        PLY                     
        RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 	
DATA_03B75C:
	dcb $0C,$1C
DATA_03B75E:
	dcb $01,$02

GET_DRAW_INFO:
	STZ $186C,X             ; Reset sprite offscreen flag, vertical 
        STZ $15A0,X             ; Reset sprite offscreen flag, horizontal 
        LDA $E4,X               ; \ 
        CMP $1A                 ;  | Set horizontal offscreen if necessary 
        LDA $14E0,X             ;  | 
        SBC $1B                 ;  | 
        BEQ ADDR_03B774         ;  | 
        INC $15A0,X             ; / 
ADDR_03B774:
        LDA $14E0,X             ; \ 
        XBA                     ;  | Mark sprite invalid if far enough off screen 
        LDA $E4,X               ;  | 
        REP #$20                ; Accum (16 bit) 
        SEC                     ;  | 
        SBC $1A                 ;  | 
        CLC                     ;  | 
        ADC.W #$0040            ;  | 
        CMP.W #$0180            ;  | 
        SEP #$20                ; Accum (8 bit) 
        ROL                     ;  | 
        AND.B #$01              ;  | 
        STA $15C4,X             ;  | 
        BNE ADDR_03B7CF         ; /  
        LDY.B #$00              ; \ set up loop: 
        LDA $1662,X             ;  |  
        AND.B #$20              ;  | if not smushed (1662 & 0x20), go through loop twice 
        BEQ ADDR_03B79A         ;  | else, go through loop once 
        INY                     ; /                        
ADDR_03B79A:
        LDA $D8,X               ; \                        
        CLC                     ;  | set vertical offscree 
        ADC DATA_03B75C,Y       ;  |                       
        PHP                     ;  |                       
        CMP $1C                 ;  | (vert screen boundry) 
        ROL $00                 ;  |                       
        PLP                     ;  |                       
        LDA $14D4,X             ;  |                       
        ADC.B #$00              ;  |                       
        LSR $00                 ;  |                       
        SBC $1D                 ;  |                       
        BEQ ADDR_03B7BA         ;  |                       
        LDA $186C,X             ;  | (vert offscreen)      
        ORA DATA_03B75E,Y       ;  |                       
        STA $186C,X             ;  |                       
ADDR_03B7BA:
        DEY                     ;  |                       
        BPL ADDR_03B79A         ; /                        
        LDY $15EA,X             ; get offset to sprite OAM                           
        LDA $E4,X               ; \ 
        SEC                     ;  |                                                     
        SBC $1A                 ;  |                                                    
        STA $00                 ; / $00 = sprite x position relative to screen boarder 
        LDA $D8,X               ; \                                                     
        SEC                     ;  |                                                     
        SBC $1C                 ;  |                                                    
        STA $01                 ; / $01 = sprite y position relative to screen boarder 
        RTS                     ; Return 

ADDR_03B7CF:
        PLA                     ; \ Return from *main gfx routine* subroutine... 
        PLA                     ;  |    ...(not just this subroutine) 
        RTS                     ; / 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
DATA_01AC0D:
	dcb $40,$B0
DATA_01AC0F:
	dcb $01,$FF
DATA_01AC11:
        dcb $30,$C0
DATA_01AC19:
        dcb $01,$FF

SubOffScreen:
	JSR IsSprOnScreen       ; \ if sprite is not off screen, return                                       
        BEQ Return01ACA4        ; /                                                                           
        LDA $5B                 ; \  vertical level                                    
        AND #$01                ;  |                                                                           
        BNE VerticalLevel       ; /                                                                           
        LDA $D8,X               ; \                                                                           
        CLC                     ;  |                                                                           
        ADC #$50                ;  | if the sprite has gone off the bottom of the level...                     
        LDA $14D4,X             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)   
        ADC #$00                ;  |                                                                           
        CMP #$02                ;  |                                                                           
        BPL OffScrEraseSprite   ; /    ...erase the sprite                                                    
        LDA $167A,X             ; \ if "process offscreen" flag is set, return                                
        AND #$04                ;  |                                                                           
        BNE Return01ACA4        ; /                                                                           
        LDA $13                   
        AND #$01                
        STA $01                   
        TAY                       
        LDA $1A                   
        CLC                       
        ADC DATA_01AC11,Y       
        ROL $00                   
        CMP $E4,X                 
        PHP                       
        LDA $1B                   
        LSR $00                   
        ADC DATA_01AC19,Y       
        PLP                       
        SBC $14E0,X             
        STA $00                   
        LSR $01                   
        BCC ADDR_01AC7C           
        EOR #$80                
        STA $00                   
ADDR_01AC7C:
        LDA $00                   
        BPL Return01ACA4          
OffScrEraseSprite:
	LDA $14C8,X             ; \ If sprite status < 8, permanently erase sprite 
        CMP #$08                ;  | 
        BCC OffScrKillSprite    ; / 
        LDY $161A,X             
        CPY #$FF                
        BEQ OffScrKillSprite      
        LDA #$00                
        STA $1938,Y             
OffScrKillSprite:
	STZ $14C8,X             ; Erase sprite 
Return01ACA4:
	RTS                       

VerticalLevel:
	LDA $167A,X             ; \ If "process offscreen" flag is set, return                
        AND #$04                ;  |                                                           
        BNE Return01ACA4        ; /                                                           
        LDA $13                 ; \                                                           
        LSR                     ;  |                                                           
        BCS Return01ACA4        ; /                                                           
        LDA $E4,X               ; \                                                           
        CMP #$00                ;  | If the sprite has gone off the side of the level...      
        LDA $14E0,X             ;  |                                                          
        SBC #$00                ;  |                                                          
        CMP #$02                ;  |                                                          
        BCS OffScrEraseSprite   ; /  ...erase the sprite      
        LDA $13                   
        LSR                       
        AND #$01                
        STA $01                   
        TAY                       
	LDA $1C                   
        CLC                       
        ADC DATA_01AC0D,Y       
        ROL $00                   
        CMP $D8,X                 
        PHP                       
        LDA $001D               
        LSR $00                   
        ADC DATA_01AC0F,Y       
        PLP                       
        SBC $14D4,X             
        STA $00                   
        LDY $01                   
        BEQ ADDR_01ACF3           
        EOR #$80                
        STA $00                   
ADDR_01ACF3:
        LDA $00                   
        BPL Return01ACA4          
        BMI OffScrEraseSprite  

IsSprOnScreen:
	LDA $15A0,X             ; \ A = Current sprite is offscreen 
        ORA $186C,X             ; /  
        RTS                     ; Return 
		
