;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Tap-Tap, from Yoshi's Island, by Dispari Scuro
;;
;; Place normally to make a walking tap-tap, or with x3 to make a jumping tap-tap.
;; Tap-tap can be hit by items and thrown around, but gets back up.
;;
;; Based on the following sprites:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shy Guy, by mikeyk
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Yoshi's Island O Spike
;; Programmed by SMWEdit
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		UpdateSpritePos = $01802A  
		SprSprInteract = $018032   
		MarioSprInteract = $01A7DC
		GetSpriteClippingA = $03B69F
		CheckForContact = $03B72B
		DisplayContactGfx = $01AB99 
		FinishOAMWrite = $01B7B3
		ShowSpinJumpStars = $07FC3B
		GivePoints = $02ACE5
		HurtMario = $00F5B7
	
        	ExtraProperty1 = $7FAB28
		ExtraProperty2 = $7FAB34
		RAM_SpriteDir = $157C
		RAM_SprTurnTimer = $15AC

		ACTSTATUS = $C2		; 0 = waiting, 1 = rolling
		SitTimer = $1528	; The amount of time tap-tap sits on the ground after being stunned
		GENERICTMP = $04	; temporary RAM
		KnockoutBank = $1DF9	; \ Sound to play when hit by something
		KnockoutSound = $13	; /
		ExtraInfo = $7FAB10
		TapTapType = $1504	; 0 = walking, 1 = jumping
		JumpTimer = $160E	; The amount of time tap-tap waits before jumping again if a jumping type

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "INIT"		
	LDA #$01		
	STA $151C,x

	STZ TapTapType,x		; Default = walking

	LDA ExtraInfo,x			; \ Get Extra Info
	AND #$04			; /
	BNE Scratch_1			; \
	LDA #$00			; | Extra Info is kinda wacky
	BRA Scratch_2			; /	
Scratch_1:
	LDA #$01
Scratch_2:
	STA TapTapType,x		; DONE!
	
        JSR SubHorzPos			; Face Mario
        TYA
        STA $157C,x
        RTL                 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        dcb "MAIN"                                    
        PHB                  
        PHK                  
        PLB                  
	CMP #09
	BCS HandleStunnd
        JSR SpriteMainSub    
        PLB                  
        RTL

HandleStunnd:
		LDA $167A,x				; Set to interact with Mario
		AND #$7F
		STA $167A,x

		LDY $15EA,X				; Replace Goomba's tile
		PHX
		LDX Tilemap
		LDA $302,Y
		CMP #$A8
		BEQ SetTile
		LDX Tilemap
		
SetTile:
		TXA
 		STA $302,Y
		PLX	
		PLB
		RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpeedX:
		dcb $08,$F8,$0C,$F4	
		
KILLED_X_SPEED:
		dcb $F0,$10

Return:
	RTS
	
SpriteMainSub:
	LDA TapTapType,x
	BEQ StartGraphics
	JSR SubHorzPos		; \
	TYA			; | Jumping tap-taps always face Mario
        STA $157C,x		; /

StartGraphics:
	JSR SubGfx
	
        LDA $9D                 ; \ if sprites locked, return
        BNE Return              ; /
	LDA $14C8,x
	CMP #$08
	BNE Return

        JSR SubOffScreen		; handle off screen situation
	INC $1570,x
	
SetXSpeed:
		LDA ACTSTATUS,x		; \ Don't set speed normally if being thrown
		BEQ ResumeSpeed		; /
		LDA $B6,x		; \
		BEQ MaybeGetUp		; |
		LDA $B6,x		; |
		CMP #$80		; | Continually decrease speed
		BCC GoDown		; |
		INC $B6,x		; | until the sprite is stationary
		BRA UpdatePosition	; |
GoDown:					; |
		DEC $B6,x		; |
		BRA UpdatePosition	; /
MaybeGetUp:
		STZ $B6,x		; \
		LDA SitTimer,x		; | If sit timer is going...
		BEQ StandUp		; |
		DEC SitTimer,x		; | decrement it...
		BRA UpdatePosition	; /
StandUp:	
		LDA #$E0		; \ Make tap-top hop
		STA $AA,x		; /
		DEC ACTSTATUS,x		; No longer stunned
		BRA UpdatePosition

ResumeSpeed:
		LDA TapTapType,x	; \
		BEQ WalkingType		; |
		STZ $B6,x		; | Jumping Tap-Tap
		LDA $AA,x		; |
		BNE UpdatePosition	; /
		LDA JumpTimer,x
		BEQ MakeJump
		DEC JumpTimer,x
		BRA UpdatePosition
MakeJump:
		LDA #$C0		; \
		STA $AA,x		; | Make the Tap-Tap jump...
		LDA #$20		; |
		STA JumpTimer,x		; | ...and set the pause timer
		BRA UpdatePosition	; /
WalkingType:
		LDY $157C,x             ; Set x speed based on direction
		LDA ExtraProperty1,x
		AND #$01
		BEQ NoFastSpeed			; Increase speed if bit 0 is set
		INY
		INY
		
NoFastSpeed:
        LDA SpeedX,y           
        STA $B6,x
UpdatePosition:
	JSL UpdateSpritePos     ; Update position based on speed values
	LDA $1588,x             ; If sprite is in contact with an object...
        AND #$03                  
        BEQ NoObjContact	
        JSR SetSpriteTurning    ;    ...change direction

	LDA ACTSTATUS,x
	BEQ NoObjContact
	STZ $B6,x
        
NoObjContact:
	JSR MaybeStayOnLedges
		LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ NotOnGround
		STZ $AA,x
		STZ $151C,x			; Reset turning flag (used if sprite stays on ledges)
NotOnGround:
	JSR Knockout
	JSL MarioSprInteract	; Check for mario/sprite contact (carry set = contact)
        BCC Return1             ; return if no contact
	
        JSR SubVertPos          ; \
        LDA $0E                 ;  | if mario isn't above sprite, and there's vertical contact...
        CMP #$E6                ;  |     ... sprite wins
        BPL SpriteWins          ; /
        LDA $7D                 ; \ if mario speed is upward, return
        BMI Return1             ; /

        LDA ExtraProperty1,x 	; Check property byte to see if sprite can be spin jumped
        AND #$10
        BEQ SpinKillDisabled    
        LDA $140D               ; Branch if mario is spin jumping
        BNE SpinKill
        
SpinKillDisabled:	
		LDA $187A
		BNE RideSprite
		LDA ExtraProperty1,x
		AND #$20
		BEQ RideSprite
		BIT $16					; Don't pick up sprite if not pressing button
        BVC RideSprite
		LDA #$0B				; Sprite status = Carried
		STA $14C8,x
		LDA #$FF				; Set time until recovery
		STA $1540,x
		RTS

RideSprite:	        
NO_YOSHI:
LABEL9:
        RTS                     

SpriteWins:
        LDA $1490               ; Branch if Mario has a star
        BNE MarioHasStar        
        JSL HurtMario
        
Return1:
	RTS                    

SpinKill:
		JSR SUB_STOMP_PTS       ; give mario points
		LDA #$F8				; Set Mario Y speed
		STA $7D
        JSL DisplayContactGfx   ; display contact graphic
        LDA #$04                ; \ status = 4 (being killed by spin jump)
        STA $14C8,x             ; /   
        LDA #$1F                ; \ set spin jump animation timer
        STA $1540,x             ; /
        JSL ShowSpinJumpStars
        LDA #$08                ; \ play sound effect
        STA $1DF9               ; /
        RTS                     ; return

MarioHasStar:
        LDA #$02                ; \ status = 2 (being killed by star)
        STA $14C8,x             ; /
        LDA #$D0                ; \ set y speed
        STA $AA,x               ; /
        JSR SubHorzPos          ; get new direction
        LDA KILLED_X_SPEED,y    ; \ set x speed based on direction
        STA $B6,x               ; /
        INC $18D2               ; increment number consecutive enemies killed
        LDA $18D2               ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NoReset             ;  |
        LDA #$08                ;  |
        STA $18D2               ; / 
          
NoReset:
		JSL GivePoints
        LDY $18D2               ; \ 
        CPY #$08                ;  | if consecutive enemies stomped < 8 ...
        BCS NO_SOUND2           ;  |
        LDA STAR_SOUNDS,y       ;  |    ... play sound effect
        STA $1DF9               ; /
        
NO_SOUND2:
	RTS							; final return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MaybeStayOnLedges:	
		LDA ExtraProperty1,x	; Stay on ledges if bit 1 is set
		AND #$02                
		BEQ NoFlipDirection
		LDA $1588,x             ; If the sprite is in the air
		ORA $151C,x             ;   and not already turning
		BNE NoFlipDirection
		JSR SetSpriteTurning 	;   flip direction
        	LDA #$01                ;   set turning flag
		STA $151C,x
		 
NoFlipDirection:
		RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ThrowSpeedX:
		dcb $D0,$30

Knockout:
		LDY #$0C	; load number of times to go through loop
KO_LOOP		CPY #$00		; \ zero? if so,
		BEQ END_KO_LOOP		; / end loop
		DEY			; decrease # of times left+get index
		;STX GENERICTMP		; \  if sprite is
		;CPY GENERICTMP		;  | this sprite
		;BEQ KO_LOOP		; /  then ignore it
		LDA $14C8,y		; \  if sprite's status
		CMP #$09		;  | is less than 9 (9,A,B = shell modes)
		BCC KO_LOOP		; /  ignore sprite
		LDA $1686,y		; \  if sprite doesn't
		AND #%00001000		;  | interact with others
		BNE KO_LOOP		; /  don't continue
		JSL $03B69F		; \
		PHX			;  | if sprite is
		TYX			;  | not touching
		JSL $03B6E5		;  | this sprite
		PLX			;  |
		JSL $03B72B		;  |
		BCC KO_LOOP		; /
		LDA $14C8,y		; \  speed doesn't matter
		CMP #$0B		;  | if mario is holding
		BEQ MOVING		; /  the shell (status=B)
		LDA $AA,y		; \ continue if sprite
		BNE MOVING		; / has Y speed
		LDA $B6,y		; \ continue if sprite
		BNE MOVING		; / has X speed
		BRA KO_LOOP		; no speed / not holding -> don't kill
MOVING		LDA $1656,y		; \  force shell
		ORA #%10000000		;  | to disappear
		STA $1656,y		; /  in smoke
		LDA #$02		; \ set shell into
		STA $14C8,y		; / death mode (status=2)
		INC ACTSTATUS,x		; next sprite mode
		
		LDA $E4,y	; Mario = 94	; \
		SEC				; | Check to see which side of the sprite...
		SBC $E4,x			; |
		STA $0F				; | the tap-tap is on so it knows which...
		LDA $14E0,y			; |
		SBC $14E0,x	; Mario = 95	; | way to be thrown.
		BPL NoIncY			; |
		PHY				; | Turn to face the direction...
		LDY #$01			; |
		BRA ContinueOn:			; | they were hit from.
	NoIncY:					; |
		PHY				; |
		LDY #$00			; |
	ContinueOn:				; |
		TYA				; |
		STA $157C,x			; /

		LDA #$C0		; \ Throw the tap-tap upward
		STA $AA,x		; /
		LDY $157C,x             ; Set x speed based on direction
		LDA ThrowSpeedX,y           
        	STA $B6,x
		PLY

		LDA #KnockoutSound	; \ Play a sound
		STA KnockoutBank	; /

		LDA #$60		; \ Set timer for how long Tap-tap sits on the ground
		STA SitTimer,x		; /

NO_INVERT_DIR	BRA KO_LOOP		; repeat loop
END_KO_LOOP	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetSpriteTurning:
		LDA #$08                ; Set turning timer 
		STA RAM_SprTurnTimer,X   
        LDA $157C,x
        EOR #$01
        STA $157C,x

Return0190B1:	
        RTS
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Tilemap:
		dcb $84,$86,$87			; Walking 1 (body), Left Foot, Right Foot
		dcb $84,$86,$87			; Walking 2 (body), Left Foot, Right Foot
		dcb $84,$87,$96			; Walking 3 (body), Left Foot, Right Foot
		dcb $84,$96,$96			; Walking 4 (body), Left Foot, Right Foot
		dcb $84,$96,$96			; Walking 5 (body), Left Foot, Right Foot
		dcb $84,$96,$96			; Walking 6 (body), Left Foot, Right Foot
		dcb $84,$87,$96			; Walking 7 (body), Left Foot, Right Foot
		dcb $84,$96,$87			; Walking 8 (body), Left Foot, Right Foot
		
		dcb $84,$86,$87
		dcb $84,$86,$87
		dcb $84,$87,$96
		dcb $84,$96,$96
		dcb $84,$96,$96
		dcb $84,$96,$96
		dcb $84,$87,$96
		dcb $84,$96,$87

Stunmap:	dcb $CC,$8C,$AC,$CE		; facing E, facing SE, facing S (XY flip), facing SW (XY flip)
		dcb $CC,$8C,$AC,$CE		; facing W (XY flip), facing NW (XY flip), facing N, facing NE

Jumpmap:	dcb $84,$86,$86			; Standing (body), left foot, right foot
		dcb $84,$86,$86

		dcb $84,$87,$87			; Jumping (body), left foot, right foot
		dcb $84,$87,$87

Horiz_Disp:
		dcb $00,$FF,$0D
		dcb $00,$03,$08
		dcb $00,$00,$06
		dcb $00,$07,$FF
		dcb $00,$FE,$08
		dcb $00,$FE,$08
		dcb $00,$FC,$0A
		dcb $00,$FD,$0E
		
		dcb $00,$09,$FB
		dcb $00,$04,$FF
		dcb $00,$08,$02
		dcb $00,$01,$08
		dcb $00,$0A,$00
		dcb $00,$0A,$00
		dcb $00,$0C,$FE
		dcb $00,$0B,$FA

JumpHoriz:	dcb $00,$00,$08			; Standing (body), left foot, right foot
		dcb $00,$00,$08

		dcb $00,$FE,$06			; Jumping (body), left foot, right foot
		dcb $00,$02,$0A

Verti_Disp:
		dcb $FD,$0B,$08
		dcb $FC,$0B,$07
		dcb $FC,$07,$0A
		dcb $FC,$0A,$04
		dcb $FC,$06,$0A
		dcb $FC,$06,$0A
		dcb $FD,$04,$0A
		dcb $FF,$0A,$08
		
		dcb $FD,$0B,$08
		dcb $FC,$0B,$07
		dcb $FC,$07,$0A
		dcb $FC,$0A,$04
		dcb $FC,$06,$0A
		dcb $FC,$06,$0A
		dcb $FD,$04,$0A
		dcb $FF,$0A,$08

JumpVerti:	dcb $FD,$0C,$0C			; Standing (body), left foot, right foot
		dcb $FD,$0C,$0C

		dcb $FD,$0C,$0C			; Jumping (body), left foot, right foot
		dcb $FD,$0C,$0C
		
Tile_Size:
        	dcb $02,$00,$00			; 00 = 8x8
		dcb $02,$00,$00			; 02 = 16x16
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
        
        	dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00
		dcb $02,$00,$00

JumpTiles:	dcb $02,$00,$00			; Standing (body), left foot, right foot
		dcb $02,$00,$00

		dcb $02,$00,$00			; Jumping (body), left foot, right foot
		dcb $02,$00,$00
        
Tile_Flip:
		dcb $00,$00,$00			; 00 = no flip
		dcb $00,$00,$00			; 40 = horizontal flip
		dcb $00,$40,$40			; 80 = vertical flip
		dcb $00,$40,$80			; C0 = horizontal and vertical flip
		dcb $00,$00,$40
		dcb $00,$00,$40
		dcb $00,$00,$40
		dcb $00,$00,$00
		
		dcb $40,$40,$40
		dcb $40,$40,$40
		dcb $40,$00,$00
		dcb $40,$00,$C0
		dcb $40,$40,$00
		dcb $40,$40,$00
		dcb $40,$40,$00
		dcb $40,$40,$40

StunFlip:
		dcb $00,$00,$C0,$C0
		dcb $C0,$C0,$00,$00

JumpFlip:	dcb $00,$00,$00			; Standing (body), left foot, right foot
		dcb $40,$40,$40

		dcb $00,$40,$40			; Jumping (body), left foot, right foot
		dcb $40,$00,$00

SubGfx:
		JSR GetDrawInfo         ; after: Y = index to sprite tile map ($300)
                                ;      $00 = sprite x position relative to screen boarder 
                                ;      $01 = sprite y position relative to screen boarder  
       
        LDA $1602,x
        STA $03                 ; | $03 = index to frame start (0 or 4)                   
       
NOT_DEAD:
		LDA ACTSTATUS,x
		BEQ NotStunned
		;=============
		;=============
	LDA $157C,x             ; $02 = direction
        STA $02

        LDA $14                 ; Set animation frame based on frame counter
        LSR A                   
        LSR A                   
	LSR A
        ;CLC                     
        ;ADC $15E9               
        AND #$07		; 8 frames
        STA $03                 

	PHX
	LDA $00                 ; Tile x position = sprite x location ($00)
        STA $0300,y             

        LDA $01                 ; Tile y position = sprite y location ($01)
        STA $0301,y             

	LDA SitTimer,x
	CMP #$40
	BCS StillRolling
	LDA $15F6,x             ; Yile properties xyppccct, format
        LDX $02                 ; If direction == 0...
        BNE NoFlipStun                
        ORA #$40                ;    ...flip tile
NoFlipStun:
        ORA $64                 ; Add in tile priority of level
        STA $0303,y
	LDX #$00
	BRA FinishDrawingStun

StillRolling:
	LDA $15F6,x             ; get palette info
        PHX
	LDX $03
	ORA StunFlip,x		; flip tile if necessary
        ORA $64                 ; add in tile priority of level
        STA $0303,y             ; store tile properties
	PLX

        LDX $03                 ; Store tile
FinishDrawingStun:
        LDA Stunmap,x           
        STA $0302,y             
        PLX

        LDY #$02                ; Set tiles to 16x16
        LDA #$00                ; We drew 1 tile
        JSL FinishOAMWrite      
        RTS
		;=============
		;============= 

NotStunned:
		LDA $14					; Frame Counter
		LSR A					; Change every 4 frames
		LSR A
		AND #$07				; 8 walking "frames"
		STA $0F					; Scratch RAM to "multiply by 3"
		ASL A					; x2
		CLC
		ADC $0F
		STA $03
		
		LDA TapTapType,x			; \ If walking type...
		BNE NoSounds				; / ...do not play sounds
		LDA $14					; \
		AND #$10				; |
		BEQ NormalDirCheck			; | Play walking sound every 16 frames
		LDA #$1B				; |
		STA $1DF9				; /
		BRA NormalDirCheck
        
NoSounds:
	STZ $03			; Correct frame counter for jumping type

	LDA $AA,x		; \
	BEQ NotInAir		; |
	LDA $03			; |
	CLC			; | Correct tile skip when in the air for jumping
	ADC #$06		; |
	STA $03			; /

NotInAir:
	LDA $157C,x		; \ $02 = sprite direction
	STA $02			; /
	BNE WhichLoop		; \
	LDA $03			; |
	CLC			; | Correct tile skip for direction for jumping
	ADC #$03		; |
	STA $03			; /

	BRA WhichLoop

NormalDirCheck:
        LDA $157C,x             ; \ $02 = sprite direction
        STA $02                 ; /
        BNE WhichLoop
        LDA $03
        CLC
        ADC #$18 				; skip some tiles if facing the other way
        STA $03

WhichLoop
	LDA TapTapType,x
	BNE LOOP_STARTj

LOOP_START:
	PHX                     ; push sprite index
        LDX #$02                ; loop counter = (number of tiles per frame) - 1

LOOP_START_2:
        PHX						; push current tile number
        
        TXA                     ; \ X = index of frame start + current tile
        CLC                     ;  |
        ADC $03                 ;  |
        TAX                     ; /
        
        PHY                     ; \
        TYA                     ; |
        LSR A                   ; |
        LSR A                   ; |
        TAY                     ; | Set tile to be either 8x8 ($00) or 16x16 ($02)
        LDA TILE_SIZE,x         ; |
        STA $0460,y             ; |
        PLY                     ; /

        LDA $00                 ; \ tile x position = sprite x location ($00)
        CLC                     ;  |
        ADC Horiz_Disp,x        ;  |
        STA $0300,y             ; /
        
        LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
        CLC                     ;  |
        ADC Verti_Disp,x        ;  |
        STA $0301,y             ; /
        
        LDA TILEMAP,x           ; \ store tile
        STA $0302,y             ; / 
        
        PHX
        LDX $15E9
        LDA $15F6,x             ; get palette info
        PLX
        ORA Tile_Flip,x		; flip tile if necessary
        ORA $64                 ; add in tile priority of level
        STA $0303,y             ; store tile properties
      
	PLX                     ; \ pull, X = current tile of the frame we're drawing
        INY                     ;  | increase index to sprite tile map ($300)...
        INY                     ;  |    ...we wrote 1 16x16 tile...
        INY                     ;  |    ...sprite OAM is 8x8...
        INY                     ;  |    ...so increment 4 times
        DEX                     ;  | go to next tile of frame and loop
        BPL LOOP_START_2        ; /

        PLX                     ; pull, X = sprite index
        LDY #$FF                ; \ We've already set 460, so use FF!
        LDA #$02                ; | A = number of tiles drawn - 1
        JSL $01B7B3             ; / don't draw if offscreen
        RTS                     ; return              

LOOP_STARTj:
	PHX                     ; push sprite index
        LDX #$02                ; loop counter = (number of tiles per frame) - 1

LOOP_START_2j:
        PHX						; push current tile number
        
        TXA                     ; \ X = index of frame start + current tile
        CLC                     ;  |
        ADC $03                 ;  |
        TAX                     ; /
        
        PHY                     ; \
        TYA                     ; |
        LSR A                   ; |
        LSR A                   ; |
        TAY                     ; | Set tile to be either 8x8 ($00) or 16x16 ($02)
        LDA TILE_SIZE,x         ; |
        STA $0460,y             ; |
        PLY                     ; /

	LDA $00			; \
	CLC			; | Horizontal offset
	ADC JumpHoriz,x		; |
	STA $0300,y		; /

	LDA $01			; \
	CLC			; | Vertical offset
	ADC JumpVerti,x		; |
	STA $0301,y		; /

	LDA Jumpmap,x
	STA $0302,y

ReturnToGraphics
	PHX
	LDX $15E9
	LDA $15F6,x
	PLX
	ORA JumpFlip,x
	ORA $64
	STA $0303,y

        PLX                     ; \ pull, X = current tile of the frame we're drawing
        INY                     ;  | increase index to sprite tile map ($300)...
        INY                     ;  |    ...we wrote 1 16x16 tile...
        INY                     ;  |    ...sprite OAM is 8x8...
        INY                     ;  |    ...so increment 4 times
        DEX                     ;  | go to next tile of frame and loop
        BPL LOOP_START_2j       ; /

        PLX                     ; pull, X = sprite index
        LDY #$FF                ; \ We've already set 460, so use FF!
        LDA #$02                ; | A = number of tiles drawn - 1
        JSL $01B7B3             ; / don't draw if offscreen
        RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from SMW
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS
		dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS:
		PHY                      
        LDA $1697               ; \
        CLC                     ;  | 
        ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
        INC $1697               ; increase consecutive enemies stomped
        TAY                     ;
        INY                     ;
        CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
        BCS NO_SOUND            ; /    ... don't play sound 
        LDA STAR_SOUNDS,y       ; \ play sound effect
        STA $1DF9               ; /   
NO_SOUND:
		TYA                     ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NO_RESET            ;  |
        LDA #$08                ; /
NO_RESET:
		JSL GivePoints
        PLY                     
        RTS                     ; return
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubVertPos:
	LDY #$00
        LDA $96
        SEC
        SBC $D8,x
        STA $0F
        LDA $97
        SBC $14D4,x
        BPL VertIncY
        INY
VertIncY:
        RTS   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL HorzIncY
	INY 
HorzIncY:
	RTS 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 	
DATA_03B75C:
	dcb $0C,$1C
DATA_03B75E:
	dcb $01,$02

GetDrawInfo:
	STZ $186C,X             ; Reset sprite offscreen flag, vertical 
        STZ $15A0,X             ; Reset sprite offscreen flag, horizontal 
        LDA $E4,X               ; \ 
        CMP $1A                 ;  | Set horizontal offscreen if necessary 
        LDA $14E0,X             ;  | 
        SBC $1B                 ;  | 
        BEQ ADDR_03B774         ;  | 
        INC $15A0,X             ; / 
ADDR_03B774:
        LDA $14E0,X             ; \ 
        XBA                     ;  | Mark sprite invalid if far enough off screen 
        LDA $E4,X               ;  | 
        REP #$20                ; Accum (16 bit) 
        SEC                     ;  | 
        SBC $1A                 ;  | 
        CLC                     ;  | 
        ADC.W #$0040            ;  | 
        CMP.W #$0180            ;  | 
        SEP #$20                ; Accum (8 bit) 
        ROL                     ;  | 
        AND.B #$01              ;  | 
        STA $15C4,X             ;  | 
        BNE ADDR_03B7CF         ; /  
        LDY.B #$00              ; \ set up loop: 
        LDA $1662,X             ;  |  
        AND.B #$20              ;  | if not smushed (1662 & 0x20), go through loop twice 
        BEQ ADDR_03B79A         ;  | else, go through loop once 
        INY                     ; /                        
ADDR_03B79A:
        LDA $D8,X               ; \                        
        CLC                     ;  | set vertical offscree 
        ADC DATA_03B75C,Y       ;  |                       
        PHP                     ;  |                       
        CMP $1C                 ;  | (vert screen boundry) 
        ROL $00                 ;  |                       
        PLP                     ;  |                       
        LDA $14D4,X             ;  |                       
        ADC.B #$00              ;  |                       
        LSR $00                 ;  |                       
        SBC $1D                 ;  |                       
        BEQ ADDR_03B7BA         ;  |                       
        LDA $186C,X             ;  | (vert offscreen)      
        ORA DATA_03B75E,Y       ;  |                       
        STA $186C,X             ;  |                       
ADDR_03B7BA:
        DEY                     ;  |                       
        BPL ADDR_03B79A         ; /                        
        LDY $15EA,X             ; get offset to sprite OAM                           
        LDA $E4,X               ; \ 
        SEC                     ;  |                                                     
        SBC $1A                 ;  |                                                    
        STA $00                 ; / $00 = sprite x position relative to screen boarder 
        LDA $D8,X               ; \                                                     
        SEC                     ;  |                                                     
        SBC $1C                 ;  |                                                    
        STA $01                 ; / $01 = sprite y position relative to screen boarder 
        RTS                     ; Return 

ADDR_03B7CF:
        PLA                     ; \ Return from *main gfx routine* subroutine... 
        PLA                     ;  |    ...(not just this subroutine) 
        RTS                     ; / 

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
DATA_01AC0D:
	dcb $40,$B0
DATA_01AC0F:
	dcb $01,$FF
DATA_01AC11:
        dcb $30,$C0
DATA_01AC19:
        dcb $01,$FF

SubOffScreen:
		JSR IsSprOnScreen       ; \ if sprite is not off screen, return                                       
        BEQ Return01ACA4        ; /                                                                           
        LDA $5B                 ; \  vertical level                                    
        AND #$01                ;  |                                                                           
        BNE VerticalLevel       ; /                                                                           
        LDA $D8,X               ; \                                                                           
        CLC                     ;  |                                                                           
        ADC #$50                ;  | if the sprite has gone off the bottom of the level...                     
        LDA $14D4,X             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)   
        ADC #$00                ;  |                                                                           
        CMP #$02                ;  |                                                                           
        BPL OffScrEraseSprite   ; /    ...erase the sprite                                                    
        LDA $167A,X             ; \ if "process offscreen" flag is set, return                                
        AND #$04                ;  |                                                                           
        BNE Return01ACA4        ; /                                                                           
        LDA $13                   
        AND #$01                
        STA $01                   
        TAY                       
        LDA $1A                   
        CLC                       
        ADC DATA_01AC11,Y       
        ROL $00                   
        CMP $E4,X                 
        PHP                       
        LDA $1B                   
        LSR $00                   
        ADC DATA_01AC19,Y       
        PLP                       
        SBC $14E0,X             
        STA $00                   
        LSR $01                   
        BCC ADDR_01AC7C           
        EOR #$80                
        STA $00                   
ADDR_01AC7C:
        LDA $00                   
        BPL Return01ACA4          
OffScrEraseSprite:
	LDA $14C8,X             ; \ If sprite status < 8, permanently erase sprite 
        CMP #$08                ;  | 
        BCC OffScrKillSprite    ; / 
        LDY $161A,X             
        CPY #$FF                
        BEQ OffScrKillSprite      
        LDA #$00                
        STA $1938,Y             
OffScrKillSprite:
	STZ $14C8,X             ; Erase sprite 
Return01ACA4:
	RTS                       

VerticalLevel:
	LDA $167A,X             ; \ If "process offscreen" flag is set, return                
        AND #$04                ;  |                                                           
        BNE Return01ACA4        ; /                                                           
        LDA $13                 ; \                                                           
        LSR                     ;  |                                                           
        BCS Return01ACA4        ; /                                                           
        LDA $E4,X               ; \                                                           
        CMP #$00                ;  | If the sprite has gone off the side of the level...      
        LDA $14E0,X             ;  |                                                          
        SBC #$00                ;  |                                                          
        CMP #$02                ;  |                                                          
        BCS OffScrEraseSprite   ; /  ...erase the sprite      
        LDA $13                   
        LSR                       
        AND #$01                
        STA $01                   
        TAY                       
	LDA $1C                   
        CLC                       
        ADC DATA_01AC0D,Y       
        ROL $00                   
        CMP $D8,X                 
        PHP                       
        LDA $001D               
        LSR $00                   
        ADC DATA_01AC0F,Y       
        PLP                       
        SBC $14D4,X             
        STA $00                   
        LDY $01                   
        BEQ ADDR_01ACF3           
        EOR #$80                
        STA $00                   
ADDR_01ACF3:
        LDA $00                   
        BPL Return01ACA4          
        BMI OffScrEraseSprite  

IsSprOnScreen:
	LDA $15A0,X             ; \ A = Current sprite is offscreen 
        ORA $186C,X             ; /  
        RTS                     ; Return 
		
