; =============
; UNDINE SPRITE
; =============

; Ram uses:

; $02   general calcuation
; $00C2 stores amount of frames for animation
; $1504 internal sprite state:  $X0 moving, $X1 summon frostwave, $X2 summon icelance, $X3 hit, X = hits remaining
; $1528 spawned flag
; $1540 counter to next state change
; $157C X Speed
; $1594 Y Speed
; $1602 animation and direction select
; $160E y equilibrium position

; Index format xxxfaadtt: x unused, f frame 0-1, a animation 0-3, d direction 0-1, t tile 0-3
; Animation 0: Cast, Animation 1: Left, Animation 2: Hit, Animation 3: Bow

; =========
; CONSTANTS
; =========

!UNDINE_FROSTBLOCK	= #$01
!UNDINE_ICELANCESP	= #$02

!UNDINE_HITS		= #$30

!UNDINE_MOVE		= #$00
!UNDINE_FROSTWAVE	= #$01
!UNDINE_ICELANCE	= #$02
!UNDINE_HIT		= #$03

!UNDINE_MOVETIME	= #$A0
!UNDINE_FROSTWAVETIME	= #$80
!UNDINE_ICELANCETIME	= #$60
!UNDINE_HITTIME		= #$30

!UNDINE_MAXXSPEEDPOS	= #$30
!UNDINE_MAXXSPEEDNEG	= #$D0
!UNDINE_MAXYSPEEDPOS	= #$18
!UNDINE_MAXYSPEEDNEG	= #$E8

!UNDINE_LEFTBOUND	= #$70
!UNDINE_RIGHTBOUND	= #$90

!UNDINE_EXTRAPROP	= $7FAB28

; ===============
; INITIALIZIATION
; ===============

		print "INIT ",pc
		LDA !UNDINE_MOVE	; Initially moving state
		ORA !UNDINE_HITS	; Add in hits
		STA $1504,x		; Store move state
		LDA !UNDINE_MOVETIME	; Load time to state change
		STA $1540,x		; Store the timer
		LDA #$00		; Animation has 1 Frame, (#$00: 1 frame, #$20 2 frames)
		STA $00C2,x		; Store Animation frame Count
		JSR SUB_HORZ_POS	; Get initial direction
		TYA
		ASL
		ASL
		ORA #$08		; Select animation 1
		STA $1602,x		; Set animation and direction
		STZ $157C,x		; Set x speed to zero
		LDA !UNDINE_MAXXSPEEDNEG; Load maximum negative speed
		STA $1594,x		; Store y speed
		LDA $D8,x		; Get y position
		STA $160E,x		; Store y equilibrium
		LDA !UNDINE_MAXYSPEEDNEG
		STA $1594,x
		RTL

; ====================
; SPRITE MAIN FUNCTION
; ====================

		print "MAIN ",pc
		PHB			; Push Old Databank
		PHK			; Push Current Databank
		PLB			; Pull Current Databank
		JSR UNDINE_GFX		; Display the sprite

		JSL $01A7DC		; Check Mario interaction
		BCC NoMarioContact
		JSL $00F5B7		; Hurt Mario

NoMarioContact:	LDA $1504,x		; Get hitpoints
		AND #$F0
		CMP #$00		; Now dead
		BNE StatusCheck
		STZ $14C8,x		; Set state to dead
		JSL $03A6C8		; Kill all enemies

		LDA !UNDINE_EXTRAPROP,x	; Secret exit depending on extra bit
                BIT #$01		; We want normal exit
                BEQ ActivateGoal
                   
		LDA #$01		; Configure secret exit
                STA $141C

ActivateGoal:	INC $13C6		; Stop Mario
		LDA #$FF
		STA $1493		; Activate Goal
		LDA #$0B		; Set music
		STA $1DFB
                PLB
		RTL			; Finish Level

StatusCheck:	LDA $14C8,x		; Load Sprite State
		CMP #$08 		; Check for normal State
		BNE RETURN 		; Don't process sprite
		LDA $9D 		; Load sprite locking 
		BNE RETURN 		; Don't process sprite
		JSR SUB_OFF_SCREEN_X0	; Process Sprite of screen 

		LDA $1504,x		; Get state byte
		AND #$0F		; Get the state bits

		CMP !UNDINE_FROSTWAVE	; Call according subroutine
		BEQ FROSTWAVE
		CMP !UNDINE_ICELANCE
		BEQ ICELANCE
		CMP !UNDINE_HIT
		BEQ HIT
		JSR MoveFunc
		BRA UPDATESTATES
FROSTWAVE:	JSR FrostWaveFunc
		BRA RETURN
ICELANCE:	JSR IceLanceFunc
		BRA RETURN
HIT:		JSR HitFunc
		BRA RETURN

UPDATESTATES:	JSR UNDINE_STATES	; Update the State

RETURN:		PLB			; Pull Old Databank
		RTL

; ==============
; State Function
; ==============

UNDINE_TRANSITIONS:		db !UNDINE_MOVE,!UNDINE_FROSTWAVE,!UNDINE_FROSTWAVE,!UNDINE_ICELANCE
UNDINE_TRANSTIME:		db !UNDINE_MOVETIME,!UNDINE_FROSTWAVETIME,!UNDINE_FROSTWAVETIME,!UNDINE_ICELANCETIME
UNDINE_TRANSANIM:		db $08,$18,$18,$00
UNDINE_TRANSANIMT:		db $00,$00,$00,$40


; $1602 animation and direction select
; Index format xxxfaadtt: x unused, f frame 0-1, a animation 0-3, d direction 0-1, t tile 0-3
; Animation 0: Cast, Animation 1: Move, Animation 2: Hit, Animation 3: Bow


UNDINE_STATES:	LDA $1540,x		; Get state timer
		CMP #$00		; Time ran up?
		BNE StateEnd
		LDA $1504,x		; Load current state
		AND #$F0		; Get hitpoints
		STA $1504,x		; Store hitpoints
		LDA $E4,x		; Get x position
		AND #$03		; Get lowest 3 bits
		TAY			; Transfer it to Y register
		LDA UNDINE_TRANSITIONS,y; Load new state
		CMP !UNDINE_MOVE
		BEQ SkipSound
		LDA #$10
		STA $1DF9
		LDA UNDINE_TRANSITIONS,y
SkipSound:	ORA $1504,x		; Add in hitpoints
		STA $1504,x		; Store new state
		LDA UNDINE_TRANSTIME,y	; Load time for new state
		STA $1540,x		; Store new time
		LDA $1602,x
		AND #$04
		STA $02
		LDA UNDINE_TRANSANIM,y	; Get new animation
		ORA $02
		STA $1602,x		; Store new animation
		LDA UNDINE_TRANSANIMT,y	; Get amount of frames
		STA $00C2,x
StateEnd:	RTS

; =============
; Move Function
; =============

MoveFunc:	LDA $160E,x		; Load y equilibrium
		SEC
		SBC $D8,x		; Subtract y position
		BMI RotateNeg
		LSR
		LSR
		LSR
		BRA AddSpeed
RotateNeg:	LSR
		LSR
		LSR
		ORA #$E0
AddSpeed:	CLC
		ADC $1594,x		; Add current speed
		BMI LimitNegative	; Check negative speed
		CMP !UNDINE_MAXYSPEEDPOS
		BCS LimitPositive
		BRA StoreYSpeed
LimitPositive:	LDA !UNDINE_MAXYSPEEDPOS
		BRA StoreYSpeed
LimitNegative:	CMP !UNDINE_MAXYSPEEDNEG
		BCS StoreYSpeed
		LDA !UNDINE_MAXYSPEEDNEG
StoreYSpeed:	STA $1594,x		; Save current Speed
		STA $AA,x		; Update y Speed
		LDA $1602,x		; Get current x direction
		BIT #$04
		BNE GoLeft		; Process left
		LDA $157C,x		; Load current speed
		CMP !UNDINE_MAXXSPEEDPOS; If not at maxspeed
		BEQ StoreXSpeed
		INC A			; Increase speed
		BRA StoreXSpeed
GoLeft:		LDA $157C,x		; Load current speed
		CMP !UNDINE_MAXXSPEEDNEG; If not at maxspeed
		BEQ StoreXSpeed
		DEC A			; Decrease speed
StoreXSpeed:	STA $157C,x		; Save local speed
		STA $B6,x		; Update speed
		JSR SUB_HORZ_POS	; Get facing direction
		TYA
		ASL
		ASL			; Configure direction
		STA $02			; Save direction

		LDA $E4,x		; Load current x position
		CMP !UNDINE_LEFTBOUND
		BPL CheckRightBND
		STZ $02			; Go right, were close to leftbound
		BRA LoadAnimation
CheckRightBND:	CMP !UNDINE_RIGHTBOUND
		BMI LoadAnimation
		LDA #$04
		STA $02			; Go left, were close to rightbound

LoadAnimation:	LDA $1602,x		; Load animation and direction
		AND #$FB		; Remove direction
		ORA $02			; Add in new direction
		STA $1602,x		; Store animation and directiony
		JSL $01802A		; Update sprite position

		JSR CheckFrostBlk

		RTS


; ==================
; CheckFrostBlock
; ==================

CheckFrostBlk:	LDY #$0B                ; Cycle all other sprites
SpriteLoop:	PHX
		TYX
		LDA $7FAB9E,x		; Load Spritenumber
		PLX
		CMP !UNDINE_FROSTBLOCK	; Check for frostblock
		BNE NextSprite		; Only process frostblocks
		LDA $14C8,y
		CMP #$0A		; Is the block kicked?
		BEQ CheckInteract
		CMP #$09		; Is the block stationary
		BEQ CheckInteract
NextSprite:	DEY
		BPL SpriteLoop
		RTS

CheckInteract:	PHX                     ; Push x to Stack
		TYX                     ; Transfer Y to X
		JSL $03B6E5             ; Get Other Sprite Clipping
		PLX                     ; Pull X from Stack
		JSL $03B69F             ; Get Current Sprite Clipping
		JSL $03B72B             ; Check Clippings
		BCC NextSprite

		LDA #$20		; Play sound effect
		STA $1DF9

		LDA $1504,x		; Get Hitpoints
		AND #$F0
		LSR
		LSR
		LSR
		LSR
		DEC A
		ASL
		ASL
		ASL
		ASL
		STA $1504,x
		LDA !UNDINE_HIT
		ORA $1504,x
		STA $1504,x
		LDA !UNDINE_HITTIME
		STA $1540,x
		LDA $1602,x
		AND #$04
		STA $02
		LDA #$10
		ORA $02
		STA $1602,x		; Store new animation

		PHX
		TYX
		LDA #$03
		STA $160E,x		; Tell frostblock to shatter
		PLX
		RTS

; ==================
; FrostWave Function
; ==================

FrostWaveFunc:	LDA $1528,x		; Already spawned block?
		BIT #$01
		BNE SkipSpawn
		LDA #$01                ; Set Spawned flag
		STA $1528,x

		JSL $02A9DE
		BMI SkipSpawn           ; If sprite slots are full, don't generate.

		PHX                     ; Free x register
		TYX
		LDA !UNDINE_FROSTBLOCK	; Load Frostblock              
		STA $7FAB9E,x	
		PLX                     ; Pull x from Stack
		
		LDA #$01
		STA $14C8,y             ; New Sprite is not initialized

		LDA $E4,x               ; Get current x position
		SEC
		SBC #$0E                ; Add in offset
		BCS StoreSpawnX
		STA $00E4,y             ; Save new X Position
		LDA $15E0,x
		DEC A			; Had overflow reduce highbit
		STA $14E0,y
		BRA YPosition
StoreSpawnX:	STA $00E4,y             ; Save new X Position
		LDA $14E0,x
		STA $14E0,y             ; High X position doesnt change

YPosition:	LDA $D8,x               ; Get current y position
		SEC
		SBC #$08                ; Add in Offset
		BCS StoreSpawnY
		STA $00D8,y
		LDA $14D4,x
		DEC A
		STA $14D4,y
		BRA CreateFrost
StoreSpawnY:	STA $00D8,y             ; Save new Y Position
		LDA $14D4,x
		STA $14D4,y             ; High Y position doesnt change

CreateFrost:	PHX                     ; Push x to stack
		TYX                     ; Transfer y to x
		JSL $07F7D2             ; Reset sprite tables, to generate this one.
		JSL $0187A7             ; Reset custom sprite tables, too.
		LDA #$88		; Configure Custom Sprite Spawn
		STA  $7FAB10,x
		PLX                     ; Pull x from stack

SkipSpawn:	LDA $1540,x		; Get state timer
		CMP #$00		; Time ran up?
		BNE FrostWaveEnd
		LDA $1504,x		; Load state
		AND #$F0		; Get Hitpoints
		STA $1504,x
		LDA !UNDINE_MOVETIME
		STA $1540,x
		LDA $1602,x
		AND #$04
		STA $02
		LDA #$08		; Change to moving animation
		ORA $02
		STA $1602,x
		STZ $1528,x		; Clear spawned flag
FrostWaveEnd:	JSR CheckFrostBlk
		RTS

; =================
; IceLance Function
; =================

ICELANCE_XPOS:	db	$20,$60,$A0

IceLanceFunc:	LDA $1528,x		; Already spawned icelances?
		BIT #$01
		BNE SkipIceSpawn
		LDA #$01                ; Set Spawned flag
		STA $1528,x

		LDX #$02		; We want 3 icelances

IceSpawnLoop:	JSL $02A9DE		; Get Slot for new Sprite
		BMI SkipIceSpawn	; If sprite slots are full, don't generate.

		PHX
		TYX
		LDA !UNDINE_ICELANCESP	; Load IceLance
		STA $7FAB9E,x
		PLX

		LDA $94			; Get Marios x position low byte
		AND #$03		; Get lowest 2 bits
		ASL
		ASL
		ASL
		ASL			; Shift to right position
		CLC
		ADC ICELANCE_XPOS,x	; Add desired position
		STA $00E4,y             ; Save new X Position
		LDA #$00		; First Screen
		STA $14E0,y             ; High X position

		LDA #$A0
		STA $00D8,y             ; Save new Y Position
		LDA #$01
		STA $14D4,y

		LDA #$01
		STA $14C8,y             ; New Sprite is not initialized

		PHX
		TYX                     ; Transfer y to x
		JSL $07F7D2             ; Reset sprite tables, to generate this one.
		JSL $0187A7             ; Reset custom sprite tables, too.
		LDA #$88		; Configure Custom Sprite Spawn
		STA $7FAB10,x
		PLX

		TXA
		CMP #$00		; Last sprite gets priority
		BNE ContinueLoop
		LDA #$01
		STA $1602,y

ContinueLoop:	DEX
		BPL IceSpawnLoop

SkipIceSpawn:	LDX $15E9		; Get current sprite index
		LDA $1540,x		; Get state timer
		CMP #$00		; Time ran up?
		BNE IceLanceEnd
		LDA $1504,x		; Load state
		AND #$F0		; Get Hitpoints
		STA $1504,x
		LDA !UNDINE_MOVETIME
		STA $1540,x
		LDA $1602,x
		AND #$04
		STA $02
		LDA #$08		; Change to moving animation
		ORA $02
		STA $1602,x
		STZ $00C2,x		; Other animations have only 1 frame
		STZ $1528,x		; Clear spawned flag
IceLanceEnd:	JSR CheckFrostBlk
		RTS

; ============
; Hit Function
; ============

HitFunc:	LDA $1540,x		; Get state timer
		CMP #$00		; Time ran up?
		BNE HitEnd
		LDA $1504,x		; Load state
		AND #$F0		; Get Hitpoints
		STA $1504,x
		LDA !UNDINE_MOVETIME
		STA $1540,x
		LDA $1602,x
		AND #$04
		STA $02
		LDA #$08		; Change to moving animation
		ORA $02
		STA $1602,x
HitEnd:		RTS

; ============
; GFX FUNCTION
; ============

UNDINE_TILEMAP:
UNDINE_TILEMAPRIGHTCAST1:	db $80,$82,$C4,$C6
UNDINE_TILEMAPLEFTCAST1:	db $80,$82,$A0,$A2
UNDINE_TILEMAPRIGHT:		db $84,$86,$C0,$C2
UNDINE_TILEMAPLEFT:		db $84,$86,$A4,$A6
UNDINE_TILEMAPRIGHTHIT:		db $C8,$CA,$C4,$C6
UNDINE_TILEMAPLEFTHIT:		db $C8,$CA,$A0,$A2
UNDINE_TILEMAPRIGHTBOW:		db $80,$82,$C4,$C6
UNDINE_TILEMAPLEFTBOW:		db $80,$82,$A0,$A2
UNDINE_TILEMAPRIGHTCAST2:	db $88,$8A,$AA,$C2
UNDINE_TILEMAPLEFTCAST2:	db $88,$8A,$A8,$A6
!UNDINE_TILESCOUNT		=  #$03

UNDINE_TILEXPOS: db $FA,$0A,$FA,$0A
UNDINE_TILEYPOS: db $00,$00,$10,$10

UNDINE_GFX:	JSR GET_DRAW_INFO	; Get necessary OAM Pointer, Y = OAM Index, X = Sprite Index
		PHX			; Push X to Stack
		LDX !UNDINE_TILESCOUNT	; X is loop counter, X = Loop Counter
TILE_LOOP:	LDA $00			; Get X Position
		CLC			; Clear Carry
		ADC UNDINE_TILEXPOS,x	; Calculate X Offset
		STA $0300,y		; Store X Offset
		LDA $01			; Get Y Position
		CLC			; Clear Carry
		ADC UNDINE_TILEYPOS,x	; Calculate Y Offset
		STA $0301,y		; Store Y Offset
		TXA			; Transfer X to A
		PHX 			; Push X to Stack
		LDX $15E9		; Get current Sprite index
		STA $02			; Store current Tile number
		LDA $15F6,x		; Get the palette from table
		ORA $64			; Take care of priorities
		STA $0303,y		; Store properity byte
		LDA $14			; Get Frame Counter
		AND $00C2,x		; Get Animation Frame Index
		LSR
		ORA $1602,x		; Add in animation information
		ORA $02			; Add in tile information
		TAX			; Transfer A to X
		LDA UNDINE_TILEMAP,x	; Load current tile from Map
		PLX			; Pull X from Stack
		STA $0302,y		; Store current Frame tile
		INY			; Next Tile
		INY
		INY
		INY
		DEX			; Next Tile
		BPL TILE_LOOP		; Branch if X positive
		PLX			; Pull X from Stack
		LDA !UNDINE_TILESCOUNT	; Load the amount of tiles
		LDY #$02		; Select 16x16 tiles drawing
		JSL $01B7B3		; Call tile render function
		RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:		db $40,$B0
SPR_T13:		db $01,$FF
SPR_T14:		db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
			db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:		db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
			db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:	    STA $03		    ;  |            
		    BRA START_SUB	    ;  |
SUB_OFF_SCREEN_X0:  STZ $03		    ; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS:		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16:				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642
